-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
-- Helper for the programmer.
--

require("strict")
require("utils")
class = require('middleclass')

local Database = class('Database')

local sqlite3 = require("lsqlite3")



-- update the schema
function Database:_update_db_schema()
    -- read the db version, if we cannot then it must be new
    local db_version = 0
    
    local DBInfo_exists = false
    local function DBInfo_exists_callback() DBInfo_exists = true return 0 end
    self.db:exec("SELECT name FROM sqlite_master WHERE type='table' AND name='DBInfo'", DBInfo_exists_callback)
    
    if DBInfo_exists then
        for v in self.db:urows("SELECT Version FROM DBInfo WHERE Key=1") do
            db_version = v
        end
    end
    
    debugMessage("Read db_version "..db_version.." from db")
    
    if db_version < 1 then
        -- create schema for v1
        debugMessage("Create schema for db_version 1")
        self.db:exec "CREATE TABLE DBInfo (Key INTEGER PRIMARY KEY DEFAULT 1, Version INTEGER DEFAULT 0)"
        self.db:exec "INSERT OR REPLACE INTO DBInfo DEFAULT VALUES"
        
        self.db:exec "CREATE TABLE EyebeastEscapeProgress (Player STRING PRIMARY KEY, Level INTEGER)"
        
        db_version = 1
    end
    
    if db_version < 2 then
        -- create schema for v2
        debugMessage("Create schema for db_version 2")
        
        -- schema changes for v2
        self.db:exec "CREATE TABLE EyebeastEscapeTimes (Player STRING, Level STRING , Time INTEGER, PRIMARY KEY (Player, Level))"
        
        db_version = 2
    end
    
    --[[
    if db_version < x then
        -- create schema for vx
        -- debugMessage("Create schema for db_version x")
        -- schema changes for vx
        -- db_version = x
    end
    --]]
    
    -- for each new
    self.db:exec("UPDATE DBInfo SET Version="..db_version.." WHERE Key=1")

end

function Database:initialize(db_filename)

    -- will open (or create) the db file
    self.db = sqlite3.open(gulp_cpp.SaveDataPath(db_filename):str())

    -- update the schema
    self:_update_db_schema()
end

function Database:get_eyebeast_escape_progress(player_id)
    for level in self.db:urows("SELECT Level FROM EyebeastEscapeProgress WHERE Player = '"..get_player_name(player_id).."'") do
        return level
    end
end

function Database:save_eyebeast_escape_progress(player_id, level)
    self.db:exec("INSERT OR REPLACE INTO EyebeastEscapeProgress (Player, Level) VALUES ('"..get_player_name(player_id).."', '"..level.."')")
end

function Database:get_eyebeast_escape_level_times(player_id, level)
    -- get the world record and the personal best for the given level
    local wr = 999.9
    local pb = 999.9
    
    -- get world record
    for t in self.db:urows("SELECT Time FROM EyebeastEscapeTimes WHERE Level = '"..level.."'") do
        if t < wr then wr = t end
    end
    
    -- get personal best
    for t in self.db:urows("SELECT Time FROM EyebeastEscapeTimes WHERE Level = '"..level.."' AND Player = '"..get_player_name(player_id).."'") do
        if t < pb then pb = t end
    end
    
    return wr, pb
    
end

function Database:save_eyebeast_escape_time(player_id, level, time)
    self.db:exec("INSERT OR REPLACE INTO EyebeastEscapeTimes (Player, Level, Time) VALUES ('"..get_player_name(player_id).."', '"..level.."', "..time..")")
end

return Database

