-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

require("strict")
require("library/SDL_scancode")

local ActionInput = {
    GAME_UP_ACTION = 0x01,
    GAME_DOWN_ACTION = 0x02,
    GAME_LEFT_ACTION = 0x04,
    GAME_RIGHT_ACTION = 0x08,
    GAME_FIRE_ACTION = 0x10,
    GAME_MENU_ACTION = 0x20,
    GAME_EMOTE_ACTION = 0x40,
    }

local key_action = {}
local action_map = 0
local move_map = 0
local fire_map = 0

local MOVE_MAP = 0x0F
local FIRE_MAP = 0x70

function ActionInput.init_movement_keys()
    
    key_action[SDL_SCANCODE.UP] = ActionInput.GAME_UP_ACTION
    key_action[SDL_SCANCODE.W] = ActionInput.GAME_UP_ACTION
    key_action[SDL_SCANCODE._7] = ActionInput.GAME_UP_ACTION

    key_action[SDL_SCANCODE.DOWN] = ActionInput.GAME_DOWN_ACTION
    key_action[SDL_SCANCODE.S] = ActionInput.GAME_DOWN_ACTION
    key_action[SDL_SCANCODE._6] = ActionInput.GAME_DOWN_ACTION

    -- left keys
    key_action[SDL_SCANCODE.LEFT] = ActionInput.GAME_LEFT_ACTION
    key_action[SDL_SCANCODE.A] = ActionInput.GAME_LEFT_ACTION
    key_action[SDL_SCANCODE._5] = ActionInput.GAME_LEFT_ACTION

    -- right keys
    key_action[SDL_SCANCODE.RIGHT] = ActionInput.GAME_RIGHT_ACTION
    key_action[SDL_SCANCODE.D]     = ActionInput.GAME_RIGHT_ACTION
    key_action[SDL_SCANCODE._8]     = ActionInput.GAME_RIGHT_ACTION

    -- fire keys
    key_action[SDL_SCANCODE.SPACE] = ActionInput.GAME_FIRE_ACTION
    key_action[SDL_SCANCODE._9]     = ActionInput.GAME_FIRE_ACTION
    key_action[SDL_SCANCODE.KP_0]  =  ActionInput.GAME_FIRE_ACTION
    
    -- game menu key
    key_action[SDL_SCANCODE.RETURN]   = ActionInput.GAME_MENU_ACTION
    key_action[SDL_SCANCODE._0]        = ActionInput.GAME_MENU_ACTION
    key_action[SDL_SCANCODE.KP_ENTER] =  ActionInput.GAME_MENU_ACTION
    
    -- emote key
    key_action[SDL_SCANCODE._4]     = ActionInput.GAME_EMOTE_ACTION
    key_action[SDL_SCANCODE.E]      = ActionInput.GAME_EMOTE_ACTION
    key_action[SDL_SCANCODE.KP_PLUS]= ActionInput.GAME_EMOTE_ACTION

    -- diagonals
    key_action[SDL_SCANCODE.KP_7] =  ActionInput.GAME_UP_ACTION + ActionInput.GAME_LEFT_ACTION
    key_action[SDL_SCANCODE.KP_9] =  ActionInput.GAME_UP_ACTION + ActionInput.GAME_RIGHT_ACTION
    key_action[SDL_SCANCODE.KP_1] =  ActionInput.GAME_DOWN_ACTION + ActionInput.GAME_LEFT_ACTION
    key_action[SDL_SCANCODE.KP_3] =  ActionInput.GAME_DOWN_ACTION + ActionInput.GAME_RIGHT_ACTION
    
    key_action[SDL_SCANCODE.KP_4] =  ActionInput.GAME_LEFT_ACTION
    key_action[SDL_SCANCODE.KP_8] =  ActionInput.GAME_UP_ACTION
    key_action[SDL_SCANCODE.KP_2] =  ActionInput.GAME_DOWN_ACTION
    key_action[SDL_SCANCODE.KP_6] =  ActionInput.GAME_RIGHT_ACTION

    action_map = 0 
    move_map = 0
    fire_map = 0
end

require("bit32")

local function game_move_processing(action)

    if gulp.movetarget then
        -- split action_map into move and fire maps
        local old_move_map = move_map
        local old_fire_map = fire_map
        
        move_map = bit32.band(action_map, MOVE_MAP)
        fire_map = bit32.band(action_map, FIRE_MAP)
        
        if gulp.movetarget.move and move_map ~= old_move_map then
            gulp.movetarget:move(move_map)
        end
        
        if gulp.movetarget.fire and fire_map ~= old_fire_map then
            gulp.movetarget:fire(fire_map)
        end
        
    end

end

-- we've separated this in order to allow us to tweak the key processing
function ActionInput.game_key_processing(scancode, down_event)
    
    -- if it's one of the game keys, otherwise ignore, don't reprocess
    local action = key_action[scancode]
    if action then

        local old_action_map = action_map
        
        if down_event then
            action_map = bit32.bor(action_map, action)
        else
            action_map = bit32.band(action_map, bit32.bnot(action))
        end

        if old_action_map == action_map then
            -- don't send new events up, since this might misinterpret this
            -- as a new action, hence guaranteeing one move minimum.
            -- This has the advantage over a repeat key eliminator because
            -- it also covers the 'press-right two ways' problems.
            return
        end
        game_move_processing(action, down_event)
    end
end

function ActionInput.process_mouse(action)
    local old_action_map = action_map
    action_map = action
    game_move_processing(action)
end


-- action targets ... movetarget:
--  move_up
--  move_down
--  move_right
--  move_left
--  fire
--  no_fire
--  no_movement


return ActionInput
