-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

local internal_avatars = {
    -- avatar, playable
    fox = { require("avatars/fox_avatar"), true },
    napple = { require("avatars/napple_avatar"), true },
    eyebeast = { require("avatars/eyebeast_avatar"), true },
    minotaur = { require("avatars/minotaur_avatar"), true },
    gangsta = { require("avatars/gangsta_avatar"), true },
    evil_egg = { require("avatars/evil_egg_avatar"), true },
    gulpy = { require("avatars/gulpy_avatar"), true },
    nurse = { require("avatars/nurse_avatar"), true },
    mayor = { require("avatars/mayor_avatar"), true },
    smily = { require("avatars/smilyface_avatar"), true },
    soph = { require("avatars/dragon_avatar"), true },
    
    slime = { require("avatars/slime_avatar"), false },
    kobold = { require("avatars/kobold_avatar"), false },
}

local avatars = {}

for key,line in pairs(internal_avatars) do
    
    -- should be using the glyph size of the glyph set the avatar is loaded from
    
    local avatar = line[1]
    local playable = line[2]
    avatar.gr_center_x = avatar.avatar_center_in_cells[1]
    avatar.gr_center_y = avatar.avatar_center_in_cells[2]
    avatar.gr_size_x = avatar.avatar_size_in_cells[1]
    avatar.gr_size_y = avatar.avatar_size_in_cells[2]
    
    avatar.playable_character = playable
    avatars[key] = avatar
    
    -- animation
    avatar.animation_sets = 
    { 
        ['e'] = avatar.east_animation or { avatar.graphic_unicode_point }, 
        ['s'] = avatar.south_animation or { avatar.graphic_unicode_point },
        ['n'] = avatar.north_animation or { avatar.graphic_unicode_point },
        ['w'] = avatar.west_animation or { avatar.graphic_unicode_point },
        ['e_st'] = avatar.east_still_animation or { avatar.graphic_unicode_point },
        ['w_st'] = avatar.west_still_animation or { avatar.graphic_unicode_point },
        ['n_st'] = avatar.north_still_animation or { avatar.graphic_unicode_point },
        ['s_st'] = avatar.south_still_animation or { avatar.graphic_unicode_point },
    }

    avatar.animation_length = 
    {
        ['e'] = #avatar.animation_sets['e'], 
        ['s'] = #avatar.animation_sets['s'],
        ['n'] = #avatar.animation_sets['n'],
        ['w'] = #avatar.animation_sets['w'],
        ['e_st'] = #avatar.animation_sets['e_st'],
        ['w_st'] = #avatar.animation_sets['w_st'],
        ['n_st'] = #avatar.animation_sets['n_st'],
        ['s_st'] = #avatar.animation_sets['s_st'],
    }
    
    avatar.walk_distance_per_animation_cycle = avatar.walk_distance_per_animation_cycle or 1
    avatar.time_for_stationary_animation_cycle = avatar.time_for_stationary_animation_cycle or 1
    
    -- handle compound glyphs
    if avatar.compound_glyphs then
        avatar.compound_animation_set = {}
        avatar.compound_animation_length = {}
        avatar.compound_glyph_visible = {}
        avatar.compound_animation_layer = {}
        avatar.compound_animation_column_offset = {}
        avatar.compound_animation_line_offset = {}
        avatar.compound_animation_stationary_time = {}
        avatar.compound_animation_distance = {}
        
        for _, value in ipairs(avatar.compound_glyphs) do
            local name = value.name
            
            avatar.compound_glyph_visible[name] = value.initially_visible or false
            avatar.compound_animation_stationary_time[name] = value.compound_animation_stationary_time
            avatar.compound_animation_distance[name] = value.compound_animation_distance
            
            avatar.compound_animation_set[name] = 
            {
                ['e'] = value.east_animation, 
                ['s'] = value.south_animation,
                ['n'] = value.north_animation,
                ['w'] = value.west_animation ,
                ['e_st'] = value.east_still_animation,
                ['w_st'] = value.west_still_animation,
                ['n_st'] = value.north_still_animation,
                ['s_st'] = value.south_still_animation,
            }
            avatar.compound_animation_layer[name] = 
            {
                ['e'] = value.east_animation.layer, 
                ['s'] = value.south_animation.layer,
                ['n'] = value.north_animation.layer,
                ['w'] = value.west_animation.layer ,
                ['e_st'] = value.east_still_animation.layer,
                ['w_st'] = value.west_still_animation.layer,
                ['n_st'] = value.north_still_animation.layer,
                ['s_st'] = value.south_still_animation.layer,
            }
            avatar.compound_animation_line_offset[name] = 
            {
                ['e'] = value.east_animation.line_offset or 0, 
                ['s'] = value.south_animation.line_offset or 0,
                ['n'] = value.north_animation.line_offset or 0,
                ['w'] = value.west_animation.line_offset or 0,
                ['e_st'] = value.east_still_animation.line_offset or 0,
                ['w_st'] = value.west_still_animation.line_offset or 0,
                ['n_st'] = value.north_still_animation.line_offset or 0,
                ['s_st'] = value.south_still_animation.line_offset or 0,
            }
            avatar.compound_animation_column_offset[name] = 
            {
                ['e'] = value.east_animation.column_offset or 0, 
                ['s'] = value.south_animation.column_offset or 0,
                ['n'] = value.north_animation.column_offset or 0,
                ['w'] = value.west_animation.column_offset or 0,
                ['e_st'] = value.east_still_animation.column_offset or 0,
                ['w_st'] = value.west_still_animation.column_offset or 0,
                ['n_st'] = value.north_still_animation.column_offset or 0,
                ['s_st'] = value.south_still_animation.column_offset or 0,
            }
            avatar.compound_animation_length[name] = 
            {
                ['e'] = #avatar.compound_animation_set[name]['e'], 
                ['s'] = #avatar.compound_animation_set[name]['s'],
                ['n'] = #avatar.compound_animation_set[name]['n'],
                ['w'] = #avatar.compound_animation_set[name]['w'],
                ['e_st'] = #avatar.compound_animation_set[name]['e_st'],
                ['w_st'] = #avatar.compound_animation_set[name]['w_st'],
                ['n_st'] = #avatar.compound_animation_set[name]['n_st'],
                ['s_st'] = #avatar.compound_animation_set[name]['s_st'],
            }
        end
    end
end

return avatars
