-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

local avatars = require("avatars/avatar_loader")
local TXT = require("text_strings")

local gold_coin_glyph = to_utf8(get_glyph('blackwall', 159))

local colour_coded_avatar = 'evil_egg'

--local menu_type = 'new'
local menu_type = 'old'

local function _select_avatar(arg)
    
    if menu_type == 'old' then
        gulp.selected_avatar = arg
        gulp.selected_colour = gulp.colour.BRIGHT_WHITE.simple_colour
    else
        local colours = 
        {
            ['31'] = gulp.colour.RED.simple_colour,
            ['32'] = gulp.colour.GREEN.simple_colour,
            ['33'] = gulp.colour.YELLOW.simple_colour,
            ['34'] = gulp.colour.BLUE.simple_colour,
        }
        -- record the selected avatar
        gulp.selected_avatar = colour_coded_avatar
        gulp.selected_colour = colours[arg]
    end
    
    -- close the main menu, and sub-menus
    --
    -- this is all a bit of a hack. 
    -- but much better than it was!
    
    if gulp.main_menu then
        gulp.main_menu:close()
    end
    
    -- start the game proper
    gulp.start_game()
    
    return false -- not nested menu (close original menu)

end


local function avatar_menu(game_name)

    local menu
    
    if menu_type == 'old' then
        menu = Menu{
            background = gulp.colour.BLUE.simple_colour, 
            foreground = gulp.colour.BRIGHT_WHITE.simple_colour,
            select_colour = gulp.colour.MAGENTA.simple_colour,
            border_style = 2,
            line = 6,
            column = 2,
            { "Pick your avatar " },
            { "" },
        }
    else
        menu = Menu{
            background = gulp.colour.BLACK.simple_colour, 
            foreground = gulp.colour.BRIGHT_WHITE.simple_colour,
            select_colour = gulp.colour.MAGENTA.simple_colour,
            border_style = 2,
            line = 12,
            column = 6,
            { "Pick your colour " },
        }
    end
    
    if game_name ~= nil then
        menu:insert_item(1, { "Game code: "..game_name } )
        menu:insert_item(2, { "" } )
    end
    
    
    if menu_type == 'old' then
        for key,avatar in pairs(avatars) do
            if avatar.playable_character then
                local glyph = to_utf8(avatar.graphic_unicode_point)
                
                -- strings in menus should use bright white for fg colour
                -- this is because the engine has changed - the fg colour no longer selects 
                -- a different texture and instead modulates the overall colour of the glyph
                
                local string = " \x1b[37m\x1b[1m" .. glyph .. "\x1b[37m\x1b[1m " .. avatar.avatar_name .. TXT.name_type_concat .. avatar.avatar_type
                local entry = {string, _select_avatar, key }
                menu:insert_item(entry)
            end
        end
    else
        local colours = 
        {
            ['31'] = 'Red',
            ['32'] = 'Green',
            ['33'] = 'Yellow',
            ['34'] = 'Blue',
        }
        for key,colour in pairs(colours) do
            local avatar = avatars[colour_coded_avatar]
            local glyph = to_utf8(avatar.graphic_unicode_point)
            local string = " \x1b[" .. key .. "m" .. glyph .. "\x1b[37m\x1b[1m " .. colour 
            local entry = {string, _select_avatar, key }
            menu:insert_item(entry)
        end
    end
    
    -- Look for escape key, if the user aborts the menu then need to clean up
    menu.keypressed = function(self, sym)
        if sym == SDLK.ESCAPE then
            self:close()
            gulp.end_game()
            return true
        end
        return Menu.keypressed(self, sym) -- will allow other menu to get keypress
    end    
    
    return true     -- make this a nested submenu of main menu
end


return avatar_menu
