-- Forlorn Fox
-- Copyright © 2017 Rob Probin.
-- No redistribution without permission.
--

--local check_debugger = require("debug_select")
--check_debugger("boot")
local exec_functions = require("utilities/exec_functions")

-- Return: number returned assumes that we should quit, and return that number to the OS.
function gulp.boot(app)
    --gulp_cpp.enable_print_time_since_last_call()    -- used to enable/debug boot timing
    local STARTUP_PAUSE_FOR_DEBUGGER = false
    if STARTUP_PAUSE_FOR_DEBUGGER then
        usleep(5000 * 1000)
    end

    local argc = #arg
    
    local renderer_index = -1;    -- index of rendering driver to use, or -1 for the first one supporting the requested flags
    
    gulp.nogui = false

    if argc == 1 and arg[1] == "-v" then
        
        -- print version
        gulp_cpp.debugMessage(string.format("%s", gulp.forlorn_fox_engine_version))
        return 0

    elseif argc >= 2 then

        for i = 1, (argc-1) do  -- (argc-1) because needs parameter afterwards
            if arg[i] == "--renderer" then
                renderer_index = tonumber(arg[i+1])
                -- -1 to initialize the first one supporting the requested flags
                if not renderer_index then renderer_index = -1 end
                gulp_cpp.debugMessage(string.format("Selecting renderer: %d", renderer_index))
            end
        end
        
    end
    
    for i = 1, argc do 
        if arg[i] == "--nogui" then
            gulp.nogui = true
        end
    end

    --debugMessage(gulp_cpp.SaveDataPath:get_base_pref_path())
    
    
    --gulp_cpp.print_time_since_last_call("+ARGS ")

    -- The file I/O and threading subsystems are initialized by default. 
    -- Logging works without initialization, too. 
    -- Can we init AUDIO from the terminal?
    local s = gulp.SDL_SubSystems
    if gulp_cpp.SDL_InitSubSystem(s.SDL_INIT_EVENTS + s.SDL_INIT_TIMER) ~= 0 then
        gulp_cpp.debugMessage("Could initialize Event+Timer SDL services\n")
        return 1
    end

    -- seed random number generator
    gulp_cpp.initialise_random_seed()

    if gulp.nogui then
        return "nogui"
    end
    
    -- SDL_INIT_EVENTS is initialised with a call to SDL_INIT_VIDEO
    -- Should we initialise SDL_INIT_JOYSTICK and SDL_INIT_HAPTIC and SDL_INIT_GAMECONTROLLER ?
    -- or SDL_INIT_EVERYTHING? 
    -- Systems are ref counted... see https://wiki.libsdl.org/SDL_InitSubSystem
    if gulp_cpp.SDL_InitSubSystem(s.SDL_INIT_VIDEO + s.SDL_INIT_AUDIO + s.SDL_INIT_JOYSTICK + s.SDL_INIT_GAMECONTROLLER) ~= 0 then
        gulp_cpp.debugMessage("Could initialize Video+Audio_Init SDL services\n")
        gulp_cpp.debugMessage(gulp_cpp.SDL_GetError())
        return 1
    end

    -- allow override of switching
    local k = gulp_cpp.SDL_GetModState()        -- keyboard subsystem is inited by video init.
    if (bit32.band(gulp.SDL_Keymod.KMOD_LSHIFT, k) or bit32.band(gulp.SDL_Keymod.KMOD_RSHIFT, k)) == 0 then
        exec_functions.execute_run_alt()
    end
    
    -- create window and renderer
    if (not gulp_cpp.CreateGameWindow()) then
        gulp_cpp.debugMessage("Could not create game window\n")
        return 1
    end
    
    local vsync_guess = true
    local r = gulp.SDL_RendererFlags
    local renderer = gulp_cpp.SDL_CreateRenderer(gulp_cpp.GetGameWindow(), renderer_index, bit32.bor(r.SDL_RENDERER_PRESENTVSYNC, r.SDL_RENDERER_ACCELERATED))
    if renderer == 0 then

        gulp_cpp.debugMessage("Accelerated+Vsync renderer not avaiable. SDL says:\n")
        gulp_cpp.debugMessage(gulp_cpp.SDL_GetError())
        gulp_cpp.debugMessage("\n")
        --accept vsync renderer
        renderer = gulp_cpp.SDL_CreateRenderer(gulp_cpp.GetGameWindow(), renderer_index, r.SDL_RENDERER_PRESENTVSYNC); -- 0)
        if renderer == 0 then
            gulp_cpp.debugMessage("Vsync renderer not avaiable. SDL says:\n")
            gulp_cpp.debugMessage(gulp_cpp.SDL_GetError())
            gulp_cpp.debugMessage("\n")
            -- accept any
            renderer = gulp_cpp.SDL_CreateRenderer(gulp_cpp.GetGameWindow(), renderer_index, 0)
            vsync_guess = false
        end
    end
    if renderer == 0 then
        gulp_cpp.debugMessage("Could not create renderer. SDL says:\n")
        gulp_cpp.debugMessage(gulp_cpp.SDL_GetError())
        gulp_cpp.debugMessage("\n")
        return 1
    end
    
    -- clear the screen immediately so that any start up delay is 'mitigated'
    -- on Linux windows construction, so that we don't end up with horrid 'background' window.
    gulp_cpp.SDL_RenderClear(renderer)
    gulp_cpp.SDL_RenderPresent(renderer)


    app:SetRenderer(renderer, vsync_guess)

    gulp_cpp.UpdateScreenData_game_window()
    --gulp_cpp.print_time_since_last_call("+SDL/Window/Renderer ")

    -- return nil
end
