-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

-- This is the main Forlorn Fox version
gulp.forlorn_fox_version = "0.09-DEV"

-- change the SaveDataPath to the correct path
gulp_cpp.SaveDataPath.set_org_name("RedGameWorks")
gulp_cpp.SaveDataPath.set_app_name("ForlornFox")
gulp_cpp.SaveDataPath.set_subfolder(gulp.forlorn_fox_version)

local function override_paths(base_path, object_file_extension)
    --
    -- now we override PATH and CPATH
    gulp_cpp.AppResourcePath.override(base_path .. 'data/')

    package.path = 
    gulp_cpp.AppResourcePath("scripts/?.lua"):str() .. ";" ..
    gulp_cpp.AppResourcePath("scripts/?/init.lua"):str() .. ";" ..
    gulp_cpp.AppResourcePath("scripts/thirdparty/?.lua"):str() .. ";" ..
    gulp_cpp.AppResourcePath("scripts/thirdparty/?/init.lua"):str()

    -- do we allow binary modules to be overriden?
    -- maybe bad for certain store rules
    package.cpath =
    gulp_cpp.AppResourcePath("cmodules/?" .. object_file_extension):str() ..  ";" ..
    gulp_cpp.AppResourcePath("cmodules/?/loadall" .. object_file_extension):str()

end

local function add_save_path()
    package.path =
    gulp_cpp.SaveDataPath("scripts/?.lua"):str() .. ";" ..
    gulp_cpp.SaveDataPath("scripts/?/init.lua"):str() .. ";" ..
    gulp_cpp.SaveDataPath("scripts/thirdparty/?.lua"):str() .. ";" ..
    gulp_cpp.SaveDataPath("scripts/thirdparty/?/init.lua"):str() .. ";" ..
    package.path
end

-- create a map of which parameters exist
argk = {} for k,v in ipairs(arg) do argk[v]=k end

-- Replace the paths on a debugging build with the source data directory
--
if argk['--DEVELOPER_PATHS'] then
    gulp_cpp.SaveDataPathDeveloper("ignore"):set_developer_mode(true)

    local base_path = gulp_cpp.AppResourcePath(""):str()
--print("PREVIOUS PATHS:")
    --print("AppResourcePath=", base_path)
--print("package.path=", package.path)
--print("package.cpath=", package.cpath)

    local piece_to_remove, lib_extension
    if gulp.platform == "MacOSX" then
        piece_to_remove = "Forlorn_Fox_Mac"
        lib_extension = ".so"
    elseif gulp.platform == "Linux" then
        piece_to_remove = "Gulpman_Linux"
        lib_extension = ".so"
    elseif gulp.platform == "Windows" then
        piece_to_remove = "Forlorn_Fox_Win"
        lib_extension = ".dll"
    elseif gulp.platform == "OpenPandora" then
        piece_to_remove = "Forlorn_Fox_Pandora"
        lib_extension = ".so"
    else
        print("--DEVELOPER_PATHS: Unknown platform", gulp.platform)
        os.exit(1) -- ok to abort, developer mode
    end

    local index = base_path:find(piece_to_remove, 1, true)
    if index then
        local new_path = base_path:sub(1, index-1)
        --debugMessage(new_path)
        override_paths(new_path, lib_extension)
    end

end

add_save_path()

--[[
print("OVERRRIDEN PATHS:")
print("AppResourcePath=", gulp_cpp.AppResourcePath("app.txt"):str())
print("package.path=", package.path)
print("package.cpath=", package.cpath)
print("SaveDataPath=", gulp_cpp.SaveDataPath("save.txt"):str())
print("LoadPath=", gulp_cpp.LoadPath("load.txt"):str())
print("SaveDatPathDeveloper=", gulp_cpp.SaveDataPathDeveloper("dev.txt"):str())
--]]


