-- Forlorn Fox
-- Copyright © 2016 Rob Probin.
-- No redistribution without permission.
--
-- Kobold
-- Created by Rob Probin, Sept 2016
--
require('strict')
class = require('middleclass')

local CavesMob = require("controllers/caves/caves_mob")

local Kobold = class('Kobold', CavesMob)

require('controllers/collisions')


-- Rob's Kobold
--local east_animation = { get_glyph("caves1",0x06), get_glyph("caves1",0x07), get_glyph("caves1",0x08), get_glyph("caves1",0x07) }

-- legs
--local east_animation = { get_glyph("caves1",0x48), get_glyph("caves1",0x49), get_glyph("caves1",0x4A), get_glyph("caves1",0x4B), 
--                         get_glyph("caves1",0x4C), get_glyph("caves1",0x4D), get_glyph("caves1",0x4E), get_glyph("caves1",0x4F)}

-- Manic miner
--local east_animation = { get_glyph("caves1",0x30), get_glyph("caves1",0x31), get_glyph("caves1",0x32), get_glyph("caves1",0x31) }

-- Manic miner++
--local east_animation = { get_glyph("caves1",0x40), get_glyph("caves1",0x41), get_glyph("caves1",0x42), get_glyph("caves1",0x43), 
--                         get_glyph("caves1",0x44), get_glyph("caves1",0x45), get_glyph("caves1",0x46), get_glyph("caves1",0x47) }

-- Manic miner+++
--local east_animation = { get_glyph("caves1",0x50), get_glyph("caves1",0x51), get_glyph("caves1",0x52), get_glyph("caves1",0x53), 
--                         get_glyph("caves1",0x54), get_glyph("caves1",0x55), get_glyph("caves1",0x56), get_glyph("caves1",0x57) }

-- Tony's Kobold
local east_animation = { get_glyph("caves1",0x60), get_glyph("caves1",0x61), get_glyph("caves1",0x62), get_glyph("caves1",0x63), 
                         get_glyph("caves1",0x64), get_glyph("caves1",0x65), get_glyph("caves1",0x66), get_glyph("caves1",0x67) }
local west_animation = { get_glyph("caves1",0x70), get_glyph("caves1",0x71), get_glyph("caves1",0x72), get_glyph("caves1",0x73), 
                         get_glyph("caves1",0x74), get_glyph("caves1",0x75), get_glyph("caves1",0x76), get_glyph("caves1",0x77) }
local south_animation= { get_glyph("caves1",0x68), get_glyph("caves1",0x69), get_glyph("caves1",0x6a), get_glyph("caves1",0x6b), 
                         get_glyph("caves1",0x6c), get_glyph("caves1",0x6d), get_glyph("caves1",0x6e), get_glyph("caves1",0x6f) }
local north_animation= { get_glyph("caves1",0x78), get_glyph("caves1",0x79), get_glyph("caves1",0x7a), get_glyph("caves1",0x7b), 
                         get_glyph("caves1",0x7c), get_glyph("caves1",0x7d), get_glyph("caves1",0x7e), get_glyph("caves1",0x7f) }

--local south_animation = { get_glyph("caves1",0x26), get_glyph("caves1",0x27) }
--local north_animation = { get_glyph("caves1",0x36), get_glyph("caves1",0x37) }
--local west_animation = { get_glyph("caves1",0x16), get_glyph("caves1",0x17), get_glyph("caves1",0x18), get_glyph("caves1",0x17) }
--local west_animation = { get_glyph("caves1",0x16), get_glyph("caves1",0x17), get_glyph("caves1",0x18), get_glyph("caves1",0x17) }

local animation_sets = 
{ 
    ['e'] = east_animation, 
    ['s'] = south_animation,
    ['n'] = north_animation,
    ['w'] = west_animation,
}

local animation_length = 
{
    ['e'] = #east_animation, 
    ['s'] = #south_animation,
    ['n'] = #north_animation,
    ['w'] = #west_animation,
}

local walk_distance_per_animation_cycle = (6/16)
local walk_distance_per_animation_cycle = (12/16)

function Kobold.adjust_animation(adjust)
    walk_distance_per_animation_cycle = walk_distance_per_animation_cycle + adjust
end



function Kobold:initialize(map_name, combat_data)
    
    
    CavesMob.initialize(self, "kobold", map_name, combat_data)
    
    self.mode = "e"
    self.animation_distance_walked = 0
end

function Kobold:walk_until_wall(distance_NS, distance_EW)
    local offset_line = self.avatar_data.gr_center_y 
    local offset_column = self.avatar_data.gr_center_x
    if distance_NS > 0 then
        offset_line = offset_line + self.avatar_data.gr_size_y / 2
    elseif distance_NS < 0 then
        offset_line = offset_line - self.avatar_data.gr_size_y / 2
    end
    if distance_EW > 0 then
        offset_column = offset_column + self.avatar_data.gr_size_x / 2
    elseif distance_EW < 0 then
        offset_column = offset_column - self.avatar_data.gr_size_x / 2
    end
    
    -- not overly efficient as a loop ... do as a calculation?
    local step = 0.1
    local d_NS, d_EW
    while distance_NS ~= 0 or distance_EW ~= 0 do
        if math.abs(distance_NS) > step then 
            d_NS = step
            distance_NS = distance_NS - step
        else
            d_NS = distance_NS
            distance_NS = 0
        end
        if math.abs(distance_EW) > step then 
            d_EW = step
            distance_EW = distance_EW - step
        else
            d_EW = distance_EW
            distance_EW = 0
        end
        local line = self.line + d_NS
        local column = self.column + d_EW
        --local l = line+offset_line
        --local c = column+offset_column
        --gulp.debug_print_from_server("Kobold:walk_until_wall", "mazerect", c-0.2, l-0.2, c+0.2, l+0.2, "MAGENTA") -- .. math.random(1000)
        --local l_ = line + self.avatar_data.gr_center_y
        --local c_ = column + self.avatar_data.gr_center_x
        --gulp.debug_print_from_server("Kobold:center", "mazerect", c_-0.2, l_-0.2, c_+0.2, l_+0.2, "YELLOW")
        if is_wall(self.map_name, line+offset_line, column+offset_column) then
            return true
        end
        
        local collision = get_collision("Mob"..self.mob_number, self.map_name, self.line, self.column, d_NS, d_EW) 
        if collision then
            return false
        end
       
        self.line = line
        self.column = column
    end
    return false
end

function Kobold:change_direction(new_direction)
    self.mode = new_direction
    self.pause = 0.2 + math.random(10) * 0.1
    
    self.animation_distance_walked = 0
end

function Kobold:walk_animation(distance)
    
    local distance_per_frame = (walk_distance_per_animation_cycle / animation_length[self.mode])
    
    self.animation_distance_walked = self.animation_distance_walked + math.abs(distance)
    
    local frame = math.floor((self.animation_distance_walked / distance_per_frame) % animation_length[self.mode]) + 1
    self:set_glyph(1, animation_sets[self.mode][frame])
    
end

    

function Kobold:update(dt)
    
    if self.pause then
        self.pause = self.pause - dt
        if self.pause >= 0 then     -- equal to so we don't get a zero dt
            return
        end
        dt = -self.pause
        self.pause = nil
    end
    
    local walk_speed = 1
    local walk_distance = dt * walk_speed
    
    --gulp.debug_print_from_server("kobold", "text", 2, 2, string.format("%f %f", self.line, self.column))

    if self.mode == "e" then    -- patrol east
        if self.column >= 15 then
            self:change_direction("s")
        else
            self:walk_animation(walk_distance)
            local wall = self:walk_until_wall(0, walk_distance)
            if wall then self:change_direction("s") end
        end
    elseif self.mode == "s" then    -- patrol south
        if self.line >= 7 then
            self:change_direction("w")
        else
            self:walk_animation(walk_distance)
            local wall = self:walk_until_wall(walk_distance, 0)
            if wall then self:change_direction("w") end
        end
    elseif self.mode == "w" then    -- patrol west
        if self.column <= 8 then
            self:change_direction("n")
        else
            self:walk_animation(-walk_distance)
            local wall = self:walk_until_wall(0, -walk_distance)
            if wall then self:change_direction("n") end
        end
    elseif self.mode == "n" then    -- patrol north
        if self.line <= 2 then
            self:change_direction("e")
        else
            self:walk_animation(-walk_distance)
            local wall = self:walk_until_wall(-walk_distance, 0)
            if wall then self:change_direction("e") end
        end
    elseif self.mode == "r" then    -- return to patrol
    elseif self.mode == "g" then    -- get player
    elseif self.mode == "a" then    -- attack player
    end
    
    
    self:set_location(self.line, self.column)
    
    -- do the combat menu stuff
    CavesMob.update(self, dt)
end

function Kobold:get_hit_description(damage)
    return self:get_hit_other_description(damage, "you")
end

function Kobold:get_hit_other_description(damage, name, index)
    
    local descriptions = 
    {   
        {   
            "The kobold swings wildly with no effect!",
            "The kobold swings wildly with no effect!" 
        },{
            "\x1b[31mThe kobold slashes you viciously",
            "The kobold slashes "..name.." viciously"
        },{
            "\x1b[31mThe kobold grunts and stabs you",
            "The kobold grunts and stabs "..name, 
        },
    }
    
    if damage == 0 then
        index = 1
    else
        index = index or (math.random(2)+1)
    end
    
    if name == 'you' then
        print(index)
        return descriptions[index][1], index
    else
        print(index)
        return descriptions[index][2], index
    end
end


function Kobold:get_death_description(player, damage)
    return self:_get_other_death_description(player, damage, true)
end
    
function Kobold:get_other_death_description(player, damage)
    return self:_get_other_death_description(player, damage, false)
end

function Kobold:_get_other_death_description(player, damage, me)
    
    local descriptions = 
    {   
        {   
            "\x1b[31mThe kobold dies for no apparent reason",
            "\x1b[31mThe kobold dies for no apparent reason" 
        },{
            "\x1b[31mThe kobold succumbs to her wounds",
            "\x1b[31mThe kobold succumbs to her wounds"
        },{
            "\x1b[31mThe kobold dies messily on your sword",
            "\x1b[31mThe kobold dies messily on "..player.name.."'s sword" 
        },{
            "\x1b[31mYou hack off the kobold's head",
            "\x1b[31m"..player.name.."hacks off the kobold's head" 
        },
    }
    
    local index
    
    if damage == 0 then
        index = 1
    elseif damage <= 2 then
        index = 2
    elseif damage <= 5 then
        index = 3
    else
        index = 4
    end
    
    if me then 
        return descriptions[index][1]
    else
        return descriptions[index][2]
    end
end

function Kobold:get_kill_description(player, damage)
    return self:_get_other_kill_description(player, damage, true)
end
    
function Kobold:get_other_kill_description(player, damage)
    return self:_get_other_kill_description(player, damage, false)
end

function Kobold:_get_other_kill_description(player, damage, me)
    
    local descriptions = 
    {   
        {   
            "\x1b[31mYou peacefully expire in your sleep",
            "\x1b[31m"..player.name.." peacefully expires in their sleep" 
        },{
            "\x1b[31mYou succumbs to your wounds",
            "\x1b[31m"..player.name.." succumbs to their wounds"
        },{
            "\x1b[31mYou die messily on the kobold's sword",
            "\x1b[31m"..player.name.." dies messily on the kobold's sword" 
        },{
            "\x1b[31mThe kobold hacks off your head",
            "\x1b[31mThe kobold hacks off "..player.name.."'s head" 
        },
    }
    
    local index
    
    if damage == 0 then
        index = 1
    elseif damage <= 2 then
        index = 2
    elseif damage <= 5 then
        index = 3
    else
        index = 4
    end
    
    if me then 
        return descriptions[index][1]
    else
        return descriptions[index][2]
    end
end



function Kobold:get_start_combat_description()
    if math.random(2) > 1 then
        return "\x1b[31mThe kobold leaps to attack!"
    else
        return "\x1b[31mThe kobold waves her sword warily"
    end
end

function Kobold:get_type()
    return 'kobold'
end

function Kobold:choose_player_to_hit(combat)
    local player = random_entry(combat.players)
    if player.combat_state == 'dying' then
        player = nil
    end
    return player
end




--require("utilities/hotswap_reloader").register_class_file("controllers/Kobold.lua", Kobold) --, copy_data_from_old, hotswap_reload)


return Kobold

