-- Forlorn Fox
-- Copyright © 2016 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
require('utils')

local colliding_objects = {}

function initialise_collisions()
    colliding_objects = {}
end


function register_for_collisions(ref, object, collision_rect)
    
    if collision_rect == nil then
        if object.avatar_data then
            collision_rect = deep_copy(object.avatar_data.collision_rect)
        end
    end
    
    if collision_rect == nil then
        return 
    end
    
    local rect = {}
    
    if collision_rect.centre ~= nil then
        -- handle centre defined collision rects
        rect.top    = collision_rect.centre.line - (collision_rect.size.line/2)
        rect.bottom = collision_rect.centre.line + (collision_rect.size.line/2)
        rect.left   = collision_rect.centre.column - (collision_rect.size.column/2)
        rect.right  = collision_rect.centre.column + (collision_rect.size.column/2)
        
        rect.v_centre = collision_rect.centre.line
        rect.h_centre = collision_rect.centre.column
    else
        -- handle top left defined collision rects
        rect.top    = collision_rect.line
        rect.bottom = collision_rect.line + (collision_rect.size.line)
        rect.left   = collision_rect.column 
        rect.right  = collision_rect.column + (collision_rect.size.column)
    
        rect.v_centre = collision_rect.line + (collision_rect.size.line / 2)
        rect.h_centre = collision_rect.column  + (collision_rect.size.column / 2)
end
    
    
    
    colliding_objects[ref] = { object=object, rect=rect }
    
end

function deregister_for_collisions(ref)
    colliding_objects[ref] = nil
end


function get_collision(in_ref, map_name, line, column, dl, dc)
    local collision = false
    
    if colliding_objects[in_ref] then
        --colliding_objects[in_ref].map_name = map_name       -- update map name for this object
        --local in_object = colliding_objects[in_ref].object
        local in_collision_rect = colliding_objects[in_ref].rect
        
        local old_in_line_min = line+in_collision_rect.top
        local old_in_line_max = line+in_collision_rect.bottom
        
        local in_line     = line+in_collision_rect.v_centre + dl
        local in_line_min = old_in_line_min + dl
        local in_line_max = old_in_line_max + dl
        
        
        local old_in_column_min = column+in_collision_rect.left
        local old_in_column_max = column+in_collision_rect.right
        
        local in_column     = column+in_collision_rect.h_centre + dc
        local in_column_min = old_in_column_min + dc
        local in_column_max = old_in_column_max + dc
        
        
        for ref, entry in pairs(colliding_objects) do
            --[[
            if ref == in_ref then
                gulp.debug_print("collisionx"..ref, "mazerect", in_column_min, in_line_min, in_column_max, in_line_max, "BLUE")
            end
            --]]
            if ref ~= in_ref then                         -- don't collide with self
                local object = entry.object
                if object.map_name == map_name then           -- only collide on the same map
                    
                    local collision_rect = entry.rect
                    
                    local line     = object.line + collision_rect.v_centre
                    local line_min = object.line + collision_rect.top
                    local line_max = object.line + collision_rect.bottom
                    local half_height = (collision_rect.bottom - collision_rect.top) / 2
                    
                    local column     = object.column + collision_rect.h_centre
                    local column_min = object.column + collision_rect.left
                    local column_max = object.column + collision_rect.right
                    local half_width = (collision_rect.right - collision_rect.left) / 2
                    
                    local edge = half_height
                    if half_height > half_width then edge = half_width end
                    
                    --[[
                    if gulp.LuaStateCodeMode=='SERVER' then 
                        gulp.debug_print("collisionx"..ref, "mazerect", column_min, line_min, column_max, line_max, "RED")
                    else
                        gulp.debug_print("collisionx"..ref, "mazerect", column_min, line_min, column_max, line_max, "GREEN")
                    end
                    --]]
                    
                    
                    if line_min > in_line_max or line_max < in_line_min or column_min > in_column_max or column_max < in_column_min then
                    -- no collision
                    else
                        
                        -- we know there is a collision
                        -- would there have been one without the vertical or horizontal component of movement?
                        local no_h_collision = (line_min > old_in_line_max or line_max < old_in_line_min or column_min > in_column_max or column_max < in_column_min)
                        local no_v_collision = (line_min > in_line_max or line_max < in_line_min or column_min > old_in_column_max or column_max < old_in_column_min)
                        
                        --[[
                        local dls = '0'
                        if dl < 0 then dls = '-ve' end 
                        if dl > 0 then dls = '+ve' end 
                        
                        local dcs = '0'
                        if dc < 0 then dcs = '-ve' end 
                        if dc > 0 then dcs = '+ve' end 
                        
                        print('collision: dl is ' .. dls .. ', dc is ' .. dcs)
                        --]]
                        
                        -- of course it is only a collision if we have both vert and horiz components overlapping
                        -- but we want to slide if it is possible to set one of them to 0, and result in no collision... 
                        -- then we slide with the other direction
                        
                        
                        -- check if our vector moves the centre of our collision rect away from the centre of the collliding rect
                        -- if so, we can still move
                        if dl < 0 and in_line > (line_min + edge) and not no_v_collision then 
                            collision = true
                            dl = 0
                        end   -- heading N, inrect S of rect
                        
                        if dl > 0 and in_line < (line_max - edge) and not no_v_collision then 
                            collision = true
                            dl = 0
                        end   -- heading S, inrect N of rect
                        
                        if dc < 0 and in_column > (column_min + edge) and not no_h_collision then 
                            collision = true
                            dc = 0
                        end   -- heading W, inrect E of rect
                        
                        if dc > 0 and in_column < (column_max - edge) and not no_h_collision then 
                            collision = true
                            dc = 0
                        end   -- heading E, inrect W of rect
                    
                        --[[
                        local col = ''
                        if not collision then col = 'NO ' end
                        print('  ' .. col .. 'collision: dl=' .. dl .. ' dc=' .. dc)
                        --]]
                    
                    end
                end
            end
        end
        
        --[[
        if gulp.LuaStateCodeMode=='SERVER' then 
            gulp.debug_print("collision"..in_ref, "mazerect", in_column_min, in_line_min, in_column_max, in_line_max, "YELLOW")
        else
            gulp.debug_print("collision"..in_ref, "mazerect", in_column_min, in_line_min, in_column_max, in_line_max, "MAGENTA")
        end
        --]]
    end
    
    return collision, dl, dc
                
end

