-- Forlorn Fox
-- Copyright © 2020 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
-- Caves of Chaos combat menu
--


require("strict")
local class = require("middleclass")
local Meter = class("Meter")
local text_sizes = require("utilities/text_sizes")


function Meter:initialize(meter_data, owner, l_offset, c_offset) -- owner and offsets only required for clients side meters
    
    gulp._delegate_list:add_delegate(self)
    
    self.full = meter_data.max_time
    self.time = meter_data.initial_time            -- initial time
    self.running = meter_data.running
    
    if gulp.LuaStateCodeMode == 'CLIENT' then    
        owner:add_control(self)
        self.width = meter_data.width    
        self.graduations = meter_data.graduations
        self.direction = meter_data.direction
        self.bar_colour = meter_data.bar_colour
        self.grad_colour = meter_data.grad_colour

        local spaces=""
        for s = 1,self.width do spaces = spaces.." " end

        self.meter = Menu
        {
            background = gulp.colour.WHITE.simple_colour, 
            foreground = gulp.colour.BLACK.simple_colour,
            select_colour = gulp.colour.MAGENTA.simple_colour,
            border_style = 2,
            
            owner = owner,
            control = self,
            
            -- centralised if no offsets were provided
            centralised = (l_offset==nil and c_offset==nil),
            c_centralised = (c_offset==nil and l_offset~=nil),
            l_centralised = (l_offset==nil and c_offset~=nil),
            
            l_offset = l_offset,
            c_offset = c_offset,
            
            text_sizes = owner.text_sizes,
            
            
            { spaces }, 
            
        }
        self.meter.on_draw = function(self, gr)
            local control = self.menu.control
            local _unit = (control.width/control.full)
            
            local _start = self.column + 1 

            -- graduations
            local split = control.full/control.graduations
            for g = split,(split*control.graduations-1), split do
                gr:FillRect(control.grad_colour, (_start+_unit*g)-0.1, self.line+0.9, (_start+_unit*g)+0.1, self.line+2.15)
            end
            
            local _end 
            if control.direction == 'to_left' then
                _end = self.column + 1 + control.width
                _start = _end - (_unit*control.time)
            else
                _end = _start + (_unit*control.time)
            end
           
            if(_start < _end) then
                gr:FillRect(control.bar_colour, _start, self.line+1, _end, self.line+2)
            end
        end
    end
end

function Meter:update(dt)
    if self.running then
        self.time = self.time + dt
        if self.time > self.full then self.time = self.full end
    end
end

function Meter:set(t, running)
    if t > self.time then 
        self.time = self.full
    else
        self.time = t
    end
    self.running = running
end

function Meter:close()
    if gulp.LuaStateCodeMode == 'CLIENT' then    
        self.meter:close()
    end
end


return Meter
    
    
