-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

gulp.LuaStateCodeMode = "SERVER"

-- uncomment to get a slew of debug information about level loading
-- gulp.DebugLevelLoading = true


local check_debugger = require("debug_select")
check_debugger("server")

 --require('mobdebug').start() -- http://studio.zerobrane.com/doc-remote-debugging.html
 -- lets do some network debugging too!
-- require('mobdebug').coro()
 
require("strict")

-- global city up here...
require('utils')
require('map_control')
require("graphics/glyph_sets")

require("mobs/mob_list")

GulpNetworking = require("gulp_networking")
gulp_networking = GulpNetworking:new()
require('map_controllers/game_manager')
require("networking/netstat")
require("gulp_constants")
require('controllers/collisions')

FF_database = require('FF_database')
local db_filename = "ff_db.sqlite"
gulp.db = FF_database:new(db_filename)

-- hacks to allow Mob and Building to initialise on server
function new_hidden_mob_display_element(maze,glyph,line,column,layer,colour) 
    -- for server, we need to just remember the glyph and layer,
    -- which is sent to clients when glyphs are updated
    return { ['glyph']=glyph, ['layer']=layer, ['visible']=false }
end

function new_mob_display_element(maze,glyph,line,column,layer,colour) 
    -- for server, we need to just remember the glyph and layer,
    -- which is sent to clients when glyphs are updated
    return { ['glyph']=glyph, ['layer']=layer, ['visible']=true }
end

function new_maze_display_element(maze,glyph,line,column,layer,colour) 
    -- for server, we need to just remember the glyph and layer,
    -- which is sent to clients when glyphs are updated
    return { ['glyph']=glyph, ['layer']=layer, ['visible']=true }
end

local dispatcher = require("utilities/dispatcher")
local ServerMapLoader = require('map_controllers/ServerMapLoader')

gulp.map_loader = ServerMapLoader()

--
-- start server running, if required
--
local function _start_game_server(server_info)
    
    
    -- some default setup of gulp which the server needs...
    -- this is a bit crap... todo make this better
    gulp.game_managers = {}
    gulp.server_identifier = tostring(math.random()) .. tostring(os.time())   -- @todo: needs to be saved?
    gulp.gameName = "My World"
    
    local server_started = true
    gulp.server_networking_enabled = false
    if server_info.port then
        gulp.server_networking_enabled = true
        server_started = gulp_networking:enableServer(server_info.port)
        
        ---[[ don't need this for server only type game
        if server_started then
            if server_info.LAN_broadcast then
                -- DISABLE FOR FIXED SERVER GAME WORLD
                --print("Starting LAN broadcast")
                --gulp_networking.LANb.start_server_game_broadcast(server_info.port)
            end
            if server_info.internet_broadcast then
                -- DISABLE FOR FIXED SERVER GAME WORLD
                --gulp_networking.Internetb.start_server_game_broadcast(server_info.port)
            end
        end
        --]]
    end

    return server_started
end


local _start_stop_test = "STOP"     -- defensive coding

function handle_start_request(game_status)
    
    gulp.enableKeepAlive = game_status.enableKeepAlive
    gulp.server_initial_map_name = game_status.start_map_name
    
    -- launch server 
    if not _start_game_server(game_status) then
        -- do some error thing
        -- handle better ... likely cause is already running a server
        
        gulp.ui_queue:send({qcommand = "server_couldnt_start"})

    else
    
        gulp.map_loader:LoadMap(gulp.server_initial_map_name)
        gulp._delegate_list:add_delegate(gulp.map_loader)
    
        gulp._delegate_list:add_delegate(gulp_networking)

        -- start processing in game messages
        server_receiver_set_in_game_mode()
        
        -- defensive coding
        if _start_stop_test ~= "STOP" then
            error("FATAL: Unexpected start request")
        end
        _start_stop_test = "START"

    end
end

--
function handle_stop_request(message)
    -- defensive coding
    if _start_stop_test ~= "START" then
        error("FATAL: Stop request without start request")
    end
    _start_stop_test = "STOP"
    
    -- drop all subsequent in-game messages in queue
    server_receiver_set_out_game_mode()
    
    -- drop all joined players
    for _,player in all_players() do
        player:remove()
    end

    -- drop the low level player registrations and sockets
    gulp_networking.cleanupClients()
    
    -- reverse everything the start message does
    gulp._delegate_list:drop_delegate(gulp_networking)
    gulp._delegate_list:drop_delegate(gulp.map_loader)
    
    gulp.map_loader:DropAllMaps()
    
    
    initialise_collisions()
    
    if gulp.server_networking_enabled then
        -- turn off networking if enabled...
        gulp_networking:disableServer()
        gulp_networking.LANb.stop_server_game_broadcast()
        gulp_networking.Internetb.stop_server_game_broadcast()
        gulp.server_networking_enabled = false
    end
    
    -- some default setup of gulp which the server needs...
    -- this is a bit crap... todo make this better
    gulp.server_identifier = nil
    gulp.gameName = nil
    
    for name,_ in pairs(gulp.game_managers) do
        drop_game_manager(name)
    end
    
    gulp.enableKeepAlive = nil
    gulp.server_initial_map_name = nil
    
    
end

local LocalConnectionFakeSocket = require("networking/local_connection_fake_socket")

function gulp.server_load(ui_queue, server_queue)
    gulp.time = 0

    gulp.ui_queue = ui_queue
    gulp.server_queue = server_queue
    --print("server_load", ui_queue, server_queue)
    
    if not gulp._delegate_list then
        gulp._delegate_list = dispatcher()
    end
    
    -- connection stuff
    gulp.server_networking_enabled = false
    gulp.local_connection_fake_socket = LocalConnectionFakeSocket:new(ui_queue)
end


local function handle_local_network_packet_server(message)
    if message.mt then
        local dispatcher = gulp.server_receivers[message.mt]
        if dispatcher then
            dispatcher(gulp.local_connection_fake_socket, message)
        end
    else
        print("SERVER: No mt supplied in network message?")
    end
end

local function handle_run_lua(message)
    if type(message.lua) == "string" then
        local f = load(message.lua)
        local success, err = pcall(f)
        if not success then
            print("Error!!!")
            print(err)
        end
    else
        print("Error in server run")
    end
end

local function server_dt_update(dt)
    gulp.time = gulp.time + dt

    gulp_networking.LANb.broadcast_server_update(dt)
    gulp_networking.Internetb.broadcast_server_update(dt)
    
    gulp._delegate_list:passive_dispatcher('update', dt)
end


local server_function_dispatch = {
    start = handle_start_request,
    stop = handle_stop_request,
    --qlocal = handle_local_network_packet_server, -- remove for now with no local server
    run_lua = handle_run_lua,
    stop_thread = function(message) gulp.server_thread_run = false end,
    update = function(message) server_dt_update(message.dt) end,
    print_server_info = function(message)
        print("hi from server_main.lua")
        gulp_networking:print_server_debug_info()
        print('---------')
        local socket = require "socket"
        print("current time = ", socket.gettime())
        local copas10 = require('copas')
        copas10.list_all()
        print('---------')
        end
}


local function server_processing()

    while not gulp.server_queue:is_empty() do
        
        local message = gulp.server_queue:read()
        --print("SERVER:", message)
        local f = server_function_dispatch[message.qcommand]
        if f then
            f(message)
        else
            error("unknown command in gulp.server_update() " .. tostring(f))
        end    
    end
    
    local stepCopas_result = gulp_networking:stepCopas(0.01) -- wait for 10ms until we process next bactch of messages from UI
    
    if stepCopas_result == false then
        gulp_cpp.SDL_Delay(10)              -- nothing happening, can pause
    --elseif stepCopas_result == nil then
        --print("------", stepCopas_result)
        --value = false
    --elseif stepCopas_result == true then
        --print("---- data", gulp_cpp.SDL_GetTicks())     -- when enabled only prints when erver traffic
    end
    
    if gulp.server_debug_print_table then
        gulp_networking:broadcast{mt="debug_print", data=gulp.server_debug_print_table}
        gulp.server_debug_print_table = nil
    end
    
end

-- This one reduces the processing power of the server, but at the price of 10ms latency.
-- Three options:
-- (1) SDL_Delay or read_timeout(10) or something ... but network reads are delayed.
-- (2) Put the network receive on a seperate thread/LuaState and push data into the queue.
-- (3) Run as fast as possible
-- (4) Some other method where we wait for a common signal
-- 
-- NOTES: 21 April 2020
-- The problem was that we can’t wait for the internal message queue and also the socket (with select). 
-- For basic forlorn fox we might have used localhost sockets rather than a memory copy thread-safe queue.
-- However, since there is no local server, I think we can delay messages from the UI to the server by at least 0.01s (10ms).
-- 

--[[
local function server_processing()
    
    repeat
        -- we use this wait for read as a time to suspend until something happens.
        -- However, we want to check other stuff occasionally. If we get lots of 
        -- stuff in the queue, we will process more. Ideally we want to 
        -- to not wait if there are packets waiting...
        local message = gulp.server_queue:read_timeout(10)
        if message == nil then
            break
        end
        
        --print("SERVER:", message)
        local f = server_function_dispatch[message.qcommand]
        if f then
            f(message)
        else
            error("unknown command in gulp.server_update() " .. tostring(f))
        end    
    
    until gulp.server_queue:is_empty()  -- only loop if there is more stuff to process
    
    gulp_networking:stepCopas()
    
    if gulp.server_debug_print_table then
        gulp_networking:broadcast{mt="debug_print", data=gulp.server_debug_print_table}
        gulp.server_debug_print_table = nil
    end
    
end
--]]


-- for non-threaded model
function gulp.server_update(dt)
    server_processing()
    server_dt_update(dt)    
end

-- for threaded model
function gulp.server_thread()
    gulp.server_thread_run = true
    while gulp.server_thread_run do
        server_processing()
    end
    
    return 0
end
