-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

require("strict")
require("bit32")
class = require("middleclass")
require("graphics/graphics")
require("graphics/glyph_sets")


local Button = require("gui/button")
local LockButton = require("gui/lockbutton")
local ToggleButton = require("gui/togglebutton")
local ColourButton = require("gui/colourbutton")
local ColourToggleButton = require("gui/colour_toggle_button")

local MouseMove = require("player/mouse_move")

local ColourPicker = require("glyph_editor/colour_picker")

local GlyphEditor = class("GlyphEditor")
require('library/SDL_keycode')


local edit_buffer_texture_offset = 0
local copy_buffer_texture_offset = 1

local resolution_button     -- keep hold of this so we can auto click it

function GlyphEditor:keypressed(sym)
    local glyph_editor_key_actions = {
        [SDLK.ESCAPE] = GlyphEditor.finish,
    }
    local action = glyph_editor_key_actions[sym]
    if action then
        action(self, sym)
        return true
    end
    return false
end



local DEFAULT_START_GLYPH = 4

-- glyph set offsets
local gso_editor = glyph_sets.editor.character_base_code
local gso_editor2 = glyph_sets.editor2.character_base_code
local gso_glyph_editor_buffers16 = glyph_sets.glyph_editor_temp16.character_base_code
local gso_glyph_editor_buffers32 = glyph_sets.glyph_editor_temp32.character_base_code


local glyph_editor_cell = 28+gso_editor
local glyph_editor_cell_bottom_closer_utf8 = to_utf8(29+gso_editor)
local glyph_editor_cell_bottom_closer = 29+gso_editor
local glyph_editor_cell_right_closer = 30+gso_editor
local glyph_editor_cell_magic_dot_utf8 = to_utf8(31+gso_editor)
local glyph_editor_cell_magic_dot = 31+gso_editor

local multicolour_grid_glyph = 7+gso_editor
local transparent_grid_glyph = 25+gso_editor
local multicolour_palette_glyph = 26+gso_editor
local transparent_palette_glyph = 27+gso_editor

local glyph_editor_cell32 = 29+gso_editor2
local glyph_editor_fill = 32    -- a space character
local multicolour_grid_glyph32 = 30+gso_editor2
local transparent_grid_glyph32 = 31+gso_editor2
local multicolour_grid_glyph32for16grid = 14+gso_editor2
local transparent_grid_glyph32for16grid = 15+gso_editor2

local editor_fg_colour = 'BLACK'


local edit_glyph =          0+gso_glyph_editor_buffers16
local copy_buffer_glyph =   1+gso_glyph_editor_buffers16


function GlyphEditor:unfloored_convert_to_click(mx, my, location)
    if (my >= location.line and my < location.line+location.ysize) and
        (mx >= location.column and mx < location.column+location.xsize) then
        return true, mx-location.column, my-location.line
    else
        return false
    end
end

function GlyphEditor:convert_to_click(mx, my, location)
    local ret, x, y = self:unfloored_convert_to_click(mx, my, location)
    if not ret then 
        return false
    end 
    return true, math.floor(x), math.floor(y)
end

--[[
function GlyphEditor:check_button(mx, my, button)
    
    local button_x_size = button.xsize
    
    if(button.text ~= nil) then
        button_x_size = button_x_size + string.len(button.text)
    end
        
    if(my >= button.line and my < button.line+button.ysize) then
        if(mx >= button.column and mx< button.column+button_x_size) then
            return true
        end
    end
    return false
end
--]]

function GlyphEditor:draw_object(line_in, col_in, object)
    local glyph = object.first_glyph
    local line, column
    
    for line=line_in, line_in+object.ysize-1 do
        for column=col_in, col_in+object.xsize-1 do
            print_glyph_at(line, column, glyph)
            glyph = glyph + 1
        end
        glyph = glyph + 32 - object.xsize
    end
end

function GlyphEditor:draw_static_object(object)
    self:draw_object(object.line, object.column, object)
end



function GlyphEditor:get_pixel(l, c)
    if (l > self.glyph.size-1) or (c > self.glyph.size-1) then
        return 1
    end
    
    return math.floor(l)+(math.floor(c)*self.glyph.size)+1
end


function GlyphEditor:rotate()
    -- rotate the glyph, copied from Rob's C++ algorithm
    local size = self.glyph.size
    
    for x = 0, (size/2)-1 do
        for y = 0, (size/2)-1 do
            local p1 = self.glyph.pixels[self:get_pixel(x,y)]   
            self.glyph.pixels[self:get_pixel(x,y)] = self.glyph.pixels[self:get_pixel(y,size-(x+1))]
            self.glyph.pixels[self:get_pixel(y, size-(x+1))] = self.glyph.pixels[self:get_pixel(size-(x+1),size-(y+1))]
            self.glyph.pixels[self:get_pixel(size-(x+1),size-(y+1))] = self.glyph.pixels[self:get_pixel(size-(y+1),x)]
            self.glyph.pixels[self:get_pixel(size-(y+1),x)] = p1
        end
    end
end

function GlyphEditor:vertical_flip()
    -- mirror the glyph along a vertical line
    local size = self.glyph.size
    
    for x = 0, (size/2)-1 do
        for y = 0, size-1 do
            local p1 = self.glyph.pixels[self:get_pixel(x,y)]
            self.glyph.pixels[self:get_pixel(x,y)] = self.glyph.pixels[self:get_pixel(size-(x+1),y)]
            self.glyph.pixels[self:get_pixel(size-(x+1),y)] = p1
        end
    end
end

function GlyphEditor:horizontal_flip()
    -- mirror the glyph along a horizontal line
    local size = self.glyph.size
    
    for x = 0, (size)-1 do
        for y = 0, (size/2)-1 do
            local p1 = self.glyph.pixels[self:get_pixel(x,y)]
            self.glyph.pixels[self:get_pixel(x,y)] = self.glyph.pixels[self:get_pixel(x,size-(y+1))]
            self.glyph.pixels[self:get_pixel(x,size-(y+1))] = p1
        end
    end
end

function GlyphEditor:translate(direction)
    -- shift the glyph by 1 pixel
    local size = self.glyph.size
    
    if direction == 'up' then
        for line = 0, (size)-2 do      -- don't do the last line
            for column = 0, size-1 do
                local p1 = self.glyph.pixels[self:get_pixel(line+1,column)]
                self.glyph.pixels[self:get_pixel(line,column)] = p1
            end
        end
        for column = 0, size-1 do
            self.glyph.pixels[self:get_pixel(size-1,column)] = colour_to_pixel(self.alt_colour, self.colour_format)
        end
    elseif direction == 'down' then
        for line = (size)-1, 1, -1 do      -- don't do the last line
            for column = 0, size-1 do
                local p1 = self.glyph.pixels[self:get_pixel(line-1,column)]
                self.glyph.pixels[self:get_pixel(line,column)] = p1
            end
        end
        for column = 0, size-1 do
            self.glyph.pixels[self:get_pixel(0,column)] = colour_to_pixel(self.alt_colour, self.colour_format)
        end
    elseif direction == 'left' then
        for line = 0, (size)-1 do      -- don't do the last column
            for column = 0, size-2 do
                local p1 = self.glyph.pixels[self:get_pixel(line,column+1)]
                self.glyph.pixels[self:get_pixel(line,column)] = p1
            end
        end
        for line = 0, size-1 do
            self.glyph.pixels[self:get_pixel(line,size-1)] = colour_to_pixel(self.alt_colour, self.colour_format)
        end
    elseif direction == 'right' then
        for line = 0, (size)-1 do      -- don't do the last column
            for column = size-1, 1, -1 do
                local p1 = self.glyph.pixels[self:get_pixel(line,column-1)]
                self.glyph.pixels[self:get_pixel(line,column)] = p1
            end
        end
        for line = 1, size-1 do
            self.glyph.pixels[self:get_pixel(line,0)] = colour_to_pixel(self.alt_colour, self.colour_format)
        end
    end
    
end


function GlyphEditor:clear_current_glyph()
    local size = self.glyph.size
    
    for x = 0, (size)-1 do
        for y = 0, (size)-1 do
            self.glyph.pixels[self:get_pixel(x,y)] = colour_to_pixel(gulp.colour.hot_pink_means_transparent, self.colour_format)
        end
    end
end

function GlyphEditor:floodfill(y, x, right)
    
    -- pick colour depending on the button clicked
    local colour = self.colour
    if right then
        colour = self.alt_colour
    end
    
    local new_pixel = colour_to_pixel(colour, self.colour_format)
    local old_pixel = self.glyph.pixels[self:get_pixel(x,y)]
    
    -- don't try to floodfill an area the colour it already is, 
    -- that would recurse until the system crashes ;-)
    if(new_pixel == old_pixel) then return end
    
    self:continue_floodfill(x, y, new_pixel, old_pixel)
    
    -- update the preview
    self:update_edit_buffer()
    
    -- make sure the display grid is up to date with the operation
    self:update_grid()
    
end

function GlyphEditor:continue_floodfill(x, y, new_pixel, old_pixel)
    
    if(self.glyph.pixels[self:get_pixel(x,y)] == old_pixel) then
        
        self.glyph.pixels[self:get_pixel(x,y)] = new_pixel
        
        if(x > 0) then
            self:continue_floodfill(x-1, y, new_pixel, old_pixel)
        end
        
        if(y > 0) then
            self:continue_floodfill(x, y-1, new_pixel, old_pixel)
        end
        
        if(x < (self.glyph.size-1)) then
            self:continue_floodfill(x+1, y, new_pixel, old_pixel)
        end
        
        if(y < (self.glyph.size-1)) then
            self:continue_floodfill(x, y+1, new_pixel, old_pixel)
        end
    end
end



function GlyphEditor:update_edit_buffer()
    local gridx, gridy
    for gridy=0, self.glyph.size-1 do
        for gridx=0, self.glyph.size-1 do
            local pixel = self.glyph.pixels[self:get_pixel(gridy, gridx)]
            self:update_editor_texture_pixel(edit_buffer_texture_offset, gridx, gridy, pixel)
        end
    end
end


function hexstring(s)
    return string.format("%x", s)
end


function GlyphEditor:dump(glyph, format_data)
    -- debug dump
    print("Glyph size is " .. glyph.size)
    print("First pixel is " .. hexstring(glyph.pixels[1]))
    print("format_data: Amask = " .. hexstring(format_data.Amask))
    print("             Bmask = " .. hexstring(format_data.Bmask))
    print("             Gmask = " .. hexstring(format_data.Gmask))
    print("             Rmask = " .. hexstring(format_data.Rmask))
    print("format_data: Ashift = " .. hexstring(format_data.Ashift))
    print("             Bshift = " .. hexstring(format_data.Bshift))
    print("             Gshift = " .. hexstring(format_data.Gshift))
    print("             Rshift = " .. hexstring(format_data.Rshift))
end



function GlyphEditor:initialize()
    self.red = 128
    self.green = 128
    self.blue = 128
end

function GlyphEditor:save_glyph()
    -- sanity check
    if self.glyph_set_sdl_surface == nil or
       self.glyph_set == nil or
       self.selected_glyph == nil or
       self.glyph == nil then
        print("Cannot save glyph:", self.glyph_set_sdl_surface, self.glyph_set, self.selected_glyph, self.glyph)
        return
    end
    
    -- save the glyph to the surface
    if gulp_cpp.save_glyph(self.glyph_set_sdl_surface, self.glyph_set, self.selected_glyph, self.glyph) then
    
        -- write the surface to file
        gulp_cpp.save_glyph_file(self.glyph_set_sdl_surface, self.glyph_set)
    
        -- reload the game textures with the new file
        gulp.app:get_standard_graphics_context():load_textures_from_glyph_set(self.glyph_set)
    end
end


function GlyphEditor:finish()
    
    MouseMove.restore_game_controls(self._game_mouse_state)
    MouseMove.restore_listed_regions(self._other_mouse_state)
    
    gulp._draw_map = self.old_draw_map
    gulp._draw_status_lines = self.draw_status_lines
    
    -- disable the buttons, removes them from draw list and allows them to be garbage collected
    self:set_button_states(self.editor_button_list, 'kill')
    self:set_button_states(self.glyph_set_load_button_list, 'kill')
    self:set_button_states(self.colour_picker_button_list, 'kill')
    
    -- stops grid being displayed, allows the elements to be garbage collected
    -- do this after buttons because it actually causes garbage collection - can clear up the buttons too
    self:remove_grid_from_draw_list()
    
    gulp._delegate_list:drop_delegate(self)
    self:save_glyph()
    
    if self.glyph_set_sdl_surface then 
        gulp_cpp.SDL_FreeSurface(self.glyph_set_sdl_surface) 
        self.glyph_set_sdl_surface = nil
    end
    --[[
    if gulp.localplayer then
        -- there will be no localplayer if the glyph editor is entered from the main menu
        -- also, no need to change the offset... so all is good
        local map_name = gulp.localplayer.map_name
        local map = get_map(map_name)
        gulp_cpp.SetLevelSize(map.width, map.height)
    end
    --]]
    
    
end


local preview_colour_position = {   
                            ['BLACK']=2, 
                            ['BLUE']=3, 
                            ['RED']=4, 
                            ['MAGENTA']=5, 
                            ['GREEN']=6, 
                            ['CYAN']=7, 
                            ['YELLOW']=8, 
                            ['BRIGHT_BLACK']=9, 
                            ['BRIGHT_BLUE']=10, 
                            ['BRIGHT_RED']=11, 
                            ['BRIGHT_MAGENTA']=12, 
                            ['BRIGHT_GREEN']=13, 
                            ['BRIGHT_CYAN']=14, 
                            ['BRIGHT_YELLOW']=15, 
                            ['BRIGHT_WHITE']=16,
                            ['WHITE']=17
                        }

local palette_colour_position = {   
                            'BLACK', 
                            'BLUE', 
                            'RED', 
                            'MAGENTA', 
                            'GREEN', 
                            'CYAN', 
                            'YELLOW', 
                            'WHITE'
                        }
                        
                        
-- *********************** START GRID SCREEN ****************************************

-- screen area definitions
local grid_loc =        { line=2,  column=3,    ysize=16,   xsize=16 }
local palette_loc =     { line=2,  column=19.5, ysize=16,   xsize=4  }
local glyph_set_loc =   { line=20, column=3,    ysize=4,    xsize=16 }
local glyph_set_label = { line=21, column=21    }
local copy_buffer =     { line=6,  column=27    }

-- extraneous drawing defintions
local tools_top_glyph = 60+gso_editor
local tools_bottom_glyph = 124+gso_editor     
local tools_left_glyph = 91+gso_editor
local tools_right_glyph = 93+gso_editor
local tools_bottom_line = 123+gso_editor

local copy_paste_top =      {line=5,    column=26,  ysize=1,    xsize=3,    first_glyph=34+gso_editor}
local copy_paste_bottom =   {line=7,    column=26,  ysize=1,    xsize=3,    first_glyph=98+gso_editor}
local tools_top =           {line=2,    column=27,  ysize=1,    xsize=1,    first_glyph=tools_top_glyph}
local tools_bottom =        {line=4,    column=26,  ysize=1,    xsize=3,    first_glyph=tools_bottom_line}
local tools_left =          {line=3,    column=26,  ysize=1,    xsize=1,    first_glyph=tools_left_glyph}
local tools_right =         {line=3,    column=28,  ysize=1,    xsize=1,    first_glyph=tools_right_glyph}
local translate_middle =    {line=9,    column=29.5,ysize=1,    xsize=1,    first_glyph=0x59+gso_editor}

local tools_left_draw_button_down =     {line=3,    column=26,  ysize=1,    xsize=1,    first_glyph=83+gso_editor}
local tools_right_flood_button_down =   {line=3,    column=28,  ysize=1,    xsize=1,    first_glyph=87+gso_editor}
local tools_left_colour_button_down =   {line=3,    column=26,  ysize=1,    xsize=1,    first_glyph=84+gso_editor}
local tools_right_bottom_button_down =  {line=3,    column=28,  ysize=1,    xsize=1,    first_glyph=86+gso_editor}

-- glyph definitions
local multi_multi_palette_glyph = 104+gso_editor
local multi_trans_palette_glyph = 105+gso_editor
local multi_colour_palette_glyph = 106+gso_editor
local trans_multi_palette_glyph = 107+gso_editor
local trans_trans_palette_glyph = 108+gso_editor
local trans_colour_palette_glyph = 109+gso_editor
local colour_multi_palette_glyph = 110+gso_editor
local colour_trans_palette_glyph = 111+gso_editor
local colour_colour_palette_glyph = 112+gso_editor

local background_colour_glyph = 72+gso_editor
local background_multi_glyph = 73+gso_editor
local background_trans_glyph = 74+gso_editor
local foreground_colour_glyph = 75+gso_editor
local foreground_multi_glyph = 76+gso_editor
local foreground_trans_glyph = 77+gso_editor

local background_pick_glyph = 104+gso_editor
local background_pick_multi_glyph = 105+gso_editor
local background_pick_trans_glyph = 106+gso_editor
local foreground_pick_glyph = 107+gso_editor
local foreground_pick_multi_glyph = 108+gso_editor
local foreground_pick_trans_glyph = 109+gso_editor

local selector_square = { ysize=2,  xsize=2,    first_glyph=16+gso_editor }

local box_top_left_corner = 32+gso_editor
local box_top_right_corner = 33+gso_editor
local box_bottom_left_corner = 64+gso_editor
local box_bottom_right_corner = 96+gso_editor

local zero_right_label_glyph = 32+gso_editor2
local zero_left_label_glyph = 0+gso_editor2


function GlyphEditor:update(dt)
    
    if(self.done == true) then
        self:finish()
    
    elseif (self.done == false) then
        -- test for self.done ensures all the animation stuff is already initialised
        -- draw the animation
        
        self.time = self.time+dt
        
        
        if(self.animation_speed > 0 and self.first_animation_glyph ~= nil) then
            if(self.time > (1/self.animation_speed)) then
                
                -- new animation frame
                self.animation_frame = self.animation_frame + 1
                if(self.animation_frame >= self.animation_frames) then
                    self.animation_frame = 0
                end
                
                -- adjust time
                self.time = self.time - (1/self.animation_speed)
                
            end
        else
            -- no animation just now
            self.time = 0
        end
    end
    return true         -- we handled the update
end

local animation_start_col = 23
local animation_start_line = 11
local animation_box_width = 8
local animation_box_height = 7

function GlyphEditor:create_animation_buttons()
    
    local first_glyph_button =  { line=animation_start_line+2,  column=animation_start_col+(animation_box_width/2)-1.75,     ysize=1,    xsize=2,    first_glyph=38+gso_editor,  click=self.first_glyph_button_function,     parent=self }
    local colour_button =       { line=animation_start_line+2,  column=animation_start_col+(animation_box_width/2)+0.75,   ysize=1,    xsize=2,    first_glyph=0x30+gso_editor2, first_glyph_down=0x34+gso_editor2, click=self.animation_colour_button_function,     parent=self }
    local speed_up_button =     { line=animation_start_line+2.5,column=animation_start_col+1,   ysize=1,    xsize=1,    first_glyph=47+gso_editor,  click=self.speed_up_button,     parent=self }
    local speed_down_button =   { line=animation_start_line+4.5,column=animation_start_col+1,   ysize=1,    xsize=1,    first_glyph=111+gso_editor,     click=self.speed_down_button,   parent=self }
    local frame_up_button =     { line=animation_start_line+2.5,column=animation_start_col+animation_box_width-1,   ysize=1,    xsize=1,    first_glyph=47+gso_editor,  click=self.frames_up_button,    parent=self }
    local frame_down_button =   { line=animation_start_line+4.5,column=animation_start_col+animation_box_width-1,   ysize=1,    xsize=1,    first_glyph=111+gso_editor,     click=self.frames_down_button,  parent=self }
    
    -- keep reference so we can change background
    self.animation_colour_button = ToggleButton:new(colour_button)
    
    table.insert(self.editor_button_list, Button:new(first_glyph_button))
    table.insert(self.editor_button_list, self.animation_colour_button)
    table.insert(self.editor_button_list, Button:new(speed_up_button))
    table.insert(self.editor_button_list, Button:new(speed_down_button))
    table.insert(self.editor_button_list, Button:new(frame_up_button))
    table.insert(self.editor_button_list, Button:new(frame_down_button))
    

end

function GlyphEditor:draw_box(line, column, width, height)
    
    print_glyph_at(line, column, box_top_left_corner) 
    print_glyph_at(line, column+width, box_top_right_corner) 
    print_glyph_at(line+height, column, box_bottom_left_corner) 
    print_glyph_at(line+height, column+width, box_bottom_right_corner) 
    
    for l=line+1, line+height-1 do
        print_glyph_at(l, column, tools_left_glyph) 
        print_glyph_at(l, column+width, tools_right_glyph) 
    end
    
    for c=column+1, column+width-1 do
        print_glyph_at(line, c, tools_top_glyph) 
        print_glyph_at(line+height, c, tools_bottom_glyph) 
    end
    
end

function GlyphEditor:draw_animation_controls()

    local animation_surround= { line=animation_start_line+4.5,  column=animation_start_col+(animation_box_width/2)-1,   ysize=3,    xsize=3,    first_glyph=10+gso_editor2 }
    local animation_label   = { line=animation_start_line+1,    column=animation_start_col+1,   ysize=1,    xsize=3,    first_glyph=101+gso_editor }
    local speed_label       = { line=animation_start_line+3.5,  column=animation_start_col+2,   ysize=1,    xsize=2,    first_glyph=112+gso_editor }
    local frame_label       = { line=animation_start_line+3.5,  column=animation_start_col+animation_box_width-3,   ysize=1,    xsize=2,    first_glyph=70+gso_editor }
    
    self:draw_static_object(animation_label)
    self:draw_static_object(speed_label)
    self:draw_static_object(frame_label)
    self:draw_static_object(animation_surround)
    
    self:draw_box(animation_start_line, animation_start_col, animation_box_width, animation_box_height) 
    
    -- draw the animation speed and frames
    print_glyph_at(speed_label.line, speed_label.column-1, zero_right_label_glyph+self.animation_speed)
    print_glyph_at(frame_label.line, frame_label.column+2, zero_left_label_glyph+self.animation_frames)
    
    -- draw the current frame
    if(self.first_animation_glyph ~= nil) then
        set_fg_colour(self.animation_colour)
        print_glyph_at(animation_surround.line+1, animation_surround.column+1, self.first_animation_glyph+self.animation_frame)
        set_fg_colour('BLACK')
    end
    
end

function GlyphEditor:speed_up_button()
    if(self.animation_speed < 9) then
        self.animation_speed = self.animation_speed + 1
    end
end

function GlyphEditor:speed_down_button()
    if(self.animation_speed > 0) then   
        self.animation_speed = self.animation_speed - 1
    end
end

function GlyphEditor:frames_up_button()
    if(self.animation_frames < 9) then
        self.animation_frames = self.animation_frames + 1
    end
end

function GlyphEditor:frames_down_button()
    if(self.animation_frames > 0) then
        self.animation_frames = self.animation_frames - 1
    end
end

function GlyphEditor:first_glyph_button_function()
    self.first_animation_glyph = self.selected_glyph+self.glyph_set.character_base_code
    self.animation_frame = 0
end


function GlyphEditor:create_editor_buttons()
    
local load_button =     { line=20,  column=0.5,     ysize=1,    xsize=2,    first_glyph=20+gso_editor,  click=self.load_button_function,    parent=self }
local exit_button =     { line=0,   column=30,      ysize=1,    xsize=2,    first_glyph=8+gso_editor,   click=self.finish,                  parent=self }
local clear_button =    { line=8,   column=26.125,  ysize=1,    xsize=2,    first_glyph=10+gso_editor,  click=self.clear_button_function, parent=self   }
local save_button =     { line=9,   column=26.125,  ysize=1,    xsize=2,    first_glyph=12+gso_editor,  click=self.save_button_function, parent=self    }
local revert_button =   { line=10,  column=26.125,  ysize=1,    xsize=2,    first_glyph=14+gso_editor,  click=self.revert_button_function, parent=self  }
local copy_button =     { line=6,   column=25,      ysize=1,    xsize=2,    first_glyph=65+gso_editor,  click=self.copy_button_function, parent=self    }
local paste_button =    { line=6,   column=28,      ysize=1,    xsize=2,    first_glyph=68+gso_editor,  click=self.paste_button_function, parent=self   }
local rotate_button =   { line=10,  column=24,      ysize=1,    xsize=2,    first_glyph=40+gso_editor,  click=self.rotate_button_function, parent=self  }
local hflip_button =    { line=8,   column=24,      ysize=1,    xsize=2,    first_glyph=42+gso_editor,  click=self.hflip_button_function, parent=self   }
local vflip_button =    { line=9,   column=24,      ysize=1,    xsize=2,    first_glyph=44+gso_editor,  click=self.vflip_button_function, parent=self   }

local colour_pick_button = { line=palette_loc.line+6, column=palette_loc.column+2, ysize=1,    xsize=1,    first_glyph=get_glyph("editor3", 0x03),  click=self.colour_picker_function, click_arg=1, parent=self   }
local grid_pick_button =   { line=palette_loc.line+6, column=palette_loc.column,   ysize=1,    xsize=1,    first_glyph=get_glyph("editor3", 0x02),  click=self.grid_picker_function,   click_arg=1, parent=self   }

local translate_left_button =  { line=9,  column=28.5, ysize=1, xsize=1, first_glyph=0x58+gso_editor, click=self.translate_button_function, click_arg='left',  parent=self }
local translate_right_button = { line=9,  column=30.5, ysize=1, xsize=1, first_glyph=0x5a+gso_editor, click=self.translate_button_function, click_arg='right', parent=self }
local translate_up_button =    { line=8,  column=29.5, ysize=1, xsize=1, first_glyph=0x2f+gso_editor, click=self.translate_button_function, click_arg='up',    parent=self }
local translate_down_button =  { line=10, column=29.5, ysize=1, xsize=1, first_glyph=0x6f+gso_editor, click=self.translate_button_function, click_arg='down',  parent=self }

local draw_button =     { line=2,   column=24,  ysize=1,    xsize=3,    first_glyph=57+gso_editor,  click=self.draw_button_function, parent=self,
                                                                    first_glyph_down=50+gso_editor, mode='down' }   -- initially active
local flood_button =    { line=2,   column=28,  ysize=1,    xsize=3,    first_glyph=61+gso_editor,  click=self.flood_button_function, parent=self,
                                                                    first_glyph_down=54+gso_editor  }   
local colour_button =   { line=4,   column=24,  ysize=1,    xsize=3,    first_glyph=121+gso_editor, click=self.colour_button_function, parent=self,
                                                                    first_glyph_down=114+gso_editor, 
                                                                    first_glyph_further_down=80+gso_editor, active_bg_colour='BRIGHT_YELLOW'}   
local res_button =   { line=18,  column=17,  ysize=1,xsize=2,    first_glyph=64+gso_editor2, click=self.resolution_button_function, parent=self,
                                                                    first_glyph_down=96+gso_editor2, mode='down'    }   -- initially in 16x16 mode
local glyph_set_scroll_up =     { line=glyph_set_loc.line+1,column=glyph_set_loc.column+glyph_set_loc.xsize,   ysize=1,    xsize=1,    first_glyph=47+gso_editor,  click=self.glyph_set_scroll, click_arg='up',   parent=self }
local glyph_set_scroll_down =   { line=glyph_set_loc.line+2,column=glyph_set_loc.column+glyph_set_loc.xsize,   ysize=1,    xsize=1,    first_glyph=111+gso_editor, click=self.glyph_set_scroll, click_arg='down', parent=self }
    
local alpha_button =   { line=18,  column=14,  ysize=1,xsize=2,    first_glyph=66+gso_editor2, click=self.alpha_button_function, parent=self }

    -- normal buttons
    table.insert(self.editor_button_list, Button:new(load_button))
    table.insert(self.editor_button_list, Button:new(exit_button))
    table.insert(self.editor_button_list, Button:new(clear_button))
    table.insert(self.editor_button_list, Button:new(save_button))
    table.insert(self.editor_button_list, Button:new(revert_button))
    table.insert(self.editor_button_list, Button:new(copy_button))
    table.insert(self.editor_button_list, Button:new(paste_button))
    table.insert(self.editor_button_list, Button:new(rotate_button))
    table.insert(self.editor_button_list, Button:new(hflip_button))
    table.insert(self.editor_button_list, Button:new(vflip_button))
    
    for pick=1, 10 do
        table.insert(self.editor_button_list, Button:new(colour_pick_button))
        colour_pick_button.line = colour_pick_button.line + 1
        colour_pick_button.click_arg = colour_pick_button.click_arg + 1
        
        table.insert(self.editor_button_list, Button:new(grid_pick_button))
        grid_pick_button.line = grid_pick_button.line + 1
        grid_pick_button.click_arg = grid_pick_button.click_arg + 1
    end

    table.insert(self.editor_button_list, Button:new(glyph_set_scroll_up))
    table.insert(self.editor_button_list, Button:new(glyph_set_scroll_down))
    table.insert(self.editor_button_list, Button:new(translate_left_button))
    table.insert(self.editor_button_list, Button:new(translate_right_button))
    table.insert(self.editor_button_list, Button:new(translate_up_button))
    table.insert(self.editor_button_list, Button:new(translate_down_button))
    
    
    -- lock buttons - we need a reference to these, as well as adding 
    -- them to the table so that enable and disable work appropriately
    self.draw_button        = LockButton:new(draw_button)
    self.flood_button       = LockButton:new(flood_button)
    table.insert(self.editor_button_list, self.draw_button)
    table.insert(self.editor_button_list, self.flood_button)

    -- even more special button - table entry and reference required as above
    -- disable this for now - do we need it?    
    -- self.colour_button  = ColourButton:new(colour_button)
    -- table.insert(self.editor_button_list, self.colour_button) 
    
    -- toggle buttons - we could keep a reference to these if we wanted to control 
    -- then from here, but they do the job themselves so we don't have to
    resolution_button = ToggleButton:new(res_button)
    table.insert(self.editor_button_list, resolution_button)

    table.insert(self.editor_button_list, ColourToggleButton:new(alpha_button))
end

-- arrays that hold the display elements for the grid pixels, and the grid lines that overlay them
local grid = {}
local grid_lines = {}

-- array that hold the pixel colours in the grid so we can fetch them with the colour picker
local grid_colour_array = {}
    


function GlyphEditor:update_grid_pixel(gridx, gridy, pixel)
    local glyph
    if grid ~= nil and grid[gridy] ~= nil and grid[gridy][gridx] ~= nil and pixel ~= nil then
        local colour, special = pixel_to_colour(pixel, self.colour_format)
        local dummy_fg_colour = get_simple_colour_by_name('BLACK')

        -- these glyph calculations rely on the glyphs being numerical adjacent
        if(special == 'multicolour') then
            if self.grid_size==32 then
                if resolution_button.mode == 'up' then
                    glyph = multicolour_grid_glyph32
                else
                    glyph = multicolour_grid_glyph32for16grid
                end
            else
                glyph = multicolour_grid_glyph
            end
        elseif (special == 'transparent') then
            if self.grid_size==32 then
                if resolution_button.mode == 'up' then
                    glyph = transparent_grid_glyph32
                else
                    glyph = transparent_grid_glyph32for16grid
                end
            else
                glyph = transparent_grid_glyph
            end
        else 
            glyph = glyph_editor_fill
        end
        
        -- update the pixel DrawListElement itself
        grid[gridy][gridx]:set_glyph(glyph)
        grid[gridy][gridx]:set_bg_colour(colour)
        grid_colour_array[gridy][gridx] = colour
    end
end

function GlyphEditor:update_grid_pixel_alpha(gridx, gridy, pixel)
    if grid ~= nil and grid[gridy] ~= nil and grid[gridy][gridx] ~= nil and pixel ~= nil then
        local colour, special = pixel_to_colour(pixel, self.colour_format)
        local dummy_fg_colour = get_simple_colour_by_name('BLACK')

        -- Adjust the colour based on level of transparency
        local alpha_colour
        if colour.a == 255 then
            -- dark blue for full transparency
            alpha_colour = get_SDL_Color_by_name('BLUE')
        elseif colour.a == 0 then
            alpha_colour = get_SDL_Color_by_name('WHITE')
        else
            -- partial transparency...
            alpha_colour = get_SDL_Color_by_name('CYAN')
            local sdl_colour = gulp_cpp.SDL_Color()
            sdl_colour.a = 255
            sdl_colour.r = 0
            sdl_colour.g = colour.a
            sdl_colour.b = colour.a
        end
        
        local glyph
        -- we use the transparent glyph for full transparency
        if colour.a == 0 then
            if self.grid_size==32 then
                if resolution_button.mode == 'up' then
                    glyph = transparent_grid_glyph32
                else
                    glyph = transparent_grid_glyph32for16grid
                end
            else
                glyph = transparent_grid_glyph
            end
        else 
            glyph = glyph_editor_fill
        end

        
        -- update the pixel DrawListElement itself
        grid[gridy][gridx]:set_glyph(glyph)
        grid[gridy][gridx]:set_bg_colour(alpha_colour)
        grid_colour_array[gridy][gridx] = colour
    end
end

function GlyphEditor:update_grid()
    if not self.alpha_mode then
        for line, columns in pairs(grid) do
            for column, element in pairs(grid[line]) do
                self:update_grid_pixel(column, line, self.glyph.pixels[self:get_pixel(line, column)])
            end
        end
    else
        for line, columns in pairs(grid) do
            for column, element in pairs(grid[line]) do
                self:update_grid_pixel_alpha(column, line, self.glyph.pixels[self:get_pixel(line, column)])
            end
        end
    end
end

function GlyphEditor:update_grid_for_colour_picker(mode)
    
    mode = mode or "normal"
    if mode == "alternate" then
        self:RGB_update_grid_for_colour_picker(mode)
    elseif mode == "shades" then
        self:SHADES_update_grid_for_colour_picker(mode)
    elseif mode == "named" then
        self:Named_update_grid_for_colour_picker(mode)
    else
        self:STANDARD_update_grid_for_colour_picker(mode)
    end
    
end

local x11_colours = require('graphics/x11_colours')

-- should be sorted
-- black at end is an off-by-one error
-- some names missing?
function GlyphEditor:Named_update_grid_for_colour_picker(mode)
    -- NOTICE: the RGB packing in this function is a function internal format ONLY
    local cells = 32*32
    local num_colours = 0
    local colour_index = 1
    local colour_array = {}
    for _,colour in pairs(x11_colours) do
        local rgb = bit32.lshift(colour.r, 16) + bit32.lshift(colour.g, 8) + colour.b
        colour_array[colour_index] = rgb
        num_colours = num_colours + 1
        colour_index = colour_index + 1
    end
    table.sort(colour_array)
    local num_x11_colours = num_colours
    
    local internal_colours = {}
    for name,colour in pairs(gulp.colour) do
        local rgb = bit32.lshift(colour.r, 16) + bit32.lshift(colour.g, 8) + colour.b
        table.insert(internal_colours, rgb)
    end
    table.sort(internal_colours)
    -- finally append (so we have X11 colours in order, followed by spectrum+extra colours in order)
    for _,rgb in ipairs(internal_colours) do
        colour_array[colour_index] = rgb
        num_colours = num_colours + 1
        colour_index = colour_index + 1
    end

    local cells_per_colour = cells / num_colours
    local colours_per_cell = 1 / cells_per_colour
    local current_colour = 1
    for i = 0,cells do
        local colour = colour_array[math.floor(current_colour)]
        local column = i % 32
        local line = math.floor(i / 32)
        if colour == nil then
            colour = 0
        end
        local sdl_colour = gulp_cpp.SDL_Color()
        sdl_colour.a = 255
        sdl_colour.r = bit32.extract(colour, 16, 8)
        sdl_colour.g = bit32.extract(colour, 8, 8)
        sdl_colour.b = bit32.extract(colour, 0, 8)
        --local name = get_name_of_colour(sdl_colour)
        --print(string.format("%x, %s", colour, name))
        --if colour == 0xadff2f then
        --    print("xxx")
        --end
        colour = colour_to_pixel(sdl_colour, self.colour_format)
        self:update_grid_pixel(column, line, colour)
        current_colour = current_colour + colours_per_cell
    end
end

function GlyphEditor:RGB_update_grid_for_colour_picker(mode)
    for line = 0,31 do
        for column = 0,31 do
            -- 10 bit RGB gives: 3 4 3
            local blue = bit32.extract(line, 2, 3)  -- 0-7
            local green = bit32.extract(line, 0, 2) * 4 + bit32.extract(column, 3, 2) -- 0x15
            local red = bit32.extract(column, 0, 3) -- 0-7
            
            local sdl_colour = gulp_cpp.SDL_Color()
            sdl_colour.b = math.floor((blue * 255) / 7)
            sdl_colour.g = math.floor((green * 255) / 15)
            sdl_colour.r = math.floor((red * 255) / 7)
            sdl_colour.a = 255
            local colour = colour_to_pixel(sdl_colour, self.colour_format)
            self:update_grid_pixel(column, line, colour)
        end
    end
end

function GlyphEditor:SHADES_update_grid_for_colour_picker(mode)
    -- 32 = 5 bits. lowest 3 bits always 1 ... or should use a log based approach?
    -- 
    local tab = 
    {
        -- equal channels (5)
        0xFFFFFF,
        0xFFFF00,
        0xFF00FF,
        0xFF0000,
        
        0x00FFFF,
        0x00FF00,
        0x0000FF,
        
        -- 25 others ... 1 + half, plus 1 + half + quarter ... various combinations.
        -- 1 + half shades (6)
        0xFF8000,
        0xFF0080,
        0x00FF80,
        0x80FF00,
        0x8000FF,
        0x0080FF,
    
        -- 1 + 2 halves (3)
        0xFF8080,
        0x80FF80,
        0x8080FF,

        -- 1 + half + quarter (6)
        0xFF8040,
        0x80FF40,
        0x4080FF,
        0xFF4080,
        0x40FF80,
        0x8040FF,
        
        -- 1 + quarter shades (6)
        0xFF4000,
        0xFF0040,
        0x00FF40,
        0x40FF00,
        0x4000FF,
        0x0040FF,

        -- 1 + three-quarter shades (6)
        0xFFC000,
        0xFF00C0,
        0x00FFC0,
        0xC0FF00,
        0xC000FF,
        0x004CFF,
    }

    for line = 0,31 do
        local tc = tab[line+1]
        for column = 0,31 do
            local blue = bit32.extract(tc, 16, 8)
            local green = bit32.extract(tc, 8, 8)
            local red = bit32.extract(tc, 0, 8)
            
            local fraction = column/31
            
            local sdl_colour = gulp_cpp.SDL_Color()
            sdl_colour.a = 255
            sdl_colour.b = math.floor(blue * fraction)
            sdl_colour.g = math.floor(green * fraction)
            sdl_colour.r = math.floor(red * fraction)
            
            local colour = colour_to_pixel(sdl_colour, self.colour_format)
            self:update_grid_pixel(column, line, colour)
        end
    end
end


function GlyphEditor:STANDARD_update_grid_for_colour_picker(mode)
--[[ 
    -- add buttons to select colours for the corners
    -- or pick a good selection and use the following modes
    
    if mode == 'bgrw' then
        -- this is default
    elseif mode == 'rgbw' then
        diagonal, right, bottom  = red, green, blue
    elseif mode == 'gbrw' then
        diagonal, right, bottom  = green, blue, red
    elseif mode == 'mcyw' then
        diagonal, right, bottom  = magenta, cyan, yellow
    elseif mode == 'cymw' then
        diagonal, right, bottom  = cyan, yellow, magenta
    elseif mode == 'ymcw' then
        diagonal, right, bottom  = yellow, magenta, cyan
    elseif mode == 'ybrw' then
        diagonal, right, bottom  = yellow, blue, red
    elseif mode == 'ybgw' then
        diagonal, right, bottom  = yellow, blue, green
    end
--]]    
    
    -- put white in middle, 
    -- corners are red, green, blue and black
    -- then calculate every other pixel based on distance from the middle and corners
    -- what fun!
    
    local red       = 0x000011
    local green     = 0x001100
    local blue      = 0x110000
    local magenta   = 0x110011
    local cyan      = 0x111100
    local yellow    = 0x001111

    local diagonal = cyan
    local right = blue
    local bottom = green
    local corner = red
 
    local colours = 
    {
        [0] = 
        {
            [0]=
            {   -- top left quadrant 
                diagonal = cyan,
                right = blue,
                bottom = green,
                corner = red
            },
            [1]=
            {   -- top right quadrant 
                diagonal = magenta,
                right = blue,
                bottom = red,
                corner = green
            },
        },
        [1] =
        {
            [0]=
            {   -- bottom left quadrant 
                diagonal = yellow,
                right = red,
                bottom = green,
                corner = blue
            },
            [1]=
            {   -- bottom right quadrant 
                diagonal = 0,
                right = red,
                bottom = red,
                corner = cyan
            }
        }
    }
    if mode == "green" then
        colours[1][1] = {
                diagonal = 0,
                right = green,
                bottom = green,
                corner = magenta
            }
    end
    
    
 
    for qline = 0,1 do
        for qcolumn = 0,1 do
            -- fill quarter of array
            local diagonal = colours[qline][qcolumn].diagonal
            local right = colours[qline][qcolumn].right
            local bottom = colours[qline][qcolumn].bottom
            local corner = colours[qline][qcolumn].corner
            
            for line = 0,15 do
                for column = 0,15 do
                    local bottom_value = bottom * line
                    local right_value = right * column
                    local diagonal_value = (diagonal * 0x0F) - (diagonal * (math.abs(line-column)))
                    local corner_value = (corner * 0x0F) - (corner * (15 - math.min(line, column)))
                    if corner_value < 0 then corner_value = 0 end
                    
                    local function get_colour_value(mask)
                        local colour =  bit32.band(bottom_value, mask) + 
                                        bit32.band(right_value, mask) + 
                                        bit32.band(diagonal_value, mask) +
                                        bit32.band(corner_value, mask)
                        if colour > mask then colour = mask end
                        return colour
                    end
                    
                    local sdl_colour = gulp_cpp.SDL_Color()
                    sdl_colour.a = 255
                    sdl_colour.b = bit32.rshift(get_colour_value(0xFF0000), 16)
                    sdl_colour.g = bit32.rshift(get_colour_value(0x00FF00), 8)
                    sdl_colour.r = get_colour_value(0x0000FF)
                    
                    -- calculate colour address
                    local print_column = column
                    local print_line = line
                    if qcolumn == 1 then print_column = 31 - column end
                    if qline == 1 then print_line = 31 - line end
                    local colour = colour_to_pixel(sdl_colour, self.colour_format)
                    self:update_grid_pixel(print_column, print_line, colour)
                end
            end
        end
    end
end

function GlyphEditor:add_32_grid_to_draw_list()
    local glyph
    for line = 0, 31 do
        grid[line] = {}
        grid_colour_array[line] = {}

        for column = 0, 31 do
                
            local colour, special = pixel_to_colour(self.glyph.pixels[self:get_pixel(line, column)], self.colour_format)
            local dummy_fg_colour = get_simple_colour_by_name('BLACK')
    
            -- these glyph calculations rely on the glyphs being numerical adjacent
            if(special == 'multicolour') then
                if resolution_button.mode == 'up' then
                    glyph = multicolour_grid_glyph32
                else
                    glyph = multicolour_grid_glyph32for16grid
                    --glyph = multicolour_grid_glyph
                end
            elseif (special == 'transparent') then
                if resolution_button.mode == 'up' then
                    glyph = transparent_grid_glyph32
                else
                    glyph = transparent_grid_glyph32for16grid
                    --glyph = transparent_grid_glyph
                end
            else 
                glyph = glyph_editor_fill
            end
            
            grid[line][column] = new_absolute_display_element(glyph, (line*0.5)+grid_loc.line, (column*0.5)+grid_loc.column, 
                                                        (line*32)+column, dummy_fg_colour, colour, false)
        end
    end
    
    -- the actual grid
    local cell_glyph = glyph_editor_cell32
    if resolution_button.mode == 'down' then 
        cell_glyph = glyph_editor_cell
    end
    
    for line = 0, 15 do
        grid_lines[line] = {}

        
        
        for column = 0, 15 do
            grid_lines[line][column] = new_absolute_display_element(cell_glyph, line+grid_loc.line, column+grid_loc.column, 
                                                1025, get_simple_colour_by_name('BLACK'), get_SDL_Color_by_name('RED'), true)
        end
        
        -- stick a closer on the end
        local column = 16
        grid_lines[line][column] = new_absolute_display_element(glyph_editor_cell_right_closer, line+grid_loc.line, column+grid_loc.column, 
                                                1025, get_simple_colour_by_name('BLACK'), get_SDL_Color_by_name('RED'), true)
    end
    
    -- stick a close on the bottom too
    local line = 16
    grid_lines[line] = {}
    for column = 0, 15 do
        grid_lines[line][column] = new_absolute_display_element(glyph_editor_cell_bottom_closer, line+grid_loc.line, column+grid_loc.column, 
                                                1025, get_simple_colour_by_name('BLACK'), get_SDL_Color_by_name('RED'), true)
    end
        
    -- and the very last magic dot in the corner    
    local column = 16
    grid_lines[line][column] = new_absolute_display_element(glyph_editor_cell_magic_dot, line+grid_loc.line, column+grid_loc.column, 
                                                1025, get_simple_colour_by_name('BLACK'), get_SDL_Color_by_name('RED'), true)
end


function GlyphEditor:add_16_grid_to_draw_list()
    local glyph
    for line = 0, 15 do
        grid[line] = {}
        grid_colour_array[line] = {}

        for column = 0, 15 do
                
            local colour, special = pixel_to_colour(self.glyph.pixels[self:get_pixel(line, column)], self.colour_format)
            local dummy_fg_colour = get_simple_colour_by_name('BLACK')
    
            -- these glyph calculations rely on the glyphs being numerical adjacent
            if(special == 'multicolour') then
                glyph = multicolour_grid_glyph
            elseif (special == 'transparent') then
                glyph = transparent_grid_glyph
            else 
                glyph = glyph_editor_fill
            end
            
            grid[line][column] = new_absolute_display_element(glyph, (line)+grid_loc.line, (column)+grid_loc.column, 
                                                        (line*32)+column, dummy_fg_colour, colour, false)
        end
    end
    
    -- the actual grid
    for line = 0, 15 do
        grid_lines[line] = {}

        for column = 0, 15 do
            grid_lines[line][column] = new_absolute_display_element(glyph_editor_cell, line+grid_loc.line, column+grid_loc.column, 
                                                1025, get_simple_colour_by_name('BLACK'), get_SDL_Color_by_name('RED'), true)
        end
        
        -- stick a closer on the end
        local column = 16
        grid_lines[line][column] = new_absolute_display_element(glyph_editor_cell_right_closer, line+grid_loc.line, column+grid_loc.column, 
                                                1025, get_simple_colour_by_name('BLACK'), get_SDL_Color_by_name('RED'), true)
    end
    
    -- stick a close on the bottom too
    local line = 16
    grid_lines[line] = {}
    for column = 0, 15 do
        grid_lines[line][column] = new_absolute_display_element(glyph_editor_cell_bottom_closer, line+grid_loc.line, column+grid_loc.column, 
                                                1025, get_simple_colour_by_name('BLACK'), get_SDL_Color_by_name('RED'), true)
    end
        
    -- and the very last magic dot in the corner    
    local column = 16
    grid_lines[line][column] = new_absolute_display_element(glyph_editor_cell_magic_dot, line+grid_loc.line, column+grid_loc.column, 
                                                1025, get_simple_colour_by_name('BLACK'), get_SDL_Color_by_name('RED'), true)

end

function GlyphEditor:remove_grid_from_draw_list()
    for line, columns in pairs(grid) do
        for column, element in pairs(grid[line]) do
            element:hide()
            --element = nil             -- this does not allow garbage collection!!!
            grid[line][column] = nil    -- but this does...
        end
    end

    -- the actual grid (including the closers)
    for line, columns in pairs(grid_lines) do
        for column, element in pairs(grid_lines[line]) do
            element:hide()
            grid_lines[line][column] = nil
        end
    end
    
    collectgarbage()
end


function GlyphEditor:glyph_set_scroll(direction)
    
    local gs = self.glyph_set
    local glyph_set_height = ((gs.glyph_set_lines * gs.characters_per_line) / glyph_set_loc.xsize)
    local scroll_limit = glyph_set_height - glyph_set_loc.ysize
    
    if direction=='up' and self.glyph_set_scroll_line > 0 then
        self.glyph_set_scroll_line = self.glyph_set_scroll_line - 1
    elseif direction=='down' and self.glyph_set_scroll_line < scroll_limit then
        self.glyph_set_scroll_line = self.glyph_set_scroll_line + 1
    end
    
end

        
function GlyphEditor:draw_glyph_set()
    local glyph=self.glyph_set.character_base_code + (glyph_set_loc.xsize * self.glyph_set_scroll_line)
    local bright_background = false
    local selected_glyph_line, selected_glyph_col
    
    for line=glyph_set_loc.line, glyph_set_loc.line+glyph_set_loc.ysize-1 do
        for column=glyph_set_loc.column, glyph_set_loc.column+glyph_set_loc.xsize-1 do
            -- adjust the background to make the glyph boundries obvious
            set_bg_colour(bright_background and 'WHITE' or 'BRIGHT_WHITE')
            
            print_glyph_at(line, column, glyph)
            
            if(glyph == self.selected_glyph+self.glyph_set.character_base_code) then
                selected_glyph_line, selected_glyph_col = line, column
            end
            
            -- set up for next iteration
            glyph = glyph + 1
            bright_background = not bright_background
        end
        bright_background = not bright_background
    end
    
    set_bg_colour('WHITE')
    
    print_string_at(glyph_set_label.line, glyph_set_label.column, self.glyph_set.name) 
    local string = string.format("0x%02x",self.selected_glyph)
    print_string_at(glyph_set_label.line+1, glyph_set_label.column+1, string) 
    
    
    -- print the selector square
    if(selected_glyph_line) then
        set_bg_colour_transparent()
        self:draw_object(selected_glyph_line-0.5, selected_glyph_col-0.5, selector_square)
        set_bg_colour('WHITE')
    end
            
end

function GlyphEditor:draw_preview()
    -- print the edit buffer display texture
    for colour, value in pairs(gulp.colour) do
        if(preview_colour_position[colour] ~= nil) then
            set_fg_colour(colour)
            if(colour == 'WHITE') then
                set_bg_colour('BRIGHT_WHITE')
            end
            print_glyph_at(preview_colour_position[colour], 1, edit_glyph)
            if(colour == 'WHITE') then
                set_bg_colour('WHITE')
            end
        end
    end
    
    set_fg_colour(editor_fg_colour)
end

function GlyphEditor:draw_copy_buffer()
    -- print the copy buffer contents
    print_glyph_at(copy_buffer.line, copy_buffer.column, copy_buffer_glyph)
end


function GlyphEditor:draw_palette()
    
    if self.submode == 'animation_pick' then
        -- draw a border in white
        set_fg_colour('BRIGHT_WHITE')
        self:draw_box(palette_loc.line-1, palette_loc.column-1, 5, 5)
        set_fg_colour('BLACK')
    end
    
    local function draw_colour(line, column, colour)
        local bright_colour = 'BRIGHT_' .. colour
        
        set_bg_colour(colour)
        print_glyph_at(palette_loc.line + line - 1, palette_loc.column + column - 1, 0x20)
        set_bg_colour(bright_colour)
        print_glyph_at(palette_loc.line + line - 1, palette_loc.column + column, 0x20)
    end
    
    
    for line = 1, 4 do
        -- top 4 lines - these are the standard colours
        draw_colour(line, 1, palette_colour_position[(line*2) - 1])
        draw_colour(line, 3, palette_colour_position[(line*2)])
    end

    -- special line for multicolour / transparent
    set_bg_colour('WHITE')
    print_glyph_at(palette_loc.line + 4.5, palette_loc.column+1, multicolour_palette_glyph)
    print_glyph_at(palette_loc.line + 4.5, palette_loc.column+2, transparent_palette_glyph)

    for uc = 1, 10 do
        -- special lines for user colours
        set_bg_colour(self.user_colours[uc])
        print_glyph_at(palette_loc.line + 5 + uc, palette_loc.column+1, 0x20)
    end
                
    set_bg_colour('WHITE')

end

function GlyphEditor:draw_tools_cluster()
    -- draw the overlapping colour selection glyphs
    -- bottom one first
    -- nominally at tools_left.line, tools_left.column+1
    
    local glyph
    
    if(self.alt_colour == gulp.colour.dark_pink_means_multicoloured) then
        glyph = background_multi_glyph
    elseif(self.alt_colour == gulp.colour.hot_pink_means_transparent) then
        glyph = background_trans_glyph
    else
        set_bg_colour(self.alt_colour)
        glyph = background_colour_glyph
    end
    
    if(self.submode == 'background_pick') then
        glyph = glyph+32    -- next line for the selection glyphs!
    end
    
    print_glyph_at(tools_left.line+0.25, tools_left.column+1.25, glyph)
    
    if(self.colour == gulp.colour.dark_pink_means_multicoloured) then
        glyph = foreground_multi_glyph
    elseif(self.colour == gulp.colour.hot_pink_means_transparent) then
        glyph = foreground_trans_glyph
    else
        set_bg_colour(self.colour)
        glyph = foreground_colour_glyph
    end
    
    if(self.submode == 'foreground_pick') then
        glyph = glyph+32    -- next line for the selection glyphs!
    end

    print_glyph_at(tools_left.line-0.25, tools_left.column+0.75, glyph)

    
    set_bg_colour('WHITE')
    
    
    
    
    self:draw_static_object(tools_left)
    self:draw_static_object(tools_right)
    self:draw_static_object(tools_top)
    self:draw_static_object(tools_bottom)
    self:draw_static_object(translate_middle)

    if(self.submode=='flood') then
        self:draw_static_object(tools_right_flood_button_down)      -- overwrite default drawn above
    end
    
    if(self.submode=='draw') then
        self:draw_static_object(tools_left_draw_button_down)        -- overwrite default drawn above
    end
    
    if(self.submode=='foreground_pick') then
        set_bg_colour('BRIGHT_YELLOW')
        self:draw_static_object(tools_left_colour_button_down)      -- overwrite default drawn above
    end
    
    if(self.submode=='background_pick') then
        self:draw_static_object(tools_left_colour_button_down)      -- overwrite default drawn above
        set_bg_colour('BRIGHT_YELLOW')
        self:draw_static_object(tools_bottom)                       -- overwrite background colour used above
    end
        
    set_bg_colour('WHITE')

end


function GlyphEditor:draw_editor()
    
    
    set_bg_colour('WHITE')
    set_fg_colour(editor_fg_colour)
    
    blank_screen()
    
    print_string_at(0, 4, "Glyph Editor")
    
    -- draw main areas of the screen
    --self:draw_edit_grid() 
    self:draw_preview()
    self:draw_copy_buffer()
    self:draw_palette()
    self:draw_glyph_set()       
    self:draw_tools_cluster()
    self:draw_animation_controls()
    
    -- draw some other stuff
    self:draw_static_object(copy_paste_top)
    self:draw_static_object(copy_paste_bottom)
    
    
end

function GlyphEditor:colour_pixel(gridx, gridy, alt)
    self:colour_pixel_ex(gridx, gridy, false, alt)
end


function GlyphEditor:colour_pixel_ex(gridx, gridy, force_last_pixel, alt)
    
    local pixel
    
    if(force_last_pixel == true and self.last_pixel ~= nil) then
        pixel = self.last_pixel
    else
        if alt then
            pixel = colour_to_pixel(self.alt_colour, self.colour_format)
        else
            pixel = colour_to_pixel(self.colour, self.colour_format)
        end
        
        self.last_pixel = pixel
    end
    
    local function set_pixel(gridx, gridy, pixel)
        -- write the new pixel value into the pixel array
        self.glyph.pixels[self:get_pixel(gridy, gridx)] = pixel
    
        -- update the edit buffers
        self:update_editor_texture_pixel(edit_buffer_texture_offset, gridx, gridy, pixel)
        -- update the draw list
        self:update_grid_pixel(gridx, gridy, pixel)
    end
    
    if self.grid_size == 32 and resolution_button.mode == 'down' then
        -- 32 bit grid in 16 bit mode, so colour 4 pixels
        local x = gridx - (gridx%2)
        local y = gridy - (gridy%2)
        set_pixel(x, y, pixel)
        set_pixel(x, y+1, pixel)
        set_pixel(x+1, y, pixel)
        set_pixel(x+1, y+1, pixel)
    else
        set_pixel(gridx, gridy, pixel)
    end
    
end 



function GlyphEditor:grid_convert_to_click(mx, my)

    local grid_click, gridx, gridy = self:unfloored_convert_to_click(mx, my, grid_loc)
    
    if(grid_click) then
        
        if self.grid_size == 32 or self.mode == 'colour_picker' then
            gridx = gridx*2
            gridy = gridy*2
        end 
        
        gridx = math.floor(gridx)
        gridy = math.floor(gridy)
    
        return grid_click, gridx, gridy
    end
    
    return false
end

function GlyphEditor:check_grid_click(mx, my, right)
    
    local grid_click, gridx, gridy = self:grid_convert_to_click(mx, my)
    
    if(grid_click) then
        if(self.submode == 'flood') then
            self:floodfill(gridx, gridy, right)
        elseif(self.submode == 'background_pick') then
            -- pick the pixel colour from the grid
            self:change_submode(self.old_submode)
        elseif(self.submode == 'foreground_pick') then
            -- pick the pixel colour from the grid
            self:change_submode(self.old_submode)
        elseif(self.submode == 'gridpick') then
            -- pick a user colour from the normal grid
            
            -- make sure colours arrays are up to date
            self:update_grid()
            
            -- adjust submode - set it to the user colour grid pick button pressed
            self.submode = self.subsubmode
            
            -- get the colour
            local c = grid_colour_array[gridy][gridx]
            self.red = c.r
            self.green = c.g
            self.blue = c.b
            
            -- and use the colour picker screen action button function to do the work
            self:pick_colour_picker_colour()
        elseif(self.mode == 'colour_picker') then
            -- pick the pixel colour from the grid
            local c = grid_colour_array[gridy][gridx]
            self.red = c.r
            self.green = c.g
            self.blue = c.b
        else
            self:colour_pixel(gridx, gridy, right)
        end
    
    
    
    end
    
end

function GlyphEditor:check_palette_click(mx, my, right)
    
    local palette_click, palette_x, palette_y = self:convert_to_click(mx, my, palette_loc)
    
    if(palette_click) then
        palette_x = palette_x + 1  -- universally apply offset to account for Lua's 
        palette_y = palette_y + 1  -- RIDICULOUS array starting position rules ;-)
        
        local picked_colour
        local simple_colour
        
        if palette_y <= 4 then
            if palette_x <= 2 then
                simple_colour = palette_colour_position[palette_y*2 -1]
            else
                simple_colour = palette_colour_position[palette_y*2]
            end
            if palette_x == 2 or palette_x == 4 then
                simple_colour = 'BRIGHT_'..simple_colour
            end
            picked_colour = gulp.colour[simple_colour]
        elseif palette_y == 5 or palette_y == 6 then
            if palette_x == 2 then
                -- multicoloured
                picked_colour = gulp.colour.dark_pink_means_multicoloured
            elseif palette_x == 3 then
                -- transparent
                picked_colour = gulp.colour.hot_pink_means_transparent
            end
        elseif palette_y > 6 then
            -- user colours
            picked_colour = self.user_colours[palette_y-6]
        end
        
        -- update the correct colour with what was just picked
        if picked_colour then
            if(self.submode == 'foreground_pick') then
                self.colour = picked_colour
                self:change_submode(self.old_submode)
            elseif(self.submode == 'background_pick') then
                self.alt_colour = picked_colour
                self:change_submode(self.old_submode)
            elseif(self.submode == 'animation_pick') then
                if simple_colour then
                    self.animation_colour = simple_colour
                    self.animation_colour_button:adjust_bg_colour(get_SDL_Color_by_name(simple_colour))
                end
                self:change_submode('draw')
            else
                -- there is space in the palette area - this might be nil
                if picked_colour then
                    if right then
                        self.alt_colour = picked_colour
                    else
                        self.colour  = picked_colour
                    end
                end
            end
        end
    end
end


function GlyphEditor:update_editor_texture(texture_offset)
    local graphics = gulp.app:get_standard_graphics_context()
    if graphics then
        graphics:update_texture_set_glyph(edit_glyph, texture_offset, self.glyph_set.glyph_size,
            self.glyph_set_sdl_surface, self.selected_glyph, self.glyph_set.characters_per_line)  
    end
end

function GlyphEditor:update_editor_texture_pixel(texture_offset, x, y, pixel)
    local graphics = gulp.app:get_standard_graphics_context()
    if graphics then
        graphics:update_texture_set_pixel(edit_glyph, texture_offset, x, y, pixel)  
    end
end

function GlyphEditor:check_glyph_selector_click(mx, my)
    
    local glyph_click, glyphx, glyphy = self:convert_to_click(mx, my, glyph_set_loc)
    
    if(glyph_click) then
        glyphy = glyphy + self.glyph_set_scroll_line
        self:save_glyph()
        self.selected_glyph = glyphx + (glyphy*glyph_set_loc.xsize)
        self.glyph = gulp_cpp.load_glyph(self.glyph_set_sdl_surface, self.glyph_set, self.selected_glyph)
        self:update_grid()
        self:update_editor_texture(edit_buffer_texture_offset)
    end
end

function GlyphEditor:editor_mousepressed(mx, my, handled, right)
    
    self:check_grid_click(mx, my, right)
    self:check_glyph_selector_click(mx, my)
    self:check_palette_click(mx, my, right)
    
    return true     -- swallow the mousepress
end

function GlyphEditor:colour_picker_mousepressed(mx, my, handled)
    
    self:check_grid_click(mx, my)
    
    return true     -- swallow the mousepress
end

function GlyphEditor:editor_mouse_down_moved(mx, my, handled, right)
    
    local grid_click, gridx, gridy = self:grid_convert_to_click(mx, my)
    if(grid_click) then
        if(self.submode == 'draw') then
            self:colour_pixel_ex(gridx, gridy, true, right)
        end
    end
        
    return true
end     

function GlyphEditor:change_submode(new_submode)
    self.submode = new_submode
    
    self.draw_button:release()
    self.flood_button:release()
    
    if new_submode ~= 'animation_pick' then
        self.animation_colour_button:release()
    end
    
    
    -- disabled
    -- self.colour_button:release()
    
    -- cover the case where we return to draw mode automatically
    -- without a click on the draw button
    if(new_submode == 'draw') then
        self.draw_button:activate()
    end
end

    

function GlyphEditor:load_button_function()
    -- save current edits before we do anything else
    self:save_glyph()
    -- display a whole new screen!
    self:change_mode('glyph_set_load')
end

function GlyphEditor:colour_picker_function(button)
    -- save current edits before we do anything else
    self:save_glyph()
    -- display a whole new screen!
    self:change_mode('colour_picker')
    
    -- record the actual pick button that was pressed
    self.submode = button
    self:copy_colour_picker_existing_colours()
end

function GlyphEditor:grid_picker_function(button)
    self.subsubmode = button        
    self:change_submode('gridpick')
end

function GlyphEditor:draw_button_function()
    self:change_submode('draw')
end

function GlyphEditor:flood_button_function()
    self:change_submode('flood')
end

function GlyphEditor:resolution_button_function(mode)
    -- refresh the grid draw list
    self:remove_grid_from_draw_list()
    self:add_32_grid_to_draw_list()
end

function GlyphEditor:alpha_button_function(mode, x, y)
    self.alpha_mode = not self.alpha_mode
    --self:update_edit_buffer()   
    self:update_grid()
end


function GlyphEditor:colour_button_function()
    if(self.submode == 'foreground_pick') then
        self:change_submode('background_pick')
    elseif(self.submode == 'background_pick') then
        self:change_submode('foreground_pick')
    else
        self.old_submode = string.sub(self.submode, 1)      -- copy string
        self:change_submode('foreground_pick')
    end
end

function GlyphEditor:animation_colour_button_function()
    if(self.submode == 'animation_pick') then
        self:change_submode('draw')
    else
        self:change_submode('animation_pick')
    end
end

function GlyphEditor:clear_button_function()
    self:clear_current_glyph()
    self:update_edit_buffer()   
    self:update_grid()
end
    
function GlyphEditor:rotate_button_function()
    self:rotate()
    self:update_edit_buffer()   
    self:update_grid()
end
    
function GlyphEditor:hflip_button_function()
    self:horizontal_flip()
    self:update_edit_buffer()   
    self:update_grid()
end
    
function GlyphEditor:vflip_button_function()
    self:vertical_flip()
    self:update_edit_buffer()   
    self:update_grid()
end
    
function GlyphEditor:translate_button_function(direction)
    self:translate(direction)
    self:update_edit_buffer()   
    self:update_grid()
end
    
function GlyphEditor:save_button_function()
    self:save_glyph()
end
    
function GlyphEditor:revert_button_function()
    self.glyph = gulp_cpp.load_glyph(self.glyph_set_sdl_surface, self.glyph_set, self.selected_glyph)
    self:update_grid()
    self:update_editor_texture(edit_buffer_texture_offset)
end
    
function GlyphEditor:copy_button_function()
    self.copy_glyph = self:glyph_copy(self.glyph, self.glyph.size, 'copy')
end
    
function GlyphEditor:paste_button_function()
    if(self.copy_glyph ~= nil) then
        self.glyph = self:glyph_copy(self.copy_glyph, self.glyph.size, 'paste')
        self:save_glyph()
        self:update_editor_texture(edit_buffer_texture_offset)
        self:update_grid()
    end
end

-- *********************** END GRID SCREEN ****************************************

local RGB_SELECTOR_LINE = 12
local RGB_SELECTOR_COLUMN = 20

function GlyphEditor:draw_colour_picker(gr)
    set_bg_colour('WHITE')
    set_fg_colour(editor_fg_colour)
    
    blank_screen()
    
    print_string_at(0, 4, "Colour Picker")
    
    set_fg_colour('RED')
    print_string_at(RGB_SELECTOR_LINE, RGB_SELECTOR_COLUMN, string.format("%3d", self.red))
    set_fg_colour('GREEN')
    print_string_at(RGB_SELECTOR_LINE, RGB_SELECTOR_COLUMN+4, string.format("%3d", self.green))
    set_fg_colour('BLUE')
    print_string_at(RGB_SELECTOR_LINE, RGB_SELECTOR_COLUMN+8, string.format("%3d", self.blue))
    set_fg_colour(editor_fg_colour)
    
    local sdl_colour = gulp_cpp.SDL_Color()
    sdl_colour.a = 255
    sdl_colour.r = self.red
    sdl_colour.g = self.green
    sdl_colour.b = self.blue
    gr:FillRect(get_SDL_Color_by_name('BLACK'), RGB_SELECTOR_COLUMN+5, RGB_SELECTOR_LINE+3.5, RGB_SELECTOR_COLUMN+12, RGB_SELECTOR_LINE+7)
    gr:FillRect(sdl_colour, RGB_SELECTOR_COLUMN, RGB_SELECTOR_LINE+4.5, RGB_SELECTOR_COLUMN+11, RGB_SELECTOR_LINE+6)
    
    local name = get_name_of_colour(sdl_colour)
    if name then
        local column = 32-#name
        if column < 0 then column = 0 end
        print_string_at(RGB_SELECTOR_LINE+7, column, name)
    end
end

function GlyphEditor:red_up(delta)
    self.red = self.red + delta
    if self.red > 255 then
        self.red = 255
    end
end
function GlyphEditor:red_down(delta)
    self.red = self.red - delta
    if self.red < 0 then
        self.red = 0
    end
end
function GlyphEditor:green_up(delta)
    self.green = self.green + delta
    if self.green > 255 then
        self.green = 255
    end
end
function GlyphEditor:green_down(delta)
    self.green = self.green - delta
    if self.green < 0 then
        self.green = 0
    end
end
function GlyphEditor:blue_up(delta)
    self.blue = self.blue + delta
    if self.blue > 255 then
        self.blue = 255
    end
end
function GlyphEditor:blue_down(delta)
    self.blue = self.blue - delta
    if self.blue < 0 then
        self.blue = 0
    end
end

function GlyphEditor:pick_colour_picker_colour(alt)
    local sdl_colour = gulp_cpp.SDL_Color()
    sdl_colour.a = 255
    sdl_colour.r = self.red
    sdl_colour.g = self.green
    sdl_colour.b = self.blue

    self.user_colours[self.submode] = sdl_colour
    if alt and alt==1 then
        self.alt_colour = sdl_colour
    else
        self.colour = sdl_colour
    end
    self:change_mode('editor')
    self:change_submode('draw')
end

function GlyphEditor:copy_colour_picker_existing_colours()
    local sdl_colour = self.user_colours[self.submode]
    self.red = sdl_colour.r
    self.green = sdl_colour.g
    self.blue = sdl_colour.b
end


function GlyphEditor:darker_click()
    self.red = math.floor(self.red * 0.9)
    self.green = math.floor(self.green * 0.9)
    self.blue = math.floor(self.blue * 0.9)
end

function GlyphEditor:create_colour_picker_buttons()
    
    local cancel_button =   {   line=22,  column=3,  ysize=1,    xsize=1,    
                                first_glyph = 22+gso_editor, 
                                text = 'Cancel',
                                click = self.load_screen_cancel_button_action,      -- works for this screen too
                                parent = self, enabled = false}
    
    table.insert(self.colour_picker_button_list, Button:new(cancel_button))
    
    local std_button =   {   line=1,  column=RGB_SELECTOR_COLUMN,  ysize=1,    xsize=1,    
                                first_glyph = 22+gso_editor, 
                                text = 'Std',
                                click = self.update_grid_for_colour_picker, click_arg="normal",
                                parent = self, enabled = false}
    table.insert(self.colour_picker_button_list, Button:new(std_button))
    
    local green_button =   {   line=1,  column=RGB_SELECTOR_COLUMN+5,  ysize=1,    xsize=1,    
                                first_glyph = 22+gso_editor, 
                                text = 'Green',
                                click = self.update_grid_for_colour_picker, click_arg="green",
                                parent = self, enabled = false}
    table.insert(self.colour_picker_button_list, Button:new(green_button))
    
    local alt_button =   {   line=2,  column=RGB_SELECTOR_COLUMN,  ysize=1,    xsize=1,    
                                first_glyph = 22+gso_editor,
                                text = 'Alt',
                                click = self.update_grid_for_colour_picker, click_arg="alternate",
                                parent = self, enabled = false}
    table.insert(self.colour_picker_button_list, Button:new(alt_button))
    
    local shades_button =   {   line=2,  column=RGB_SELECTOR_COLUMN+5,  ysize=1,    xsize=1,    
                                first_glyph = 22+gso_editor, 
                                text = 'Shades',
                                click = self.update_grid_for_colour_picker, click_arg="shades",
                                parent = self, enabled = false}
    table.insert(self.colour_picker_button_list, Button:new(shades_button))

    local named_button =   {   line=3,  column=RGB_SELECTOR_COLUMN,  ysize=1,    xsize=1,    
                                first_glyph = 22+gso_editor,
                                text = 'Named',
                                click = self.update_grid_for_colour_picker, click_arg="named",
                                parent = self, enabled = false}
    table.insert(self.colour_picker_button_list, Button:new(named_button))

    
    local colour_pick_button1 = { line=22, column=RGB_SELECTOR_COLUMN, ysize=1, xsize=2, text="SET", first_glyph=0x32+gso_editor2,  click=self.pick_colour_picker_colour, click_arg=0, parent=self   }
    table.insert(self.colour_picker_button_list, Button:new(colour_pick_button1))

    local colour_pick_button2 = { line=22, column=RGB_SELECTOR_COLUMN+6, ysize=1, xsize=2, text="ALT", first_glyph=0x32+gso_editor2,  click=self.pick_colour_picker_colour, click_arg=1, parent=self   }
    table.insert(self.colour_picker_button_list, Button:new(colour_pick_button2))

    local ln = RGB_SELECTOR_LINE

    local darker_button =   {   line=ln-4,  column=RGB_SELECTOR_COLUMN,  ysize=1,    xsize=1,    
                                first_glyph = 22+gso_editor, 
                                text = 'Darker',
                                click = self.darker_click,
                                parent = self, enabled = false}
    table.insert(self.colour_picker_button_list, Button:new(darker_button))

    local red_up = {   line=ln-1,  column=21,  ysize=1,    xsize=1,    
                                first_glyph = 47+gso_editor, 
                                click = self.red_up, click_arg=1,
                                parent = self, enabled = false}
    local red_down = {   line=ln+1,  column=21,  ysize=1,    xsize=1,    
                                first_glyph = 111+gso_editor, 
                                click = self.red_down, click_arg=1,
                                parent = self, enabled = false}
    local green_up = {   line=ln-1,  column=25,  ysize=1,    xsize=1,    
                                first_glyph = 47+gso_editor, 
                                click = self.green_up, click_arg=1,
                                parent = self, enabled = false}
    local green_down = {   line=ln+1,  column=25,  ysize=1,    xsize=1,    
                                first_glyph = 111+gso_editor, 
                                click = self.green_down, click_arg=1,
                                parent = self, enabled = false}
    local blue_up = {   line=ln-1,  column=29,  ysize=1,    xsize=1,    
                                first_glyph = 47+gso_editor, 
                                click = self.blue_up, click_arg=1,
                                parent = self, enabled = false}
    local blue_down = {   line=ln+1,  column=29,  ysize=1,    xsize=1,    
                                first_glyph = 111+gso_editor, 
                                click = self.blue_down, click_arg=1,
                                parent = self, enabled = false}
    
    table.insert(self.colour_picker_button_list, Button:new(red_up))
    table.insert(self.colour_picker_button_list, Button:new(red_down))
    table.insert(self.colour_picker_button_list, Button:new(green_up))
    table.insert(self.colour_picker_button_list, Button:new(green_down))
    table.insert(self.colour_picker_button_list, Button:new(blue_up))
    table.insert(self.colour_picker_button_list, Button:new(blue_down))
    
    local red_up16 = {   line=ln-2,  column=21,  ysize=1,    xsize=1,    
                                first_glyph = 47+gso_editor, 
                                click = self.red_up, click_arg=16,
                                parent = self, enabled = false}
    local red_down16 = {   line=ln+2,  column=21,  ysize=1,    xsize=1,    
                                first_glyph = 111+gso_editor, 
                                click = self.red_down,  click_arg=16,
                                parent = self, enabled = false}
    local green_up16 = {   line=ln-2,  column=25,  ysize=1,    xsize=1,    
                                first_glyph = 47+gso_editor, 
                                click = self.green_up,  click_arg=16,
                                parent = self, enabled = false}
    local green_down16 = {   line=ln+2,  column=25,  ysize=1,    xsize=1,    
                                first_glyph = 111+gso_editor, 
                                click = self.green_down,  click_arg=16,
                                parent = self, enabled = false}
    local blue_up16 = {   line=ln-2,  column=29,  ysize=1,    xsize=1,    
                                first_glyph = 47+gso_editor, 
                                click = self.blue_up,  click_arg=16,
                                parent = self, enabled = false}
    local blue_down16 = {   line=ln+2,  column=29,  ysize=1,    xsize=1,    
                                first_glyph = 111+gso_editor, 
                                click = self.blue_down,  click_arg=16,
                                parent = self, enabled = false}

    table.insert(self.colour_picker_button_list, Button:new(red_up16))
    table.insert(self.colour_picker_button_list, Button:new(red_down16))
    table.insert(self.colour_picker_button_list, Button:new(green_up16))
    table.insert(self.colour_picker_button_list, Button:new(green_down16))
    table.insert(self.colour_picker_button_list, Button:new(blue_up16))
    table.insert(self.colour_picker_button_list, Button:new(blue_down16))

end


-- *********************** START LOAD SCREEN ****************************************

-- load screen buttons


function GlyphEditor:create_load_screen_buttons()
    
    local cancel_button_line = 2
    
    for glyph_set,data in pairs(glyph_sets) do
    
        -- only allow editing of glyphs sets that come from file
        if data.load_from_file then
            local button =  {   line=data.button_line,  column=3,  ysize=1,    xsize=1,    
                                first_glyph = 22+gso_editor, 
                                text = data.name,
                                click = self.load_action,
                                click_arg = glyph_set,
                                parent = self, enabled = false
                            }
                    
            table.insert(self.glyph_set_load_button_list, Button:new(button))

            if data.button_line > cancel_button_line then cancel_button_line = data.button_line end

        end
        
    end
    
    -- shift cancel button after the last button
    cancel_button_line = cancel_button_line + 2
    
    local cancel_button =   {   line=cancel_button_line,  column=3,  ysize=1,    xsize=1,    
                                first_glyph = 22+gso_editor, 
                                text = 'Cancel',
                                click = self.load_screen_cancel_button_action,
                                parent = self, enabled = false}
    
    table.insert(self.glyph_set_load_button_list, Button:new(cancel_button))
    
end



function GlyphEditor:draw_glyph_set_load(gr)
    set_bg_colour('WHITE')
    set_fg_colour(editor_fg_colour)
    
    blank_screen()
    
    print_string_at(0, 4, "Load glyph Set")
    
end

function GlyphEditor:load_glyph_set(gs)
    
    self.glyph_set = gs
    
    if self.glyph_set_sdl_surface then 
        gulp_cpp.SDL_FreeSurface(self.glyph_set_sdl_surface) 
        self.glyph_set_sdl_surface = nil
    end
    
    self.grid_size = gs.glyph_size
    
    self.selected_glyph = DEFAULT_START_GLYPH
    self.glyph_set_scroll_line = 0
    
    local graphics = gulp.app:get_standard_graphics_context()
    if graphics then
        self.glyph_set_sdl_surface = gulp_cpp.load_glyph_file(self.glyph_set)
        if self.glyph_set_sdl_surface == nil then
            print ("Error - failed to load glyph set")
            self:finish()
        end
    end
    
    self.colour_format = gulp_cpp.get_colour_format(self.glyph_set_sdl_surface)
    self.glyph = gulp_cpp.load_glyph(self.glyph_set_sdl_surface, self.glyph_set, self.selected_glyph)
    
    
end

function GlyphEditor:load_action(click_arg)
    self:load_glyph_set(glyph_sets[click_arg])
    self:change_mode('editor')
    self:update_editor_texture(edit_buffer_texture_offset)
end

function GlyphEditor:load_screen_cancel_button_action()
    self:change_mode('editor')
    self:change_submode('draw')
end

function GlyphEditor:glyph_set_load_mousepressed(mx, my, handled)
    
    -- should catch leaking click here? until we have a window class holding all the buttons, or something?
    
    --return false
    return true
end

function GlyphEditor:glyph_set_load_mouse_down_moved(x, y, handled)
    -- do nothing
    return true
end

-- *********************** END LOAD SCREEN ****************************************


function GlyphEditor:set_button_states(button_list, state)
    for i, button in pairs(button_list) do
        button:set_enabled_state(state)
    end
end

function GlyphEditor:change_mode(new_mode)
    
    if self.mode == 'editor' or self.mode == 'colour_picker' then
        self:remove_grid_from_draw_list()
    end
        
    self.mode = new_mode
    
    if(self.mode == 'editor') then
    
        self:set_button_states(self.editor_button_list, 'enabled')
        self:set_button_states(self.glyph_set_load_button_list, 'disabled')
        self:set_button_states(self.colour_picker_button_list, 'disabled')
        if self.grid_size == 32 then
            self:add_32_grid_to_draw_list()
            edit_glyph =          0+gso_glyph_editor_buffers32
            copy_buffer_glyph =   1+gso_glyph_editor_buffers32
            resolution_button.mode = 'down'       -- 32x32 mode
            resolution_button:ClickFunction()       
            resolution_button:set_enabled_state('enabled')
        else
            self:add_16_grid_to_draw_list()
            edit_glyph =          0+gso_glyph_editor_buffers16
            copy_buffer_glyph =   1+gso_glyph_editor_buffers16
            resolution_button:set_enabled_state('disabled')
        end
    
    elseif(self.mode == 'glyph_set_load') then
        
        self:set_button_states(self.editor_button_list, 'disabled')
        self:set_button_states(self.glyph_set_load_button_list, 'enabled')
        self:set_button_states(self.colour_picker_button_list, 'disabled')
    
    elseif(self.mode == 'colour_picker') then
        
        self:set_button_states(self.editor_button_list, 'disabled')
        self:set_button_states(self.glyph_set_load_button_list, 'disabled')
        self:set_button_states(self.colour_picker_button_list, 'enabled')
        self:add_32_grid_to_draw_list()
        resolution_button.mode = 'down'       -- 32x32 mode
        resolution_button:ClickFunction()       
        self:update_grid_for_colour_picker()
    
    end

end

    
                        
function GlyphEditor:draw(gr)
    
    set_graphics_context(gr)
    
    if(self.mode == 'editor') then
        self:draw_editor()
    elseif(self.mode == 'glyph_set_load') then
        self:draw_glyph_set_load()
    elseif(self.mode == 'colour_picker') then
        self:draw_colour_picker(gr)
    end
    
    return false            -- we want to draw other stuff
end

function GlyphEditor:glyph_copy(source_glyph, dest_glyph_size, mode)
    local copy = {}
    local copy_pixels = {}
      
    local graphics = gulp.app:get_standard_graphics_context()
    local x, y, index
    local transparent = colour_to_pixel(gulp.colour.hot_pink_means_transparent, self.colour_format)
    
    local update_copy_texture = (mode ~= 'paste')
    
    -- if we did a copy 16 on this glyph, we better paste 16 as well
    if mode == 'paste' and source_glyph.mode == 'copy16' then
        mode = 'paste16'
    end
    
    
    -- first update the copy buffers (if necessary)
    -- always update both 16 bit and 32 bit copy buffers
    if update_copy_texture and graphics then
        if source_glyph.size == 16 then
            for k,pixel in pairs(source_glyph.pixels) do
                -- source is 16 bit, updating 16 bit copy buffer is trivial
                x = math.floor((k-1) / source_glyph.size)
                y = (k-1) % source_glyph.size
                graphics:update_texture_set_pixel(gso_glyph_editor_buffers16, copy_buffer_texture_offset, x, y, pixel)
                
                if mode == 'copy16' then
                    -- for 32 bit copy buffer in copy 16 mode, place the 16 bit character in the top left
                    -- and blank the rest of the buffer
                    -- facilitates shrinking glyphs
                    graphics:update_texture_set_pixel(gso_glyph_editor_buffers32, copy_buffer_texture_offset, x, y, pixel)
                    graphics:update_texture_set_pixel(gso_glyph_editor_buffers32, copy_buffer_texture_offset, x+16, y, transparent)
                    graphics:update_texture_set_pixel(gso_glyph_editor_buffers32, copy_buffer_texture_offset, x, y+16, transparent)
                    graphics:update_texture_set_pixel(gso_glyph_editor_buffers32, copy_buffer_texture_offset, x+16, y+16, transparent)
                    
                else
                    -- for 32 bit copy buffer, normally do pixel doubling
                    x = x * 2
                    y = y * 2
                    graphics:update_texture_set_pixel(gso_glyph_editor_buffers32, copy_buffer_texture_offset, x, y, pixel)
                    graphics:update_texture_set_pixel(gso_glyph_editor_buffers32, copy_buffer_texture_offset, x+1, y, pixel)
                    graphics:update_texture_set_pixel(gso_glyph_editor_buffers32, copy_buffer_texture_offset, x, y+1, pixel)
                    graphics:update_texture_set_pixel(gso_glyph_editor_buffers32, copy_buffer_texture_offset, x+1, y+1, pixel)
                end
            end
        elseif source_glyph.size == 32 then
            for k,pixel in pairs(source_glyph.pixels) do
                -- source is 32 bit, 32 bit copy buffer is trivial
                x = math.floor((k-1) / source_glyph.size)
                y = (k-1) % source_glyph.size
                graphics:update_texture_set_pixel(gso_glyph_editor_buffers32, copy_buffer_texture_offset, x, y, pixel)
                
                -- for 16 bit copy buffer, just take the top left corner
                if x<dest_glyph_size and y<dest_glyph_size then
                    graphics:update_texture_set_pixel(gso_glyph_editor_buffers16, copy_buffer_texture_offset, x, y, pixel)
                end
            end
        end
    end
    
    -- then update the copy_glyph
    if source_glyph.size == dest_glyph_size then
        -- simple case
        for k,pixel in pairs(source_glyph.pixels) do
            copy_pixels[k] = pixel
        end
    elseif mode == 'paste16' then
        -- special mode for pasting 16 bit glyphs into the top left hand corner of 32 bit glyph sets
        for x = 0, source_glyph.size-1 do
            for y = 1, source_glyph.size do
                copy_pixels[(x*dest_glyph_size)+y] = source_glyph.pixels[(x*source_glyph.size)+y]
                copy_pixels[(x*dest_glyph_size)+y+16] = transparent
                copy_pixels[((x+16)*dest_glyph_size)+y] = transparent
                copy_pixels[((x+16)*dest_glyph_size)+y+16] = transparent
            end
        end
    elseif source_glyph.size == 32 and dest_glyph_size == 16 then
        -- start the outer loop at zero and the inner loop at 1
        -- this gives us a 1 based array for the final result
        -- hurrah for 1 based arrays!
        for x = 0, dest_glyph_size-1 do
            for y = 1, dest_glyph_size do
                copy_pixels[(x*dest_glyph_size)+y] = source_glyph.pixels[(x*source_glyph.size)+y]
            end
        end
    elseif source_glyph.size == 16 and dest_glyph_size == 32 then
        for k,pixel in pairs(source_glyph.pixels) do
            -- x and y are coordinates in the 32 bit destination glyph
            -- need -1 on k because source glyph pixel array is 1 based
            -- multiply by to to convert from 16 to 32 bit
            x = math.floor((k-1) / source_glyph.size) * 2
            y = ((k-1) % source_glyph.size) * 2
            
            -- and write 4 pixels - the one calculated, and one to right, below, and below right
            -- always remembering to add an extra 1 each time to keep the array 1 based
            -- hurrah for 1 based arrays!
            copy_pixels[(x*dest_glyph_size)+(y)+1] = pixel
            copy_pixels[(x*dest_glyph_size)+(y+1)+1] = pixel
            copy_pixels[((x+1)*dest_glyph_size)+(y)+1] = pixel
            copy_pixels[((x+1)*dest_glyph_size)+(y+1)+1] = pixel
        end
    end
    
    copy.size = dest_glyph_size
    copy.pixels = copy_pixels
    copy.mode = mode
    
    return copy
end


function GlyphEditor:mousepressed(x, y, handled, right)
    
    if(not handled) then
        if(self.mode == 'editor') then
            handled = self:editor_mousepressed(x, y, handled, right)
        elseif(self.mode == 'glyph_set_load') then
            handled = self:glyph_set_load_mousepressed(x, y, handled)
        elseif(self.mode == 'colour_picker') then
            handled = self:colour_picker_mousepressed(x, y, handled)
        end
        self.x = x
        self.y = y
    end
    
    
    return handled
end


function GlyphEditor:mouse_down_moved(x, y, handled)
    
    if(not handled) then
        
        if(x ~= self.x or y ~= self.y) then
        
            if(self.mode == 'editor') then
                handled = self:editor_mouse_down_moved(x, y, handled)
            elseif(self.mode == 'glyph_set_load') then
                handled = self:glyph_set_load_mouse_down_moved(x, y, handled)
            end
        end
        
        self.x = x
        self.y = y
    end
    
    
    return handled
end


function GlyphEditor:launch_glyph_editor()
    print("Welcome to the glyph editor")
    print("We hope you enjoy your visit")
    
    self._other_mouse_state = MouseMove.suspend_non_game_regions()
    self._game_mouse_state = MouseMove.disable_game_controls()
    
    self.selected_glyph = DEFAULT_START_GLYPH
    self.glyph_set_scroll_line = 0
    self.grid_size = 16
    
    self:load_glyph_set(glyph_sets.spectrum)
    
    self.submode = 'draw'
    
    self.colour = gulp.colour.dark_pink_means_multicoloured
    self.alt_colour = gulp.colour.hot_pink_means_transparent
    self.animation_colour = gulp.colour.WHITE
    self.user_colours = {
        [1]=gulp.colour['BRIGHT_RED'],
        [2]=gulp.colour['BRIGHT_GREEN'],
        [3]=gulp.colour['BRIGHT_BLUE'],
        [4]=gulp.colour['BLACK'],
        [5]=gulp.colour['BRIGHT_RED'],
        [6]=gulp.colour['BRIGHT_GREEN'],
        [7]=gulp.colour['BRIGHT_BLUE'],
        [8]=gulp.colour['BLACK'],
        [9]=gulp.colour['BRIGHT_BLUE'],
        [10]=gulp.colour['BLACK']
    }
    
    self.editor_button_list = {}
    self.glyph_set_load_button_list = {}
    self.colour_picker_button_list = {}
    
    self.animation_speed = 3    -- fps
    self.animation_frames = 1
    self.animation_frame = 0
    self.time = 0
    

    self.old_draw_map = gulp._draw_map
    self.draw_status_lines = gulp._draw_status_lines

    gulp._draw_map = false
    gulp._draw_status_lines = false
    

    self:dump(self.glyph, self.colour_format)
    
    gulp._delegate_list:add_delegate(self)
    

    self.done = false
    
    --blank_display()
    
    self:create_editor_buttons()
    self:create_load_screen_buttons()
    self:create_animation_buttons()
    self:create_colour_picker_buttons()
    
    -- use change_mode() to make sure the correct buttons are displayed at first
    self:change_mode('editor')
    self:update_editor_texture(edit_buffer_texture_offset)
    
    
    
end



return GlyphEditor

