
-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
--[[

    Set of graphics helper functions
    
--]]

local x11_colours = require('graphics/x11_colours')

--local colours = { 'BLACK', 'BLUE', 'RED', 'MAGENTA', 'GREEN', 'CYAN', 'YELLOW', 'WHITE', 
--          'BRIGHT_BLACK', 'BRIGHT_BLUE', 'BRIGHT_RED', 'BRIGHT_MAGENTA', 'BRIGHT_GREEN', 
--          'BRIGHT_CYAN', 'BRIGHT_YELLOW', 'BRIGHT_WHITE' }

local full_colour_lookup_map = nil
local name_lookup_full_colour_map = nil

local function setup_full_colour_map()
    full_colour_lookup_map = {}
    name_lookup_full_colour_map = {}
    -- insert FG colours
    for name,colour in pairs(gulp.colour) do
        local sdl_colour = gulp_cpp.SDL_Color()
        sdl_colour.a = colour.a 
        sdl_colour.r = colour.r 
        sdl_colour.g = colour.g 
        sdl_colour.b = colour.b
        full_colour_lookup_map[name] = sdl_colour
        -- internal format ONLY
        local rgb_colour = bit32.lshift(colour.r, 16) + bit32.lshift(colour.g, 8) + colour.b
        
        if name_lookup_full_colour_map[rgb_colour] then
            name_lookup_full_colour_map[rgb_colour] = name_lookup_full_colour_map[rgb_colour] .. "/" .. name
        else
            name_lookup_full_colour_map[rgb_colour] = name
        end
    end
    -- insert X11 colours
    for name,sdl_colour in pairs(x11_colours) do
        full_colour_lookup_map[name] = sdl_colour
        -- internal format ONLY
        local rgb_colour = bit32.lshift(sdl_colour.r, 16) + bit32.lshift(sdl_colour.g, 8) + sdl_colour.b
        if name_lookup_full_colour_map[rgb_colour] then
            name_lookup_full_colour_map[rgb_colour] = name_lookup_full_colour_map[rgb_colour] .. "/" .. name
        else
            name_lookup_full_colour_map[rgb_colour] = name
        end
    end
end

function get_simple_colour_by_name(colour)
    local c = gulp.colour[colour]
    if c then
        return c.simple_colour or 0
    end
    return 0
end

local function get_colour_by_name(colour)
    return gulp.colour[colour]
end

function get_SDL_Color_by_name(colour)
    if not full_colour_lookup_map then
        setup_full_colour_map()
    end
    
    return full_colour_lookup_map[colour]
end


function get_name_of_colour(colour)
    local r, g, b --, a
    if type(colour) == 'table' or type(colour) == 'userdata' then
        -- assume a table which is contains keys { r b g }
        -- or a SDL colour
        --a = colour.a 
        r = colour.r 
        g = colour.g 
        b = colour.b

    elseif type(colour) == 'number' then
        -- simple colour
        
        print("NOT IMPLEMENTED YET")
        return nil
    else
        return nil
    end
    
    -- internal format ONLY
    local rgb_colour = bit32.lshift(r, 16) + bit32.lshift(g, 8) + b
    return name_lookup_full_colour_map[rgb_colour]
end
