-- Particle Effects
-- Rob Probin, Oct 2016

require("strict")
local class = require("middleclass")
require("graphics/graphics")
---------------------------------------------------------------------------

local MagicParticles= class("ParticleEffectsMagic")

function MagicParticles:initialize(manager, maze, line, column)
    self._manager = manager
    self._current_line = line
    self._current_column = column
    self._particles = {}
--    self._delete = false
    self._t = 0
    self._first = 1
    self._last = 0
    self._maze = maze
    self:_add_new_particle()
end

--[[
local mp_colours = { 
    colour_to_SDL_Color(gulp.colour.X11_PURPLE),
    colour_to_SDL_Color(gulp.colour.BLUE),
    colour_to_SDL_Color(gulp.colour.RED),
    colour_to_SDL_Color(gulp.colour.MAGENTA)
}--]]
---[[
local mp_colours = {
    gulp.colour.X11_PURPLE.simple_colour,
    gulp.colour.BLUE.simple_colour,
    gulp.colour.RED.simple_colour,
    gulp.colour.MAGENTA.simple_colour
}--]]

local image_glyph = get_glyph('caves2', 0x20)
local image_base_character_code = get_glyph_set_base_code('caves2')

function MagicParticles:_add_new_particle()
    local speed_in_cells_per_sec = 1
    local direction = math.random(0,256)/256 * (2 * math.pi)
    local dX_cps = speed_in_cells_per_sec * math.sin(direction)
    local dY_cps = speed_in_cells_per_sec * math.cos(direction)
    
    local new_p = {
        age = 0,
        line = self._current_line, -- +gulp.TOP_UI_HEIGHT,
        column = self._current_column,
        colour = mp_colours[math.random(#mp_colours)],
        dx_cps = dX_cps,
        dy_cps = dY_cps
    }
    
    local layer = 500
    local display_element = new_mob_display_element(self._maze, image_glyph, new_p.line, new_p.column, layer, new_p.colour)
    display_element:set_bg_transparent()
    new_p.display_element = display_element
    
    self._last = self._last + 1
    self._particles[self._last] = new_p
    -- print("ADD", self._first, self._last, gulp.LuaStateCodeMode)
    -- for i,_ in pairs(self._particles) do print(i) end
    
end

-- local max_age = 5
local max_age = 20

function MagicParticles:update(dt)
    -- assumes oldest is always first to be removed
    for i = self._first, self._last do
        local dot = self._particles[i]
        
        dot.age = dot.age + dt
        if dot.age >= max_age then
            -- print("DELETE", i)
            self._particles[i] = nil
            dot.display_element:hide()
            self._first = self._first + 1
        else
            dot.line = dot.line + dt * dot.dy_cps
            dot.column = dot.column + dt * dot.dx_cps
            dot.display_element:set_line(dot.line)
            dot.display_element:set_column(dot.column)
        end
    end
    
    -- create new ones occasionally
    self._t = self._t + dt
    if self._t > 0.87 then
        self._t = self._t - 0.87
        self:_add_new_particle()
    end
end

function MagicParticles:draw(gr)
    
    --for i in self._first, self._last do
    --    local dot = self.particles[i]
    
    --local white_colour = gulp_cpp.SDL_Color()
    --white_colour.a = 255
    --white_colour.r = 255
    --white_colour.g = 255
    --white_colour.b = 255

    --gr:DrawPoint(white_colour, 1.5, 1.5)
    --[[
    gr:set_bg_transparent()

    -- order doesn't matter
    for _, dot in pairs(self._particles) do
        if dot and dot.age < max_age then
            --gr:DrawPoint(white_colour, dot.column, dot.line)
            --gr:FillRect(dot.colour, dot.column, dot.line, dot.column+1, dot.line+1)
            local alpha = math.max(0, (1-(dot.age / max_age)) * 255)
            
            local colour = gulp_cpp.SDL_Color()
            colour.a = alpha
            colour.r = dot.colour.r 
            colour.g = dot.colour.g 
            colour.b = dot.colour.b
            gr:DrawLine(colour, dot.column, dot.line, dot.column+dot.dx_cps/8, dot.line+dot.dy_cps/8)
            --gr:DrawPoint(colour, dot.column, dot.line)
            
            gr:SetTextureAlphaMod(image_base_character_code, alpha);
            gr:set_fg_colour(dot.colour)
            gr:SetTextureAlphaMod(image_base_character_code, 255);
            gr:go_to(dot.line, dot.column)
            gulp_cpp.print_glyph(gr, image_glyph)
        
        end
    end
    
    gr:set_bg_opaque()
    --]]
end

function MagicParticles:move_source(line, column)
    self._current_line = line
    self._current_column = column
end

function MagicParticles:delete()
    for i = self._first, self._last do
        local dot = self._particles[i]
        dot.display_element:hide()
        self._particles[i] = nil
    end
    self._first = 1
    self._last = 0
    
    --self._delete = true
    --self._manager:delete_ref(self)
end

---------------------------------------------------------------------------

-- local ParticleEffectsManager = class("ParticleEffectsManager")

--
local ParticleEffectsManager = { _list = {} }
--function ParticleEffectsManager:initialize()
--    self._list = {}
--end

function ParticleEffectsManager.update(dt)
    local self = ParticleEffectsManager
    for _, obj in ipairs(self._list) do
        obj:update(dt)
    end
end

function ParticleEffectsManager.draw(gr)
    local self = ParticleEffectsManager
    for _, obj in ipairs(self._list) do
        obj:draw(gr)
    end
end

--function ParticleEffectsManager:add_magic_particle(line, column)
--    local direction = math.random (0, 360)
--end

--function ParticleEffectsManager:add_start_magic(line, column)
--    for _ in range(1, 10) do
--        self:add_magic_particle)(line, column)
--    end
--end

--function ParticleEffectsManager:add_more_magic(line, colummn)
--    self:add_magic_particle(line, column)
--end

function ParticleEffectsManager.new_magic(maze, line, column)
    local self = ParticleEffectsManager
    local new_magic = MagicParticles(self, maze, line, column )
    table.insert(self._list, new_magic)
    return new_magic
end

--function ParticleEffectsManager.delete(obj)  
--end

function ParticleEffectsManager.delete_all()
    local self = ParticleEffectsManager
    for _, obj in ipairs(self._list) do
        obj:delete()
    end
    ParticleEffectsManager = { _list = {} }
end

--[[
function ParticleEffectsManager:add_smoke_particle(line, column)
end

function ParticleEffectsManager:maybe_add_smoke_particle(dt, line, column)
    --after some time self:base_time + dt
    --add_smoke_particle(line, column)
end

function ParticleEffectsManager:new_smoke()
end
--]]

--[[
local ParticleEffects = {
    manager = ParticleEffectsManager:new()
    new_smoke = ParticleEffects:new_smoke()
    new_magic = ParticleEffects:new_magic()
}
--]]

--[[
local ParticleEffectsManagerInstance = nil

--
-- get partial effects manager instance singleton
local function get_partial_effects()
    if not ParticleEffectsManagerInstance then
        ParticleEffectsManagerInstance = ParticleEffectsManager:new()
    end
    return ParticleEffectsManagerInstance
end
--]]

return ParticleEffectsManager
