-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

require ("strict")
class = require("middleclass")
require("../graphics/graphics")

local Button = class("Button")

function Button:initialize(button_data)
    
    self.button_display_elements = {}
    
    for k,v in pairs(button_data) do
        self[k]=v
    end

    self:create_display_elements()

    -- enable button by default
    if self.enabled == nil or self.enabled == true then
        self:enable()
    end
    

end

function Button:destroy()
    -- disable the button
    self:disable()
    
    -- and drop all the display elements so they can be garbage collected
    for line, columns in pairs(self.button_display_elements) do
        for column, element in pairs(self.button_display_elements[line]) do
            self.button_display_elements[line][column] = nil
        end
    end

end


function Button:set_enabled_state(state)
    if state == 'kill' then
        self:destroy()
    elseif state == 'enabled' then
        self:enable()
    else
        self:disable()
    end
end

function Button:enable()
    gulp._delegate_list:add_delegate(self)
    self.enabled = true
    self:show_display_elements()
end

function Button:disable()
    self:hide_display_elements()
    self.enabled = false
    gulp._delegate_list:drop_delegate(self)
end


    
function Button:_internal_create_display_elements(glyph)
    local line, column
    --local glyph = self.first_glyph
    
    for line=self.line, self.line + self.ysize-1 do
        self.button_display_elements[line] = {}
        for column=self.column, self.column + self.xsize-1 do
            self.button_display_elements[line][column] = new_hidden_absolute_display_element(glyph, line, column, 1026, 
                                get_simple_colour_by_name('BLACK'), get_SDL_Color_by_name('WHITE'), false)
            --print_glyph_at(line, column, glyph)
            glyph = glyph + 1
        end
        glyph = glyph + 32 - self.xsize
    end
    
    -- make display list elements for each character in the string
    if(self.text ~= nil) then
        --print_string_at(self.line, self.column+self.xsize, self.text)
        local char_location
        line = self.line
        column = self.column+self.xsize
        --print ("line=" .. line .. " column=" .. column)
        for char_location=1, string.len(self.text) do
            local char_code = self.text:byte(char_location)
            --print ("char_location=" .. char_location .. " char_code=" .. char_code)
            self.button_display_elements[line][column] = new_hidden_absolute_display_element(char_code, line, column, 1000, 
                                get_simple_colour_by_name('BLACK'), get_SDL_Color_by_name('WHITE'), false)
            column = column+1
        end
    end
end 

function Button:adjust_bg_colour(bg_colour)
    for line=self.line, self.line + self.ysize-1 do
        for column=self.column, self.column + self.xsize-1 do
            self.button_display_elements[line][column]:set_bg_colour(bg_colour)
        end
    end
end

function Button:create_display_elements()
    self:_internal_create_display_elements(self.first_glyph)
end

function Button:show_display_elements()
    for line, columns in pairs(self.button_display_elements) do
        for column, element in pairs(self.button_display_elements[line]) do
            element:show()
        end
    end
end

function Button:hide_display_elements()
    for line, columns in pairs(self.button_display_elements) do
        for column, element in pairs(self.button_display_elements[line]) do
            element:hide()
        end
    end
end

function Button:_internal_draw(glyph)
end
--[[
function Button:_internal_draw(glyph)
    local line, column
    
    for line=self.line, self.line + self.ysize-1 do
        for column=self.column, self.column + self.xsize-1 do
            print_glyph_at(line, column, glyph)
            glyph = glyph + 1
        end
        glyph = glyph + 32 - self.xsize
    end
    
    if(self.text ~= nil) then
        print_string_at(self.line, self.column+self.xsize, self.text)
    end
end 

function Button:draw()
    --local glyph = self.first_glyph
    self:_internal_draw(self.first_glyph)
end
--]]
function Button:update(dt)
    -- should be moved to base class
    return false    -- don't consume the update call - we want to update other stuff too
end

function Button:mousepressed(x, y, handled)
if not handled then
        local button_x_size = self.xsize
        
        if(self.text ~= nil) then
            button_x_size = button_x_size + string.len(self.text)
        end
            
        if(y >= self.line and y < self.line + self.ysize) then
            if(x >= self.column and x < self.column + button_x_size) then
                
                return true     -- button was clicked - don't check other objects
            end
        end
    end
    
    return handled    -- button was not clicked - don't consume click
end

function Button:mousereleased(x, y, handled)
    
if not handled then
        local button_x_size = self.xsize
        
        if(self.text ~= nil) then
            button_x_size = button_x_size + string.len(self.text)
        end
            
        if(y >= self.line and y < self.line + self.ysize) then
            if(x >= self.column and x < self.column + button_x_size) then
                
                -- do something with the click!
                self.click(self.parent, self.click_arg)
                
                return true     -- button was clicked - don't check other objects
            end
        end
    end
    
    return handled    -- button was not clicked - don't consume click
end

return Button
