-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

require ("strict")
class = require("middleclass")
require("../graphics/graphics")

local Button = require("../gui/button")

local ToggleButton = Button:subclass('ToggleButton')


function ToggleButton:initialize(button_data)
    
    Button.initialize(self, button_data)
    
    -- initialize some data if it wasn't passed as arguments
    if self.mode == nil then
        self.mode = 'up'
    end
    
end
    
function ToggleButton:ClickFunction()
    -- switch our mode
    if self.mode == 'up' then
        self:activate()
        --self.mode = 'down'
    else
        self:release()
        --self.mode = 'up'
    end

    -- do something with the click!
    if self.click then 
        self.click(self.parent, self.mode, self.click_arg)
    end
    
end

function ToggleButton:mousereleased(x, y, handled)
    
    if not handled then
        
        local button_x_size = self.xsize
        
        if(self.text ~= nil) then
            button_x_size = button_x_size + string.len(self.text)
        end
            
        if(y >= self.line and y < self.line + self.ysize) then
            if(x >= self.column and x < self.column + button_x_size) then
                
                self:ClickFunction()
                return true     -- button was clicked - don't check other objects
            end
        end
    end
        
    return handled    -- button was not clicked - don't consume click
    
end

function ToggleButton:mousepressed(x, y, handled)
    
    if not handled then
        
        local button_x_size = self.xsize
        
        if(self.text ~= nil) then
            button_x_size = button_x_size + string.len(self.text)
        end
            
        if(y >= self.line and y < self.line + self.ysize) then
            if(x >= self.column and x < self.column + button_x_size) then
                
                return true     -- button was clicked - don't check other objects
            end
        end
    end
        
    return handled    -- button was not clicked - don't consume click
    
end

function ToggleButton:create_display_elements()
    if self.mode=='down' then
        Button._internal_create_display_elements(self, self.first_glyph_down)
    else
        Button._internal_create_display_elements(self, self.first_glyph)
    end
end

--[[
function ToggleButton:draw()
    if self.mode=='up' then
        Button._internal_draw(self, self.first_glyph)
    else
        Button._internal_draw(self, self.first_glyph_down)
    end
end
--]]
function ToggleButton:activate()
    self.mode = 'down'
    self:adjust_display_elements(self.first_glyph_down)
end

function ToggleButton:release()
    self.mode = 'up'
    self:adjust_display_elements(self.first_glyph)
end

function ToggleButton:adjust_display_elements(glyph)

    for line=self.line, self.line + self.ysize-1 do
        for column=self.column, self.column + self.xsize-1 do
            self.button_display_elements[line][column]:set_glyph(glyph)
            glyph = glyph + 1
        end
        glyph = glyph + 32 - self.xsize
    end

end


return ToggleButton