--[[/*
  Simple DirectMedia Layer
  Copyright (C) 1997-2013 Sam Lantinga <slouken@libsdl.org>

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
*/]]
-- Modified by Rob Probin October 2014 for use from Lua for Forlorn Fox

--[[/**
 *  \file SDL_keycode.h
 *
 *  Defines constants which identify keyboard keys and modifiers.
 */]]

--#ifndef _SDL_keycode_h
--#define _SDL_keycode_h

--#include "SDL_stdinc.h"
--#include "SDL_scancode.h"
require("library/SDL_scancode")
--[[[/**
 *  \brief The SDL virtual key representation.
 *
 *  Values of this type are used to represent keyboard keys using the current
 *  layout of the keyboard.  These values include Unicode values representing
 *  the unmodified character that would be generated by pressing the key, or
 *  an SDLK.* constant for those keys that do not generate characters.
 */]]
--typedef Sint32 SDL_Keycode;

SDLK_SCANCODE_MASK = (2 ^ 30)
function SDL_SCANCODE_TO_KEYCODE(X)  return (X + SDLK_SCANCODE_MASK) end

SDLK = {}
--enum
--{
    SDLK.UNKNOWN = 0

    SDLK.RETURN = string.byte('\r')
    SDLK.ESCAPE = string.byte('\x1b')
    SDLK.BACKSPACE = string.byte('\b')
    SDLK.TAB = string.byte('\t')
    SDLK.SPACE = string.byte(' ')
    SDLK.EXCLAIM = string.byte('!')
    SDLK.QUOTEDBL = string.byte('"')
    SDLK.HASH = string.byte('#')
    SDLK.PERCENT = string.byte('%')
    SDLK.DOLLAR = string.byte('$')
    SDLK.AMPERSAND = string.byte('&')
    SDLK.QUOTE = string.byte('\'')
    SDLK.LEFTPAREN = string.byte('(')
    SDLK.RIGHTPAREN = string.byte(')')
    SDLK.ASTERISK = string.byte('*')
    SDLK.PLUS = string.byte('+')
    SDLK.COMMA = string.byte(',')
    SDLK.MINUS = string.byte('-')
    SDLK.PERIOD = string.byte('.')
    SDLK.SLASH = string.byte('/')
    SDLK._0 = string.byte('0')
    SDLK._1 = string.byte('1')
    SDLK._2 = string.byte('2')
    SDLK._3 = string.byte('3')
    SDLK._4 = string.byte('4')
    SDLK._5 = string.byte('5')
    SDLK._6 = string.byte('6')
    SDLK._7 = string.byte('7')
    SDLK._8 = string.byte('8')
    SDLK._9 = string.byte('9')
    SDLK.COLON = string.byte(':')
    SDLK.SEMICOLON = string.byte(';')
    SDLK.LESS = string.byte('<')
    SDLK.EQUALS = string.byte('=')
    SDLK.GREATER = string.byte('>')
    SDLK.QUESTION = string.byte('?')
    SDLK.AT = string.byte('@')
    --[[/*
       Skip uppercase letters
     */]]
    SDLK.LEFTBRACKET = string.byte('[')
    SDLK.BACKSLASH = string.byte('\\')
    SDLK.RIGHTBRACKET = string.byte(']')
    SDLK.CARET = string.byte('^')
    SDLK.UNDERSCORE = string.byte('_')
    SDLK.BACKQUOTE = string.byte('`')
    SDLK.a = string.byte('a')
    SDLK.b = string.byte('b')
    SDLK.c = string.byte('c')
    SDLK.d = string.byte('d')
    SDLK.e = string.byte('e')
    SDLK.f = string.byte('f')
    SDLK.g = string.byte('g')
    SDLK.h = string.byte('h')
    SDLK.i = string.byte('i')
    SDLK.j = string.byte('j')
    SDLK.k = string.byte('k')
    SDLK.l = string.byte('l')
    SDLK.m = string.byte('m')
    SDLK.n = string.byte('n')
    SDLK.o = string.byte('o')
    SDLK.p = string.byte('p')
    SDLK.q = string.byte('q')
    SDLK.r = string.byte('r')
    SDLK.s = string.byte('s')
    SDLK.t = string.byte('t')
    SDLK.u = string.byte('u')
    SDLK.v = string.byte('v')
    SDLK.w = string.byte('w')
    SDLK.x = string.byte('x')
    SDLK.y = string.byte('y')
    SDLK.z = string.byte('z')

    SDLK.CAPSLOCK = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.CAPSLOCK)

    SDLK.F1 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F1)
    SDLK.F2 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F2)
    SDLK.F3 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F3)
    SDLK.F4 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F4)
    SDLK.F5 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F5)
    SDLK.F6 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F6)
    SDLK.F7 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F7)
    SDLK.F8 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F8)
    SDLK.F9 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F9)
    SDLK.F10 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F10)
    SDLK.F11 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F11)
    SDLK.F12 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F12)

    SDLK.PRINTSCREEN = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.PRINTSCREEN)
    SDLK.SCROLLLOCK = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.SCROLLLOCK)
    SDLK.PAUSE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.PAUSE)
    SDLK.INSERT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.INSERT)
    SDLK.HOME = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.HOME)
    SDLK.PAGEUP = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.PAGEUP)
    SDLK.DELETE = 127
    SDLK.END = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.END)
    SDLK.PAGEDOWN = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.PAGEDOWN)
    SDLK.RIGHT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.RIGHT)
    SDLK.LEFT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.LEFT)
    SDLK.DOWN = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.DOWN)
    SDLK.UP = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.UP)

    SDLK.NUMLOCKCLEAR = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.NUMLOCKCLEAR)
    SDLK.KP_DIVIDE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_DIVIDE)
    SDLK.KP_MULTIPLY = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_MULTIPLY)
    SDLK.KP_MINUS = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_MINUS)
    SDLK.KP_PLUS = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_PLUS)
    SDLK.KP_ENTER = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_ENTER)
    SDLK.KP_1 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_1)
    SDLK.KP_2 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_2)
    SDLK.KP_3 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_3)
    SDLK.KP_4 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_4)
    SDLK.KP_5 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_5)
    SDLK.KP_6 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_6)
    SDLK.KP_7 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_7)
    SDLK.KP_8 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_8)
    SDLK.KP_9 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_9)
    SDLK.KP_0 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_0)
    SDLK.KP_PERIOD = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_PERIOD)

    SDLK.APPLICATION = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.APPLICATION)
    SDLK.POWER = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.POWER)
    SDLK.KP_EQUALS = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_EQUALS)
    SDLK.F13 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F13)
    SDLK.F14 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F14)
    SDLK.F15 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F15)
    SDLK.F16 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F16)
    SDLK.F17 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F17)
    SDLK.F18 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F18)
    SDLK.F19 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F19)
    SDLK.F20 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F20)
    SDLK.F21 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F21)
    SDLK.F22 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F22)
    SDLK.F23 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F23)
    SDLK.F24 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.F24)
    SDLK.EXECUTE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.EXECUTE)
    SDLK.HELP = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.HELP)
    SDLK.MENU = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.MENU)
    SDLK.SELECT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.SELECT)
    SDLK.STOP = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.STOP)
    SDLK.AGAIN = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.AGAIN)
    SDLK.UNDO = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.UNDO)
    SDLK.CUT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.CUT)
    SDLK.COPY = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.COPY)
    SDLK.PASTE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.PASTE)
    SDLK.FIND = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.FIND)
    SDLK.MUTE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.MUTE)
    SDLK.VOLUMEUP = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.VOLUMEUP)
    SDLK.VOLUMEDOWN = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.VOLUMEDOWN)
    SDLK.KP_COMMA = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_COMMA)
    SDLK.KP_EQUALSAS400 =
        SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_EQUALSAS400)

    SDLK.ALTERASE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.ALTERASE)
    SDLK.SYSREQ = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.SYSREQ)
    SDLK.CANCEL = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.CANCEL)
    SDLK.CLEAR = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.CLEAR)
    SDLK.PRIOR = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.PRIOR)
    SDLK.RETURN2 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.RETURN2)
    SDLK.SEPARATOR = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.SEPARATOR)
    SDLK.OUT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.OUT)
    SDLK.OPER = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.OPER)
    SDLK.CLEARAGAIN = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.CLEARAGAIN)
    SDLK.CRSEL = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.CRSEL)
    SDLK.EXSEL = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.EXSEL)

    SDLK.KP_00 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_00)
    SDLK.KP_000 = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_000)
    SDLK.THOUSANDSSEPARATOR =
        SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.THOUSANDSSEPARATOR)
    SDLK.DECIMALSEPARATOR =
        SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.DECIMALSEPARATOR)
    SDLK.CURRENCYUNIT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.CURRENCYUNIT)
    SDLK.CURRENCYSUBUNIT =
        SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.CURRENCYSUBUNIT)
    SDLK.KP_LEFTPAREN = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_LEFTPAREN)
    SDLK.KP_RIGHTPAREN = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_RIGHTPAREN)
    SDLK.KP_LEFTBRACE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_LEFTBRACE)
    SDLK.KP_RIGHTBRACE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_RIGHTBRACE)
    SDLK.KP_TAB = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_TAB)
    SDLK.KP_BACKSPACE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_BACKSPACE)
    SDLK.KP_A = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_A)
    SDLK.KP_B = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_B)
    SDLK.KP_C = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_C)
    SDLK.KP_D = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_D)
    SDLK.KP_E = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_E)
    SDLK.KP_F = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_F)
    SDLK.KP_XOR = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_XOR)
    SDLK.KP_POWER = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_POWER)
    SDLK.KP_PERCENT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_PERCENT)
    SDLK.KP_LESS = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_LESS)
    SDLK.KP_GREATER = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_GREATER)
    SDLK.KP_AMPERSAND = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_AMPERSAND)
    SDLK.KP_DBLAMPERSAND =
        SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_DBLAMPERSAND)
    SDLK.KP_VERTICALBAR =
        SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_VERTICALBAR)
    SDLK.KP_DBLVERTICALBAR =
        SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_DBLVERTICALBAR)
    SDLK.KP_COLON = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_COLON)
    SDLK.KP_HASH = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_HASH)
    SDLK.KP_SPACE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_SPACE)
    SDLK.KP_AT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_AT)
    SDLK.KP_EXCLAM = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_EXCLAM)
    SDLK.KP_MEMSTORE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_MEMSTORE)
    SDLK.KP_MEMRECALL = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_MEMRECALL)
    SDLK.KP_MEMCLEAR = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_MEMCLEAR)
    SDLK.KP_MEMADD = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_MEMADD)
    SDLK.KP_MEMSUBTRACT =
        SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_MEMSUBTRACT)
    SDLK.KP_MEMMULTIPLY =
        SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_MEMMULTIPLY)
    SDLK.KP_MEMDIVIDE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_MEMDIVIDE)
    SDLK.KP_PLUSMINUS = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_PLUSMINUS)
    SDLK.KP_CLEAR = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_CLEAR)
    SDLK.KP_CLEARENTRY = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_CLEARENTRY)
    SDLK.KP_BINARY = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_BINARY)
    SDLK.KP_OCTAL = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_OCTAL)
    SDLK.KP_DECIMAL = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_DECIMAL)
    SDLK.KP_HEXADECIMAL =
        SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KP_HEXADECIMAL)

    SDLK.LCTRL = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.LCTRL)
    SDLK.LSHIFT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.LSHIFT)
    SDLK.LALT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.LALT)
    SDLK.LGUI = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.LGUI)
    SDLK.RCTRL = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.RCTRL)
    SDLK.RSHIFT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.RSHIFT)
    SDLK.RALT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.RALT)
    SDLK.RGUI = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.RGUI)

    SDLK.MODE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.MODE)

    SDLK.AUDIONEXT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.AUDIONEXT)
    SDLK.AUDIOPREV = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.AUDIOPREV)
    SDLK.AUDIOSTOP = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.AUDIOSTOP)
    SDLK.AUDIOPLAY = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.AUDIOPLAY)
    SDLK.AUDIOMUTE = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.AUDIOMUTE)
    SDLK.MEDIASELECT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.MEDIASELECT)
    SDLK.WWW = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.WWW)
    SDLK.MAIL = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.MAIL)
    SDLK.CALCULATOR = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.CALCULATOR)
    SDLK.COMPUTER = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.COMPUTER)
    SDLK.AC_SEARCH = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.AC_SEARCH)
    SDLK.AC_HOME = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.AC_HOME)
    SDLK.AC_BACK = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.AC_BACK)
    SDLK.AC_FORWARD = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.AC_FORWARD)
    SDLK.AC_STOP = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.AC_STOP)
    SDLK.AC_REFRESH = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.AC_REFRESH)
    SDLK.AC_BOOKMARKS = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.AC_BOOKMARKS)

    SDLK.BRIGHTNESSDOWN =
        SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.BRIGHTNESSDOWN)
    SDLK.BRIGHTNESSUP = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.BRIGHTNESSUP)
    SDLK.DISPLAYSWITCH = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.DISPLAYSWITCH)
    SDLK.KBDILLUMTOGGLE =
        SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KBDILLUMTOGGLE)
    SDLK.KBDILLUMDOWN = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KBDILLUMDOWN)
    SDLK.KBDILLUMUP = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.KBDILLUMUP)
    SDLK.EJECT = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.EJECT)
    SDLK.SLEEP = SDL_SCANCODE_TO_KEYCODE(SDL_SCANCODE.SLEEP)
--};

--[[/**
 * \brief Enumeration of valid key mods (possibly OR'd together).
 */]]
--typedef enum
--{
KMOD = {}
    KMOD.NONE = 0x0000
    KMOD.LSHIFT = 0x0001
    KMOD.RSHIFT = 0x0002
    KMOD.LCTRL = 0x0040
    KMOD.RCTRL = 0x0080
    KMOD.LALT = 0x0100
    KMOD.RALT = 0x0200
    KMOD.LGUI = 0x0400
    KMOD.RGUI = 0x0800
    KMOD.NUM = 0x1000
    KMOD.CAPS = 0x2000
    KMOD.MODE = 0x4000
    KMOD.RESERVED = 0x8000
--} SDL_Keymod;

KMOD.CTRL  =  (KMOD.LCTRL + KMOD.RCTRL)
KMOD.SHIFT = (KMOD.LSHIFT + KMOD.RSHIFT)
KMOD.ALT   = (KMOD.LALT + KMOD.RALT)
KMOD.GUI   = (KMOD.LGUI + KMOD.RGUI)

--#endif /* _SDL_keycode_h */

--/* vi: set ts=4 sw=4 expandtab: */
