--[[/*
  Simple DirectMedia Layer
  Copyright (C) 1997-2013 Sam Lantinga <slouken@libsdl.org>

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
*/]]
-- Modified by Rob Probin October 2014 for use from Lua for Forlorn Fox

--[[/**
 *  \file SDL_scancode.h
 *
 *  Defines keyboard scancodes.
 */]]

--#ifndef _SDL_scancode_h
--#define _SDL_scancode_h

--#include "SDL_stdinc.h"

--[[/**
 *  \brief The SDL keyboard scancode representation.
 *
 *  Values of this type are used to represent keyboard keys, among other places
 *  in the \link SDL_Keysym::scancode key.keysym.scancode \endlink field of the
 *  SDL_Event structure.
 *
 *  The values in this enumeration are based on the USB usage page standard:
 *  http://www.usb.org/developers/devclass_docs/Hut1_12v2.pdf
 */]]
 
 SDL_SCANCODE = {}
 
--typedef enum
--{
    SDL_SCANCODE.UNKNOWN = 0

    --[[/**
     *  \name Usage page 0x07
     *
     *  These values are from usage page 0x07 (USB keyboard page).
     */]]
    --[[/* @{ */]]

    SDL_SCANCODE.A = 4
    SDL_SCANCODE.B = 5
    SDL_SCANCODE.C = 6
    SDL_SCANCODE.D = 7
    SDL_SCANCODE.E = 8
    SDL_SCANCODE.F = 9
    SDL_SCANCODE.G = 10
    SDL_SCANCODE.H = 11
    SDL_SCANCODE.I = 12
    SDL_SCANCODE.J = 13
    SDL_SCANCODE.K = 14
    SDL_SCANCODE.L = 15
    SDL_SCANCODE.M = 16
    SDL_SCANCODE.N = 17
    SDL_SCANCODE.O = 18
    SDL_SCANCODE.P = 19
    SDL_SCANCODE.Q = 20
    SDL_SCANCODE.R = 21
    SDL_SCANCODE.S = 22
    SDL_SCANCODE.T = 23
    SDL_SCANCODE.U = 24
    SDL_SCANCODE.V = 25
    SDL_SCANCODE.W = 26
    SDL_SCANCODE.X = 27
    SDL_SCANCODE.Y = 28
    SDL_SCANCODE.Z = 29

    SDL_SCANCODE._1 = 30
    SDL_SCANCODE._2 = 31
    SDL_SCANCODE._3 = 32
    SDL_SCANCODE._4 = 33
    SDL_SCANCODE._5 = 34
    SDL_SCANCODE._6 = 35
    SDL_SCANCODE._7 = 36
    SDL_SCANCODE._8 = 37
    SDL_SCANCODE._9 = 38
    SDL_SCANCODE._0 = 39

    SDL_SCANCODE.RETURN = 40
    SDL_SCANCODE.ESCAPE = 41
    SDL_SCANCODE.BACKSPACE = 42
    SDL_SCANCODE.TAB = 43
    SDL_SCANCODE.SPACE = 44

    SDL_SCANCODE.MINUS = 45
    SDL_SCANCODE.EQUALS = 46
    SDL_SCANCODE.LEFTBRACKET = 47
    SDL_SCANCODE.RIGHTBRACKET = 48
    SDL_SCANCODE.BACKSLASH = 49 --[[/**< Located at the lower left of the return
                                  *   key on ISO keyboards and at the right end
                                  *   of the QWERTY row on ANSI keyboards.
                                  *   Produces REVERSE SOLIDUS (backslash) and
                                  *   VERTICAL LINE in a US layout, REVERSE
                                  *   SOLIDUS and VERTICAL LINE in a UK Mac
                                  *   layout, NUMBER SIGN and TILDE in a UK
                                  *   Windows layout, DOLLAR SIGN and POUND SIGN
                                  *   in a Swiss German layout, NUMBER SIGN and
                                  *   APOSTROPHE in a German layout, GRAVE
                                  *   ACCENT and POUND SIGN in a French Mac
                                  *   layout, and ASTERISK and MICRO SIGN in a
                                  *   French Windows layout.
                                  */]]
    SDL_SCANCODE.NONUSHASH = 50 --[[/**< ISO USB keyboards actually use this code
                                  *   instead of 49 for the same key, but all
                                  *   OSes I've seen treat the two codes
                                  *   identically. So, as an implementor, unless
                                  *   your keyboard generates both of those
                                  *   codes and your OS treats them differently,
                                  *   you should generate SDL_SCANCODE.BACKSLASH
                                  *   instead of this code. As a user, you
                                  *   should not rely on this code because SDL
                                  *   will never generate it with most (all?)
                                  *   keyboards.
                                  */]]
    SDL_SCANCODE.SEMICOLON = 51
    SDL_SCANCODE.APOSTROPHE = 52
    SDL_SCANCODE.GRAVE = 53 --[[/**< Located in the top left corner (on both ANSI
                              *   and ISO keyboards). Produces GRAVE ACCENT and
                              *   TILDE in a US Windows layout and in US and UK
                              *   Mac layouts on ANSI keyboards, GRAVE ACCENT
                              *   and NOT SIGN in a UK Windows layout, SECTION
                              *   SIGN and PLUS-MINUS SIGN in US and UK Mac
                              *   layouts on ISO keyboards, SECTION SIGN and
                              *   DEGREE SIGN in a Swiss German layout (Mac:
                              *   only on ISO keyboards), CIRCUMFLEX ACCENT and
                              *   DEGREE SIGN in a German layout (Mac: only on
                              *   ISO keyboards), SUPERSCRIPT TWO and TILDE in a
                              *   French Windows layout, COMMERCIAL AT and
                              *   NUMBER SIGN in a French Mac layout on ISO
                              *   keyboards, and LESS-THAN SIGN and GREATER-THAN
                              *   SIGN in a Swiss German, German, or French Mac
                              *   layout on ANSI keyboards.
                              */]]
    SDL_SCANCODE.COMMA = 54
    SDL_SCANCODE.PERIOD = 55
    SDL_SCANCODE.SLASH = 56

    SDL_SCANCODE.CAPSLOCK = 57

    SDL_SCANCODE.F1 = 58
    SDL_SCANCODE.F2 = 59
    SDL_SCANCODE.F3 = 60
    SDL_SCANCODE.F4 = 61
    SDL_SCANCODE.F5 = 62
    SDL_SCANCODE.F6 = 63
    SDL_SCANCODE.F7 = 64
    SDL_SCANCODE.F8 = 65
    SDL_SCANCODE.F9 = 66
    SDL_SCANCODE.F10 = 67
    SDL_SCANCODE.F11 = 68
    SDL_SCANCODE.F12 = 69

    SDL_SCANCODE.PRINTSCREEN = 70
    SDL_SCANCODE.SCROLLLOCK = 71
    SDL_SCANCODE.PAUSE = 72
    SDL_SCANCODE.INSERT = 73 --[[/**< insert on PC, help on some Mac keyboards (but
                                   does send code 73, not 117) */]]
    SDL_SCANCODE.HOME = 74
    SDL_SCANCODE.PAGEUP = 75
    SDL_SCANCODE.DELETE = 76
    SDL_SCANCODE.END = 77
    SDL_SCANCODE.PAGEDOWN = 78
    SDL_SCANCODE.RIGHT = 79
    SDL_SCANCODE.LEFT = 80
    SDL_SCANCODE.DOWN = 81
    SDL_SCANCODE.UP = 82

    SDL_SCANCODE.NUMLOCKCLEAR = 83 --[[/**< num lock on PC, clear on Mac keyboards
                                     */]]
    SDL_SCANCODE.KP_DIVIDE = 84
    SDL_SCANCODE.KP_MULTIPLY = 85
    SDL_SCANCODE.KP_MINUS = 86
    SDL_SCANCODE.KP_PLUS = 87
    SDL_SCANCODE.KP_ENTER = 88
    SDL_SCANCODE.KP_1 = 89
    SDL_SCANCODE.KP_2 = 90
    SDL_SCANCODE.KP_3 = 91
    SDL_SCANCODE.KP_4 = 92
    SDL_SCANCODE.KP_5 = 93
    SDL_SCANCODE.KP_6 = 94
    SDL_SCANCODE.KP_7 = 95
    SDL_SCANCODE.KP_8 = 96
    SDL_SCANCODE.KP_9 = 97
    SDL_SCANCODE.KP_0 = 98
    SDL_SCANCODE.KP_PERIOD = 99

    SDL_SCANCODE.NONUSBACKSLASH = 100 --[[/**< This is the additional key that ISO
                                        *   keyboards have over ANSI ones,
                                        *   located between left shift and Y.
                                        *   Produces GRAVE ACCENT and TILDE in a
                                        *   US or UK Mac layout, REVERSE SOLIDUS
                                        *   (backslash) and VERTICAL LINE in a
                                        *   US or UK Windows layout, and
                                        *   LESS-THAN SIGN and GREATER-THAN SIGN
                                        *   in a Swiss German, German, or French
                                        *   layout. */]]
    SDL_SCANCODE.APPLICATION = 101 --[[/**< windows contextual menu, compose */]]
    SDL_SCANCODE.POWER = 102 --[[/**< The USB document says this is a status flag,
                               *   not a physical key - but some Mac keyboards
                               *   do have a power key. */]]
    SDL_SCANCODE.KP_EQUALS = 103
    SDL_SCANCODE.F13 = 104
    SDL_SCANCODE.F14 = 105
    SDL_SCANCODE.F15 = 106
    SDL_SCANCODE.F16 = 107
    SDL_SCANCODE.F17 = 108
    SDL_SCANCODE.F18 = 109
    SDL_SCANCODE.F19 = 110
    SDL_SCANCODE.F20 = 111
    SDL_SCANCODE.F21 = 112
    SDL_SCANCODE.F22 = 113
    SDL_SCANCODE.F23 = 114
    SDL_SCANCODE.F24 = 115
    SDL_SCANCODE.EXECUTE = 116
    SDL_SCANCODE.HELP = 117
    SDL_SCANCODE.MENU = 118
    SDL_SCANCODE.SELECT = 119
    SDL_SCANCODE.STOP = 120
    SDL_SCANCODE.AGAIN = 121   --[[/**< redo */]]
    SDL_SCANCODE.UNDO = 122
    SDL_SCANCODE.CUT = 123
    SDL_SCANCODE.COPY = 124
    SDL_SCANCODE.PASTE = 125
    SDL_SCANCODE.FIND = 126
    SDL_SCANCODE.MUTE = 127
    SDL_SCANCODE.VOLUMEUP = 128
    SDL_SCANCODE.VOLUMEDOWN = 129
--[[/* not sure whether there's a reason to enable these */]]
--[[/*     LOCKINGCAPSLOCK = 130  */]]
--[[/*     LOCKINGNUMLOCK = 131 */]]
--[[/*     LOCKINGSCROLLLOCK = 132 */]]
    SDL_SCANCODE.KP_COMMA = 133
    SDL_SCANCODE.KP_EQUALSAS400 = 134

    SDL_SCANCODE.INTERNATIONAL1 = 135 --[[/**< used on Asian keyboards, see
                                            footnotes in USB doc */]]
    SDL_SCANCODE.INTERNATIONAL2 = 136
    SDL_SCANCODE.INTERNATIONAL3 = 137 --[[/**< Yen */]]
    SDL_SCANCODE.INTERNATIONAL4 = 138
    SDL_SCANCODE.INTERNATIONAL5 = 139
    SDL_SCANCODE.INTERNATIONAL6 = 140
    SDL_SCANCODE.INTERNATIONAL7 = 141
    SDL_SCANCODE.INTERNATIONAL8 = 142
    SDL_SCANCODE.INTERNATIONAL9 = 143
    SDL_SCANCODE.LANG1 = 144 --[[/**< Hangul/English toggle */]]
    SDL_SCANCODE.LANG2 = 145 --[[/**< Hanja conversion */]]
    SDL_SCANCODE.LANG3 = 146 --[[/**< Katakana */]]
    SDL_SCANCODE.LANG4 = 147 --[[/**< Hiragana */]]
    SDL_SCANCODE.LANG5 = 148 --[[/**< Zenkaku/Hankaku */]]
    SDL_SCANCODE.LANG6 = 149 --[[/**< reserved */]]
    SDL_SCANCODE.LANG7 = 150 --[[/**< reserved */]]
    SDL_SCANCODE.LANG8 = 151 --[[/**< reserved */]]
    SDL_SCANCODE.LANG9 = 152 --[[/**< reserved */]]

    SDL_SCANCODE.ALTERASE = 153 --[[/**< Erase-Eaze */]]
    SDL_SCANCODE.SYSREQ = 154
    SDL_SCANCODE.CANCEL = 155
    SDL_SCANCODE.CLEAR = 156
    SDL_SCANCODE.PRIOR = 157
    SDL_SCANCODE.RETURN2 = 158
    SDL_SCANCODE.SEPARATOR = 159
    SDL_SCANCODE.OUT = 160
    SDL_SCANCODE.OPER = 161
    SDL_SCANCODE.CLEARAGAIN = 162
    SDL_SCANCODE.CRSEL = 163
    SDL_SCANCODE.EXSEL = 164

    SDL_SCANCODE.KP_00 = 176
    SDL_SCANCODE.KP_000 = 177
    SDL_SCANCODE.THOUSANDSSEPARATOR = 178
    SDL_SCANCODE.DECIMALSEPARATOR = 179
    SDL_SCANCODE.CURRENCYUNIT = 180
    SDL_SCANCODE.CURRENCYSUBUNIT = 181
    SDL_SCANCODE.KP_LEFTPAREN = 182
    SDL_SCANCODE.KP_RIGHTPAREN = 183
    SDL_SCANCODE.KP_LEFTBRACE = 184
    SDL_SCANCODE.KP_RIGHTBRACE = 185
    SDL_SCANCODE.KP_TAB = 186
    SDL_SCANCODE.KP_BACKSPACE = 187
    SDL_SCANCODE.KP_A = 188
    SDL_SCANCODE.KP_B = 189
    SDL_SCANCODE.KP_C = 190
    SDL_SCANCODE.KP_D = 191
    SDL_SCANCODE.KP_E = 192
    SDL_SCANCODE.KP_F = 193
    SDL_SCANCODE.KP_XOR = 194
    SDL_SCANCODE.KP_POWER = 195
    SDL_SCANCODE.KP_PERCENT = 196
    SDL_SCANCODE.KP_LESS = 197
    SDL_SCANCODE.KP_GREATER = 198
    SDL_SCANCODE.KP_AMPERSAND = 199
    SDL_SCANCODE.KP_DBLAMPERSAND = 200
    SDL_SCANCODE.KP_VERTICALBAR = 201
    SDL_SCANCODE.KP_DBLVERTICALBAR = 202
    SDL_SCANCODE.KP_COLON = 203
    SDL_SCANCODE.KP_HASH = 204
    SDL_SCANCODE.KP_SPACE = 205
    SDL_SCANCODE.KP_AT = 206
    SDL_SCANCODE.KP_EXCLAM = 207
    SDL_SCANCODE.KP_MEMSTORE = 208
    SDL_SCANCODE.KP_MEMRECALL = 209
    SDL_SCANCODE.KP_MEMCLEAR = 210
    SDL_SCANCODE.KP_MEMADD = 211
    SDL_SCANCODE.KP_MEMSUBTRACT = 212
    SDL_SCANCODE.KP_MEMMULTIPLY = 213
    SDL_SCANCODE.KP_MEMDIVIDE = 214
    SDL_SCANCODE.KP_PLUSMINUS = 215
    SDL_SCANCODE.KP_CLEAR = 216
    SDL_SCANCODE.KP_CLEARENTRY = 217
    SDL_SCANCODE.KP_BINARY = 218
    SDL_SCANCODE.KP_OCTAL = 219
    SDL_SCANCODE.KP_DECIMAL = 220
    SDL_SCANCODE.KP_HEXADECIMAL = 221

    SDL_SCANCODE.LCTRL = 224
    SDL_SCANCODE.LSHIFT = 225
    SDL_SCANCODE.LALT = 226 --[[/**< alt, option */]]
    SDL_SCANCODE.LGUI = 227 --[[/**< windows, command (apple), meta */]]
    SDL_SCANCODE.RCTRL = 228
    SDL_SCANCODE.RSHIFT = 229
    SDL_SCANCODE.RALT = 230 --[[/**< alt gr, option */]]
    SDL_SCANCODE.RGUI = 231 --[[/**< windows, command (apple), meta */]]

    SDL_SCANCODE.MODE = 257    --[[/**< I'm not sure if this is really not covered
                                 *   by any of the above, but since there's a
                                 *   special KMOD_MODE for it I'm adding it here
                                 */]]

    --[[/* @} *//* Usage page 0x07 */]]

    --[[/**
     *  \name Usage page 0x0C
     *
     *  These values are mapped from usage page 0x0C (USB consumer page).
     */]]
    --/* @{ */

    SDL_SCANCODE.AUDIONEXT = 258
    SDL_SCANCODE.AUDIOPREV = 259
    SDL_SCANCODE.AUDIOSTOP = 260
    SDL_SCANCODE.AUDIOPLAY = 261
    SDL_SCANCODE.AUDIOMUTE = 262
    SDL_SCANCODE.MEDIASELECT = 263
    SDL_SCANCODE.WWW = 264
    SDL_SCANCODE.MAIL = 265
    SDL_SCANCODE.CALCULATOR = 266
    SDL_SCANCODE.COMPUTER = 267
    SDL_SCANCODE.AC_SEARCH = 268
    SDL_SCANCODE.AC_HOME = 269
    SDL_SCANCODE.AC_BACK = 270
    SDL_SCANCODE.AC_FORWARD = 271
    SDL_SCANCODE.AC_STOP = 272
    SDL_SCANCODE.AC_REFRESH = 273
    SDL_SCANCODE.AC_BOOKMARKS = 274

    --[[/* @} *//* Usage page 0x0C */]]

    --[[/**
     *  \name Walther keys
     *
     *  These are values that Christian Walther added (for mac keyboard?).
     */]]
    --[[/* @{ */]]

    SDL_SCANCODE.BRIGHTNESSDOWN = 275
    SDL_SCANCODE.BRIGHTNESSUP = 276
    SDL_SCANCODE.DISPLAYSWITCH = 277 --[[/**< display mirroring/dual display
                                           switch, video mode switch */]]
    SDL_SCANCODE.KBDILLUMTOGGLE = 278
    SDL_SCANCODE.KBDILLUMDOWN = 279
    SDL_SCANCODE.KBDILLUMUP = 280
    SDL_SCANCODE.EJECT = 281
    SDL_SCANCODE.SLEEP = 282

    SDL_SCANCODE.APP1 = 283
    SDL_SCANCODE.APP2 = 284

    --[[/* @} *//* Walther keys */]]

    --[[/* Add any other keys here. */]]

    SDL_SCANCODE.SDL_NUM_SCANCODES = 512 --[[/**< not a key, just marks the number of scancodes
                                 for array bounds */]]
--} SDL_Scancode;


--#endif --[[/* _SDL_scancode_h */]]

--[[/* vi: set ts=4 sw=4 expandtab: */]]
