-- Forlorn Fox main file - for running a client application
-- Copyright © 2014-2020 Rob Probin & Tony Park
-- No redistribution without permission.


-- require statements
require("sound_man")

utf8 = require("utilities/gulp_utf8")

local GlyphEditor = require("glyph_editor/glyph_editor")
local FunctionKeyMonitor = require("utilities/function_key_monitor")


local glyph_edit = GlyphEditor:new()

require("graphics/graphics")

require("mobs/mob_list")


require('map_control')
ClientMapLoader = require('map_controllers/ClientMapLoader')

gulp.map_loader = ClientMapLoader()

-- Networking
GulpNetworking = require("gulp_networking")
gulp_networking = GulpNetworking:new()
gulp_control_networking = nil
-- End Networking

main_menu = require("main_menu/main_menu")
require("networking/netstat")
local file_utils = require("utilities/file_utils")
require('utils')

local action_input = require("action_input")
local MouseMove = require("player/mouse_move")

local LuaMainThreaded = require("utilities/LuaMainThreaded")

local prefs = require("utilities/prefs")
local dispatcher = require("utilities/dispatcher")
require("gulp_constants")
local LocalConnectionFakeSocket = require("networking/local_connection_fake_socket")

local text_sizes = require("utilities/text_sizes")

function gulp.set_wrap_limits()
    local gr = gulp.app:get_standard_graphics_context()
    -- Minus 0.9 to ensure text can be seen at edge of screen
    gr:set_wrap_limits(0, math.floor(text_sizes:lines()), 
        0, text_sizes:columns()-0.9)
end



-- Pointless to override/delegate, because it's only called before everything :-)
function gulp.load(app)
    
    gulp_cpp.SetWindowTitle(gulp.window_title)
    
    gulp.time = 0
    gulp.app = app
        
    file_utils.create_save_folders()
    prefs.load()
    
    load_graphics()
    load_sounds()
    
    action_input.init_movement_keys()
    
    gulp._delegate_list = dispatcher()
    
    gulp._user_movement_target_list = {}
    gulp.game_managers = {}
    
    
    set_absolute_draw_list(app:get_absolute_draw_list())
    
    main_common.decode_command_line_arguments()
    
    --app:get_standard_graphics_context():set_game_screen_offset(true)
    
    gulp.function_key_monitor = FunctionKeyMonitor:new()

    -------------------------------------------------------------------------------
    gulp.ui_queue = {}
    gulp.server_queue = {}
    gulp.lua_game_world = {}

    -- connect to the control process on the server
    gulp_control_networking = GulpNetworking:new()
    gulp_control_networking.target_IPAddr = gulp.ServerIP        -- needs to be altered to URL of the website
    gulp_control_networking.target_port = gulp.prefs.network_port             -- default?? 
    gulp_control_networking:enableClient(gulp_control_networking.target_IPAddr, gulp_control_networking.target_port, 'control')
    
    gulp.main_menu = main_menu()

    -- remove this for now, but we might want to reinstate 'offline' game mode at some point
     --gulp.local_connection_fake_socket = LocalConnectionFakeSocket:new(gulp.server_queue)

    --[[ -- Queue test with no return Qs
    print("test 1")
    local q = gulp_cpp.LuaStateQueue("main_queue")
    if not q:is_empty() then print("Error", q:is_empty()) end
    q:send(true)
    if q:is_empty() then print("Error", q:is_empty()) end
    local t = q:read()
    print(q, type(t), t)
    if(t ~= true) then print("Error") end
    
    print("test 2")
    -- try sending to the newq
    local newq = gulp_cpp.LuaStateQueue("secondary_queue")
    if not newq:is_empty() then print("Error", newq:is_empty()) end
    newq:send("Hello")
    if newq:is_empty() then print("Error", newq:is_empty()) end
    local t = newq:read()
    print(q, newq, type(t), t)
    if(t ~= "Hello") then print("Error") end
    
    print("test 3")
    -- try sending to the newq
    if not newq:is_empty() then print("Error", newq:is_empty()) end
    q:send("Goodbye")
    if newq:is_empty() then print("Error", newq:is_empty()) end
    local t = q:read()
    print(q, newq, type(t), t)
    print(q:get_identifier(), newq:get_identifier())
    if(t ~= "Goodbye") then print("Error") end
    
    print("test 4")
    if not q:is_empty() then print("Error", q:is_empty()) end
    q:send(1234)
    if q:is_empty() then print("Error", q:is_empty()) end
    local t = q:read()
    print(type(t), t)
    if(t ~= 1234) then print("Error") end

    print("test 5")
    if not q:is_empty() then print("Error", q:is_empty()) end
    q:send(nil)
    if q:is_empty() then print("Error", q:is_empty()) end
    local t = q:read()
    print(type(t), t)
    if(t ~= nil) then print("Error") end

    print("test 6")
    if not q:is_empty() then print("Error", q:is_empty()) end
    q:send({ 1, 2, 3, { fred = 39, sid = 45, "hi", 
                [10]="five", ["!!__!!"] = "O_o", ["gee"]="GEE",
                [""]="Hello",
                { [true] = 999, [false] = 123 }, { true, false } } } )
    if q:is_empty() then print("Error", q:is_empty()) end
    local t = q:read()
    print(type(t), t)
    print_table(t)
    --]]
    --[[ -- Queue test with return Qs
    local q = gulp_cpp.LuaStateQueue("main_queue")
    print(q:is_empty())
    q:send(q, true)
    print(q:is_empty())
    local newq, t = q:read()
    print(q, newq, type(t), t)
    if(t ~= true) then print("Error") end
    
    -- try sending to the newq
    print(newq:is_empty())
    q:send(newq, "Hello")
    print(newq:is_empty())
    local newq2, t = newq:read()
    print(q, newq, newq2, type(t), t)
    if(t ~= "Hello") then print("Error") end
    
    -- try sending to the newq
    print(newq:is_empty())
    q:send(newq, "Goodbye")
    print(newq:is_empty())
    local newq3, t = q:read()
    print(q, newq, newq2, newq3, type(t), t)
    print(q:get_identifier(), newq:get_identifier(), newq2:get_identifier(), newq3:get_identifier())
    if(t ~= "Goodbye") then print("Error") end
    
    print(q:is_empty())
    q:send(q, 1234)
    print(q:is_empty())
    local newq, t = q:read()
    print(type(t), t)
    if(t ~= 1234) then print("Error") end

    print(q:is_empty())
    q:send(q, nil)
    print(q:is_empty())
    local newq, t = q:read()
    print(type(t), t)
    if(t ~= nil) then print("Error") end

    print(q:is_empty())
    q:send(q, { 1, 2, 3, { fred = 39, sid = 45, "hi", 
                [10]="five", ["!!__!!"] = "O_o", ["gee"]="GEE",
                [""]="Hello",
                { [true] = 999, [false] = 123 }, { true, false } } } )
    print(q:is_empty())
    local newq, t = q:read()
    print(type(t), t)
    print_table(t)
    --]]
    
    gulp.set_wrap_limits()
    
--[[
    -- joystick basic support
    -- Check for joysticks
    local joysticks = {}
    for i = 1, gulp_cpp.SDL_NumJoysticks() do
        -- Open joystick
        joysticks[i] = gulp_cpp.SDL_JoystickOpen(i-1)

--        // Close if opened
--        if (SDL_JoystickGetAttached(joy)) {
--            SDL_JoystickClose(joy);
--        }
    end

    for i,joy in ipairs(joysticks) do
        if(joy) then
            print("Opened Joystick", i)
            print("Name: ", gulp_cpp.SDL_JoystickName(joy))
            print("Number of Axes: ", gulp_cpp.SDL_JoystickNumAxes(joy))
            print("Number of Buttons: ", gulp_cpp.SDL_JoystickNumButtons(joy))
            print("Number of Balls: ", gulp_cpp.SDL_JoystickNumBalls(joy))
            print("Number of hats:", gulp_cpp.SDL_JoystickNumHats(joy))
        else
            print("Couldn't open Joystick ", i)
        end
    end
--]]

end



--local _angle = 0

local function handle_local_network_packet_ui(message)
    if message.mt then
        local dispatcher = gulp.client_receivers[message.mt]
        if dispatcher then
            dispatcher(message)
        end
    else
        print("UI: No mt supplied in network message?")
    end
end

local showingError = false
local function handle_internet_broadcast_error(message)
    -- If not already showing the menu then show it
    if showingError == false then
        gulp.error_menu = Menu {
            background = gulp.colour.BRIGHT_RED.simple_colour, 
            foreground = gulp.colour.BRIGHT_WHITE.simple_colour,
            border_style = 2,
            { message.error_string },
            { "" },
            { "Restart game to try again" },
            { "" },
    --                        { " QUIT to Menu ", function() showingError = false gulp.save_and_quit_game() end },
            { " Continue", function() 
                    showingError = false 
                    end },
        }
        gulp.error_menu.on_close = function()
            showingError = false
        end
    end
    
    -- Stop us showing the network error again
    showingError = true
end

local function handle_internet_broadcast_now_registered()
  -- If the error message is being shown close
  if showingError then
    gulp.error_menu:close()
    showingError = false
  end  
end

local function handle_server_couldnt_start(message)
    
    --gulp.temp_menu = 
    local m = Menu {
            line = 1.5,
            column = 1.5,
            background = gulp.colour.BRIGHT_RED.simple_colour, 
            foreground = gulp.colour.BRIGHT_WHITE.simple_colour,
            border_style = 2,
            { "Couldn't start server" },
            { "" },
            { "Port already in use!" }, --"Failed to bind to port" },
            { "Is another game running?" },
            { "" },
            { " OK", 1 }
        }
    m.on_close = function()
        gulp.temp_menu = nil  
        
        -- we know the local server IS NOT running
        gulp.local_server_started = false

        gulp.end_game()                    
        
        end

        
end


--
-- Queue command dispatch table
-- 
local ui_function_dispatch = {
    qlocal = handle_local_network_packet_ui,
    internet_broadcast_error = handle_internet_broadcast_error,
    internet_broadcast_now_registered = handle_internet_broadcast_now_registered,
    server_couldnt_start = handle_server_couldnt_start,
    request_game = gulp.HandleRequestGame,
}

local particle_effects = require("graphics/particle_effects")

local server_dt_update = { qcommand = "update", dt = 0 }

local GUI_update_time = 0;
local GUI_update_time_counter_start = gulp_cpp.PerformanceFrequencyCounter()
local GUI_update_time_counter_end = gulp_cpp.PerformanceFrequencyCounter()

local print_time_clock = gulp_cpp.SDL_GetTicks()
local os_clock_start = os.clock()

function gulp.update(dt)
    GUI_update_time_counter_start:set_now()
    --[[
    gulp.debug:set_lua_info_string(string.format("%f %f %f   %f %f %f", 
            gulp_cpp.current_column_start, 
            gulp_cpp.current_columns, 
            gulp_cpp.current_column_end, 
            gulp_cpp.current_line_start, 
            gulp_cpp.current_line_end,
            gulp_cpp.current_lines))
    --]]
    
    --if gulp.localplayer then
    --    gulp.debug:set_lua_info_string(string.format("L = %f, C = %f", gulp.localplayer.line, gulp.localplayer.column))
    --    gulp.debug:set_lua_info_string(string.format("clo = %f, cco = %f", gulp.localplayer.current_line_offset, gulp.localplayer.current_column_offset))
    --end
    
    -- tell the server as soon as possible, so that the latency is minimised
    -- NOTE:Does the controller need delta T? Maybe not, it might be entirely reactive 
    -- this is needed to call GulpNetworking:update for keepAlive
    server_dt_update.dt = dt
    for _, q in pairs(gulp.server_queue) do       -- changed from ipairs to pairs. Order is not specific, and it means we can take them out the list.
        q:send(server_dt_update)
    end
            
    gulp.time = gulp.time + dt

    local function dispatch_queue(q)
        while not q:is_empty() do
            local message = q:read()
            --print("UI:", message)
            local f = ui_function_dispatch[message.qcommand]
            if f then
                f(message)
            else
                error("unknown command in gulp.update() " .. tostring(f))
            end
        end
    end
    
    for _, q in ipairs(gulp.ui_queue) do
        dispatch_queue(q)
    end
    
    if gulp.server then
        dispatch_queue(gulp.ui_queue['control'])
    end
    
    --[[
    _angle = _angle + (36 * dt) 
    if _angle > 360 then
        _angle = _angle - 360
    end
    --]]
    
    --gulp_cpp.LuaMain_push_number(gulp.lua_game_world, dt);
    --gulp_cpp.run_gulp_function_if_exists(gulp.lua_game_world, "server_update", 1, 0);
    --gulp_cpp.LuaMain_push_number(gulp.lua_network_low_level, dt  );
    --gulp_cpp.run_gulp_function_if_exists(gulp.lua_network_low_level, "net_update", 1, 0);
    
    -- Kenny added temp step tick code for copas
    -- make sure receive stuff is done before objects are updated
    gulp_networking:stepCopas()
    
    gulp._delegate_list:update_dispatcher(dt)

    -- make sure send stuff is sent straight away
    --gulp_networking:stepCopas()
    
    main_common.hotswap_reloader.monitor_loop(dt)
    
    particle_effects.update(dt)
    
    -- measure update time
    GUI_update_time_counter_end:set_now()
    GUI_update_time = GUI_update_time + GUI_update_time_counter_end:result_of_subtract_arg(GUI_update_time_counter_start)
end


local border_colour = get_SDL_Color_by_name('BLACK')

function gulp.splash(app)
    
    -- called before load and draw
    -- maybe this function should be called preload?
    -- engine gives a single call to draw before it calls load - presumably to display splash screen
    
    gulp._delegate_list = dispatcher()
    text_sizes.setup_defaults(32, 24)
    set_absolute_draw_list(app:get_absolute_draw_list())
    
    local window_width = gulp_cpp.GetWindowWidth()
    local window_height = gulp_cpp.GetWindowHeight()
    set_adl_rect(0, 0, window_width, window_height)
    
end

local GUI_draw_time = 0;
local GUI_draw_time_counter_start = gulp_cpp.PerformanceFrequencyCounter()
local GUI_draw_time_counter_end = gulp_cpp.PerformanceFrequencyCounter()

function gulp.draw(gr)
    
    GUI_draw_time_counter_start:set_now()

    --[[
    gr:printExT(2 ,2 ,6 ,33, { angle = _angle} )
    gr:printExT(1 ,4 ,5 ,66, { flip = 1 } )
    gr:printExT(1 ,6 ,4 ,66, { flip = 2 } )
    gr:printExT(1 ,8 ,3 ,66, { flip = 3 } )
    gr:printExT(1 ,10 ,6 ,33, { angle = _angle, scale_x = 0.5 + (_angle/36), scale_y = 0.5 + (_angle/36) } )
    gr:printExT(1 ,12 ,3 ,65, { angle = _angle, scale_x = 4 } )
    --]]
    set_graphics_context(gr)
    
    -- draw the map and the status lines - if enabled
    if gulp._draw_map and gulp.localplayer then
        local map_name = gulp.localplayer.map_name
        if check_map_exists(map_name) then
            local map_to_render = get_maze(map_name)
            if map_to_render then
              --check_mdls(map_to_render, "draw")
              map_to_render:print(gr)
              particle_effects.draw(gr)
            end
        end
    end
    
    -- draw UI elements etc
    -- do this before the passive dispatcher so menus go on top of everything
    render_draw_lists(gr)
    
    local ret = gulp._delegate_list:passive_dispatcher('draw', gr)
    
    MouseMove.draw_regions(gr)

    render_debug_data(gr)
        
    -- measure update time
    GUI_draw_time_counter_end:set_now()
    GUI_draw_time = GUI_draw_time + GUI_draw_time_counter_end:result_of_subtract_arg(GUI_draw_time_counter_start)
    local clock_diff = (gulp_cpp.SDL_GetTicks() - print_time_clock) / 1000
    if clock_diff > 10 then
        --print(string.format("GUI Update = %.1f %%", 100 * GUI_update_time / clock_diff))
        --print(string.format("GUI Draw = %.1f %%", 100 * GUI_draw_time / clock_diff))
        --print(string.format("Total Processor usage = %.1f %%", 100 * (os.clock()-os_clock_start)/clock_diff))
        print_time_clock = gulp_cpp.SDL_GetTicks()
        GUI_draw_time = 0
        GUI_update_time = 0
        os_clock_start = os.clock()
    end
    
    return ret
end

function gulp.mousepressed(x, y, handled, right)
    return gulp._delegate_list:action_dispatcher('mousepressed', x, y, handled, right)
end

function gulp.mousereleased(x, y, handled, right)
    return gulp._delegate_list:action_dispatcher('mousereleased', x, y, handled, right)
end

function gulp.mouse_down_moved(x, y, handled, right)
    return gulp._delegate_list:action_dispatcher('mouse_down_moved', x, y, handled, right)
end

function gulp.keypressed(sym, scancode, modifier, repeated)
    local handled = gulp._delegate_list:action_dispatcher('keypressed', sym, scancode, modifier, repeated)
    
    if not handled then
        action_input.game_key_processing(scancode, true)
    end
end

function gulp.keyreleased(sym, scancode, modifier)
    local handled = gulp._delegate_list:action_dispatcher('keyreleased', sym, scancode, modifier)
    
    if not handled then
        action_input.game_key_processing(scancode, false)
    end
end


function gulp.textinput(...)
    return gulp._delegate_list:action_dispatcher('textinput', ...)
end

function gulp.textediting(...)
    return gulp._delegate_list:action_dispatcher('textediting', ...)
end



-- does this make sense to delegate?
function gulp.focus(...) -- f)
    return gulp._delegate_list:action_dispatcher('focus', ...)
end

function gulp.launch_glyph_editor()
    glyph_edit:launch_glyph_editor()
end


Player = require("player/Player")
require("main_menu/connect_menu")


function gulp._start_game_after_connect()
    MouseMove.enable_game_controls()
    gulp._draw_map = true
    gulp._draw_status_lines = true
    
    local player_id = nil
    
    -- client has a slave map loader
    gulp.map_loader:LoadInitialMap()
    
    -- create the player
    -- must be after the initial maze is loaded (nasty dependency!!)
    -- because the player accesses the C++ maze data directly
    -- in various places, including its initialize() function
    gulp.localplayer = Player(gulp.selected_avatar, gulp.selected_colour, player_id)
    gulp.add_user_movement_target(gulp.localplayer)
    
    -- Running in client mode
    gulp_networking:send_register_player_from_Client(gulp.localplayer)
    
    -- must be done after player registered
    local map_name = gulp.localplayer.map_name
    gulp.map_loader:request_neighbouring_maps(map_name)
    
    local map = get_map(map_name)
    if(map) then
        local manager = find_game_manager(map)
        if(manager) then
            manager:arrange_screen()
        end
    end
        
    
    -- delegate for networking - this will change to be 
    -- "run networking in separate lua context" and will always happen (no check)
    -- after refactoring
    -- TODO
    gulp._delegate_list:add_delegate(gulp_networking)
    
end

function gulp.close_all_movetargets()
    for _,v in pairs(gulp._user_movement_target_list) do
        -- if close exists in that movement target
        if v.close then
            v:close()
        end
    end
end

function gulp.save_and_quit_game()
    gulp.save_game()
    gulp.end_game()
    remember_game_end_for_debug()   
end

function gulp.save_game()
end


-- This might be called on a partially constructed game ...
-- so we tear down what ever has been constructed. It is also safe to call twice, 
-- e.g. from a local server message after the client has already aborted itself.
function gulp.end_game(reason, game)
    
    if game == nil then
        game = 1
        print "Ended a game without telling end_game() which game to end...!"
    end
    
    
    particle_effects.delete_all()

    if gulp.main_menu then
        gulp.main_menu:close()          -- close if not closed already
    end
    
    MouseMove.disable_game_controls()
    gulp._draw_map = false
    gulp._draw_status_lines = false

    -- close down server at some appropriate point 
    if gulp.local_server_started then
        gulp.local_server_started = false
        gulp.server_queue[game]:send( { qcommand = "stop" } )
    end
    
    -- drop all the players of all types
    for _,player in all_players() do
        player:remove()
    end
    
    
    gulp_networking:unregisterClient()
    
    gulp.close_all_movetargets()
    gulp.remove_user_movement_target(gulp.localplayer)
    
    -- drop the maps
    gulp._delegate_list:drop_delegate(gulp.map_loader)
    gulp.map_loader:DropAllMaps()
    
    if gulp.localplayer then
        gulp.localplayer:remove()
        gulp.localplayer = nil
    end
    gulp.movetarget = nil
    
    -- make sure we remove all the dropped graphical elements
    -- from the screen before we (potentially) restart the game
    collectgarbage()
    
    gulp.main_menu = main_menu()
    if reason then
        gulp.reason_for_quit_menu = Menu{
                line = 4,
                column = 4,
                background = gulp.colour.BLUE.simple_colour, 
                foreground = gulp.colour.BRIGHT_WHITE.simple_colour,
                border_style = 2,
                { reason },
                { "" },
                { "  OK  ", 99 }
            }
    end
    
    -- End network stuff
    gulp_networking:disableClient()
    
    gulp._delegate_list:drop_delegate(gulp_networking)
  
    -- clean up to avoid reuse later
    gulp.target_IPAddr = nil
    gulp.target_port = nil
    
    if gulp.text_buffer then
        gulp.text_buffer:clear()
    end
    
    -- reset back to registration mode so we can connect to a new server
    client_receiver_set_register_mode()
end


function gulp.add_user_movement_target(object)
    
    -- send a 'key up' to the current movetarget before we switch to the new one
    if gulp.movetarget ~= nil and gulp.movetarget.action then
        gulp.movetarget:action(0)
    end
    
    
    table.insert(gulp._user_movement_target_list, object)
    gulp.movetarget = gulp._user_movement_target_list[#gulp._user_movement_target_list]
end

function gulp.remove_user_movement_target(object)
    -- removes the last copy only
    local delegates = gulp._user_movement_target_list
    local i = #gulp._user_movement_target_list
    while i > 0 do
        if delegates[i] == object then
            table.remove(delegates, i)
            break
        end
        i = i - 1
    end
    
    -- make sure we have a movetarget that's in the list
    local len = #gulp._user_movement_target_list
    if len then
        gulp.movetarget = gulp._user_movement_target_list[len]
    else
        gulp.movetarget = nil
    end
end

function gulp.user_movement_not_handled(object, movement_type, ...)
    -- if object is in the list twice, this is a problem...
    
    print("haven't implemented this yet")
    os.exit(1) -- not implemnted yet
end

function gulp.quit_event(game)
    -- check if not save world, if not, save
end

function gulp.specific_game_quit(game)
    -- @todo: debug this before using
    if game == nil then
        game = 1
        print("Quitting game 1 only")
    end
    
    -- tell threads to quit
    local server_queue = gulp.server_queue[game]
    if server_queue then
        server_queue:send( { qcommand = "stop_thread" } )
    end
    
    local lua_game_world = gulp.lua_game_world[game]
    if lua_game_world then
        -- wait up to 0.5 for the server thread to respond
        for _ = 1,10 do
            gulp_cpp.SDL_Delay(50)
            if not gulp.lua_game_world[game]:IsRunning() then
                break
            end
        end
        if gulp.lua_game_world[game]:IsRunning() then
            error("Thread ".. gulp.lua_game_world[game]:GetThreadName() .. "Still running on shutdown")
        else
            gulp.lua_game_world[game] = nil
            gulp.server_queue[game] = nil
        end
        
        --print("Server Game World thread return value", gulp.lua_game_world:WaitThread())
        -- Not strictly necessary, but we do it anyway...
        --gulp.lua_game_world[game]:WaitThread()
    
    end
    
end



local function open_joystick(device)
    -- joystick basic support
    local joy = gulp_cpp.SDL_JoystickOpen(device)

    if(joy) then
        print("Opened Joystick", device)
        print("Name: ", gulp_cpp.SDL_JoystickName(joy))
        print("Number of Axes: ", gulp_cpp.SDL_JoystickNumAxes(joy))
        print("Number of Buttons: ", gulp_cpp.SDL_JoystickNumButtons(joy))
        print("Number of Balls: ", gulp_cpp.SDL_JoystickNumBalls(joy))
        print("Number of hats:", gulp_cpp.SDL_JoystickNumHats(joy))
    else
        print("Couldn't open Joystick ", device)
    end
end


-- Make a table where we can change the integer into a name
local known_event_types_name_lookup = {}
for ev_name,ev_int in pairs(gulp.SDL_EventType) do
    known_event_types_name_lookup[ev_int] = ev_name
end

local function joy_added(event)
    open_joystick(event.jdevice.which)
end
local function joy_axis_motion(event)
    local ev = event.jaxis
    --print("joy motion", ev.which, ev.axis, ev.value)
    if ev.axis == 0 then
        if ev.value > 1000 then
            gulp.keypressed(SDLK.RIGHT, SDL_SCANCODE.RIGHT, KMOD.NONE)
            gulp.keyreleased(SDLK.LEFT, SDL_SCANCODE.LEFT, KMOD.NONE)
        elseif ev.value < -1000 then
            gulp.keyreleased(SDLK.RIGHT, SDL_SCANCODE.RIGHT, KMOD.NONE)
            gulp.keypressed(SDLK.LEFT, SDL_SCANCODE.LEFT, KMOD.NONE)
        else
            gulp.keyreleased(SDLK.RIGHT, SDL_SCANCODE.RIGHT, KMOD.NONE)
            gulp.keyreleased(SDLK.LEFT, SDL_SCANCODE.LEFT, KMOD.NONE)
        end
    elseif ev.axis == 1 then
        if ev.value > 1000 then
            gulp.keyreleased(SDLK.UP, SDL_SCANCODE.UP, KMOD.NONE)
            gulp.keypressed(SDLK.DOWN, SDL_SCANCODE.DOWN, KMOD.NONE)
        elseif ev.value < -1000 then
            gulp.keypressed(SDLK.UP, SDL_SCANCODE.UP, KMOD.NONE)
            gulp.keyreleased(SDLK.DOWN, SDL_SCANCODE.DOWN, KMOD.NONE)            
        else
            gulp.keyreleased(SDLK.UP, SDL_SCANCODE.UP, KMOD.NONE)
            gulp.keyreleased(SDLK.DOWN, SDL_SCANCODE.DOWN, KMOD.NONE)
        end
    end


end
local function joy_button_down(event)
    local ev = event.jbutton
    print("joy down", ev.which, ev.button, ev.state)
    if ev.button == 0 or ev.button == 1 then
        gulp.keypressed(SDLK.SPACE, SDL_SCANCODE.SPACE, KMOD.NONE, false)
    else
        gulp.keypressed(SDLK.RETURN, SDL_SCANCODE.RETURN, KMOD.NONE, false)
    end

end
local function joy_button_up(event)
    local ev = event.jbutton
    print("joy up", ev.which, ev.button, ev.state)
    if ev.button == 1 or ev.button == 2 then
        gulp.keyreleased(SDLK.SPACE, SDL_SCANCODE.SPACE, KMOD.NONE, false)
    else
        gulp.keyreleased(SDLK.RETURN, SDL_SCANCODE.RETURN, KMOD.NONE, false)
    end
end


local extra_event_processing_table = {
    [gulp.SDL_EventType.SDL_JOYDEVICEADDED] = joy_added,
    [gulp.SDL_EventType.SDL_JOYAXISMOTION] = joy_axis_motion,
    [gulp.SDL_EventType.SDL_JOYBUTTONDOWN] = joy_button_down,
    [gulp.SDL_EventType.SDL_JOYBUTTONUP] = joy_button_up,
}


function gulp.unknown_event(event_type, event)
    local processor = extra_event_processing_table[event_type]
    if processor then
        processor(event)
    else
        local type_string = known_event_types_name_lookup[event_type]
        if type_string then
            --print(type_string .. " event")
        else
            print(string.format("Unknown event 0x%x", event_type))
        end
    end
end

--
-- SDL_WindowEventID values
--
local SDL_WINDOWEVENT_NONE = 0          -- Never used
local SDL_WINDOWEVENT_SHOWN = 1         -- Window has been shown
local SDL_WINDOWEVENT_HIDDEN = 2        -- Window has been hidden
local SDL_WINDOWEVENT_EXPOSED = 3       -- Window has been exposed and should be redrawn
local SDL_WINDOWEVENT_MOVED = 4         -- Window has been moved to data1, data2
local SDL_WINDOWEVENT_RESIZED = 5       -- Window has been resized to data1xdata2
local SDL_WINDOWEVENT_SIZE_CHANGED = 6  -- The window size has changed, either as a result of an API call or through the system or user changing the window size
local SDL_WINDOWEVENT_MINIMIZED = 7     -- Window has been minimized
local SDL_WINDOWEVENT_MAXIMIZED = 8     -- Window has been maximized
local SDL_WINDOWEVENT_RESTORED = 9      -- Window has been restored to normal size and position
local SDL_WINDOWEVENT_ENTER = 10        -- Window has gained mouse focus
local SDL_WINDOWEVENT_LEAVE = 11        -- Window has lost mouse focus
local SDL_WINDOWEVENT_FOCUS_GAINED = 12 -- Window has gained keyboard focus
local SDL_WINDOWEVENT_FOCUS_LOST = 13   -- Window has lost keyboard focus
local SDL_WINDOWEVENT_CLOSE = 14        -- The window manager requests that the window be closed

function gulp.windowevent(event, data1, data2)
    --print("Window event", event, data1, data2)
    if event == SDL_WINDOWEVENT_SIZE_CHANGED then
        --print("Resized to", data1, "x", data2)
        gulp_cpp.UpdateScreenData_game_window()
        text_sizes.resize_all()
        
        if gulp.localplayer then
            gulp.localplayer:calculate_offsets()
            
            local map = get_map(gulp.localplayer.map_name)
            if(map) then
                local manager = find_game_manager(map)
                if(manager) then
                    manager:arrange_screen()
                end
            end
        
        end
        
        gulp._delegate_list:passive_dispatcher('resize')

    end
    
    -- Do this when any window event happens.
    gulp.set_wrap_limits()
    
    --[[
    SDL_WINDOWEVENT_RESIZED
    window has been resized to data1xdata2; this is event is always preceded by 
    SDL_WINDOWEVENT_SIZE_CHANGED

    SDL_WINDOWEVENT_SIZE_CHANGED
    window size has changed, either as a result of an API call or through the system 
    or user changing the window size; this event is followed by 
    SDL_WINDOWEVENT_RESIZED if the size was changed by an external event, i.e. the 
    user or the window manager
    --]]
end



function gulp.setNetworkError()
    gulp.NetworkError = true 
end    

function gulp.clearNetworkError()
    gulp.NetworkError = false 
end    

function gulp.isNetworkError()
    return gulp.NetworkError
end    

function server_run_lua(command, game)
    
    if game == nil then
        game = 1
        print("Only printing data for game 1")
    end
    
    gulp.server_queue[game]:send( { qcommand = "run_lua", lua = command } )
end
