-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

require("strict")
local class = require("middleclass")
local TextEffects = require("utilities/text_effects")
require("graphics/graphics")

local FallingLetters = class("FallingLetters")
local text_sizes = require("utilities/text_sizes")

-- maybe this should be the theme colour?
local default_colour = "BRIGHT_MAGENTA"
if gulp.app_name == "Forlorn Fox" then
    default_colour = "ORANGE"
elseif gulp.app_name == "Egg Tag" then
    default_colour = "BRIGHT_YELLOW"
end

function FallingLetters:set_letter(row, column, letter)
    local t = TextEffects:new(row, column, letter)
    local size = 200 + math.random(0, 100)
    t:small(size)
    t:colour(default_colour)
    local angle = -10 + math.random(0,20)
    t:rotate(angle)
    t.FL_size_inc = (math.random(0,1) * 2)-1
    t.FL_size_inc = t.FL_size_inc * 10
    t.FL_angle_inc = (math.random(0,1) * 2)-1
    t.FL_angle_inc = t.FL_angle_inc * 5
    t.FL_size = size
    t.FL_angle = angle
    return t
end

function FallingLetters:set_fall()
    local colours = {
        gulp.colour.RED.simple_colour,
        gulp.colour.GREEN.simple_colour,
        gulp.colour.YELLOW.simple_colour,
        gulp.colour.BLUE.simple_colour,
    }
    
    local t = { 
        row = math.random(math.floor(text_sizes:window_start_line()), 
            math.floor(text_sizes:window_end_line())),
        column = math.random(math.floor(text_sizes:window_start_column()), 
            math.floor(text_sizes:window_end_column())),
        speed = math.random(1,11)
    }
    if gulp.app_name == "Forlorn Fox" then
        t.char = math.random(33, 127)
        t.colour = random_entry(colours)
    else
        t.char = get_glyph("mobs", 0x01)
    end
    
    return t
end

function FallingLetters:initialize()
    gulp._delegate_list:add_delegate(self)

    self.fall = {}
    for i = 0, 20 do
        table.insert(self.fall, self:set_fall())
    end
    
    local letters = {}
    
    if gulp.app_name == "Forlorn Fox" then
        table.insert(letters, self:set_letter(3, 3, "F"))
        table.insert(letters, self:set_letter(3, 5.2, "o"))
        table.insert(letters, self:set_letter(3, 7.4, "r"))
        table.insert(letters, self:set_letter(3, 9.6, "l"))
        table.insert(letters, self:set_letter(3, 11.8, "o"))
        table.insert(letters, self:set_letter(3, 14, "r"))
        table.insert(letters, self:set_letter(3, 16.2, "n"))
        table.insert(letters, self:set_letter(3, 22.4, "F"))
        table.insert(letters, self:set_letter(3, 24.6, "o"))
        table.insert(letters, self:set_letter(3, 26.8, "x"))
    elseif gulp.app_name == "Egg Tag" then
        table.insert(letters, self:set_letter(2, 3, "E"))
        table.insert(letters, self:set_letter(2, 5.2, "g"))
        table.insert(letters, self:set_letter(2, 7.4, "g"))
        table.insert(letters, self:set_letter(2, 22.4, "T"))
        table.insert(letters, self:set_letter(2, 24.6, "a"))
        table.insert(letters, self:set_letter(2, 26.8, "g"))
    else
        table.insert(letters, self:set_letter(3, 7.2, "E"))
        table.insert(letters, self:set_letter(2, 9.4, "y"))
        table.insert(letters, self:set_letter(3, 11.6, "e"))
        table.insert(letters, self:set_letter(2, 13.8, "b"))
        table.insert(letters, self:set_letter(3, 16, "e"))
        table.insert(letters, self:set_letter(2, 18.2, "a"))
        table.insert(letters, self:set_letter(3, 20.4, "s"))
        table.insert(letters, self:set_letter(2, 22.6, "t"))
        
        table.insert(letters, self:set_letter(6, 9.4, "E"))
        table.insert(letters, self:set_letter(7, 11.6, "s"))
        table.insert(letters, self:set_letter(6, 13.8, "c"))
        table.insert(letters, self:set_letter(7, 16, "a"))
        table.insert(letters, self:set_letter(6, 18.2, "p"))
        table.insert(letters, self:set_letter(7, 20.4, "e"))
    end
    
    self._letters = letters
end

function FallingLetters:draw(gr)
--    print_string_at(20, 0, tostring(self._letters[1].FL_angle))
--    print_string_at(21, 0, tostring(self._letters[1].FL_angle_inc))
--    set_fg_colour("BLUE")

    text_sizes:set_viewport(gr)

    set_fg_colour("WHITE")
    set_bg_colour_transparent()
    for _, f in ipairs(self.fall) do
        if(f.colour) then set_fg_colour(f.colour) end
        print_glyph_at(f.row, f.column, f.char)
    end
    
    --[[   Remove Whale/Dragon/Test drawing ... looks weird without it :-)
    set_fg_colour("MAGENTA")
    print_glyph_at(10,10,get_glyph('dragon',33))
    
    print_glyph_at(10,11,get_glyph('dragon',128))
    print_glyph_at(10,12,get_glyph('dragon',129))
    print_glyph_at(10,13,get_glyph('dragon',130))
    print_glyph_at(10,14,get_glyph('dragon',131))
    print_glyph_at(10,15,get_glyph('dragon',132))
    print_glyph_at(10,16,get_glyph('dragon',133))
    print_glyph_at(10,17,get_glyph('dragon',134))
    print_glyph_at(10,18,get_glyph('dragon',135))
    
    print_glyph_at(11,11,get_glyph('dragon',144))
    print_glyph_at(11,12,get_glyph('dragon',145))
    print_glyph_at(11,13,get_glyph('dragon',146))
    print_glyph_at(11,14,get_glyph('dragon',147))
    print_glyph_at(11,15,get_glyph('dragon',148))
    print_glyph_at(11,16,get_glyph('dragon',149))
    print_glyph_at(11,17,get_glyph('dragon',150))
    print_glyph_at(11,18,get_glyph('dragon',151))
    --]]
    
end

function FallingLetters:update(dt)
    local dt_limited = math.min(0.5, dt)
    for _, letter in ipairs(self._letters) do
        
        local size_inc = dt_limited * letter.FL_size_inc 
        letter.FL_size = letter.FL_size + size_inc
        if letter.FL_size > 300 then
            letter.FL_size_inc = -math.abs(letter.FL_size_inc)
        elseif letter.FL_size < 200 then
            letter.FL_size_inc = math.abs(letter.FL_size_inc)
        end
        letter:small(letter.FL_size)
        
        local angle_inc = dt_limited * letter.FL_angle_inc
        if letter.FL_angle > 10 then
            letter.FL_angle_inc = -math.abs(letter.FL_angle_inc)
        elseif letter.FL_angle < -10 then
            letter.FL_angle_inc = math.abs(letter.FL_angle_inc)
        end
        letter.FL_angle = letter.FL_angle + angle_inc
        letter:rotate(letter.FL_angle)
    end
    
    for i, f in ipairs(self.fall) do
        f.row = f.row + dt * f.speed
        if f.row > text_sizes:window_end_line()+1 then
            self.fall[i] = self:set_fall()
            self.fall[i].row = text_sizes:window_start_line()-1
        end
    end
    
end

function FallingLetters:close()
    for _, letter in ipairs(self._letters) do
        letter:hide()
    end
    gulp._delegate_list:drop_delegate(self)
end

return FallingLetters
