-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

function gulp.start_game()
    remember_game_start_for_debug()

    -- Clear network error as new game
    gulp.clearNetworkError()
    
    -- all players are clients
    gulp._start_client_game()
    
end



function gulp.failed_to_connect_menu(reason)
    gulp.main_menu = main_menu()
    gulp.reason_for_quit_menu = Menu{
            line = 4,
            column = 4,
            background = gulp.colour.BLUE.simple_colour, 
            foreground = gulp.colour.BRIGHT_WHITE.simple_colour,
            border_style = 2,
            { "Failed to connect" },
            { "" },
            { reason },
            { "" },
            { "  OK  ", 99 }
    }
end

function gulp._start_client_game()
    gulp.connecting_menu = Menu{
        {"Connecting to server"}, 
        {"For 0 seconds" },
        { "" },
        {"Cancel", function() 
                gulp.connecting_menu:cancel_connect()
                if not gulp.connecting_menu._antiflicker then
                    --gulp.main_menu = main_menu() 
                end
            end },
    }
    gulp.connecting_menu._antiflicker = false
    gulp.connecting_menu.on_close = function(self)
        self._connect_waiter:close()
    end
    gulp.connecting_menu.slow_close = function(self)
        -- delayed close to avoid flicker
        gulp.connecting_menu._antiflicker = true
        local t = self._connect_waiter:anti_flicker_start(self)
    end
    gulp.connecting_menu.keypressed = function(self, sym)
        if back_key(sym) then
            gulp.connecting_menu:close()
            gulp.main_menu = main_menu()
            return true
        else
            return Menu.keypressed(self, sym)
        end
    end
    
    gulp.connecting_menu._connect_waiter = { 
        update = function(self, dt)
            if not self.anti_flicker_mode then
                self.connect_time = self.connect_time + dt
                local new_string = "For " .. math.floor(self.connect_time) .. " seconds"
                gulp.connecting_menu:update_menu_string(2, new_string)
                if self.connect_time > 30 then
                    gulp.client_register_failed("timeout")
                    gulp.connecting_menu:cancel_connect()
                end
                if self.connect_time > 0.4 then
                    gulp.connecting_menu:show()
                end
            else
                self.anti_flicker_timer = self.anti_flicker_timer + dt
                if self.anti_flicker_timer > 0.1 then
                    self.anti_flicker_owner:close()
                end
            end
        end,
        close = function(self)
            gulp._delegate_list:drop_delegate(gulp.connecting_menu._connect_waiter)
        end,
        anti_flicker_start = function(self, owner)
            self.anti_flicker_mode = true
            self.anti_flicker_timer = 0
            self.anti_flicker_owner = owner
        end,
        connect_time = 0,
        anti_flicker_mode = false,
        
    }
    gulp.connecting_menu:hide()
    
    gulp.connecting_menu.cancel_connect = function(self)
       gulp.end_game()
    end
    gulp.connecting_menu.tell_result = function(self, success, reason)
        if not success then
            self:close()
            gulp.failed_to_connect_menu(reason)
            
            self:cancel_connect()
        else
            -- start game?
        end
    end
    
    gulp._delegate_list:add_delegate(gulp.connecting_menu._connect_waiter)
    
    if gulp.enable_client_networking == nil then
        error("enable_client_networking is nil ... coding error")
    end
    
    if gulp.enable_client_networking then
        gulp_networking:enableClient(gulp.target_IPAddr, gulp.target_port)
    else
        -- The connect should always succeed
        gulp_networking:set_local_connect_socket()
        local success = gulp_networking:finalise_client_connect()
        gulp.connecting_menu:tell_result(success, "local connect")
    end

end

function gulp.client_register_failed(reason, server_version)
    local message
    
    if reason == "timeout" then
        message = "Timed out"
    elseif reason == "magic" then
        message = "Message Broken"
    elseif reason == "version" then
        message = "Server Version " .. tostring(server_version)
    elseif reason == "?" then
        message = "Unknown reason"
    else
        message = "Invalid message"
    end
    
    gulp.connecting_menu:close()
    gulp.failed_to_connect_menu(message)
end

function gulp.client_register_succeeded()
    gulp.connecting_menu:slow_close()
    gulp._start_game_after_connect()
end

