-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
-- Main menu
--
-- Created by Rob Probin, October 2014
-- For Forlorn Fox

require("strict")
local class = require("middleclass")
local credits = require("main_menu/credits")

local main_menu = class("main_menu")
local TextEntry = require("main_menu/unicode_text_entry")
local TextEffects = require("utilities/text_effects")
local prefs = require("utilities/prefs")
local dialog = require("gui/dialog")
local text_sizes = require("utilities/text_sizes")

require("utils")
local LuaMainThreaded = require("utilities/LuaMainThreaded")

function main_menu:_show_broadcast()
    if gulp.prefs.inhibit_broadcast then
        self._my_menu._sub_menu:update_menu_string(5, "Allow LAN Broadcasts")
        self._my_menu._sub_menu:update_menu_string(6, "  (currently host silent)")
    else
        self._my_menu._sub_menu:update_menu_string(5, "Inhibit LAN Broadcasts")
        self._my_menu._sub_menu:update_menu_string(6, "  (currently host announces)")
    end
end

function main_menu:_update_toggle_broadcast()
    gulp.prefs.inhibit_broadcast = not gulp.prefs.inhibit_broadcast
    prefs.save()
    self:_show_broadcast()
    return true
end

function main_menu:_show_internet_broadcast()
    if gulp.prefs.allow_internet_broadcast then
        self._my_menu._sub_menu:update_menu_string(7, "Inhibit Internet Broadcasts")
        self._my_menu._sub_menu:update_menu_string(8, "  (currently host announces)")
    else
        self._my_menu._sub_menu:update_menu_string(7, "Allow Internet Broadcasts")
        self._my_menu._sub_menu:update_menu_string(8, "  (currently host silent)")
    end
end

function main_menu:_update_toggle_internet_broadcast()
    gulp.prefs.allow_internet_broadcast = not gulp.prefs.allow_internet_broadcast
    prefs.save()
    self:_show_internet_broadcast()
    return true
end

-- when we want to close the main menu and any nested menus directly 
-- controlled by the main menu.

function main_menu:_update_preference_string(update_menu_location_function, pref_name, max_length, min_length)
    min_length = min_length or 1
    self._name = TextEntry(0, 0, gulp.prefs[pref_name])
    self._name._sub_menu = self._my_menu._sub_menu
    self._name.text_complete = function(self, text)
        gulp.prefs[pref_name] = text
        prefs.save()
        update_menu_location_function(self._sub_menu)
    end
    self._name.character_validation = function(self, sym, text)
        if #text >= max_length then
            return false
        end
        return true
    end
    self._name.final_validation = function(self, text)
        if #text < min_length then
            return false
        end
        return true
    end
    
    return true
end


function main_menu:_update_port_number()
    self._name = TextEntry(0, 0, tostring(gulp.prefs.network_port))
    self._name._sub_menu = self._my_menu._sub_menu
    self._name.text_complete = function(self, text)
        local num = tonumber(text)
        if num then
            gulp.prefs.network_port = num
            prefs.save()
            -- update the name in the original menu
            self._sub_menu:update_menu_string(4, string.format("  (currently %s)", gulp.prefs.network_port))
        end
    end
    self._name.character_validation = function(self, sym, text)
        if #text >= 5 then
            return false
        end
        if sym < string.byte('0') or sym > string.byte('9') then
            return false
        end
        return true
    end
    self._name.final_validation = function(self, text)
        if #text == 0 or #text > 5 or tonumber(text) > 65535 or
            tonumber(text) < 1 then
            return false
        end
        return true
    end
    
    return true
end

local function update_menu_player_name(sub_menu)
    -- update the name in the original menu
    sub_menu:update_menu_string(2, string.format("  (%s)", gulp.prefs.player_name ))
end

local function update_menu_game_name(sub_menu)
    local menu_game_name = gulp.prefs.gameName
    if not gulp.prefs.gameName or gulp.prefs.gameName == "" then
        menu_game_name = "Blank = random name"
    end
    
    sub_menu:update_menu_string(4, string.format("  (%s)", menu_game_name ))
end

function main_menu:_create_settings_menu()
    self._my_menu._sub_menu = Menu{
        column = 1,
        line = 3,
        background = gulp.colour.BLACK.simple_colour, 
        foreground = gulp.colour.YELLOW.simple_colour,
        border_style = 2,
        --{ "Play 'Standard'", function() gulp.start_map_name = 'central_cavern' end },
        --{ "Play 'Coin Rush'", function() gulp.start_map_name = 'coin_rush' end },
        { "Set Player Name", main_menu._update_preference_string, self, update_menu_player_name, "player_name", 15 },
        { "" }, -- player name
        --{ "Set Server Port", main_menu._update_port_number, self },
        --{ string.format("  (currently %d)", gulp.prefs.network_port) },
        --{ "Toggle LAN Broadcast", main_menu._update_toggle_broadcast, self },
        --{ "  (currently ?)" },
        --{ "Toggle Internet Broadcast", main_menu._update_toggle_internet_broadcast, self },
        --{ "  (currently ?)" },
        { "Set Default Game Name", main_menu._update_preference_string, self, update_menu_game_name, "gameName", 25, 0 },
        { "" }, -- game name

    }
    update_menu_player_name(self._my_menu._sub_menu)
    update_menu_game_name(self._my_menu._sub_menu)
    
    if gulp.platform_type == "Desktop" then
        self._my_menu._sub_menu:insert_item{""}
        self._my_menu._sub_menu:insert_item{"Toggle Fullscreen", function() gulp_cpp.ToggleFullscreen() return true end }
        --self._my_menu._sub_menu:insert_item{"Zoom In", function() gulp_cpp.Zoom(8) return true end }
        --self._my_menu._sub_menu:insert_item{"Zoom Out", function() gulp_cpp.Zoom(-8) return true end }
    end
    
    self._my_menu._sub_menu:insert_item{""}
    --self._my_menu._sub_menu:insert_item{ "Editors", main_menu._create_editors_menu, self }
    self._my_menu._sub_menu:insert_item{ "Game Info", main_menu._create_game_info_menu, self }
    self._my_menu._sub_menu:insert_item{"Back", 0}
    
    -- set the correct title
    --self:_show_broadcast()
    
    -- Internet tracker disable by default
    --self:_show_internet_broadcast()
    
    return true
end


function main_menu:_create_editors_menu()
    self._editor_menu = Menu{
        column = 1,
        border_style = 2,
        { "\x1b[37mCommands :- " },
        { "\x1b[37m\x1b[1m  Graphics Editor", gulp.launch_glyph_editor },
        { "\x1b[37m  Map Editor", },
    }
    
    return true
end



function main_menu:_create_game_info_menu()
    -- renderer test code
    local num_r = gulp_cpp.RendererInfo.number_renderers
    local current_renderer = gulp_cpp.RendererInfo(gulp.app:get_SDL_renderer(), 0)
    
    local m = {
        border_style = 2,
        { string.format("Game Ver: %s", gulp.forlorn_fox_version) },
        { string.format("Engine ver: %s", gulp.forlorn_fox_engine_version) }, 
        { string.format("Platform: %s", gulp.platform) },
        { "" },
        { string.format("Number renderers %i", num_r) },
        { string.format("Current: %s (%s)", current_renderer.name, render_flag_decode(current_renderer)) },
        { "" },
        { "Other Renderers:" },
    }
    for i = 0, num_r-1 do
        local rend = gulp_cpp.RendererInfo(nil, i)
        table.insert(m, { string.format("Renderer %i: %s (%s)", i, rend.name, render_flag_decode(rend)) })
    end
    table.insert(m, {"Back", 0})
    
    self._game_info = Menu(m)
    return true
end


function main_menu:_do_credits()
    self._credits = credits:new()
end

local avatar_menu = require("avatars/avatar_menu")


function main_menu:_start_hosted_game(game)
    
    -- todo
    -- need to launch the server here in a different lua state
    
    local server_info = { qcommand = "start", start_map_name= gulp.start_map_name }
    
    server_info.LAN_broadcast = not gulp.prefs.inhibit_broadcast
    server_info.internet_broadcast = gulp.prefs.allow_internet_broadcast
    server_info.port = gulp.prefs.network_port + game
    server_info.enableKeepAlive = gulp.enableKeepAlive
    
    gulp.server_queue[game]:send(server_info)
    
    -- this is used to cause local client to connect to this server
    -- not an issue in our new server/client model but leave in place, this
    -- would be required for any potential 'offline' mode
    gulp.enable_client_networking = false
    
    -- this is used to shut down a local server at game end if necessary
    -- again not really relevant now, but might be wanted for offline mode
    gulp.local_server_started = true
        
    -- todo
    -- Add a new menu here that shows that the server is running, 
    -- and allows the player to join their own server game if they choose
    -- or just watch the game

    -- launch a player
    -- avatar_menu()
    
    return true -- keep original menu open (i.e. nested menu)
end

local select_game_to_join_menu = require("main_menu/select_game_to_join")

function main_menu:_select_game_to_join_menu()
    self._my_menu._sub_menu = select_game_to_join_menu()
    return true
end

local backgrounds = {
    require("main_menu/backgrounds/falling_letters"),
    require("main_menu/backgrounds/falling_letters"),
    --require("main_menu/backgrounds/maze_title")
}

local updater_instance = require("updater")

function main_menu:_update_download_menu(fraction, complete, size)
    if fraction < 0 then fraction = 0 end
    if fraction > 1 then fraction = 1 end
    local n = math.floor(20*fraction)
    local m = 20-n
    local new_string = string.format("[%s%s]", string.rep("#", n), string.rep("-", m))
    self._download_menu:update_menu_string(2, new_string)
    self._download_menu:update_menu_string(3, string.format("%3d%%    %d bytes", math.floor(fraction*100), size))
    if complete then
        self._download_menu:close()
        self._update_menu:close()
    end
end

local exec_functions = require("utilities/exec_functions")

function main_menu:_create_download_menu()
    self._download_menu = Menu{
        column = 1,
        border_style = 2,
        { string.format("Downloading %s", updater_instance:new_version()) },
        { "" },
        { "" },
        { "\x1b[37m\x1b[1mCancel", 0 },
    }
    self._download_menu._download_in_progress = true
    
    self._download_menu.on_close = function(self) 
        if self._download_in_progress then updater_instance:stop_download_file() end 
        end
    
    self._download_menu.on_update = function(menu, dt) 
        local progress, complete, size = updater_instance:get_progress() 
        if complete and updater_instance:successful_download() then
            if updater_instance:check_file() then
                local success = updater_instance:unpack_file()
                if success then
                    
                    local success = updater_instance:write_run_file()
                    if success then
                        exec_functions.execute_run_alt()
                    end
                    
                    --success = updater_instance:run_new_application()
                    if not success then
                        dialog("Failed to run version")
                    end
                else
                    dialog("Failed to unpack")
                end
            else
                dialog("Bad file")
            end
        end
        self:_update_download_menu(progress, complete, size) 
    end
    
    self:_update_download_menu(0, false, 0)
    updater_instance:download_file()
    return true
end

function main_menu:_create_app_update_menu()
    self._update_menu = Menu{
        column = 1,
        border_style = 2,
        { string.format("\x1b[37mVersion: %s", updater_instance:new_version()) },
        { string.format("\x1b[37mSize: %.1f MB", updater_instance:expected_file_size()/1024/1024) },
        { "\x1b[37m\x1b[1mDownload", main_menu._create_download_menu, self },
        { "\x1b[37m\x1b[1mCancel", 0 },
    }
    
    return true
end

local function multicolour(str, index)
    local i = (index % 7)+31
    
    local t = {}
    for c in str:gmatch "." do
        table.insert(t, string.format("\x1b[%im%s", i, c))
        i = i + 1
        if i > 37 then
            i = 31
        end
    end
    return table.concat(t)
end


-- check for server connection every 4 seconds
local registration_check_time = 4

local version_line = 5
local play_line = 2
local welcome_line = 1

function main_menu:check_for_updates(dt, m)
    
    if(not self.weve_added_menu and updater_instance:new_version_available()) then
        -- add menu in for updates
        self.update_index = 0
        m:insert_item(version_line, { multicolour("Get new version!", self.update_index), self._create_app_update_menu, self})
        self.weve_added_menu = true
        self.update_timer = 0
    elseif self.weve_added_menu then
        self.update_timer = self.update_timer + dt
        if self.update_timer > 0.1 then
            self.update_timer = 0;
            self.update_index = self.update_index + 1
            m:update_menu_string(version_line, multicolour("Get new version!", self.update_index))
        end
    end
    
    if not gulp.server then
        if gulp.ControlID == nil then
            m:update_menu_string(play_line, "Connecting...")
            registration_check_time = registration_check_time - dt
            if registration_check_time < 0 then
                registration_check_time = registration_check_time + 4
                
                if gulp_control_networking.clientSockets[1] == nil then
                    gulp_control_networking:enableClient(gulp_control_networking.target_IPAddr, gulp_control_networking.target_port, 'control')
                else
                    gulp_control_networking:SendToServer({ mt = 'register_for_control', })
                end
            end
        else
            m:update_menu_string(play_line, "Play the game")
        end
    end
    
    m:update_menu_string(welcome_line, "Welcome, "..gulp.prefs.player_name)
end

function main_menu:_create_main_menu()
    
    -- select a specific background
    -- need to create before the menu, or else the background goes in front of the menu...
    local background = random_entry(backgrounds):new()
    
    local function get_text_sizes()
        local l = -18
        local c = 33
        if gulp.platform_type == "Mobile" then
            l = 0
            c = 15
        end
        return text_sizes:new(l, c)
    end
    
    local m = Menu{
        background = gulp.colour.BLACK.simple_colour, 
        foreground = gulp.colour.WHITE.simple_colour,
        select_colour = gulp.colour.MAGENTA.simple_colour,
        border_style = 2,
        
        line = (gulp.platform_type == "Mobile") and 7 or 23.5,
        c_centralised = true,
        text_sizes = get_text_sizes(),
        
        { "Welcome, "..gulp.prefs.player_name },
        { "Settings", main_menu._create_settings_menu, self },
        { "Credits", main_menu._do_credits, self }, 
    }
    
    
    
    if gulp.server == true then
        local function add_game()
--[[            
            local new_game_index = #gulp.lua_game_world + 1
            
            gulp.ui_queue[new_game_index] = gulp_cpp.LuaStateQueue("ui_queue_"..new_game_index)
            gulp.server_queue[new_game_index] = gulp_cpp.LuaStateQueue("server_queue_"..new_game_index)
            gulp.lua_game_world[new_game_index] = LuaMainThreaded("scripts/game_world_server/server_main.lua")
            gulp.lua_game_world[new_game_index]:run("server_load", gulp.ui_queue[new_game_index], gulp.server_queue[new_game_index])
            gulp.lua_game_world[new_game_index]:run_in_thread("server_thread")
--]]            
            
            gulp.create_new_game("From server menu")
            
            --self:_start_hosted_game("host", new_game_index)
            
            m:update_menu_string(2, " Currently serving "..#gulp.lua_game_world.." games")
            
            -- keep the menu open
            return true
        end
        --m:insert_item(1, { "Add new game", add_game })
        m:insert_item(1, { " Currently serving "..#gulp.lua_game_world.." games"})
    else
        local function register_for_control()
            if gulp.ControlID == nil then
                local message = { mt = 'register_for_control', }
                gulp_control_networking:SendToServer(message)
            else
                print("Already registered with ControlID: "..gulp.ControlID)
            end
            return true     -- keep menu open
        end
        local function request_game()
            if gulp.ControlID == nil then return true end
            local message = { mt = 'request_game', control_id=gulp.ControlID, game_name = gulp.prefs.gameName}
            gulp_control_networking:SendToServer(message)
            return true     -- keep menu open
        end
        
        local function test_message()
            local message = { mt = 'test_message', text = "Tony's test message", }
            gulp_control_networking:SendToServer(message)
            return true     -- keep menu open
        end
        
        local function get_game_list()
            local message = { mt = 'get_game_list' }
            gulp_control_networking:SendToServer(message)
            return true     -- keep menu open
        end
        
        -- m:insert_item(1, { "Register with server", register_for_control })
        m:insert_item(play_line, { "Play the game", request_game })
        --m:insert_item(2, { "Join game", main_menu._select_game_to_join_menu, self})
        --m:insert_item(3, { "Test message", test_message })
        --m:insert_item(4, { "Get game list", get_game_list })
    end
    
    if gulp.platform_type ~= "Mobile" then
        m:insert_item{ "Quit game", function() gulp.app:quit_main_loop() end }
    end
        
    
    m.on_update = function(menu, dt) self:check_for_updates(dt, menu) end
    self:check_for_updates(0, m)
    
    -- assign the background to this menu so it can be destroyed on menu destruction
    m.background = background
    
    m.keypressed = function(self, sym)
        if back_key(sym) then
            if gulp.platform_type ~= "Mobile" then
                -- for now quit the app for android too
                -- it would be better to background the app, but I don't know how right now
                gulp.app:quit_main_loop()
            end
            return true
        else
            return Menu.keypressed(self, sym)
        end
    end
    
    m.on_close = function(self)
        -- @todo: check if more than one sub_menu could exist?
        if self._sub_menu then
            self._sub_menu:close()
        end
        
        self._version_text:hide()
        self.background:close()
        
        gulp.main_menu = nil
    end
    
    
    -- version text
    m._version_text = TextEffects:new(text_sizes:lines()/2, text_sizes:window_end_column()-1, "v"..gulp.forlorn_fox_version )
    --self._version_text:small()
    m._version_text:narrow()
    m._version_text:colour("RED")
    m._version_text:rotate(90)
    m._version_text:down()

    m._version_text.resize = function(self)
        m._version_text:move(text_sizes:lines()/2, text_sizes:window_end_column()-1)
    end

    return m
end

function main_menu:initialize(title)
    -- check the game is valid
    if gulp.main_menu ~= nil then
        debugMessage("ERROR: Called main_menu:initialize with gulp.main_menu already set?!?")
    end
    
    self._my_menu = self:_create_main_menu(title)

end


function main_menu:close()
    if self._my_menu then
        self._my_menu:close()
        self._my_menu = nil
    end
end

return main_menu
