-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
-- Handle internet games
--
-- Created by Kenny Osborne & Rob Probin, September 2015
-- For Forlorn Fox

-- configuration parameters

-- dependancies

require("strict")
local class = require("middleclass")
local copas = require("copas")
local lunajson = require ("thirdparty/json/lunajson")
module = false  -- work around to make ltn12.lua work properly under Lua 5.2 and strict.lua

--local synchttp = require("socket.http").request
local asynchttp = require("copas.http").request

-- load url module
local url = require("socket.url")

local manage_game_list = require("networking/manage_game_list")

local onlineServerAddress = "http://forlorn.wubbleu.co.uk/php"
local getGameListScript = "/getRegisteredServers.php"
local updateIntervalDelay = 2 -- 5 seconds
local updateListEnabled = false
local gamesListMenu     -- assume there is only one


-- Copas thread that will pop the message and send over the network    
local function getGamesListThread()
    while updateListEnabled == true do
        local addressString = string.format("%s%s", onlineServerAddress, getGameListScript)

        -- The actual request to the server
        print("**Fetching games from Internet server")
        local res, err = asynchttp(addressString)

        -- If we got data from the server
        if res then
            local active_games_list = lunajson.decode(res)

            --print("Number of active games: ", #active_games_list) 
            local game_list = {}
            for _, game in ipairs(active_games_list) do
                    
                local t = {}
                -- copy only specific ones across
                t.game_id = game["game_id"]
                t.game_name = game["game_name"]
                t.game_version = game["game_version"]
                t.game_ip_address = game["game_ip_address"]
                t.game_port = game["game_port"]

                --print("Game ID: ", t.game_id) 
                --print("Game Name: ", t.game_name)
                --print("Game Version: ", t.game_version)
                --print("Game IP Address: ", t.game_ip_address)
                --print("Game Port: ", t.game_port)
                
                t.received_last_update = gulp.time
                
                table.insert(game_list, t)
            end

            gamesListMenu._list_of_games = game_list
        end
        copas.sleep(updateIntervalDelay)
    end
    --print("Left thread")
end  

-- Function to add message to the queue for sending
local function startGetGamesList()
    -- Start thread for sending messages
    updateListEnabled = true
    local mythread = copas.addthread(getGamesListThread);
    name_network_item(mythread, "addthread - getGamesListThread")
end

-- Function to add message to the queue for sending
local function stopGetGamesList()
    updateListEnabled = false
    -- Start thread for sending messages
    --copas.deletethread(getGamesListThread);
end

local function get_servers_list()
    return gamesListMenu._list_of_games
end

local connect_function_copy

local function local_connect(...)
    gamesListMenu:close()
    connect_function_copy(...)
end


local function start_public_internet_games_menu(connect_function)
    
    connect_function_copy = connect_function
    
    gamesListMenu = Menu
    {
        line = 3,
        column = 1,
        background = gulp.colour.RED.simple_colour, 
        foreground = gulp.colour.BRIGHT_WHITE.simple_colour,
        border_style = 2,
        {" Back " , 4},
        { "\x1b[36m----------INTERNET GAMES:" },
    }

    -- Start thread for getting update
    startGetGamesList()

    gamesListMenu.on_close = function()
        stopGetGamesList()
        gamesListMenu._list_manager:terminate() -- kill the list manager
        gamesListMenu = nil
    end
    
    gamesListMenu._list_of_games = {}
    gamesListMenu._list_manager = manage_game_list:new(gamesListMenu, local_connect, get_servers_list)
    
    return true
end


return start_public_internet_games_menu

