-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
-- Select Game to Join Menu
--
-- Created by Rob Probin, January 2015
-- For Forlorn Fox


local TextEntry = require("main_menu/unicode_text_entry")

local start_public_internet_games_menu = require("main_menu/public_internet_games")

require("gulp_networking")

local manage_lan_listener = require("networking/manage_lan_listener")
local avatar_menu = require("avatars/avatar_menu")

function validatePort(port)
    if #port == 0 or #port > 5 or tonumber(port) > 65535 or
        tonumber(port) < 1 then
        return false
    end
    return true
end   

function validateIPV4Address(address)
    -- Extract the ip parts
    local ipParts = {address:match("(%d+)%.(%d+)%.(%d+)%.(%d+)")}
    if ipParts ~= nil then
        if (#ipParts == 4) then
            for _,v in pairs(ipParts) do
                if (tonumber(v) < 0 or tonumber(v) > 255) then
                    return false
                end
            end
            return true
        end
    end
    return false
end    

function validateIPV6Address(address)
    -- Simple look for :
    if string.match(address, ":") then
        return true
    else
        return false
    end
    
   --[[
   local ipParts = 0;
        
    -- Try and strip the brackets off
    local stripped_brackets = address:match("%[(.-)%]")
    
    -- Were square brackets found and a string extracted then use that to verify it is ipv6 else 
    -- test the string passed in
    if stripped_brackets then
        ipParts = {stripped_brackets:match(("([a-fA-F0-9]*):"):rep(8):gsub(":$","$"))}
    else
        ipParts = {address:match(("([a-fA-F0-9]*):"):rep(8):gsub(":$","$"))}
    end
    
    if #ipParts == 8 then
        for _,v in pairs(ipParts) do
            if #v > 0 and tonumber(v, 16) > 65535 then 
                return false
            end
        end
        return true
    end
    return false
    --]]
end    

    
function validateHostname(address) 
    --RFC 1123 for hostname 
    if string.match(address, "^%w[%w%.-]+%w$") then
        return true
    end
        return false
end

function extractAndValidateAddress(text) 
    if text == nil then 
        return false, nil, nil
    end  
    
    -- Check for the number of :'s
    local _, count = string.gsub(text, ":", "")
    
    local address, port
    
    if count < 2 then
        -- ipv4 or hostname so split the address and port out
        address, port = text:match("([^,]+):([^,]+)")    
    else
        -- ipv6
        address, port = text:match("%[([^,]+)%]:([^,]+)")    
    end   
   
    -- Check there was an address entered
    if address == nil then
        address = text
    end   
    
    -- Check a port was entered, otherwise use the default prefs one
    if port == nil then
        port = tostring(gulp.prefs.network_port)
    end   
        
    -- If vailidate port number
    if port == nil or true == validatePort(port) then
        -- Check the address is a string first then check for IPV4 or IPV6
        if type(address) == "string" then        
        
            -- Check if IPV4 address
            if true == validateIPV4Address(address) then 
                return true, address, port 
            end   

            -- Check if IPV6 address
            if true == validateIPV6Address(address) then 
                return true, address, port 
            end   

            -- Check if hostname 
            if true == validateHostname(address) then 
                return true, address, port 
            end   
        end    
    end    
    return false, nil, nil
end


local function directConnectToServer(connect_to)
    local serverAddress = TextEntry(0, 0, tostring(""))   
    serverAddress.text_complete = function(self, text)
        local result, ip, port = extractAndValidateAddress(text) 
        if result == true then
            connect_to(ip, port)
        else
            local invaildAddress = Menu
            {
                line = 3,
                column = 1,
                background = gulp.colour.RED.simple_colour, 
                foreground = gulp.colour.BRIGHT_WHITE.simple_colour,
                border_style = 2,
                {"Invalid Address"},
                {"               "},
                {"      OK       ", 99 }
            }
        end
    
    end
    return true
end

---[[
local function test_address_validation()
    local address_test_cases = {
                            {"127.0.0.1", "127.0.0.1", "10000", true},                                    -- The "localhost" IPv4 address
                            {"127.0.0.1:80", "127.0.0.1", "80", true},                                 -- The "localhost" IPv4 address, with a specified port (80)
                            {"123:5::1", "123:5::1", "10000", true},                                          -- The "localhost" IPv6 address
                            {"::1", "::1", "10000", true},                                          -- The "localhost" IPv6 address
                            {"[::1]:80", "::1", "80", true},                                     --  The "localhost" IPv6 address, with a specified port (80)
                            {"2605:2700:0:3::4713:93e3", "2605:2700:0:3::4713:93e3", "10000", true},                     -- Rosetta Code's primary server's public IPv6 address
                            {"[2605:2700:0:3::4713:93e3]:80", "2605:2700:0:3::4713:93e3", "80", true},               -- Rosetta Code's primary server's public IPv6 address, +port (80)
                            {"2001:db8:85a3:0:0:8a2e:370:7334", "2001:db8:85a3:0:0:8a2e:370:7334", "10000", true},              -- doc, IPv6 for 555-1234
                            {"2001:db8:85a3::8a2e:370:7334", "2001:db8:85a3::8a2e:370:7334", "10000", true},                -- doc
                            {"[2001:db8:85a3:8d3:1319:8a2e:370:7348]:443", "2001:db8:85a3:8d3:1319:8a2e:370:7348", "443", true},   -- doc +port
                            {"::ffff:192.168.0.1", "::ffff:192.168.0.1", "10000", true},                          -- private transitional
                            {"[::ffff:71.19.147.227]:80", "::ffff:71.19.147.227", "80", true},                   -- Rosetta Code's transitional +port
                          --  {"::", "::", nil, false},                                           -- invalid, octet > 255 (currently not detected)
                           -- {"256.0.0.0", nil, "10000", false},                                    -- invalid
                          --  {"g::1", nil, nil, false},                                         -- Bad address
                           -- {"0000", "0000", nil, false},                                         -- Bad address                                                                   
                          --  {"0000:0000", "0000:0000", nil, false},                                    -- Bad address
                            {"0000:0000:0000:0000:0000:0000:0000:0000", "0000:0000:0000:0000:0000:0000:0000:0000", "10000", true},      -- Good address
                            {"0000:0000:0000::0000:0000", "0000:0000:0000::0000:0000", "10000", true},                    -- Good address          
                          --  {"0000::0000::0000:0000", "0000::0000::0000:0000", nil, false},                        -- Bad address              
                            {"ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", "10000", true},      -- Good address
                          --  {"ffff:ffff:ffff:fffg:ffff:ffff:ffff:ffff", "ffff:ffff:ffff:fffg:ffff:ffff:ffff:ffff", nil, false},      -- Bad address
                            {"fff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", "fff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", "10000", true},       -- Good address
                            {"fff:ffff:0:ffff:ffff:ffff:ffff:ffff", "fff:ffff:0:ffff:ffff:ffff:ffff:ffff", "10000", true}}          -- Good address
                      
    local temp_port = gulp.prefs.network_port
    gulp.prefs.network_port = 10000

    for k, v in pairs(address_test_cases) do
        local input = v[1]
        local expected_address = v[2]
        local expected_port = v[3]
        local expected_result = v[4]
        
        local result, address, port =  extractAndValidateAddress(input) 
        
        -- Check result
        if expected_result ~= result or expected_address ~= address or expected_port ~= port then
            print("Validation test failed", result, address, port)
            print("expected", expected_address, expected_port)
            error("Address and port valiadation failed")
        end    
    end
    gulp.prefs.network_port = temp_port
end
--]]


local function connect_to(ip, port)
    gulp.target_IPAddr = ip
    gulp.target_port = port          -- @todo fix horrible hack that's going to break the prefs at some point!
    
    gulp.enable_client_networking = true
    
    avatar_menu()
    return true -- keep original menu open (i.e. nested menu)
end

local function select_game_to_join()
    local sub_menu = Menu{
        column = 0,
        line = 0,
        background = gulp.colour.BLACK.simple_colour, 
        foreground = gulp.colour.MAGENTA.simple_colour,
        border_style = 2,
        { "Direct Connect", directConnectToServer, connect_to},
        { "Public Internet Games", start_public_internet_games_menu, connect_to},
        { "\x1b[34mAdd Server" },
        { "\x1b[34mEdit" },
        { "\x1b[34mDelete" },
        { "Cancel", 0 },
        --{ "Refresh", 0 }, 
        { "\x1b[34m----------ADDED GAMES:" },
        -- Pre-added game addresses
        { "----------LAN GAMES:" }
        -- LAN broadcast games
    }
    
  --  sub_menu.add_old_style = function(self, func, obj, param)
  --      self:insert_item(1, { "Old-style Connect", func, obj, param })
  --  end
    
    sub_menu.listener = manage_lan_listener:new(sub_menu, connect_to)
    -- provide a reference back to the menu itself
    sub_menu.listener.sub_menu = sub_menu
    
    
    sub_menu.on_close = function(self)
        self.listener:terminate()
        self.listener = nil
    end


    return sub_menu
end

-- Run the test cases
test_address_validation()

return select_game_to_join

