-- Forlorn Fox main file - for running a headless server application
-- Copyright © 2014-2020 Rob Probin & Tony Park
-- No redistribution without permission.

-- temporary hack
if not gulp.nogui then
    require('main_client_ui_thread')
    return
end

local file_utils = require("utilities/file_utils")

local LuaMainThreaded = require("utilities/LuaMainThreaded")

local prefs = require("utilities/prefs")


function gulp.load(app)
    gulp.time = 0
    gulp.app = app
        
    file_utils.create_save_folders()
    prefs.load()
    
        
    main_common.decode_command_line_arguments()
    
    gulp.ui_queue = {}
    gulp.server_queue = {}
    gulp.lua_game_world = {}
    
    -- create a special 'controller' server
    -- this will be connected to by all clients and will allow them to request an actual game server be started
    gulp.ui_queue['control'] = gulp_cpp.LuaStateQueue("ui_queue_control")
    gulp.server_queue['control'] = gulp_cpp.LuaStateQueue("server_queue_control")
    gulp.controller = LuaMainThreaded("scripts/game_world_server/controller_server_main.lua")
    gulp.controller:run("server_load", gulp.ui_queue['control'], gulp.server_queue['control'])
    gulp.controller:run_in_thread("controller_server_thread")

    -- automatically start the controller server
    local server_info = { qcommand = "start" }
    
    server_info.port = gulp.prefs.network_port
    server_info.enableKeepAlive = gulp.enableKeepAlive
    
    gulp.server_queue['control']:send(server_info)
end



local function handle_server_couldnt_start(message)
    print("Couldn't start server")
    print("Port already in use!")
    print("Is another game running?")
    if type(message)=='string' then print("message: " .. message) end
    print()
end

local function handle_internet_broadcast_now_registered()
    print("Internet broadcast now registered")
end
local function handle_internet_broadcast_error()
    print("Internet broadcast Error")
end
local function handle_local_network_packet_ui(message)
    print("Local network packet to non-existant local client - why is this happening?")
end

--
-- Queue command dispatch table
-- 
local ui_function_dispatch = {
    qlocal = handle_local_network_packet_ui,
    internet_broadcast_error = handle_internet_broadcast_error,
    internet_broadcast_now_registered = handle_internet_broadcast_now_registered,
    server_couldnt_start = handle_server_couldnt_start,
    request_game = gulp.HandleRequestGame,
}




local server_dt_update = { qcommand = "update", dt = 0 }

function gulp.update(dt)

    server_dt_update.dt = dt
    for _, q in pairs(gulp.server_queue) do       -- changed from ipairs to pairs. Order is not specific, and it means we can take them out the list.
        q:send(server_dt_update)
    end
    
    gulp.time = gulp.time + dt

    local function dispatch_queue(q)
        while not q:is_empty() do
            local message = q:read()
            --print("UI:", message)
            local f = ui_function_dispatch[message.qcommand]
            if f then
                f(message)
            else
                error("unknown command in gulp.update() " .. tostring(f))
            end
        end
    end
    
    for _, q in ipairs(gulp.ui_queue) do
        dispatch_queue(q)
    end
    
    if gulp.server then
        dispatch_queue(gulp.ui_queue['control'])
    end
end


-- no gulp.draw(gr) if no gui??? 




