-- Forlorn Fox
-- Copyright © 2020 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
-- Caves of Chaos combat menu
--


require("strict")
local class = require("middleclass")
local CombatMenu = class("CombatMenu")
local text_sizes = require("utilities/text_sizes")

local meter = require("controls/meter")

function CombatMenu:initialize(manager_name, message)
    self.manager = manager_name
    self.combat_id = message.combat_id
    
    -- set up data for message box
    self.messages_first_line = 12
    self.messages_last_line = 19
    self.messages = {}
    for line = self.messages_first_line, self.messages_last_line do
        self.messages[line] = ""
    end
    
    self:create_menu(message)
    
    self.menu:update_field('mob_name', message.mob_name)
    self.menu:update_field('player_name', gulp.localplayer.avatar_data.avatar_name)
    self.menu:update_field('mob_hp', message.mob_hp)
    self.menu:update_field('player_hp', message.player_hp)
    
    gulp.localplayer:hold()
end

function CombatMenu:create_menu(message)
    
    self.menu=Menu
    {
        background = gulp.colour.WHITE.simple_colour, 
        foreground = gulp.colour.BLACK.simple_colour,
        select_colour = gulp.colour.MAGENTA.simple_colour,
        border_style = 2,
        
        centralised = true,
        
        text_sizes = text_sizes:new(42, 0),
        
        { "" },
        { "           _________ _________           " },        -- 41 across
        { "           HP:          HP:" },
        { "" },
        { "" },
        { "" },
        { "" },
        { "" },
        { "" },
        { "" },
        { "" },
        { "" },     -- first line of message box, line 12
        { "" },
        { "" },
        { "" },
        { "" },
        { "" },
        { "" },     
        { "" },     -- last line of message box, line 19
        --{ "                                         " },         -- 41 across
        --{ { "Fight", CombatMenu.fight, self  }, { "Heal", CombatMenu.heal, self }, { "Run", CombatMenu.run, self }, },
        
        -- display the monster (and character??) illustration(s)
        graphics = 
        {
        --    l,  c, glyph                                                   w,  h,  rot, ratio, fg_colour  
        --  { 1,  1, get_glyph("monsters", 0),                              10, 10,    0,   1.0, gulp.colour.WHITE.simple_colour },
            { 1,  1, get_glyph("monsters", 0),                              10, 10,    0,   1.0, gulp.colour.WHITE.simple_colour, message.graphics_bg },
            { 1, 32, gulp.localplayer.avatar_data.graphic_unicode_point,     1,  1,    0,  10.0, gulp.colour.WHITE.simple_colour, message.graphics_bg },
        },
        
        fields = 
        {
        --   item                l,  c,  w, def,just 
            ['mob_name']    = {  2, 11,  9, ""},
            ['player_name'] = {  2, 21,  9, "", 'right' },
            ['mob_hp']      = {  3, 15,  2, ""},
            ['player_hp']   = {  3, 28,  2, ""},
        },
    }
    
    self.menu.on_update = function(menu, dt)
        if menu.close_delay ~= nil then
            menu.close_delay = menu.close_delay - dt
            if menu.close_delay < 0 then
                Menu.close(menu)
            end
        end
    end
    
    self.menu.on_close = function(menu)
        gulp.localplayer:release()
        Menu.close(self.sub_menu)
    end
    
    local meter_data={
        running = true,
        initial_time = message.mob_meter_time, 
        max_time = 6,           -- this needs to match the mobs... should get some of this data from mob
        width = 5, 
        graduations = 3, 
        direction = 'to_right',
        bar_colour = get_SDL_Color_by_name('RED'),
        grad_colour = get_SDL_Color_by_name('BLACK'),
    }
    self.mob_meter = meter(meter_data, self.menu, 4, 12)
    
    meter_data={
        running = true,
        initial_time = 2, 
        max_time = 6, 
        width = 5, 
        graduations = 3, 
        direction = 'to_left',
        bar_colour = get_SDL_Color_by_name('GREEN'),
        grad_colour = get_SDL_Color_by_name('RED'),
    }
    self.combat_meter = meter(meter_data, self.menu, 4, 24)

    self.sub_menu = Menu
    {
        background = gulp.colour.WHITE.simple_colour, 
        foreground = gulp.colour.BLACK.simple_colour,
        select_colour = gulp.colour.MAGENTA.simple_colour,
        border_style = 2,
        
        owner = self.menu,
        l_offset = 6,
        c_centralised = true,
        
        text_sizes = text_sizes:new(42, 0),
        { "Fight", CombatMenu.fight, self }, 
        { "Heal", CombatMenu.heal, self }, 
        { "Run", CombatMenu.run, self },
    }

    self.sub_menu.on_escape = function(menu)
        -- 'do you want to run away?' menu
        CombatMenu.run(self)
        Menu.close(menu)
    end

end    

function CombatMenu:mob_attack(message)
    self.mob_meter.time = message.meter_time
    self:show_message(message.description)
    
    if message.target then
        -- subtract the damage from the player's hp
        -- hmm this should probably be a separate 'player hp' message
    end
end

function CombatMenu:run()
    
    -- attack of opportunity for the mob?
    
    if self.dead then return true end
    
    local message = { 
        mt = "game_control",
        manager = self.manager,
        combat_id = self.combat_id,
        player_id = gulp.localplayer.id,
        command = "caves_combat_run",
    }
   
    gulp_networking:SendToServer(message)
    
    -- close the outer combat menu
    Menu.close(self.menu)
    
    -- don't close the submenu here because the outer menu close closes it anyway
    return true
end

function CombatMenu:heal()
    
    if self.dead then return true end
    
    if self.combat_meter.time < 2 then return true end
    self.combat_meter.time = self.combat_meter.time - 2
    
    -- if the character has the right items, let them heal
    
    -- keep combat menu open
    return true
end

function CombatMenu:fight()
    
    if self.dead then return true end
    
    if self.combat_meter.time < 2 then return true end
    self.combat_meter.time = self.combat_meter.time - 2
    
    -- take a swing at the enemy
    
    local message = { 
        mt = "game_control",
        manager = self.manager,
        combat_id = self.combat_id,
        player_id = gulp.localplayer.id,
        command = "caves_combat_fight",
    }
   
    gulp_networking:SendToServer(message)
    
    -- keep combat menu open
    return true
end

function CombatMenu:show_message(message)
    if message == nil then print("nil message!!") end
    
    for line = self.messages_first_line, self.messages_last_line - 1 do
        self.messages[line] = self.messages[line+1]
        self.menu:update_menu_string(line, self.messages[line])
    end
    self.messages[self.messages_last_line] = message
    self.menu:update_menu_string(self.messages_last_line, message)
end

function CombatMenu:won_combat()
    -- close the menu after a short delay
    self.menu.close_delay = 4
end

function CombatMenu:mob_hp_update(hp)
    self.menu:update_field('mob_hp', hp)
end

function CombatMenu:player_hp_update(hp)
    self.menu:update_field('player_hp', hp)
    
    if hp == 0 then
        -- OMG we died
        -- all we need to do here is close the menu after an appropriate delay
        self.dead = true
        self.menu.close_delay = 4
    end
        
end


return CombatMenu