-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
class MazeEditor : public SpecialStates
{
public:
    virtual void print(MyGraphics& gr);
    MazeEditor();
    virtual bool mouse_down(int x, int y);
    virtual bool mouse_down_moved(int x, int y);
    virtual bool key_down(const SDL_KeyboardEvent& key);
    virtual void update(double dt, MySoundManager& sound, GameApplication& app);
protected:
    void toggle_block();
    void set_block(char c);
    void save();
private:
    //char maze_ptrs[edit_height+1];
    char maze[edit_width+1][edit_height];
    int line;
    int column;
    //PresentationMaze z;
    bool flashing_var;
    RepeatingTimer flash_timer;
    char last_char_written;
    int flash_save;
};



#include "MazeEditor.h"
#include "GameApplication.h"
#include "GameToScreenMapping.h"
#include <sstream>
#include <fstream>
#ifdef _MSC_VER
#include <ciso646>   // Visual Studio is not C++ standards complaint...
#endif
#include "SaveDataPathDeveloper.h"

void MazeEditor::print(MyGraphics& gr)
{
    for(int h = 0; h < edit_height; h++)
    {
        gr.go_to(h+1, 0);
        for(int w = 0; w < edit_width; w++)
        {
            char c = maze[w][h];
            simple_colour_t fg = WHITE;
            simple_colour_t bg = BLACK;
            if(w == column && h == line && flashing_var)
            {
                bg = MAGENTA;
            }
            if(c == '#')
            {
                gr.print(fg, bg, ' ');
            }
            else if(c == ' ')
            {
                gr.print(bg, fg, apple_graphic);
            }
            else
            {
                gr.print(bg, fg, c);
            }
        }
    }
    gr.set_fg_colour(BLACK);
    gr.set_bg_colour(WHITE);
    gr.go_to(edit_height+1, 0);
    print_string(gr, "WASD+Space, Touch/Mouse     ");

    gr.go_to(edit_height+2, 16);
    print_string(gr, "            ");

    gr.go_to(edit_height+2, 0);
    std::stringstream s;
    s << "line = " << line+1 << "  Column = " << column+1;
    print_string(gr, s.str());

    if(flash_save)
    {
        gr.set_fg_colour(BLACK);
        gr.set_bg_colour(BRIGHT_GREEN);
    }
    gr.go_to(edit_height+1, 28);
    print_string(gr, "SAVE");
    gr.go_to(edit_height+2, 28);
    print_string(gr, "SAVE");

}

MazeEditor::MazeEditor()
: line(0)
, column(0)
, flashing_var(false)
, last_char_written('+')
, flash_save(0)
{
    flash_timer.start_time(ZX_Spectrum_Flash_Rate_in_ms);

    // fill all with space to start with
    for(int h = 0; h < edit_height; h++)
    {
        for(int w = 0; w < edit_width; w++)
        {
            maze[w][h] = ' ';
        }
    }
    // top line and botton line wall
    for(int w = 0; w< edit_width; w++)
    {
        maze[w][0] = '#';
        maze[w][edit_height-1] = '#';
    }
    // left and right wall
    for(int h = 0; h < edit_height; h++)
    {
        maze[0][h] = '#';
        maze[edit_width-1][h] = '#';
    }
    // put the zero terminators in the end
    for(int h = 0; h < edit_height; h++)
    {
        maze[edit_width][h] = 0;
    }
    // fill the pointers
    for(int h = 0; h < edit_height; h++)
    {
    //  maze_ptrs[h] = &(maze[h]);
    }

}

void MazeEditor::toggle_block()
{
    if(column >= 0 && column < edit_width && line >= 0 && line < edit_height)
    {
        if(maze[column][line] == ' ')
        {
            last_char_written = '#';
        }
        else
        {
            last_char_written = ' ';
        }
        maze[column][line] = last_char_written;
    }
}

void MazeEditor::set_block(char c)
{
    if(column >= 0 && column < edit_width && line >= 0 && line < edit_height)
    {
        maze[column][line] = c;
    }
}


bool MazeEditor::mouse_down(int x, int y)
{
    line = static_cast<int>(y_to_line(y));
    column = static_cast<int>(x_to_column(x));
    line--;
    if((line == 21 || line == 22) && column >= 28)
    {
        flash_save = 6;
        save();
    }
    else
    {
        toggle_block();
    }

    return true;
}

bool MazeEditor::mouse_down_moved(int x, int y)
{
    int line2 = static_cast<int>(y_to_line(y)) - 1;
    int column2 = static_cast<int>(x_to_column(x));
    if(line != line2 || column != column2)
    {
        line = line2;
        column = column2;
        set_block(last_char_written);
    }

    return true;
}

bool MazeEditor::key_down(const SDL_KeyboardEvent& key)
{
    bool processed_key = false;
    SDL_Keysym keysym = key.keysym;
    if(keysym.sym == SDLK.UP || keysym.sym == SDLK.w)
    {
        line--;
        if(line == -1)
        {
            line = edit_height-1;
        }
        processed_key = true;
    }
    if(keysym.sym == SDLK.LEFT || keysym.sym == SDLK.a)
    {
        column--;
        if(column == -1)
        {
            column = edit_width-1;
        }
        processed_key = true;
    }
    if(keysym.sym == SDLK.DOWN || keysym.sym == SDLK.s)
    {
        line++;
        if(line == edit_height)
        {
            line = 0;
        }
        processed_key = true;
    }
    if(keysym.sym == SDLK.RIGHT || keysym.sym == SDLK.d)
    {
        column++;
        if(column == edit_width)
        {
            column = 0;
        }
        processed_key = true;
    }
    if(keysym.sym == SDLK.SPACE)
    {
        toggle_block();
        processed_key = true;
    }

    return processed_key;
}


void MazeEditor::update(double dt, MySoundManager& sound, GameApplication& app)
{
    if(flash_timer.expired())
    {
        flashing_var = not flashing_var;
        if(flash_save != 0)
        {
            flash_save--;
        }
    }
}

void MazeEditor::save()
{
    const char* path = SaveDataPathDeveloper("maps/new_maze.lua").c_str();
    if(path)
    {
        std::ofstream f(path);
        if(f)
        {
            f << "RawMapData" << "{" << std::endl;

            for(int h = 0; h < edit_height; h++)
            {
                f << "\t\"";
                for(int w = 0; w < edit_width; w++)
                {
                    f << maze[w][h];
                }
                f << "\"," << std::endl;
            }
            f << "}" << std::endl;
            f.close();
        }
    }
}
