-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
-- Menu generator
--

require('library/SDL_keycode')
require('strict')
class = require('middleclass')

local Menu = class('menu')
local MouseMove = require("player/mouse_move")
local text_sizes = require("utilities/text_sizes")

-- style 1: functions for ASCII border
local function _ascii_print_horizontal(width)
    ANSI_print(string.format("+%s+", string.rep("-", width)))
end
local function _ascii_print_side(index)
    ANSI_print("|")
end

-- style 2: glyph strings bubble border
local style2_top_left = to_utf8(get_glyph('townmap', 0x38))
local style2_top = to_utf8(get_glyph('townmap', 0x39))
local style2_top_right = to_utf8(get_glyph('townmap', 0x3A))
local style2_left = to_utf8(get_glyph('townmap', 0x58))
local style2_right = to_utf8(get_glyph('townmap', 0x5A))

local style2_bottom_left = to_utf8(get_glyph('townmap', 0x77))
local style2_bottom = to_utf8(get_glyph('townmap', 0x79))
local style2_bottom_right = to_utf8(get_glyph('townmap', 0x7A))

local style2_top_left_over = to_utf8(get_glyph('townmap', 0x16))
local style2_top_over = to_utf8(get_glyph('townmap', 0x17))
local style2_top_right_over = to_utf8(get_glyph('townmap', 0x18))
local style2_left_over = to_utf8(get_glyph('townmap', 0x36))
local style2_right_over = to_utf8(get_glyph('townmap', 0x37))

local style2_bottom_left_over = to_utf8(get_glyph('townmap', 0x76))
local style2_bottom_over = to_utf8(get_glyph('townmap', 0x56))
local style2_bottom_right_over = to_utf8(get_glyph('townmap', 0x57))

-- style 3:garden maze wall style
local style3_top_left = to_utf8(get_glyph('townmap', 32+27))
local style3_top_right = to_utf8(get_glyph('townmap', 32+29))

local style3_bottom_left = to_utf8(get_glyph('townmap', 96+27))
local style3_bottom_right = to_utf8(get_glyph('townmap', 96+29))

local style3_topbottom = to_utf8(get_glyph('townmap', 64+30))
local style3_leftright = to_utf8(get_glyph('townmap', 96+30))

-- Array of border styles.
-- Has 4 functions defined that do the border printing.
local border_types = {
    {
        top = _ascii_print_horizontal,
        bottom = _ascii_print_horizontal,
        left = _ascii_print_side,
        right = _ascii_print_side,
    },
    {
        top = function(width, fg, bg)
            set_bg_transparent()
            ANSI_print(string.format("%s%s%s%s%s",
                    ANSI_string_save_cursor_position,
                    style2_top_left, string.rep(style2_top, width), style2_top_right,
                    ANSI_string_recall_cursor_position))
            ANSI_set_fg_colour(bg)
            ANSI_print(string.format("%s%s%s",
                    style2_top_left_over, string.rep(style2_top_over, width), style2_top_right_over
                    ))
            ANSI_set_fg_colour(fg)
            set_bg_opaque()
            end,
        bottom = function(width, fg, bg)
            set_bg_transparent()
            ANSI_print(string.format("%s%s%s%s%s",
                    ANSI_string_save_cursor_position,
                    style2_bottom_left, string.rep(style2_bottom, width), style2_bottom_right,
                    ANSI_string_recall_cursor_position))
            ANSI_set_fg_colour(bg)
            ANSI_print(string.format("%s%s%s",
                    style2_bottom_left_over, string.rep(style2_bottom_over, width), style2_bottom_right_over
                    ))
            ANSI_set_fg_colour(fg)
            set_bg_opaque()
            end,
        left = function(index, fg, bg)
            set_bg_transparent()
            ANSI_print(style2_left .. ANSI_string_cursor_back1)
            ANSI_set_fg_colour(bg)
            ANSI_print(style2_left_over)
            ANSI_set_fg_colour(fg)
            set_bg_opaque()
            end,
        right = function(index, fg, bg)
            set_bg_transparent()
            ANSI_print(style2_right .. ANSI_string_cursor_back1)
            ANSI_set_fg_colour(bg)
            ANSI_print(style2_right_over)
            ANSI_set_fg_colour(fg)
            set_bg_opaque()
            end,
    },
    {
        top = function(width)
            ANSI_print(string.format("%s%s%s", style3_top_left, string.rep(style3_topbottom, width), style3_top_right))
            end,
        bottom = function(width)
            ANSI_print(string.format("%s%s%s", style3_bottom_left, string.rep(style3_topbottom, width), style3_bottom_right))
            end,
        left = function(index) ANSI_print(style3_leftright) end,
        right = function(index) ANSI_print(style3_leftright) end,
    },

}

function Menu:_scan_selectable()
    -- find out how many lines are selectable
    self._maximum_selectable = 0
    for _,v in ipairs(self.menu) do
        if type(v[1]) == 'table' then
            for _,e in ipairs(v) do
                if e[2] then
                    self._maximum_selectable = self._maximum_selectable + 1
                end
            end
        else 
            if v[2] then
                self._maximum_selectable = self._maximum_selectable + 1
            end
        end
    end
end

function Menu:delete_mouse_objects()
    for k,v in ipairs(self.mouse_objects) do
        MouseMove.remove_region(v)
        self.mouse_objects[k] = nil
    end
    self.mouse_objects = nil
end

function Menu:mouse_event(x, y, button_down, move_event, i)
    local m = self.mouse_regions[i]
    if m then
        self.mouse_on_top_of_region = i
        self.selected_index = i
        if not move_event and not button_down then
            --print("Unclick")
            self:_select_menu_item(m[5])
            self.mouse_on_top_of_region = nil       -- clear highlight
        end
    else
        print("Menu:mouse_event - Invalid i")
    end
end

function Menu:mouse_outside()
    self.mouse_on_top_of_region = nil
end

function Menu:add_mouse_outside_region()
    if self.mouse_outside_object then
        self:delete_mouse_outside_region()
    end
    self.mouse_outside_object = gulp_cpp.MouseTargetRectangle(gulp.app, 0, 0, gulp_cpp.GetWindowWidth(), gulp_cpp.GetWindowHeight(), self.mouse_outside, self)
    self.mouse_outside_object:care_about_drag_button_down()
    self.mouse_outside_object:care_about_drag_button_up()
    MouseMove.add_region(self.mouse_outside_object)
end

function Menu:delete_mouse_outside_region()
    MouseMove.remove_region(self.mouse_outside_object)
    self.mouse_outside_object = nil
end

function Menu:resize()
    self:scan_mouse_regions()
end

function Menu:scan_mouse_regions()
    if self.mouse_objects then
        self:delete_mouse_objects()
    end
    self.mouse_regions = {}
    
    local cell_size = self.text_sizes:get_cell_size()
    local vp_columns = self.text_sizes:columns()
    local vp_lines = self.text_sizes:lines()
    local offset_x, offset_y
    offset_x, offset_y = self.text_sizes:get_origin()
    
    if self.owner then
        -- need owner to have been updated first...!
        if self.c_offset then self.column = self.owner.column + self.c_offset end
        if self.l_offset then self.line = self.owner.line + self.l_offset end
    end
    
    if self.centralised or self.c_centralised then
        self.column = (vp_columns - (self.internal_width+2)) / 2
    end

    if self.centralised or self.l_centralised then
        self.line = (vp_lines - (#self.menu+2)) / 2
    end
    
    local line = self.line
    local table_index = 1
    
    for k,v in ipairs(self.menu) do
        line = line + 1
        
        if type(v[1]) == 'table' then
            
            local column = self.column + 1
            
            for _,e in ipairs(v) do
                local length = e[1]:len()
                if e[2] then
                    local rectangle = { (column * cell_size) + offset_x, 
                                        (line * cell_size) + offset_y, 
                                        ((column + length) * cell_size) + offset_x, 
                                        ((line + 1) * cell_size) + offset_y, 
                                        table_index }
                    table.insert(self.mouse_regions, rectangle)
                    table_index = table_index + 1
                end
                column = column + length + 1
            end
        else 
            if v[2] then
                local rectangle = { ((self.column + 1) * cell_size) + offset_x,
                                    (line * cell_size) + offset_y, 
                                    ((self.column + self.internal_width + 1) * cell_size) + offset_x, 
                                    ((line + 1) * cell_size) + offset_y, 
                                    table_index }
                table.insert(self.mouse_regions, rectangle)
                table_index = table_index + 1
            end
        end
        
    end
    
    self.mouse_objects = {}
    for k,r in ipairs(self.mouse_regions) do
        local mouse_object = gulp_cpp.MouseTargetRectangle(gulp.app, r[1], r[2], r[3], r[4], self.mouse_event, self, k)
        mouse_object:care_about_drag_button_down()
        mouse_object:care_about_drag_button_up()
        table.insert(self.mouse_objects, mouse_object)
        MouseMove.add_region(mouse_object)
    end
    self.mouse_on_top_of_region = nil   -- once we have a new set of regions, we need a move or click to recalculate
end


function Menu:draw_mouse_regions(gr)
    for _,rectangle in ipairs(self.mouse_regions) do
        gr:DrawAbsoluteRect(colour_to_SDL_Color(gulp.colour.BLUE), rectangle[1], rectangle[2], rectangle[3], rectangle[4])
    end
    if self.mouse_on_top_of_region then
        local rectangle = self.mouse_regions[self.mouse_on_top_of_region]
        gr:DrawAbsoluteRect(colour_to_SDL_Color(gulp.colour.GREEN), rectangle[1], rectangle[2], rectangle[3], rectangle[4])
    end
end

function Menu:initialize(menu)
    self._hidden  = false
    self.line =  menu.line or 2
    self.column = menu.column or 2
    self.menu = menu
    self.selected_index = 1
    
    -- submenu stuff
    self.owner = menu.owner
    self.c_offset = menu.c_offset
    self.l_offset = menu.l_offset

    if menu.close_on_escape == nil then
        self.close_on_escape = true
    else
        self.close_on_escape = menu.close_on_escape
    end
    
    -- we should do some better calculation of this based on total screen area
    self.text_sizes = menu.text_sizes or text_sizes

    self.centralised = menu.centralised or false
    self.c_centralised = menu.c_centralised or false
    self.l_centralised = menu.l_centralised or false

    self.old_line = self.line
    self.old_column = self.column

    self:set_width()
    self:_scan_selectable()

    gulp._delegate_list:add_delegate(self)
    gulp.add_user_movement_target(self)
    self.finished = false

    self.border = border_types[menu.border_style or 1]
    if not self.border then
        self.border = border_types[1]
    end

    self.controls = {}

    -- set up the mouse stuff
    self:add_mouse_outside_region()
    self:scan_mouse_regions()
    
end

function Menu:set_width()
    
    local old_width = self.internal_width
    
    self.internal_width = 1
    
    for _,v in ipairs(self.menu) do
        
        local size
        if type(v[1]) == 'table' then
            size = -1
            for _,e in ipairs(v) do
                size =  size + ANSI_printable_len(e[1]) + 1
            end
        else 
            size = ANSI_printable_len(v[1])
        end
        
        if size > self.internal_width then
            self.internal_width = size
        end
    end
    
    -- return true if width changed
    return (self.internal_width ~= old_width)
end

function Menu:update_menu_string(item, string)
    self.menu[item][1] = string
    if(self:set_width()) then
        self:scan_mouse_regions()
    end
end

-- just use one parameter to insert at the end
function Menu:insert_item(location, menu_item)
    if menu_item then
        table.insert(self.menu, location, menu_item)
    else
        -- insert at the end
        table.insert(self.menu, location)
    end
    self:set_width()
    self:_scan_selectable()
    self:scan_mouse_regions()
    
    return #self.menu
end

function Menu:remove_item(location)
    
    table.remove(self.menu, location)
    self:set_width()
    self:_scan_selectable()
    self:scan_mouse_regions()
    
    return #self.menu
end



function Menu:close()
    --print("Menu close")
    if not self.finished then
        --print("Menu finishing")
        gulp.remove_user_movement_target(self)
        gulp._delegate_list:drop_delegate(self)
        self.finished = true
        if type(self.on_close) == "function" then
            self:on_close()
        end
        self:delete_mouse_objects()
        self:delete_mouse_outside_region()
    end
    
    for _,control in pairs(self.controls) do
        control:close()
    end
    
end

function Menu:position(line, column)
    self.line = line
    self.column = column
end

function  Menu:get_result()
    return self.result_selected_value, self.result_selected_index
end

function Menu:is_finished()
    return self.finished
end



function Menu:_get_padded_text(line_table)
    local s = line_table[1]
    local padding_len = self.internal_width - ANSI_printable_len(s)
    if padding_len > 0 then
        s = s .. string.rep(" ", (padding_len))
    end
    return s
end


function Menu:update(dt)
    
    -- if line or column has changed, then scan mouse regions
    if self.old_line ~= self.line or self.old_column ~= self.column then
        self:scan_mouse_regions()
        self.old_line = self.line
        self.old_column = self.column
    end
    
    if type(self.on_update) == "function" then
        self:on_update(dt)
    end
end


function Menu:draw(gr)
    if self._hidden then return end
    
    local vp_columns = self.text_sizes:columns()
    local vp_lines = self.text_sizes:lines()

    -- if centralised, relocate menu here
    if self.centralised or self.c_centralised then
        self.column = (vp_columns - (self.internal_width+2)) / 2
    end

    if self.centralised or self.l_centralised then
        self.line = (vp_lines - (#self.menu+2)) / 2
    end

    self.text_sizes:set_viewport(gr)
    
    -- never wrap menus
    local wrap = gr:get_wrap()
    gr:wrap(false)

    local fg = self.menu.foreground or gulp.colour.WHITE.simple_colour
    local bg = self.menu.background or gulp.colour.BRIGHT_BLUE

    ANSI_reset_attributes()
    ANSI_set_fg_colour(fg)
    ANSI_set_bg_colour(bg)

    local line = self.line
    goto_position(line, self.column) line = line + 1
    self.border.top(self.internal_width, fg, bg)

    local index = 0
    for _,v in ipairs(self.menu) do
        
        goto_position(line, self.column) line = line + 1
        local selectable = false
        
        ANSI_reset_attributes()
        ANSI_set_fg_colour(fg)
        ANSI_set_bg_colour(bg)
        self.border.left(index, fg, bg)
        
        if type(v[1]) == 'table' then
            for _,e in ipairs(v) do
                selectable = false
                ANSI_set_bg_colour(bg)
                if e[2] then
                    index = index + 1
                    selectable = index == self.selected_index
                end
                if selectable then
                    if self.menu.select_colour then
                        ANSI_set_bg_colour(self.menu.select_colour)
                        ANSI_print(e[1])
                    else
                        ANSI_print(string.format("\x1b[7m%s\x1b[0m", e[1]))
                    end
                else
                    ANSI_print(e[1])
                end
                gr:go_to(gr:get_line(), gr:get_column() + 1)
            end
            gr:go_to(gr:get_line(), self.column+self.internal_width+1)
        else 
            if v[2] then
                index = index + 1
                selectable = index == self.selected_index
            end
            if selectable then
                if self.menu.select_colour then
                    ANSI_set_bg_colour(self.menu.select_colour)
                    ANSI_print(self:_get_padded_text(v))
                else
                    ANSI_print(string.format("\x1b[7m%s\x1b[0m", self:_get_padded_text(v)))
                end
            else
                ANSI_print(self:_get_padded_text(v))
            end
        end
        
        ANSI_reset_attributes()
        ANSI_set_fg_colour(fg)
        ANSI_set_bg_colour(bg)
        self.border.right(index, fg, bg)

    end

    goto_position(line, self.column) line = line + 1
    self.border.bottom(self.internal_width, fg, bg)

    -- draw graphics in menus
    --gr:set_bg_transparent()
    if self.menu.graphics then
        for _,v in ipairs(self.menu.graphics) do
            local line = self.line + v[1]
            local column = self.column + v[2]
            
            -- if specified column is negative, it means cells in from right hand edge
            if v[2] < 0 then column = column + self.internal_width end
            
            -- set the colours
            if v[8] ~= nil then set_fg_colour(v[8]) end
            if v[9] ~= nil then set_bg_colour(v[9]) end
            
            print_large_glyph_at(line, column, v[3], v[4], v[5], v[6], v[7])
            
            -- reset the colours
            ANSI_set_bg_colour(bg)
            ANSI_set_fg_colour(fg)
        end
    end
    --gr:set_bg_opaque()

    -- update fields in menus
    if self.menu.fields then
        for f,v in pairs(self.menu.fields) do
            local line = self.line + v[1]           -- +1 for border not required... (why?)
            local column = self.column + v[2] + 1   -- +1 for border
            
            local blank = ""
            for x = 1,v[3] do blank = blank.." " end
            goto_position(line, column)
            ANSI_print(blank)
            
            -- leave the blanking and the string print separate to facilitate centre justification, 
            -- which might ot be on a whole character position
            local string = tostring(v[4])
            local justified_column = column
            if v[5] == 'right' then
                justified_column = justified_column + (v[3]-string.len(string))
            end
            goto_position(line, justified_column)
            ANSI_print(string)
        end
    end
    

    -- we alter the key selector, so no other visual indication is necessary
    --self:draw_mouse_regions(gr)
    
    -- reset wrap settings
    gr:wrap(wrap)
    
    if type(self.on_draw) == "function" then
        self:on_draw(gr)
    end
    
    
end

function Menu:update_field(name, value)
    if self.menu.fields and self.menu.fields[name] then
        self.menu.fields[name][4] = value
    end
end


function Menu:_select_menu_item(selected_index)
    if self._hidden then return end

    -- passing nil means use the keyboard selected one.
    -- otherwise override with the incoming one
    selected_index = selected_index or self.selected_index

    self.result_selected_index = selected_index

    local index = 1
    for _,v in ipairs(self.menu) do
        
        local result_value
        
        local line_index = 0
        local on_line = true
        
        while on_line do
        
            if type(v[1]) == 'table' then
                line_index = line_index + 1
                if v[line_index] then
                    result_value = v[line_index][2]
                else
                    on_line = false
                    result_value = nil
                end
            else
                result_value = v[2]
                on_line = false
            end
            
            if result_value then
                if index == selected_index then
                    self.result_selected_value = result_value
                    local nested
                    if type(result_value) == "function" then
                        nested = result_value(v[3], v[4], v[5], v[6])
                    end
                    if not nested then
                        self:close()
                    end
                end
                index = index + 1
            end
            
        end
        
    end
end

function Menu:show()
    self._hidden = false
end

function Menu:hide()
    self._hidden = true
end


local GAME_UP_ACTION = 1;
local GAME_DOWN_ACTION = 2;
local GAME_LEFT_ACTION = 4;
local GAME_RIGHT_ACTION = 8;
local MOVE_MAP = 0x0F
local GAME_FIRE_ACTION = 16;

function Menu:move(move_map)
    --local move_map = bit32.band(action, MOVE_MAP)
    if move_map == GAME_UP_ACTION or move_map == GAME_LEFT_ACTION then
        self.selected_index = self.selected_index - 1
        if self.selected_index < 1 then
            self.selected_index = self._maximum_selectable
        end

    elseif move_map == GAME_DOWN_ACTION or move_map == GAME_RIGHT_ACTION then
        if self.selected_index >= self._maximum_selectable then
            self.selected_index = 1
        else
            self.selected_index = self.selected_index + 1
        end
    end

end

function Menu:fire(move_map)
    if move_map == GAME_FIRE_ACTION then
        self:_select_menu_item()
    end
end

function Menu:keypressed(sym, scancode, modifier, repeated)
    if back_key(sym) then
        if type(self.on_escape) == "function" then
            self:on_escape()
        end
        if self.close_on_escape then
            self:close()
        else
            return false        -- didn't handle this ESC
        end
    elseif sym == SDLK.RETURN or sym == SDLK.KP_ENTER then
        -- activate
        self:_select_menu_item()
    else
        return false
    end
    return true     -- handled
end

local function border_colours(app)
    if not app then
        return
    end

    local colour1, colour2
    local colour_table = {}

    local i = 1
    local k,v
    for k,v in pairs(gulp.colour) do
        colour_table[i] = v
        i= i+1
    end

    repeat
        colour1 = random_entry(colour_table).simple_colour
        colour2 = random_entry(colour_table).simple_colour
    until colour1 ~= colour2 and colour1 ~= nil and colour2 ~= nil

    set_status_line_colours(0, colour1, colour2);
    set_status_line_colours(1, colour1, colour2);
    set_status_line_colours(2, colour1, colour2);
end

function Menu:create_test(app)
    local test_menu = Menu{
        background = gulp.colour.BLUE.simple_colour,
        foreground = gulp.colour.BRIGHT_CYAN.simple_colour,
        border_style = math.random(3),
        { "\x1b[32mCommands Available :- " },
        { "" },
        { "\x1b[33m\x1b[1m  PICK UP AN OBJECT", function(app) set_status_line(1, 0, "pick up") end, app },
        { "\x1b[33m\x1b[1m  DROP AN OBJECT", function(app) set_status_line(1, 0, "drop") end, app },
        { "\x1b[33m\x1b[1m  TAKE AN OBJECT", function(app) set_status_line(1, 0, "take") end, app },
        { "\x1b[33m\x1b[1m  GIVE AN OBJECT", 3},
        { "\x1b[33m\x1b[1m  EXAMINE", 4 },
        { "\x1b[37m\x1b[1m  COMMAND A CHARACTER", 5},
        { "\x1b[37m\x1b[1m  READ SOMETHING", 6},
        { "\x1b[37m\x1b[1m  CAST A SPELL", 7},
        { "\x1b[37m\x1b[1m  BLOW SOMETHING", 8},
        { "\x1b[37m\x1b[1m  MOVE LIFT", border_colours, app }
    }
    return test_menu
end

function Menu:add_control(control)
    table.insert(self.controls, control)
end



return Menu
