-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
-- Create list of games
--
-- Created by Rob Probin, September 2015
-- For Forlorn Fox

--LANb = require("networking/lan_broadcast")

local manage_game_list = class("manage_game_list")

local time_to_expire = 7

function manage_game_list:initialize(source_menu, connect_to, get_function, get_param)
    self.menu = source_menu
    self.connect_to = connect_to
    self.get_function = get_function
    self.get_param = get_param
    gulp._delegate_list:add_delegate(self)
    
    self.active_games = {}
    self.menu_refs = {}
end


function manage_game_list:update(dt)
    -- Should we check dt to make sure plenty of time has passed?
    -- We don't really case - we are not doing anything else
    
    local servers = self.get_function(self.get_param)
    
    -- first parse the games given to add or update the times
    for _, game in ipairs(servers) do
            -- copy game into our list
            self.active_games[game.game_id] = game
    end
    
    -- now look for old games and grey them out from the menu
    for _, game in pairs(self.active_games) do
        -- don't display games that are older than 2 minutes
        local update_required = nil
        if self.menu_refs[game.game_id] then
            if (gulp.time - game.received_last_update) > time_to_expire then
                update_required = "old"
            end
        else
            -- must be a new game
            -- now update the list with new games
            update_required = "new"
        end

        if update_required then
            
            local good_ver = game.game_version == gulp.forlorn_fox_version
            local version_ok_string = to_utf8(get_glyph('editor', 3))
            local version_ok_colour = "\x1b[32m\x1b[1m"     -- bright green
            local run_func = 0
            if not good_ver then
                version_ok_string = to_utf8(get_glyph('editor', 1))
                version_ok_colour = "\x1b[31m"          -- red
                run_func = nil
            end
            if update_required == "old" then
                version_ok_colour = "\x1b[34m"         --  blue for old
            end
            local display1 = string.format(
                "%s%s  v%s %s", 
                version_ok_colour,
                tostring(game.game_name),
                tostring(game.game_version),
                tostring(version_ok_string)
            )
            local display2 = string.format(
                " %s@ %s %s", 
                version_ok_colour,
                tostring(game.game_ip_address),
                tostring(game.game_port)
            )
    
            if update_required == "old" then
                local menu_id = self.menu_refs[game.game_id]
                self.menu:update_menu_string(menu_id, display1)
                self.menu:update_menu_string(menu_id+1, display2)
            elseif good_ver then
                self.menu_refs[game.game_id] = self.menu:insert_item( { display1, self.connect_to, game.game_ip_address, game.game_port } )
                self.menu:insert_item( { display2 } )
            else
                self.menu_refs[game.game_id] = self.menu:insert_item( { display1 } )
                self.menu:insert_item( { display2 } )
            end
        end
    
    end
    
end

function manage_game_list:terminate()
    gulp._delegate_list:drop_delegate(self)
    self.menu = nil -- ensure no circular references for garbage collector ... not essential?
end

return manage_game_list
