-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

require("strict")
local class = require("middleclass")
local copas = require("copas")

-- Message queue for copas sending 
--[[
function MessagesInClientQueue(socket_ref)
    return send_message_queues[socket_ref]
end
--]]


local NetworkTransport = class("NetworkTransport")

function NetworkTransport:initialize(connection_socket)
    self.my_socket = connection_socket
    self.send_message_queue = {}
    self.close_socket = false
end

-- wrapper to add a message to the queue for sending   
function NetworkTransport:SendMessage(message)
    
    -- temporary until everthing has been converted over
    if type(message) == "string" then
        message = { mt = "old", content = message }
    end 
    local message_string = encode_for_network(message)

    local raw_socket = self.my_socket
    
    if raw_socket == nil then
        print("Socket is nil - dropping message")
        --local traceback = debug.traceback()
        --for line in traceback:gmatch("(.-)\r?\n") do
        --    print("   >>>", line)
        --end
    else
        table.insert(self.send_message_queue, message_string)
    end
end

function NetworkTransport:close()
    self.my_socket:close()
end

function NetworkTransport:receive(pattern, part)
    copas.recieve(self.my_socket, pattern, part)
end

-- Add the socket to be closed to the list 
function NetworkTransport:AddSocketToClose()
    self.close_socket = true
end
--[[
-- Copas thread that will pop the message and send over the network    
function SendMessageThread()

  -- wait until client socket is connected to start working
  while gulp_networking.clientSocket == nil do
      -- nothing
      -- should have a timeout here...?
      copas.sleep(0)
  end
  
  while true do
    -- need a check here to see if we should act
    if #gulp_networking.clientSocket.send_message_queue > 0 then
        
        local messages = table.concat(gulp_networking.clientSocket.send_message_queue)
        -- remove queue, we've sent it
        gulp_networking.clientSocket.send_message_queue = {}
        
        -- This might yield if send doesn't go.
        -- It also yields 1 in 10 (on average, actually at random).
        -- This affects ALL client transmissions not just this one.
        -- We could use a seperate thread per client, but since this is the
        -- server sending (just to different sockets), does that help?
        -- Yielding 1 in 10 means messages are delayed by one frame, 
        -- potentially.
        local s, err, lastIndex = copas.send(gulp_networking.clientSocket.my_socket, messages)
        if (err) then
            -- could be 'closed' or 'timeout' - but 
            -- copas uses timeout, so must be close!
            print ("SendMessageThread - send error on on socket: ", gulp_networking.clientSocket)
            print ("The Copas error is: ", err)
         
            if gulp.LuaStateCodeMode == 'SERVER' then
                self:HandleNetworkErrorServer("SendError")
            else
                self:HandleNetworkErrorClient("SendError")
            end
        else
            -- @todo: debug code - remove after testing
            if s ~= #messages then
                print("SendMessageThread - Didn't send all of message" , s, #messages)
            end
        end
    end
    
    if gulp_networking.clientSocket.close_socket then
        copas.sleep(0.1)

        gulp_networking.clientSocket.my_socket:close()

        print(gulp.LuaStateCodeMode, "Socket closed - return from SendMessageThread")
        return
    end
        
    copas.sleep(0) -- use 0 just to other copas threads to run  
  end
end  
--]]
function _RunSendMessageThread(networking)

    -- wait until at least one client socket is connected to start working
    -- don't need this any more...?
---[[
    while networking.clientSockets[1] == nil do
      -- nothing
      -- should have a timeout here...?
      copas.sleep(0.01)
    end
--]]
    -- send_message_queue did not used to belong to clientSocket
    -- for the server, we don't have a client socket, so this isn;t working

    print("Running ServerSendMessageThread for "..tostring(networking).." with socket "..tostring(networking.clientSockets[1].my_socket))


    while true do
        -- give the rest of the system a chance in the state where there are constant messages        
        copas.sleep(0)    -- don't wait if messages to send
        
        -- now loop until we find some messages
        local found_message = false
        local socket_messages = {}
        while not found_message do
            
            local live_socket = false
            
            -- loop round each socket we are sending on
            for i,socket in pairs(networking.clientSockets) do
                -- check if the socket is closing
                if socket.close_socket then
                    copas.sleep(0.1)                -- other clients will glitch when a client leaves the server...?
                    socket.my_socket:close()
                    networking.clientSockets[i] = nil
                else
                    -- socket is not closing... at least one socket is still live
                    live_socket = true
                    
                    -- check if the socket has any messages to send, and add them to our buffer
                    if #socket.send_message_queue > 0 then
                        socket_messages[i] = table.concat(socket.send_message_queue)        
                        found_message = true
                    end
                end
            end
            
            -- check if all sockets are closed and we are done here, return from the thread run function
            if live_socket == false then
                print(gulp.LuaStateCodeMode, "All sockets closed - return from SendMessageThread")
                if gulp.LuaStateCodeMode=='CLIENT' then
                    gulp.ControlID = nil
                end
                return
            end
            
            -- if we don't have any messages, take a 10ms break
            if not found_message then
                copas.sleep(0.01)
            end
        end
        
        -- we have something to send...!
        for i,messages in pairs(socket_messages) do
            
            local socket = networking.clientSockets[i]
            
            -- remove queue, we've sent it
            socket.send_message_queue = {}
            
            -- This might yield if send doesn't go.
            -- It also yields 1 in 10 (on average, actually at random).
            -- This affects ALL client transmissions not just this one.
            -- We could use a seperate thread per client, but since this is the
            -- server sending (just to different sockets), does that help?
            -- Yielding 1 in 10 means messages are delayed by one frame, 
            -- potentially.
            local s, err, lastIndex = copas.send(socket.my_socket, messages)
            if (err) then
                -- could be 'closed' or 'timeout' - but 
                -- copas uses timeout, so must be close!
                print ("SendMessageThread - send error on on socket: ", socket)
                print ("The Copas error is: ", err)
             
                if gulp.LuaStateCodeMode == 'SERVER' then
                    socket:HandleNetworkErrorServer("SendError", networking)
                else
                    socket:HandleNetworkErrorClient("SendError", networking)
                end
            else
                -- @todo: debug code - remove after testing
                if s ~= #messages then
                    print("SendMessageThread - Didn't send all of message" , s, #messages)
                end
            end
        end
    end
end  

function SendMessageThread()
--    print("Running SendMessageTread for "..tostring(gulp_networking).." with socket "..tostring(gulp_networking.clientSockets[1].my_socket))
    _RunSendMessageThread(gulp_networking)
end

function SendControlMessageThread()
--    print("Running SendControlMessageThread..............................................................")
    _RunSendMessageThread(gulp_control_networking)
end

function ServerSendMessageThread()
--    print("Running ServerSendMessageThread..............................................................")
    _RunSendMessageThread(gulp_networking)
end

function ServerSendControlMessageThread()
    _RunSendMessageThread(gulp_control_networking)
end

function GulpNetworking:stepCopas(timeout)
    -- No delay by default
    timeout = timeout or 0
    return copas.step(timeout)
end


function NetworkTransport:HandleNetworkErrorServer(ErrorType, networking)
    print ("Server Network Error: ", ErrorType)
    
    -- Handle any server stuff
    -- Remove the remote player as there connection has failed
    UnregisterClient(self, networking)
 
    if self.send_message_queue then
        -- Error with this socket so clear to nil to prevent us trying to use it again
        self.send_message_queue = {} -- make sure nothing left on old socket number (copas.send might switch)
    end
end

function NetworkTransport:HandleNetworkErrorClient(socketWithError, ErrorType)
    print ("Client Network Error: ", ErrorType)
    
    -- Tell the rest of the game that an error has occured.
    gulp.setNetworkError()
    
    if self.send_message_queue then
        -- Error with this socket so clear to nil to prevent us trying to use it again
        self.send_message_queue = {} -- make sure nothing left on old socket number (copas.send might switch)
    end
end


return NetworkTransport
