-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
require('strict')
require('utils')
local GoldCoin = require('objects/gold_coin')


function generate_treasure(map_name)
    
    -- values set to give 1 unburied coin and two buried coins on a non-scrolling level
    local default_density_table = {
        buried_gold_coin_density = 0.003,
        unburied_gold_coin_density = 0.0015,
    }
    
    local map = get_map(map_name)
    local height = map.auto.maze:height()
    local width = map.auto.maze:width()
    
    local function _generate(number_of_coins, treasure, buried)
        for _ = 1, number_of_coins do
            local line
            local column
            repeat
                line = math.random(height)
                column = math.random(width)
            until not is_wall(map_name, line, column)
            local item 
            
            -- this should be replaced with some sort of creator / factory in the object code
            if treasure == 'gold_coin' then
                item = GoldCoin('gold coin')
            else
                -- default item for now!
                item = GoldCoin('gold coin')
            end
            
            map.auto.pickables:add_to_floor(line+0.5, column+0.5, item, 'map_generation', buried)
            --print("added coin at ", line, column, buried)
        end
    end
    
    local function get_treasure_count(location, name)
        local number = map["number_of_"..location.."_"..name]
        if number == nil then
            local density = map[location.."_"..name.."_density"]
            if density == nil then
                density = default_density_table[location.."_"..name.."_density"]
                if density ==  nil then density = 0 end
            end
            number = height*width*density
        end
        return number
    end
    
    local number_of_coins = get_treasure_count("unburied","gold_coin")
    _generate(number_of_coins, "gold_coin", false)
    
    number_of_coins = get_treasure_count("buried","gold_coin")
    _generate(number_of_coins, "gold_coin", true)
end


