-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
require('strict')
local class = require('middleclass')
local SpeechBubble = class('SpeechBubble')
require('utils')
local text_sizes = require("utilities/text_sizes")


local style2_top = to_utf8(get_glyph('townmap', 0x39))
local style2_left = to_utf8(get_glyph('townmap', 0x58))
local style2_right = to_utf8(get_glyph('townmap', 0x5A))
local style2_bottom = to_utf8(get_glyph('townmap', 0x79))

local style2_top_left = to_utf8(get_glyph('townmap', 0x38))
local style2_top_right = to_utf8(get_glyph('townmap', 0x3A))
local style2_bottom_left = to_utf8(get_glyph('townmap', 0x77))
local style2_bottom_right = to_utf8(get_glyph('townmap', 0x7A))

local style2_top_over = to_utf8(get_glyph('townmap', 0x17))
local style2_left_over = to_utf8(get_glyph('townmap', 0x36))
local style2_right_over = to_utf8(get_glyph('townmap', 0x37))
local style2_bottom_over = to_utf8(get_glyph('townmap', 0x56))

local style2_top_left_over = to_utf8(get_glyph('townmap', 0x16))
local style2_top_right_over = to_utf8(get_glyph('townmap', 0x18))
local style2_bottom_left_over = to_utf8(get_glyph('townmap', 0x76))
local style2_bottom_right_over = to_utf8(get_glyph('townmap', 0x57))

local style2_top_left_arrow = to_utf8(get_glyph('townmap', 0x8A))
local style2_top_right_arrow = to_utf8(get_glyph('townmap', 0x8B))
local style2_bottom_left_arrow = to_utf8(get_glyph('townmap', 0x78))
local style2_bottom_right_arrow = to_utf8(get_glyph('townmap', 0x89))

local style2_top_left_arrow_over = to_utf8(get_glyph('townmap', 0x9A))
local style2_top_right_arrow_over = to_utf8(get_glyph('townmap', 0x9B))
local style2_bottom_left_arrow_over = to_utf8(get_glyph('townmap', 0x88))
local style2_bottom_right_arrow_over = to_utf8(get_glyph('townmap', 0x99))


local border_types = {
    {
        top = function(width, fg, bg, arrow)
            
            local top_left = style2_top_left
            local top_right = style2_top_right
            local top_left_over = style2_top_left_over
            local top_right_over = style2_top_right_over
            if arrow then
                if arrow == 'left' then
                    top_left = style2_top_left_arrow
                    top_left_over = style2_top_left_arrow_over
                else
                    top_right = style2_top_right_arrow
                    top_right_over = style2_top_right_arrow_over
                end
            end
            
            set_bg_transparent()
            ANSI_print(string.format("%s%s%s%s%s",
                    ANSI_string_save_cursor_position,
                    top_left, string.rep(style2_top, width), top_right,
                    ANSI_string_recall_cursor_position))
            ANSI_set_fg_colour(bg)
            ANSI_print(string.format("%s%s%s",
                    top_left_over, string.rep(style2_top_over, width), top_right_over
                    ))
            ANSI_set_fg_colour(fg)
            set_bg_opaque()
            end,
        bottom = function(width, fg, bg, arrow)
            
            local bottom_left = style2_bottom_left
            local bottom_right = style2_bottom_right
            local bottom_left_over = style2_bottom_left_over
            local bottom_right_over = style2_bottom_right_over
            if arrow then
                if arrow == 'left' then
                    bottom_left = style2_bottom_left_arrow
                    bottom_left_over = style2_bottom_left_arrow_over
                else
                    bottom_right = style2_bottom_right_arrow
                    bottom_right_over = style2_bottom_right_arrow_over
                end
            end
            
            set_bg_transparent()
            ANSI_print(string.format("%s%s%s%s%s",
                    ANSI_string_save_cursor_position,
                    bottom_left, string.rep(style2_bottom, width), bottom_right,
                    ANSI_string_recall_cursor_position))
            ANSI_set_fg_colour(bg)
            ANSI_print(string.format("%s%s%s",
                    bottom_left_over, string.rep(style2_bottom_over, width), bottom_right_over
                    ))
            ANSI_set_fg_colour(fg)
            set_bg_opaque()
            end,
        left = function(index, fg, bg)
            set_bg_transparent()
            ANSI_print(style2_left .. ANSI_string_cursor_back1)
            ANSI_set_fg_colour(bg)
            ANSI_print(style2_left_over)
            ANSI_set_fg_colour(fg)
            set_bg_opaque()
            end,
        right = function(index, fg, bg)
            set_bg_transparent()
            ANSI_print(style2_right .. ANSI_string_cursor_back1)
            ANSI_set_fg_colour(bg)
            ANSI_print(style2_right_over)
            ANSI_set_fg_colour(fg)
            set_bg_opaque()
            end,
    }

}

function SpeechBubble:initialize(text, speaker, side, level, background)
    -- why should an object have a line and column or a depth? 
    -- 
    self.text = text
    self.speaker = speaker
    self.side = side
    self.level = level
    self.background = background
    
    if background == gulp.colour.GREEN.simple_colour or
       background == gulp.colour.YELLOW.simple_colour then
        self.foreground = gulp.colour.BLACK.simple_colour
    end
    

    
    
    self.item_type = 'speech_bubble'
    
    
    -- work out the bubble size
    -- if the text is too long, probably want to shrink the glyphs
    -- but leave that for now
    
    -- side and level are hints, change if there is not enough space
    
    local length = ANSI_printable_len(text)
    
    -- parse text for words, longest word is minimum possible width
    
    -- 14 wide will always fit somewhere on screen
    -- (if character is on one of middle two columns, there is 16 columns to either left or right,
    --  two columns used for edge of speech bubble, leaves 14)
    
    -- for now assume no words longer than 10 characters, and let width = 10 ??
    
    local words, max_width = split(text, " ")
    self.lines = { "" }     -- seed first line with empty string
    
    if max_width < 10 then max_width = 10 end
    
    local line = 0
    
    -- split text into lines
    for _, word in pairs(words) do
        local line_width
        if line == 0 then
            line_width = 1000
        else
            line_width = ANSI_printable_len(self.lines[line]) + 1
        end
        
        if word['width'] + line_width <= max_width then
            -- add word to the current line
            self.lines[line] = self.lines[line].." "..word['word']
        else
            -- new line for the word
            line = line + 1
            self.lines[line] = word['word']
        end
    end
    
    -- check the width of the lines to get the actual width
    local text_height = line
    local text_width = 0
    for _, line in pairs(self.lines) do
        if text_width < ANSI_printable_len(line) then text_width = ANSI_printable_len(line) end
    end
    
    self.box_width = text_width + 2
    self.box_height = text_height + 2 
    
    -- update location on first draw
    self._moved = true
    
    gulp._delegate_list:add_delegate(self)
    self.border = border_types[1]
    
end

function SpeechBubble:close()
    gulp._delegate_list:drop_delegate(self)
end

function SpeechBubble:moved()
    self._moved = true
end


function SpeechBubble:update_location(speaker_line, speaker_column)
    
    
    -- all this text_size stuff should go to initialise and resize functions
    -- the text size should match the map window, not the full size of the map
    -- I wonder how we get that?
    
    -- I guess using map.text_sizes makes some sense because then multiple speech bubbles can share the same text_size
    -- bt maybe map should take care of it itself? might be useful for other shit too, like context sensitive menus?
    local map = get_map(self.speaker.map_name)
    local maze = map.auto.maze

    local available_height = maze:GetAvailableLevelHeight()
    local available_width = maze:GetAvailableLevelWidth()
    
    if map.text_sizes == nil then
        map.text_sizes = text_sizes:new(available_width, available_height)
    else
        map.text_sizes:resize(available_width, available_height)
    end
    
    local effective_speaker_line = speaker_line
    local effective_speaker_column = speaker_column
    
    if speaker_line < 0 then effective_speaker_line = 0 end
    if speaker_line > (available_height) then effective_speaker_line = available_height end
    if speaker_column < -1 then effective_speaker_column = -1 end
    if speaker_column > (available_width+1) then effective_speaker_column = available_width+1 end
    
    if self.side == 'left' then
        if self.box_width > effective_speaker_column then self.side = 'right' end
    end
    if self.side == 'right' then
        if self.box_width + effective_speaker_column > available_width then self.side = 'left' end
    end
    if self.level == 'top' then
        if self.box_height >= effective_speaker_line+1 then self.level = 'bottom' end
    end
    if self.level == 'bottom' then
        if self.box_height + effective_speaker_line >= available_height+1 then self.level = 'top' end
    end
    
    -- calculate
    if self.side == 'left' then
        self.box_column = effective_speaker_column - (self.box_width + 0.5)
    else
        self.box_column = effective_speaker_column + 0.5
    end
    
    if self.level == 'top' then
        self.box_line = effective_speaker_line - (self.box_height - 1)
    else
        self.box_line = effective_speaker_line + 0.5
    end
    
    -- adjustment to get speech bubble arrow to point somewhere nice-ish
    self.box_line = self.box_line - 0.75
    
end    

    
function SpeechBubble:_get_padded_text(s)
    local padding_len = (self.box_width-2) - ANSI_printable_len(s)
    if padding_len > 0 then
        s = s .. string.rep(" ", (padding_len))
    end
    return s
end

function SpeechBubble:draw(gr)
    if self._hidden then return end
    
    local wrap = gr:get_wrap()
    gr:wrap(false)
    
    local fg = self.foreground or gulp.colour.WHITE.simple_colour
    local bg = self.background or gulp.colour.BRIGHT_BLUE

    ANSI_reset_attributes()
    ANSI_set_fg_colour(fg)
    ANSI_set_bg_colour(bg)

    if self.speaker.avatar_data.speech_font then
        swap_font(gr, glyph_sets[self.speaker.avatar_data.speech_font])
    end

    local player = gulp.localplayer
    local speaker_line = self.speaker.line - player.current_line_offset
    local speaker_column = self.speaker.column - player.current_column_offset
    
    if self._moved then
        self:update_location(speaker_line, speaker_column)
        self._moved = false
    end
    
    local map = get_map(self.speaker.map_name)
    map.text_sizes:set_viewport(gr)
    
    local line = self.box_line
    local column = self.box_column
    
    goto_position(line, column) line = line + 1
    
    local arrow
    if self.level == 'bottom' then
        if self.side == 'left' then
            arrow = 'right'
        else
            arrow = 'left'
        end
    end
    
    self.border.top(self.box_width-2, fg, bg, arrow)

    local index = 1
    for _,v in ipairs(self.lines) do
        goto_position(line, column) line = line + 1
        self.border.left(index, fg, bg)

        ANSI_print(self:_get_padded_text(v))

        self.border.right(index, fg, bg)

        index = index + 1
    end

    goto_position(line, column) line = line + 1
    
    
    arrow = nil
    if self.level == 'top' then
        if self.side == 'left' then
            arrow = 'right'
        else
            arrow = 'left'
        end
    end
    self.border.bottom(self.box_width-2, fg, bg, arrow)

    gr:wrap(wrap)
    default_font(gr)

end


return SpeechBubble

