-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--
-- Tree

require('strict')
local class = require('middleclass')
require('graphics/glyph_sets')

local Tree = class('Tree')
local tree_counter = 0

function Tree:initialize(map, tree_data)
    
    gulp._delegate_list:add_delegate(self)
    
    -- tree data specifies location just above left hand element of stump in map
    self.line = tree_data.line - 3
    self.column = tree_data.column - 2
    
    self.height = 4     -- fixed
    self.width = 7      -- fixed
    
    self.done = false
    
    tree_counter = tree_counter + 1
    self.tree_number = tree_counter

    -- create the glyphs we need in C++
    self:create_glyphs(map.auto.maze)
    
    -- sort out our initial graphical state
    -- we can't do this because it uses player state that hasn't been set up on map switching
    -- seems that waiting for the first 'real' update is not a problem anyway
    --self:update(0)

end

function Tree:get_tree_glyph(line, column)
    local glyph = 0xa0 + 0x10*line + 0x03 + column
    return get_glyph("tree", glyph)
end

function Tree:create_glyphs(maze)
    
    self.glyphs = { }
    for line = 1, self.height do    -- start at zero - this is the extra line for the roof which is actually outside
        self.glyphs[line] = {}
        
        
        for column = 1, self.width do
            
            self.glyphs[line][column] = 
                new_maze_display_element(maze, line+self.line-1, column+self.column-1, 
                                            self:get_tree_glyph(line, column), 200)
        end
    end
        
end

--[[
function Tree:update_player_view_layer(player)
    
    local layer = top_layer
    if self.inside == true then layer = 800 end
        
    for line = self.line-1, self.height+self.line do
        for column = self.column, self.width+self.column do
            player:set_view_layer(line, column, layer)
        end
    end
end
--]]

function Tree:delete_glyphs()
    
    for line = 1, self.height do
        for column = 1, self.width do
            if gulp.LuaStateCodeMode == 'CLIENT' then
                self.glyphs[line][column]:hide()
            end
            self.glyphs[line][column] = nil
        end
    end
        
end

-- a finisher method
function Tree:_finish()
    gulp._delegate_list:drop_delegate(self)
    self:delete_glyphs()
end

function Tree:finish()
    self.done = true
end


function Tree:update(dt)
end


return Tree
