-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

require("strict")
local class = require("middleclass")
local text_buffer = require("player/text_buffer")
local UnicodeTextEntry = require("main_menu/unicode_text_entry")
local term = require("player/term")
local caves_wrap = require("easter_eggs/caves_term_wrapper")

local chat = class("chat", UnicodeTextEntry)

function chat:initialize(mode)
    if not gulp.text_buffer then
        gulp.text_buffer = text_buffer:new()
    end
    self._mode = mode
    gulp.text_buffer:timed_disappear(false)
    
    -- start at top/left of text aligned location
    UnicodeTextEntry.initialize(self, 0, 0)
end

function chat:on_close()
    if not self.dont_clear_timed_disappear then
        gulp.text_buffer:timed_disappear(true)
    end
    gulp.chat_widget = nil
end

function chat:_me_command(arg, cmd, text)
    if #arg > 0 then
        text = gulp.prefs.player_name .. " " .. arg
        gulp.text_buffer:add_line(text)
    else
        gulp.text_buffer:add_line("Format: /me action")
    end
end

local command_set

function chat:_cli_command()
    term.open()
end

function chat:_poke_command(arg, cmd, text)
    if arg == "35899,0" then 
        gulp.text_buffer:add_line("CLI enabled", "MAGENTA")
        command_set['cli'] = chat._cli_command
    else
        gulp.text_buffer:add_line("No infinite lives")
    end
end

function chat:_writetype_command()
    command_set['poke'] = chat._poke_command
    gulp.text_buffer:add_line("Poke enabled", "GREEN")
end

function chat:_start_caves_command(arg, cmd, text)
    caves_wrap:new()
    self.dont_clear_timed_disappear = true
end

command_set = {
    ["me"] = chat._me_command,
    ["writetyper"] = chat._writetype_command,
    ["caves"] = chat._start_caves_command,
}

function chat:dispatch_message(text)
    -- Build the message
        local chatMessage = {
        mt = 'chat_message',
        message = text,
    }
     
    -- If server then send to all clients, else if client send to
    -- the server so it can send out, else do nothing
    if gulp.LuaStateCodeMode == 'SERVER' then
        gulp_networking:broadcast(chatMessage)
        -- server generated messages need to be printed at server
        gulp.text_buffer:add_line(text)
    else
        -- if we are a client, we wait for a server to reply so they are 
        -- serialised at the server to the same order. They are printed on the
        -- broadcast reply.
        gulp_networking:SendToServer(chatMessage)
    end
end

function chat:text_complete(text)
    if text:match("^%s*$") then
        return
    end
    local cmd = text:match("^/(%g+)")
    if cmd then
        if command_set[cmd] then
            local arg = text:sub(#cmd+3)
            command_set[cmd](self, arg, cmd, text)
        else
            gulp.text_buffer:add_line("Unknown command")
        end
    else
        -- should this be player name or character name?
        if self._mode ~= "emote" then
            text = gulp.prefs.player_name .. ">" .. text
        else
            text = gulp.prefs.player_name .. " " .. text
        end
        
        -- Call the dispatcher to send over the network if needed
        chat:dispatch_message(text)
    end
end

-- notice this is a FUNCTION not a member function/method
function chat.open()
    if not gulp.chat_widget then
        gulp.chat_widget = chat:new()
    end
end

function chat.open_emote()
    if not gulp.chat_widget then
        gulp.chat_widget = chat:new("emote")
    end
end

function chat.print_chat_message(text)
    if not gulp.text_buffer then
        gulp.text_buffer = text_buffer:new()
    end
    gulp.text_buffer:add_line(text)
end

return chat
