-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

local MouseMove = {}

local In = require("action_input")


local RECTAN = 0
local TRIANG = 1
local CIRCLE = 2

local mouse_data = {}

local function click_mouse(obj, x, y, button_down, move_event, direction)
    -- (BUG FIX: y=0 invlaid because seemed to get single click of y=zero in menu bar. SDL bug?)
    if y==0 then return end
    
    --print(obj, x, y, button_down, move_event, arg_last)
    mouse_data[obj].clicked = button_down
    --print("Click_mouse", button_down)
    if button_down then
        In.process_mouse(direction)
    else
        In.process_mouse(0)
    end
end

local function click_game_menu(obj, x, y, button_down, move_event, direction)
    --print(obj, x, y, button_down, move_event, arg_last)
    mouse_data[obj].clicked = button_down
    --print("Click_mouse", button_down)
    if button_down then
        In.process_mouse(direction)
        In.process_mouse(0)
    end
end

local function click_exit_menu(obj, x, y, button_down, move_event, direction)
    --print(obj, x, y, button_down, move_event, arg_last)
    mouse_data[obj].clicked = button_down
    --print("click_exit_menu", button_down)
    if button_down then
        if gulp.localplayer then
            gulp.localplayer:escape_key()
        end
    end
end


local game_mouse_store = {}
local game_mouse_enabled = false

-- _______________________
-- |eMenu             gMenu|      | y
-- | \        Up         / |      | axis
-- |  \_________________/  |      v
-- |   |               |   |
-- Left|     Laser     | Right
-- |   |_______________|   |
-- |  /                 \  |
-- | /       Down        \ |
-- |/_____________________\|
--                         ----> x axis
--
-- Notice: the corners are processed as diagonals

-- BUG FIX: When MouseMove.init_game_regions() was called from 
-- enable_game_controls() we had a weird bug where it would crash with a C++ 
-- exception when the game was started in client-mode when it was connected to 
-- the ZBS Lua debugger and the game mouse regions were clicked in the game. 
--
-- The actual exception was generated by a function call to Lua via a LuaRef 
-- holding  a Lua function in MouseTargetBaseType::run. The LuaRef held a 
-- different lua_State* which initially confused us, and this lua_State* had 
-- the state struct member set as LUA_YIELD (1) rather than LUA_OK (0). 
--
-- The function call back to Lua was via a lua_pcallk inside LuaBridge. Since the 
-- LuaBridge function call checked for a function return value of LUA_OK from 
-- the call, it would therefore signal an error by raising an exception 
-- this check failed. 
--
-- It turns out, in the client-mode game, enable_game_controls() was being 
-- called from the client receive thread with a different lua_State (we think a 
-- coroutine version of the current lua_State, since we only have one actual lua 
-- context at this point). In single-player-mode and server-mode the main Lua 
-- coroutine was used for the enable_game_control() call. We feel this itself 
-- shouldn't have been problematic. 
-- 
-- However, when the client-mode game was running in the ZBS Lua debugger the
-- client receive thread (at least at this point in the code), Lua state had the 
-- state struct member set as LUA_YIELD for some reason we don't yet understand.
-- 
-- The workaround here is to setup the Lua table with functions on the game load 
-- rather event from the main Lua state rather than do it just-in-time
-- when enabling game controls. 

function MouseMove.init_game_regions(app)
    -- These define screen areas for touch controls
    --local available_height = gulp_cpp.GetAvailableLevelHeightIncludingUI()
    --local available_width = gulp_cpp.GetAvailableLevelWidth()
    
    --local up_down_pixels_size = available_height / 3;
    --local left_right_pixels_size = available_width / 4;
    --local menu_pixel_size = available_height / 10;

    -- test data
    --[[
    mouse_data = {
        { RECTAN, colour_to_SDL_Color(gulp.colour.GREEN), 5, 15, 10, 20, click_mouse_fire},
        { TRIANG, colour_to_SDL_Color(gulp.colour.BLUE), 20, 15, 30, 19, 25, 21, click_mouse_fire},
       { CIRCLE, colour_to_SDL_Color(gulp.colour.RED), 5, 5, 4, click_mouse_fire},
    }
    --]]
    --[[
    local x1 = left_right_pixels_size
    local x2 = available_width - left_right_pixels_size
    local y1 = up_down_pixels_size
    local y2 = available_height - up_down_pixels_size
    local xe = available_width
    local ye = available_height
    
    local red = colour_to_SDL_Color(gulp.colour.RED)
    local blue = colour_to_SDL_Color(gulp.colour.BLUE)
    local green = colour_to_SDL_Color(gulp.colour.GREEN)
    local magenta = colour_to_SDL_Color(gulp.colour.MAGENTA)
    mouse_data = {
        -- fire
        { RECTAN, red, x1, y1, x2, y2, click_mouse, In.GAME_FIRE_ACTION },
        
        -- up
        { RECTAN, red, x1, 0, x2, y1, click_mouse, In.GAME_UP_ACTION},
        --{ TRIANG, blue, 0, 0, x1, 0, x1, y1, click_mouse},
        --{ TRIANG, green, x2, 0, x2, y1, xe, 0, click_mouse},
        
        -- down
        { RECTAN, red, x1, y2, x2, ye, click_mouse, In.GAME_DOWN_ACTION},
        --{ TRIANG, blue, 0, ye, x1, y2, x1, ye, click_mouse},
        --{ TRIANG, green, x2, y2, xe, ye, x2, ye, click_mouse},
        
        -- left
        { RECTAN, red, 0, y1, x1, y2, click_mouse, In.GAME_LEFT_ACTION},
        --{ TRIANG, blue, 0, 0, x1, y1, 0, y1, click_mouse},
        --{ TRIANG, green, 0, y2, 0, ye, x1, y2, click_mouse},
        
        -- right
        { RECTAN, red, x2, y1, xe, y2, click_mouse, In.GAME_RIGHT_ACTION},
        --{ TRIANG, blue, xe, 0, x2, y1, xe, y1, click_mouse},
        --{ TRIANG, green, x2, y2, xe, y2, xe, ye, click_mouse},
        
        -- up/left
        { RECTAN, blue, 0, 0, x1, y1, click_mouse, In.GAME_UP_ACTION+In.GAME_LEFT_ACTION},
        -- up/right
        { RECTAN, green, x2, 0, xe, y1, click_mouse, In.GAME_UP_ACTION+In.GAME_RIGHT_ACTION},
        -- down/left
        { RECTAN, blue, 0, y2, x1, ye, click_mouse, In.GAME_DOWN_ACTION+In.GAME_LEFT_ACTION},
        -- down/right
        { RECTAN, green, x2, y2, xe, ye, click_mouse, In.GAME_DOWN_ACTION+In.GAME_RIGHT_ACTION},
        
        -- menu
        { CIRCLE, magenta, 0, 0, menu_pixel_size, click_exit_menu},
        { CIRCLE, magenta, available_width, 0, menu_pixel_size, click_game_menu, In.GAME_MENU_ACTION },
    }
    --]]
    --[[
    game_mouse_store = {}

    for k, line in ipairs(mouse_data) do
        local object
        if line[1] == RECTAN then
            object = gulp_cpp.MouseTargetRectangle(app, line[3], line[4], line[5], line[6], line[7], k, line[8])
        elseif line[1] == TRIANG then
            object = gulp_cpp.MouseTargetTriangle(app, line[9], k, line[10])
            object:define(line[3], line[4], line[5], line[6], line[7], line[8])
        else
            object = gulp_cpp.MouseTargetCircle(app, line[3], line[4], line[5], line[6], k, line[7])
        end
        table.insert(game_mouse_store, object)
    end
    --]]
end


function MouseMove.enable_game_controls()
    local app = gulp.app
    
    -- init_game_data() was removed from here, see comment above
    
    local old_enabled = game_mouse_enabled
    if not game_mouse_enabled then
        for _, object in ipairs(game_mouse_store) do
            app:add_mouse_target(object)
        end
        game_mouse_enabled = true
    end
    
    return old_enabled
end


function MouseMove.disable_game_controls()
    local old_enabled = game_mouse_enabled
    if game_mouse_enabled then
        for _, object in ipairs(game_mouse_store) do
           gulp.app:remove_mouse_target(object)
        end
        game_mouse_enabled = false
    end
    return old_enabled
end


function MouseMove.restore_game_controls(previous_enabled)
    if previous_enabled then
        MouseMove.enable_game_controls()
    --else
    --    MouseMove.disable_game_controls()
    end
end


-- ordered list of items
local active_region_ordered_list = {}
local list_of_suspended_region_lists = {}

function MouseMove.draw_regions(gr)
    --[[
    for k, line in ipairs(mouse_data) do
        local colour = line[2]
        if mouse_data[k].clicked then
            colour = colour_to_SDL_Color(gulp.colour.WHITE)
        end
        if line[1] == 0 then
            gr:DrawRect(colour, line[3], line[4], line[5], line[6])
        elseif line[1] == 1 then
            DrawTriangle(gr, colour, line[3], line[4], line[5], line[6], line[7], line[8])
        else
            DrawCircle(gr, colour, line[3], line[4], line[5])
        end
    end

    for i,region in ipairs(active_region_ordered_list) do
        local shape, v1, v2, v3, v4, v5, v6 = region:get_shape()
        
        local colour = colour_to_SDL_Color(gulp.colour.WHITE)
        local sdl_colour = gulp_cpp.SDL_Color()
        sdl_colour.a = 128
        sdl_colour.r = 128 
        sdl_colour.g = 255
        sdl_colour.b = 128
        
        if shape == "Rectangle" then
            gr:FillRect(sdl_colour, v1, v2, v3, v4)
            gr:DrawRect(colour, v1, v2, v3, v4)
        elseif shape == "Circle" then
            DrawCircle(gr, colour, v1, v2, v3)
        elseif shape == "Triangle" then
            DrawTriangle(gr, colour, v1, v2, v3, v4, v5, c6)
        else
            print("Unknown shape", shape)
        end
    end
    
    --]]

    -- these statements are for testing the drawing primatives
    --local colour = colour_to_SDL_Color(gulp.colour.WHITE)
    --gr:DrawPoint(colour_to_SDL_Color(gulp.colour.WHITE), 0.5, 5)
    --gr:DrawLine(colour_to_SDL_Color(gulp.colour.BLACK), 0.5, 7, 2, 8)
    --gr:DrawRect(colour_to_SDL_Color(gulp.colour.BLACK), 0.5, 9, 2, 10)
    --gr:FillRect(colour_to_SDL_Color(gulp.colour.BLACK), 0.5, 11, 2, 12)
    --DrawTriangle(gr, colour, 10, 10, 15, 20, 20, 15)

end


function MouseMove.add_region(obj)
    gulp.app:add_mouse_target(obj)
    -- add it at the end
    table.insert(active_region_ordered_list, obj)
    --print("Mouse regions", #active_region_ordered_list)
end

local function find_in_list(l, v)
    local pos = nil
    for i = 1, #l do
        if v == l[i] then
            pos = i
            break
        end
    end
    return pos
end

function MouseMove.remove_region(obj)
    if obj == nil then
        print("BUG: MouseMove.remove_region() called with nil")
        return
    end
    
    gulp.app:remove_mouse_target(obj)

    -- we need to find it in the list
    -- there is no big push to have a table lookup for this
    -- because the list shouldn't be that big...
    -- function table_invert(t)
    -- local u = { }
    -- for k, v in pairs(t) do u[v] = k end
    -- return u
    -- end
    local pos = find_in_list(active_region_ordered_list, obj)
    if pos then
        table.remove(active_region_ordered_list, pos)
    else
        -- not in the active list, maybe in the suspended lists?
        local found = false
        for l,_ in pairs(list_of_suspended_region_lists) do
            local pos = find_in_list(l, obj)
            if pos then
                -- remove this one from the table
                table.remove(l, pos)
                found = true
                break
            end
        end
        if not found then
            print("Mouse region not found in MouseMove.remove_region()")
        end
    end
    --print("Mouse regions", #active_region_ordered_list)
end

function MouseMove.suspend_non_game_regions()
    for _, obj in ipairs(active_region_ordered_list) do
        gulp.app:remove_mouse_target(obj)
    end
    local retval = active_region_ordered_list
    list_of_suspended_region_lists[retval] = true
    active_region_ordered_list = {}
    return retval
end

function MouseMove.restore_listed_regions(objects_list)
    
    if #active_region_ordered_list ~= 0 then
        print("Unexpected list in restore - need to make add not replace!")
    end
    if not objects_list then
        print("objects_list==nil in MouseMove.restore_listed_regions()")
        return
    end
    if not list_of_suspended_region_lists[objects_list] then
        print("Unknown restore list in MouseMove.restore_listed_regions()")
    else
        list_of_suspended_region_lists[objects_list] = nil
    end
    
    -- we need to add these in the same order they were 
    -- added originally, so that the overlapping works still.
    for _, obj in ipairs(objects_list) do
        gulp.app:add_mouse_target(obj)
    end
    active_region_ordered_list = objects_list
end


return MouseMove

