-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

require("strict")

local class = require("middleclass")
local text_buffer = require("player/text_buffer")
local UnicodeTextEntry = require("main_menu/unicode_text_entry")
local text_sizes = require("utilities/text_sizes")

--
-- Very slight extension to unicode text entry to
-- call the terminal
--
local term_entry = class("term_entry", UnicodeTextEntry)

function term_entry:initialize(CLI)
    self._calling_CLI = CLI
    
    -- start two lines down from text aligned location
    UnicodeTextEntry.initialize(self, 0, 2, "", gulp.colour.ORANGE.simple_colour)
end

function term_entry:text_complete(text)
    self._calling_CLI:run(text)
end

function term_entry:on_close()
    self._calling_CLI:close_from_term_entry()
end


--
-- This is the CLI terminal controller. 
--
-- This 'owns' the terminal entry 'widget' and recreates it if the user 
-- hasn't issued a blank line, so the user can carry on the terminal session.
--
local term = class("term")
quit = nil                      -- @todo: what is this???? why is a global variable?
function term:initialize()
    --self._old_quit = quit
    self._old_quit = nil
    self._running = true
    self._was_text = false
    
    quit = function() term.close(gulp.local_CLI) end

    if not gulp.text_buffer then
        gulp.text_buffer = text_buffer:new()
    end
    gulp.text_buffer:timed_disappear(false)
    self._text_sizes = text_sizes:new(50, 30)
    gulp.text_buffer:modify_text_sizes(self._text_sizes)
    
    self._widget = term_entry:new(self)
end

function term:run(text)
    -- @todo quick access to mode /cli
    if text:find("%g") then
        self._was_text = true
        
        if self.object then
            -- call function
            self.object.text_complete(self.object, text)
        else
            -- this what we will do with terminal mode
            gulp.text_buffer:add_line(text, "RED")
        end
    else
        self._was_text = false
    end
end

function term:_common_close()
    gulp.local_CLI = nil
    quit = self._old_quit
    
    if self.object then
        -- call function
        self.object.on_close(self.object)
    end
    gulp.text_buffer:timed_disappear(true)
    gulp.text_buffer:modify_text_sizes()
end

function term:close_from_term_entry()
    if self._was_text and self._running then
        -- new input, if there was text and we are still running
        self._widget = term_entry:new(self)
    else
        -- term_entry already closing
        self._widget = nil
        self:_common_close()

    end
    self._was_text = false
end

function term:close()
    self._running = false
    self._was_text = false
    if self._widget then
        self._widget:close()
    else
        self:_common_close()
    end
end

-- None meth
function term.open(object)
    if not gulp.local_CLI then
        gulp.local_CLI = term:new()
    end
    gulp.local_CLI.object = object
end




return term
