-- Forlorn Fox
-- Copyright © 2014 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

require('status_manager')
require('player/PlayerList')
require('utils')

---------------------------------
-- FUN / TEST COLOUR FUNCTIONS --
---------------------------------

local function switch_maze_colours()
    local colour1, colour2
    local colour_table = {}
    local colour_name_table = {}
    
    local i = 1
    local k,v
    for k,v in pairs(gulp.colour) do
        colour_table[i] = v
        colour_name_table[i] = k
        i= i+1
    end
    
    repeat
        
        colour1 = random_entry(colour_table).simple_colour
        colour2 = random_entry(colour_name_table)
    until colour1 ~= colour2 and colour1 ~= nil and colour2 ~= nil
    
    get_map(gulp.localplayer.map_name).auto.maze:set_default_maze_colours(colour1,get_SDL_Color_by_name(colour2))
end

local function switch_status_colours()
    
    local colour1, colour2
    local colour_table = {}
    
    local i = 1
    local k,v
    for k,v in pairs(gulp.colour) do
        colour_table[i] = v
        i= i+1
    end
    
    repeat
        colour1 = random_entry(colour_table).simple_colour
        colour2 = random_entry(colour_table).simple_colour
    until colour1 ~= colour2 and colour1 ~= nil and colour2 ~= nil
    
    set_status_line_colours(0, colour1, colour2);
    set_status_line_colours(1, colour1, colour2);
    set_status_line_colours(2, colour1, colour2);

end


function test_menu()

    local last_number_of_dice

    local function dice_roll(app, dice)
        local rolls = {}
        local total = 0
        local dnd_total = 0
        
        last_number_of_dice = dice
        
        for d = 1,dice do
            rolls[d] = math.random(6)
            total = total+rolls[d]
        end
        
        clear_status_line(1)
        clear_status_line(2)
    
        if total == dice then 
            if total == 2 then
                set_status_line(1,0, "You rolled snake-eyes!") 
            else
                set_status_line(1,0, "You rolled")
                set_status_line(2,0, "weird-freaky-"..dice.."-eyed-snake-eyes!") 
            end
        else
            local numbers = ""
            local lowest = 6
            for d = 1,dice do
                numbers = numbers..tostring(rolls[d])
                if d ~= dice then numbers = numbers..", " end
                
                if rolls[d] < lowest then lowest = rolls[d] end
                
            end
            
            set_status_line(1,0, "You rolled "..numbers.." for a")
            
            if dice == 4 then
                -- show D&D ability score as well
                set_status_line(2,0, "total of "..total.." (D&D ability:"..total-lowest..")")
            else
                set_status_line(2,0, "total of "..total)
            end
            
        end
        
        return total  -- keeps menu open (everything except nil and false is true)
    end

    local snake_eyes = "\x1b[33m\x1b[1m  SNAKE-EYES!"
    local menu
    
    local function roll_snake_eyes()
        
        if last_number_of_dice == nil then 
            clear_status_line(1)
            clear_status_line(2)
            set_status_line(1,0, "You must roll some dice first")
            return true 
        end
        
        local total
        local count = 0
        repeat
            total = dice_roll(gulp.app, last_number_of_dice)
            count = count + 1
        until (total == last_number_of_dice)
    
        menu:update_menu_string(9, snake_eyes.." ("..tostring(count).." rolls)")
    
        return true  -- keeps menu open
    end
    
    local function _switch_status_colours(app)
        switch_status_colours(app)
    end
    
    local function _switch_maze_colours(app)
        gulp.localplayer:action_switch_maze_colours()
    end
        
    local test_menu
    
    menu = Menu{
        background = gulp.colour.BLUE.simple_colour, 
        foreground = gulp.colour.BRIGHT_YELLOW.simple_colour,
        border_style = 2,
        { "\x1b[32mCommands Available :- " },
        { "" },
        { "\x1b[37m\x1b[1m  Use LOS test", function()
                gulp.localplayer.selected_fire_action = Player.action_los_test
                end },
        { "\x1b[37m\x1b[1m  Test Iterators", function()
                gulp.localplayer.selected_fire_action = Player.action_test_iterators
                end },
        { "\x1b[33m\x1b[1m  ROLL 2 SIX SIDED DICE", dice_roll, gulp.app, 2 },
        { "\x1b[33m\x1b[1m  ROLL 3 SIX SIDED DICE", dice_roll, gulp.app, 3 },
        { "\x1b[33m\x1b[1m  ROLL 4 SIX SIDED DICE", dice_roll, gulp.app, 4 },
        { "\x1b[33m\x1b[1m  ROLL 5 SIX SIDED DICE", dice_roll, gulp.app, 5 },
        { snake_eyes, roll_snake_eyes },
        { "\x1b[37m\x1b[1m  DISPLAY PLAYERS", display_playerlist_ex, gulp.localplayer.map_name, gulp.localplayer},
        { "\x1b[37m\x1b[1m  WAIT A SECOND", 7},
        { "\x1b[37m\x1b[1m  PAINTSHOP SWATCH", _switch_status_colours},
        { "\x1b[37m\x1b[1m  SECRET OPTION", _switch_maze_colours },
        { "\x1b[31m\x1b[1m  NESTED MENU EXAMPLE", function() 
                test_menu = Menu:create_test(gulp.app)
                test_menu:position(4,4)
                return true     -- this does the magic for nesting
                end  },
        { "\x1b[31m\x1b[1m  SWITCH TO MENU", function() 
                test_menu = Menu:create_test(gulp.app) 
                test_menu:position(4,4)
                end  },
        { "\x1b[33m\x1b[1m  SAVE AND QUIT", function()  gulp.save_and_quit_game() end },
    }
    

end



