-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

require("strict")
local class = require("middleclass")
local text_sizes = require("utilities/text_sizes")

local text_buffer = class("text_buffer")

function text_buffer:initialize(specified_text_sizes)
    self.text_sizes = specified_text_sizes or text_sizes
    self._lines = {}
    self._times = {}
    self._lengths = {}
    self._colour = {}
    self._last = 0
    self._size = 20
    --self._pos = 23
    self._timed_hide = true
    self._time = 0
    gulp._delegate_list:add_delegate(self)    
end

function text_buffer:modify_text_sizes(specified_text_sizes)
    self.text_sizes = specified_text_sizes or text_sizes
end

function text_buffer:_delete_item(i)
    self._lines[i] = nil
    self._times[i] = nil
    self._lengths[i] = nil
    self._colour[i] = nil
end


function text_buffer:add_line(text, colour)
    self:_delete_item(self._last - self._size)
    self._last = self._last + 1
    self._lines[self._last] = text
    self._times[self._last] = self._time -- could have used gulp_cpp.SDL_GetTicks() but wraps after 49 days
    self._lengths[self._last] = ANSI_printable_len(text)
    self._colour[self._last] = colour or "WHITE"
end

function text_buffer:update(dt)
    self._time = self._time + dt
end

function text_buffer:timed_disappear(timed_disappear_enable)
    self._timed_hide = timed_disappear_enable
end

function text_buffer:draw(gr)
    self.text_sizes:set_viewport(gr)
 
 -- Minus 0.9 to ensure text can be seen at edge of screen
    gr:set_wrap_limits(0, math.floor(self.text_sizes:lines()), 
        0, self.text_sizes:columns()-0.9)
    
    local wrap = gr:get_wrap()
    gr:wrap(true)
    
    local time = self._time - 30        -- disappear
    local time2 = self._time - 25        -- start to fade
    local hide = self._timed_hide
    local i = self._last
    --local ln = self._pos
    local ln = self.text_sizes:lines() - 1
    
    while ln >= (1) do
        local l = self._lines[i]
        if not l then break end
        
        local line_time = self._times[i]
        local fade = 255                -- text slightly transparent, usually
        local fade_bg = fade - 100      -- don't make this negative to start!
        if hide then
            if line_time < time2 then
                if line_time < time then
                    break
                end
                fade = fade * ((time - line_time) / (time - time2))
                
                -- fade-100 has a wierd effect where the box fades out before the text.
                -- If we fade_bg = fade/2 then the box doesn't fade quickly enough.
                -- 
                fade_bg = fade - 100
                if fade_bg < 0 then fade_bg = 0 end
            end
        end
        
        local height = math.floor( math.abs(self._lengths[i] - 1) / math.floor(self.text_sizes:columns()))
        ln = ln - height
        
        goto_position(ln, 0)
        set_bg_colour{r=0,g=0,b=0,a=fade_bg}
        set_fg_colour(self._colour[i])
        gr:SetTextureAlphaMod(0, fade);
        ANSI_print(l)
        gr:SetTextureAlphaMod(0, 255);
        ln = ln - 1
        i = i - 1
    end
    
    gulp.set_wrap_limits()
    gr:wrap(wrap)
end

function text_buffer:clear()
    for i = 0, (self._size + 1) do
        self:_delete_item(self._last - i)
    end
end


return text_buffer
