-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--


function generate_maze(level, map, mode)
    
    local exit_count = 0
    local finished_count = 0
    local master = nil
    local exit_stacks = {}
    local heads = {}
    local joined = {}
    local joined_to_master = {}
    local finished = {}
    
    local directions = {
            { line = -1, column =  0 },      -- up
            { line =  1, column =  0 },      -- down
            { line =  0, column = -1 },      -- left
            { line =  0, column =  1 },      -- right
        }
    
    local alldirections = {
            { line = -1, column =  0 },      -- up
            { line =  1, column =  0 },      -- down
            { line =  0, column = -1 },      -- left
            { line =  0, column =  1 },      -- right
            { line = -1, column = -1 },      -- 
            { line =  1, column =  1 },      -- 
            { line = -1, column =  1 },      -- 
            { line =  1, column = -1 },      -- 
        }
    
    local function check_location(direction, old_line, old_column, path)
        --print("from:",old_line, old_column, ", check:", line, column, " in path ", path)
        local line = old_line+direction.line
        local column = old_column+direction.column
                    
        
        -- to be valid, a location must be blank
        -- also need to check level[line], because in the case of exits in the top or bottom of the screen,
        -- the first round of pathwalking will check locations that are off the top and bottom of the level
        -- this is an example of programmer efficiency vs program efficiency, this check could be eliminated
        -- by being more clever about setting up the initial conditions for the pathwalking
        if level[line] == nil or level[line][column] ~= 'blank' then 
            --print("not blank")
            return false 
        end
        
        local diagonally_adjacent = nil
        
        -- it must also not already border the path in question
        -- and it must not be next to 2 cells in the path it is joining
        for _,dir in ipairs(alldirections) do
            
            local new_line = line+dir.line
            local new_column = column+dir.column
            local cell = level[new_line][new_column]
            
            if type(cell) == 'number' then
                
                if cell ~= path and dir.line ~= 0 and dir.column ~= 0 then
                    -- we are diagonally adjacent to another path, which
                    -- is acceptable only if we end up joining that path
                    if diagonally_adjacent == nil then
                        diagonally_adjacent = cell
                    else
                        -- if we are diagonally adjacent to more than one 
                        -- path, then give up, it is too hard
                        return false
                    end
                end
                

                -- don't include the cell that we originate from in
                -- our analysis
                if not ((new_line==old_line and column==old_column) or
                        (line==old_line and new_column==old_column))then
                    
                    if  cell == path  then 
                        --print("rejoins")
                        return false 
                    end
                    
                    if cell >= 1 and cell <= exit_count then 
                        -- we have encountered another path
                        if joined[path][cell] then 
                            -- if we already joined this path, this cell is no-go
                            --print("would join but already joined")
                            return false            
                        elseif joined_to_master[path] then
                            -- if we are already joined to master, don't join another path
                            return false
                        elseif (dir.line==0 or dir.column==0) then
                            -- we can join providing we are looking straight (no diagonal joins)
                            direction.join = {path, cell}
                            --print("potential join :", path, cell, line, column)
                        end
                    end
                end
            end
        end
        
        -- check for diagonally adjacency without a join (to the same path)
        if diagonally_adjacent ~= nil then  
            if direction.join == nil then 
                --print("path ", path, " is diagonally adjacent to ", diagonally_adjacent, " no join")
                return false
            elseif diagonally_adjacent ~= direction.join[2] then
                --print("path ", path, " is diagonally adjacent to ", diagonally_adjacent, " but joining ", direction.join[2])
                return false
            end
        end
        
        -- we haven't found a reason to disallow this path, so let it go
        return true
    end
                
            
    -- count the number of exits, create a stack for each, and record the exit locations
    for exit_char,exit in pairs(map.procedural.exits) do
        
        exit_count = exit_count + 1
        exit_stacks[exit_count] = {}
        table.insert(heads,{line=exit.line+1,column=exit.column+1})  -- crappy hacking for 0 based exits in maps!!
        
        local temp={}
        for _,_ in pairs(map.procedural.exits) do
            table.insert(temp, false)
        end
        table.insert(joined, temp)     -- all paths are unjoined to start with
        
        table.insert(finished, false)     -- all paths are unfinished to start with
        
    end

    while finished_count ~= exit_count do
        -- for each path we are building (from each exit)
        for path=1, exit_count do
            
            if(not finished[path]) then
            
                -- look at the head (i.e. where we are now) and check places we could go
                -- a neighbouring location is valid if it is blank, and does not already border THIS path
                -- it is acceptable (in fact necessary) to join another path ONCE only
            
                local line = heads[path].line
                local column = heads[path].column
                local allowable_directions = {}
                
                for _,dir in ipairs(directions) do
                    -- find all allowable directions
                    dir.join = nil
                    if (check_location(dir, line, column, path)) then
                        table.insert(allowable_directions, dir)
                    end
                end
                
                -- and select one at random
                if #allowable_directions == 0 then
                    -- backtrack on stack
                    local sp = #exit_stacks[path]
                    
                    if sp == 0 then
                        -- stack empty
                        if finished[path] == false then
                            finished[path] = true
                            finished_count = finished_count + 1
                            --print("finished ", path)
                        end
                    else
                        -- pop the stack
                        heads[path].line = exit_stacks[path][sp][1]
                        heads[path].column = exit_stacks[path][sp][2]
                        table.remove(exit_stacks[path])
                    end
                    
                        
                    
                else
                    local direction = random_entry(allowable_directions)
                    --print("path:", path, " line:", line, " column:", column, " direction:", direction.line, direction.column, " allowable:", #allowable_directions)
                    line = line+direction.line
                    column = column+direction.column
                    level[line][column] = path
                    
                    if direction.join ~= nil then
                        local path = direction.join[1]
                        local cell = direction.join[2]
                        --print("paths join    :", path, cell, line, column)
                        
                        -- the path is joined
                        joined[path][cell] = true
                        joined[cell][path] = true
                        
                        -- we use the 'master' path to ensure that all paths are connected together
                        -- even if we have joined another path, we are allowed to join new paths
                        -- until we are joined to the master
                        
                        -- however, all paths being joined to master is not an exit condition of the 
                        -- routine, so occasionally we might see some paths that do not meet the exit
                        
                        -- also, because we do not recurse when marking paths joined to master, it is
                        -- currently normal for the routine to terminate without all paths being marked
                        -- as joined to master, although typically it seems that they are
                        
                        -- if we need to add in the exit condition that all paths are connected, then
                        -- we will need to fix this recursion issue first
                        
                        -- right now I am not bothering because it seems to work well enough
                        
                        -- if master has not been set, then the path we are joining is master,
                        -- and it is itself joined
                        if master == nil then
                            -- both this path and the path we join are marked as joined     
                            master = cell
                            joined_to_master[path] = true
                            joined_to_master[cell] = true
                            
                            --print(path, " and ", cell, " joined to master")
                        else
                            -- the path we join is only marked 'joined to master' if the joining path is joined to master
                            -- this should really be recursive, lack of recursion is resulting in loops in some maps
                            if joined_to_master[cell] == true then
                                joined_to_master[path] = true
                            
                                --print(path, " joined to master")
                            end
                        end
                        
                        -- the following two chunks of code stop the carving of all the 'wrong' paths in the maze
                        -- as soon as the two paths join, so you get more like a passage with very few branches
                        if mode == 'tunnels' then
                            if finished[path] == false then
                                finished[path] = true
                                finished_count = finished_count + 1
                                --print("finished")
                            end
                            if finished[cell] == false then
                                finished[cell] = true
                                finished_count = finished_count + 1
                                --print("finished")
                            end
                        end
                    end    
                    
                    local stack_entry = {line,column}
                    table.insert(exit_stacks[path], stack_entry)
                    
                    heads[path].line = line
                    heads[path].column = column
                    
                    -- remove any joins from the direction table!!
                    directions[1].join = nil
                    directions[2].join = nil
                    directions[3].join = nil
                    directions[4].join = nil
                    
                end
            end
        end
    end
end


