
-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

require('procedural_generation/maze_generator')
require('procedural_generation/slime_maze_generator')
require('procedural_generation/connection_generator')
require('utils')

local function generate_empty_level(map)
    -- create array of specified size, complete with outside walls and exits
    local width = map.procedural.width
    local height = map.procedural.height
    local level = {}
    
    for line=1,height do
        
        local line_table = {}
        
        for column=1,width do
            local cell = 'blank'
            if line==1 or column==1 or line==height or column==width then
                cell = 'wall'
            end
            
            -- check if this is an exit
            if map.procedural.exits then
                for exit_char, exit in pairs(map.procedural.exits) do
                    if exit.column == column-1 and exit.line == line-1 then     -- -1s because of silly zero based exit placements
                        cell = exit_char
                    end
                end
            end
            
            line_table[column] = cell
        end
        
        level[line] = line_table
    end

    return level
end

local function load_predefined_chunks(level, map)
    -- copy chunks into level data
    if map.chunks then
        for _, chunk in pairs(map.chunks) do
            
            local chunk_width = chunk[1]:len()
            local chunk_height = #chunk
            
            for line=1, chunk_height do
                for column=1, chunk_width do
                    local element = chunk[line]:sub(column, column)
                    if element == 'b' then element = 'blank' end
                    level[chunk.line+line-1][chunk.column+column-1] = element
                end
            end
        end
    end
end


local function convert_to_loadable_char(cell)
    if type(cell) == 'number' then
        --[[
        if cell==1 then return 'w' end
        if cell==2 then return 'x' end
        if cell==3 then return 'y' end
        if cell==4 then return 'z' end
        if cell == 1001 then return 'W' end
        if cell == 1002 then return 'X' end
        if cell == 1003 then return 'Y' end
        if cell == 1004 then return 'Z' end
        --]]
        return ' '
    elseif cell == 'wall' then
        return '#'
    elseif cell == 'passage' then
        return ' '
    elseif cell == 'blank' then
        return '#'
    else 
        -- no mapping - handles exits and other items
        return cell
    end
end


local function convert_to_loadable(level, map)

    local width = map.procedural.width
    local height = map.procedural.height
    
    for line=1,height do
        for column=1,width do
            if map[line] == nil then
                map[line] = convert_to_loadable_char(level[line][column])
            else
                map[line] = map[line] .. convert_to_loadable_char(level[line][column])
            end
        end
    end
end

function procedurally_generate(map)
    
    local level = time_function("generate_empty_level: \t"..map.title, generate_empty_level, map)    --generate_empty_level(map)
    
    if map.procedural.map_type == 'maze' then
        time_function("generate_maze: \t"..map.title, generate_maze, level, map)    --generate_maze(level, map)
    elseif map.procedural.map_type == 'tunnels' then
        time_function("generate_maze: \t"..map.title, generate_maze, level, map, 'tunnels')    --generate_maze(level, map, 'tunnels')
    elseif map.procedural.map_type == 'slime' then
        time_function("generate_slime_map: \t"..map.title, generate_slime_map, level, map)    --generate_slime_map(level, map)
    end
    
    time_function("load_predefined_chunks: \t"..map.title, load_predefined_chunks, level, map)    --load_predefined_chunks(level, map)
    
    -- connected maps have to have procedural generation after the chunk loading
    if map.procedural.map_type == 'connected' then
        time_function("generate_connections: \t"..map.title, generate_connections, level, map)    --generate_connections(level, map)
    end
    
    time_function("convert_to_loadable: \t"..map.title, convert_to_loadable, level, map)    --convert_to_loadable(level, map)
    
end

