-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--


-- rooms are rectangular
function generate_room(level, size, x, y)
    
    for line = y, y+size-1 do
        for column = x, x+size-1 do
            level[line][column] = ' '
        end
    end

end

    local directions = {
            { line = -1, column =  0 },      -- up
            { line =  1, column =  0 },      -- down
            { line =  0, column = -1 },      -- left
            { line =  0, column =  1 },      -- right
        }

-- caverns are rough
function generate_cavern(level, map, area, x, y)
    
    for cell = 1, area do
        local line = y
        local column = x
        
        while level[line][column] == ' ' do
            local dir = random_entry(directions)
            line = line+dir.line
            column = column+dir.column
            
            if line < 1 or line > map.procedural.height then
                line = y
            end
            if column < 1 or column > map.procedural.width then
                column = x
            end
            
        end
        
        if math.random(10) == 10 then
            level[line][column] = '#'
        else
            level[line][column] = ' '
        end
    end
end


function generate_path(level, map, median_line, start_column, end_column)
    -- generate a random path 
    local column = start_column
    local line = median_line
    
    while column ~= end_column do
        -- pick a direction
        -- roll D100
        
        local back_window = 10
        local down_window = 30
        local up_window = 30
        
        -- adjust up / down probabilities based on deviation from median line
        local line_deviation = median_line - line
        if line_deviation < 0 then
            -- path is below the line
            down_window = down_window + (line_deviation*5)
        else
            -- path is above the line
            up_window = up_window - (line_deviation*5)
        end
        
        
        -- disallow cavern formation by the paths
        --print(line, column)
        if level[line-1][column-1] == median_line or level[line+1][column-1] == median_line then
            back_window = 0
        end
        if level[line-1][column-1] == median_line or level[line-1][column+1] == median_line then
            up_window = 0
        end
        if level[line+1][column-1] == median_line or level[line+1][column+1] == median_line then
            down_window = 0
        end
        
        
        
        -- select direction based on D100 roll
        local D100 = math.random(0,99)
        down_window = down_window+back_window
        up_window = up_window+down_window
        
        if D100 < back_window then
            column = column-1
        elseif D100 < (down_window) then
            line = line+1
        elseif D100 < (up_window) then
            line = line-1
        else
            column = column+1 
        end
        
        if line < 2 then line = 2 end
        if line > map.procedural.height-1 then line = map.procedural.height-1 end
        if column < 2 then column = 2 end
        if column > map.procedural.width-1 then column = map.procedural.width-1 end
        
        level[line][column] = median_line
    
    end
end


function generate_slime_map(level, map)
    
    -- entrance, exit and other predefined areas provided by generic chunk loader
    
    -- generate a few more caverns
    for cavern = 1, map.procedural.width/50 do
        generate_cavern(level, map, 81, cavern*50, math.random(7, map.procedural.height-7))
    end
    
    -- generate paths
    generate_path(level, map, 5,  17, map.procedural.width-5)
    generate_path(level, map, 11, 17, map.procedural.width-5)
    generate_path(level, map, 17, 17, map.procedural.width-5)
    
    
end


