-- Forlorn Fox
-- Copyright 2017 Rob Probin
-- No redistribution without permission.
--
-- @todo: 
--   * run alternate
--   * switch to alternate on boot
--       - Does shift key stop switch (or bring up game select menu?)
--   * Fix update when menu in background


--   * Sign manefest 
--      * vs talking to a https server? Maybe a signed manifest would be good enough?
--      * https effectively ensures it's our server ... but not necessary our manesfest (if server hacked)
--  Application Download will be covered by SHA2 and application might be covered by platform signing


--   * Abort download if half way through and very slow…        - FUTURE
--   * check if already downloaded and valid    - FUTURE


local lunajson = require ("thirdparty/json/lunajson")
require('utils')
local copas = require("copas")
local asynchttp = require("copas.http").request
require('strict')
local ltn12 = require('ltn12')
--local http = require('socket/http')

class = require('middleclass')

--local onlineServerAddress = "http://forlorn.wubbleu.co.uk/"
local onlineServerAddress = "http://forlornfox.com/dl_files/"

local game_manifest_file = "forlorn_fox_manifest.txt"
local game_manifest_sig = "forlorn_fox_manifest.sig"

require("networking/netstat")
local exec_functions = require("utilities/exec_functions")

local dialog = require("gui/dialog")
local file_utils = require("utilities/file_utils")


-- signing a manifest:
-- 
-- Single step method for creating signature:
-- openssl dgst -sha256 -sign private.pem  -out forlorn_fox_manifest.sig forlorn_fox_manifest.txt
--
-- Verify (unencrypt only)
-- openssl rsautl -verify -inkey public.pem -pubin -keyform PEM -in forlorn_fox_manifest.sig
--
-- Show the contents of the verify unencrypt:
-- openssl rsautl -verify -inkey public.pem -pubin -keyform PEM -in forlorn_fox_manifest.sig -asn1parse
--
-- Two step method (but doesn't include hash type info or random stuff)
-- openssl dgst -sha256 forlorn_fox_manifest.txt
-- openssl dgst -sha256 -binary forlorn_fox_manifest.txt > hash_manifest2
-- openssl rsautl -sign -inkey private.pem -keyform PEM -in hash_manifest2 > signature_manifest2
--
-- 

local test_manifest_string = [[
{
  "magic": 1234,
  "type": "FF_manifest",
  "latest_stable": "0.08-DEV",
  "versions": [
    {
      "version": "0.08-DEV",
      "system": "MacOSX",
      "platform_type": "Desktop",
      "location": "http://forlornfox.com/dl_files/ff/ForlornFox-0.06-DEV-Mac.zip",
      "execpath": "ForlornFox-0.06-DEV-Mac/Forlorn Fox 0.06-DEV.app/Contents/MacOS/Forlorn Fox",
      "runpath" : "ForlornFox-0.06-DEV-Mac",
      "checktype": "md5",
      "check":"2203dae31ee90f5828a82860a2656a35"
    },
    {
      "version": "0.08-DEV",
      "system": "Windows",
      "platform_type": "Desktop",
      "location": "http://forlornfox.com/dl_files/ff/ForlornFox-0.06-DEV-Win32.zip",
      "execpath": "ForlornFox-0.06-DEV-Win32/ForlornFox-0.06-DEV.exe",
      "runpath" : "ForlornFox-0.06-DEV-Win32",
      "checktype": "md5",
      "check":"91127237f7e89b90759a8c9704483396"
    },
    {
      "version": "0.08-DEV",
      "system": "Linux",
      "platform_type": "Desktop",
      "buildsize": 32,
      "location": "http://forlornfox.com/dl_files/ff/ForlornFox-0.06-DEV-Linux-i386.tar.gz",
      "execpath": "ForlornFox-0.06-DEV-Linux-i386/ForlornFox-0.06-DEV",
      "runpath" : "ForlornFox-0.06-DEV-Linux-i386",
      "checktype": "md5",
      "check":"a9d1327aab3c18a019759d1ee95ef5a9"
    },
    {
      "version": "0.08-DEV",
      "system": "Linux",
      "platform_type": "Desktop",
      "buildsize" : 64,
      "location": "http://forlornfox.com/dl_files/ff/ForlornFox-0.06-DEV-Linux-x86_64.tar.gz",
      "execpath": "ForlornFox-0.06-DEV-Linux-x86_64/ForlornFox-0.06-DEV",
      "runpath" : "ForlornFox-0.06-DEV-Linux-x86_64",
      "checktype": "md5",
      "check":"8275a73018dd133b69a533b55326ddf6"
    }
  ]
}
]]

local function print_hexstring(s)
    local out = ""
    for a in s:gmatch"." do
        out = out .. string.format("%02x", a:byte())
    end
    print(out)
    return out
end


local function get_numeric_version(ver)
    if type(ver) ~= "string" then
        return nil
    end
    local i = ver:find("[^%d%.]")
    if i ~= nil then
        ver = ver:sub(1, i-1)
    end -- else doesn't contain anything that is not a digit or a decimal point
    if ver == "" then
        return nil
    end
    
    return tonumber(ver)
end

--[[
function sink.chain(f, snk)
  return function(chunk, err)
    local r, e = snk(f(chunk))
    if not r then return nil, e end
    if not chunk then return snk(nil, err) end
    return 1
  end
end
--]]

local function make_progress_func(size_obj)
    return function (chunk)
        -- first two cases are to maintain chaining logic that
        -- support expanding filters (see below)
        if chunk == nil then
            return nil
        elseif chunk == "" then
            return ""
        else
            -- process chunk and return filtered data
            size_obj.current_size = size_obj.current_size + chunk:len()
            --print(size_obj.current_size)
            return chunk
        end
    end
end

local function get_the_file(the_url, target_file_handle, size_obj)
    --local sink_f, sink_t = ltn12.sink.table()
    local progress_sink = ltn12.sink.chain(make_progress_func(size_obj), ltn12.sink.file(target_file_handle))
    
    local res, err, header, status = asynchttp{url = the_url, sink = progress_sink }
    
    if res then     -- res will be 1 or nil for complex method
        --print(sink_t)
        --local s = table.concat(sink_t)
        return err == 200
    else
        dialog("Problem getting file", status)
    end
end


local function get_size(the_url)
    --local sink_f, sink_t = ltn12.sink.table()
    local sink_f = ltn12.sink.null()
    local res, err, header, status = asynchttp{ method = "HEAD", url = the_url, sink = sink_f }
    --local res, err, header, status = http.request{ method = "HEAD", url = the_url, sink = sink_f }
    
    if res then     -- res will be 1 or nil for complex method
        --print(sink_t)
        --local s = table.concat(sink_t)
        return header["content-length"]
    else
        print("updater get_size() download error", err, status)
    end
end

local function code_updater(result_table)
    -- get string from https server
    -- or http at the moment
    
    -- The actual request to the server
    local res, err, header,status = asynchttp(onlineServerAddress .. game_manifest_file)
    local res2, err2, header2, status2 = asynchttp(onlineServerAddress .. game_manifest_sig)

    -- If we got data from the server
    if res and res2 and err == 200 and err2 == 200 then

        -- print(res)
        
        -- http://notebook.kulchenko.com/programming/https-ssl-calls-with-lua-and-luasec
        -- https://github.com/brunoos/luasec/wiki
        --local res = test_manifest_string
        
        -- check the downloads are resonable sizes before doing anything
        if res:len() > 123456 or res2:len() > 10000 then
            print("Manifest files too big - ignoring")
            return
        end
        
        -- final the hash of the manifest file
        local manifest_hash = gulp_cpp.SHA256()
        manifest_hash:update(res)
        local manifest_hash_result = manifest_hash:final()
        --print_hexstring(manifest_hash_result)
        --print("-----------------")
        --print_hexstring(res2)
        
        local enable_manifest_sign_check = false
        if enable_manifest_sign_check then
            local decoded_hash = res2:decode_signature(public_key)
            --print("-----------------")
            --print_hexstring(decoded_hash)
            
            -- Ignore the ASN1 stuff, if present.
            -- This is a hack ... maybe decode properly?
            -- https://github.com/kunkku/lua-asn1
            if decoded_hash > 32 then
                decoded_hash = decoded_hash:sub(-32)
            end
            
            -- could try more than one public key here... like in a loop
            if manifest_hash_result ~= decoded_hash then
                print("Don't trust manifest file - ignoring")
                return
            end
        end

        local manifest = lunajson.decode(res)

        --print_table(manifest)
        --print(manifest)
        
        if manifest['magic'] == 1234 and manifest["type"] == "FF_manifest" and 
            manifest["latest_stable"] and manifest["versions"] and type(manifest["versions"]) == "table" then
                
                --print("manifest OK")
                local latest_stable_string = manifest["latest_stable"]
                local latest_stable = get_numeric_version(latest_stable_string)
                --get_numeric_version("")
                --get_numeric_version({})
                local current = get_numeric_version(gulp.forlorn_fox_version)
                
                if latest_stable ~= nil then
                    --print(latest_stable, current)
                    
                    if latest_stable > current then
                        --print("Version is later than current")
                        for k,v in ipairs(manifest['versions']) do
                            if v['version'] == latest_stable_string and 
                            v['system'] == gulp.platform and 
                            v['platform_type'] == gulp.platform_type and
                            ( v['buildsize'] == nil or v['buildsize'] == gulp.buildsize)
                            then
                                result_table._location = v['location']
                                result_table._checktype = v['checktype']
                                result_table._check = v['check']
                                result_table._exec = v['execpath']
                                result_table._runpath = v['runpath']
                                result_table._new_version = latest_stable_string
                                result_table._expected_file_size = get_size(result_table._location)
                                if result_table._expected_file_size ~= nil and tonumber(result_table._expected_file_size, 10) 
                                and result_table._location
                                and result_table._checktype
                                and result_table._check
                                and result_table._exec 
                                and result_table._runpath
                                and result_table._new_version
                                then
                                    result_table._new_version_available = true
                                end
                                break
                            end
                            
                        end
                    end
                    
                end
        end
    else
        if not res or err ~= 200 then
            -- remove after debugging
            -- This line happens quite often...
            --print("Manifest download error", err, status)
        else
            print("Manifest signature download error", err2, status2)
        end
    end
end

local Updater = class('Updater')

function Updater:initialize()
    self._new_version_available = false
end

function Updater:start()
    -- Start thread for getting the manifest
    local mythread = copas.addthread(code_updater, self)
    name_network_item(mythread, "addthread - code_updater")
end

function Updater:new_version_available()
    return self._new_version_available
end

function Updater:clear_new_version_available()
    self._new_version_available = false
end

function Updater:new_version()
    return self._new_version
end

function Updater:expected_file_size()
    return self._expected_file_size
end

local function downloader(me)
    local download_archive_filename = me._location:match"/([^/]*)$"
    if download_archive_filename == nil then
        dialog("Failed to get filename", err)
        me.complete = true
        me.download_run = false
        return
    end
    
    me._target_file = file_utils.UpdateDataPath(download_archive_filename)
    if not me._target_file then
        dialog("Failed Construct location", "Cancelling")
        me.complete = true
        me.download_run = false
        return
    end
    
    local target_file_handle, err = io.open(me._target_file, "wb")
    if not target_file_handle then
        dialog("Failed to open file", err)
        me.complete = true
        me.download_run = false
        return
    end
    -- always check at start and after copas.sleep - might be invalid.
    if me.download_run == false then
        --print("*** Download aborted before start***")
        me.complete = true
        me.download_run = false
        return
    end
    local succeeded = get_the_file(me._location, target_file_handle, me)
    if succeeded then
        me._successful_download = true
    end
    me.complete = true
    
    --[[
    for i = 0, 120 do
        -- always check at start and after copas.sleep - might be invalid.
        if me.download_run == false then
            print("*** Download aborted ***")
            break
        end
        me._update_download_progress(me, i/100, i==119)
        copas.sleep(0.1)
    end
    --]]
    me.download_run = false
end

--function Updater:_update_download_progress(fraction, finished)
--    self.current_fraction = fraction
--    self.complete = finished
--end

function Updater:get_progress()
    self.current_fraction = self.current_size / self._expected_file_size
    return self.current_fraction, self.complete, self.current_size
end

function Updater:successful_download()
    return self._successful_download
end

function Updater:download_file()
    self.current_fraction = 0
    self.current_size = 0
    self.complete = false
    --print("*** START DOWNLOAD FILE ***")
    if self.download_run ~= true then
        self.download_run = true
        self.download_thread = copas.addthread(downloader, self)
    else
        print("Still running - ignore!")
    end
end


function Updater:check_file()
    local result = exec_functions.validate(self._target_file, self._checktype)
    if result then
        --print_hexstring(result)
        --print(self._check)
        local hex_result = result:gsub(".", function(s) return string.format("%02x", s:byte()) end)
        if hex_result ~= self._check then
            dialog("Failed to validate", "Ignoring")
            return false
        end
    end
    return true
end

local unpack_tar = require("utilities/untar")
local ZipRead = require("utilities/zip_read")

function Updater:unpack_file()
    local success, err
    local output_dir = file_utils.UpdateDataPath(self._new_version)
    
    if file_utils.isdir(output_dir) then
        -- make sure old output directory is empty
        success, err = file_utils.remove_recursive(output_dir)
        if not success then
            -- only show first error
            dialog("Can't remove old app dir", err[1])
            return false
        end
    end
    
    -- make the output direcotry
    if not file_utils.create_path_if_doesnt_exist(output_dir) then
        dialog("Failed to create the new application directory", "Ignoring")
        return false
    end

    if self._target_file:sub(-7) == ".tar.gz" then
        success, err = unpack_tar(self._target_file, output_dir)
        
    elseif self._target_file:sub(-4) == ".zip" then
        local z = ZipRead(self._target_file)
        success, err = z:is_error()
        success = not success
        if success then
            success, err = z:uncompress_all(output_dir)
        end
        z:close()
        
    else
        dialog("Failed to identify update", "Ignoring unknown type")
        return false        
    end
    
    if not success then
        dialog("Failed to unpack update", err)
        
        -- don't really care if this fails - repeats will try to delete again anyway
        file_utils.remove_recursive(output_dir) 
    end
    
    -- in all cases, we remove this update file, it's obsolete
    os.remove(self._target_file)        -- don't care if it fails to delete
    
    --write_that_its_ready_to_run(output_dir)
    
    return success
end


function Updater:stop_download_file()
    self.download_run = false
    --print("****Write stop download****") 
end


function Updater:write_run_file()
    local exec = self._exec
    local output_dir = file_utils.UpdateDataPath(self._new_version)
    local full_path = file_utils.concat_path(output_dir, exec)
    --print(full_path)
    --print(exec)
    --print(output_dir)
    if not file_utils.is_file(full_path) then
        return
    end
    local run_directory = self._runpath
    local run_full_path = file_utils.concat_path(output_dir, run_directory)
    --print(run_full_path)
    if not file_utils.is_folder(run_full_path) then
        return 
    end
    local dest = gulp_cpp.SaveDataPath:get_base_pref_path()
    local t = io.open(dest .. "run_alt", "w")
    local f
    if t then
        f = t:write(full_path) if not f then return end
        f = t:write("\n") if not f then return end
        f = t:write(run_full_path) if not f then return end
        f = t:write("\n") if not f then return end
        f = t:close()
    end
    
    return true
end

--[[
function Updater:run_new_application()
    local exec = self._exec
    local output_dir = file_utils.UpdateDataPath(self._new_version)
    local full_path = file_utils.concat_path(output_dir, exec)
    print(full_path)
    print(exec)
    print(output_dir)
    if not file_utils.is_file(full_path) then
        return nil
    end    
    return exec_functions.execute_application(cwd, full_path)
end
--]]

--
-- test code
--
local x = Updater:new()
x:start()

--return Updater
return x