-- Forlorn Fox
-- Copyright 2017 Rob Probin
-- No redistribution without permission.
--

--require('strict')


local exec_functions = {}

local lookup = 
{ 
    ['md5'] = gulp_cpp.MD5,
    ['sha224'] = gulp_cpp.SHA224,
    ['sha256'] = gulp_cpp.SHA256,
    ['sha384'] = gulp_cpp.SHA384,
    ['sha512'] = gulp_cpp.SHA512
}

function exec_functions.validate(filename, validate_type)
    local hash_type = lookup[validate_type]
    if not hash_type then
        return nil
    end
    local hash = hash_type()
    if not hash then
        dialog("Unrecognised hash", validate_type)
        return nil
    end
    
    --local SDL_GetTicks = gulp_cpp.SDL_GetTicks
    --local t1 = SDL_GetTicks()
    local f, err = io.open(filename, "rb")
    if f then
        while true do
            local data = f:read(8192)
            if not data then break end
            hash:update(data)
        end
        f:close()
    else
        dialog("File read error", err)
        return nil
    end
    --local t2 = SDL_GetTicks()
    --print("Validate time", t2-t1)
    return hash:final()
end


-- execute an application
--
-- For Windows we probably want CreateProcess/CreateProcessA   (there is ShellExecute, or ShellExecuteEx but ...)
-- 
-- For Linux we probably want fork/execv
--      https://stackoverflow.com/questions/5460421/how-do-you-write-a-c-program-to-execute-another-program#5460448
--
-- For Mac, maybe we want:
--      'open' from a shell
--      Launch Services for C.  https://developer.apple.com/documentation/coreservices/launch_services
--      NSWorkspace & NSTask in Cocoa.  
--          e.g. [[NSWorkspace sharedWorkspace] launchApplication:@"pathToApplication"];
--
-- System() is rubbish for a bunch of reasons:
--     * It most likely creates another shell that gets thrown away
--     * Resource heavy
--     * Launches with the current app user/context (probably ok)
--     * Virus checkers on Windows don't really like it (apparently?)
--     * Some lack of control
--
-- But it is cross platform ... so do we try this to start with, and refine after?
--
-- ALTERNATIVES:
--
-- io.popen (prog [, mode]) but this function is system dependent and is not available on all platforms.


local function file_exists(filename)
    local f = io.open(filename, "r");
    if f then
        f:close()
        return true
    end
    return false
end


-- returns line or nil on errors
local function read_first_lines(filename, number)
    number = number or 1
    local f = io.open(filename, "r");
    if f then
	local lines = {}
	for _ = 1, number do
            table.insert(lines, f:read("*l"))
	end
        f:close()
        return table.unpack(lines)
    end
    return  nil
end

function exec_functions.execute_run_alt()
    local prefs = gulp_cpp.SaveDataPath:get_base_pref_path()
    if prefs then
        local exe_file_to_run, set_cwd = read_first_lines(prefs .. "run_alt", 2)
        --print("File to run", exe_file_to_run)
        if exe_file_to_run and file_exists(exe_file_to_run) then

            local temp_arg = {}
            for k,v in ipairs(arg) do
               temp_arg[k] = v
            end
            --temp_arg[0] = exe_file_to_run
            
            --gulp_cpp.print_time_since_last_call()
            --gulp_cpp.print_time_since_last_call()
            
            -- On boot, things that get destroyed without closing:
            --  * This LuaState
            --  * SDL Quit without quit
            --  * GameApp (including sound manager)
            
            -- On relaunch:
            --  * 2 lua states
            --  * 2 threads
            --  * sockets
            --  * SDL window (no SDL Quit)
            --  * files open? (Maybe, maybe not)
            --print(exe_file_to_run)
            --print(table.unpack(temp_arg))
    
            local success = false
            --print("PATH>", lfs.currentdir())
            if set_cwd then
                success = lfs.chdir(set_cwd)
                --success = true
            end
            --print("PATH>", lfs.currentdir())
            if success then
                --Changes the current working directory to the given path.
                --Returns true in case of success or nil plus an error string.
                        local _, err = gulp_cpp.switch_to_program(exe_file_to_run, table.unpack(temp_arg))
                        print("Switch failed", err)
                        -- if it returns, run this version...
            else
                print("set cwd failed")
            end
        end
    end
end



--[[
local function print_hexstring(s)
    local out = ""
    for a in s:gmatch"." do
        out = out .. string.format("%02x", a:byte())
    end
    print(out)
    return out
end


local hash = gulp_cpp.MD5()
hash:update("")
local x = print_hexstring(hash:final())
if x ~= "d41d8cd98f00b204e9800998ecf8427e" then print("MISMATCH") end

local hash = gulp_cpp.MD5()
hash:update("The quick brown fox jumps over the lazy dog")
local x = print_hexstring(hash:final())
if x ~= "9e107d9d372bb6826bd81d3542a419d6" then print("MISMATCH") end

local hash = gulp_cpp.MD5()
hash:update("The quick brown fox jumps over the lazy dog.")
local x = print_hexstring(hash:final())
if x ~= "e4d909c290d0fb1ca068ffaddf22cbd0" then print("MISMATCH") end

local hash = gulp_cpp.SHA224()
hash:update("The quick brown fox jumps over the lazy dog")
x = print_hexstring(hash:final())
if x ~= "730e109bd7a8a32b1cb9d9a09aa2325d2430587ddbc0c38bad911525" then print("MISMATCH") end

local hash = gulp_cpp.SHA224()
hash:update("The quick brown fox jumps over the lazy dog.")
x = print_hexstring(hash:final())
if x ~= "619cba8e8e05826e9b8c519c0a5c68f4fb653e8a3d8aa04bb2c8cd4c" then print("MISMATCH") end

local hash = gulp_cpp.SHA224()
hash:update("")
x = print_hexstring(hash:final())
if x ~= "d14a028c2a3a2bc9476102bb288234c415a2b01f828ea62ac5b3e42f" then print("MISMATCH") end

hash = gulp_cpp.SHA256()
hash:update("")
x = print_hexstring(hash:final())
if x ~= "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855" then print("MISMATCH") end

hash = gulp_cpp.SHA384()
hash:update("")
x = print_hexstring(hash:final())
if x ~= "38b060a751ac96384cd9327eb1b1e36a21fdb71114be07434c0cc7bf63f6e1da274edebfe76f65fbd51ad2f14898b95b" then print("MISMATCH") end

hash = gulp_cpp.SHA512()
hash:update("")
x = print_hexstring(hash:final())
if x ~= "cf83e1357eefb8bdf1542850d66d8007d620e4050b5715dc83f4a921d36ce9ce47d0d13c5d85f2b0ff8318d2877eec2f63b931bd47417a81a538327af927da3e" then print("MISMATCH") end
--]]

return exec_functions

