-- Forlorn Fox reloader file
-- Copyright © 2016 Rob Probin

-- we need Lua File System to check the last modification time
local lfs = require"lfs"


--if hotswap_reload_data then
--    hotswap_reload_data_old = hotswap_reload_data
--end

-- global variable, so that we can copy from it when we reload
hotswap_reload_data = 
{
    enabled = false,
    list = {},
    time = 0,
    step = 1,
    mapped_files = {}
}

local hotswap_reload = {}

--function hotswap_reload.get_datablock()
--    return hotswap_reload_data
--end

--[[
local function copy_data_from_old(old_funcs_table)
    local old = hotswap_reload_data_old
    
    -- just data copy across verbatim
    for k,v in pairs(old) do
        hotswap_reload_data[k] = v
    end
    
    -- patch all the old functions with new ones
    for name,f in pairs(hotswap_reload) do
        old_funcs_table[name] = f
    end
end
--]]

local function get_modification_time_core(file)
    local attr = file_utils.attributes(file)
    if type(attr) ~= "table" or attr.mode ~= "file" then
        return nil
    end
    return attr.modification
end

local function get_modification_time(file)
    local save_path = gulp_cpp.SaveDataPath(file):str()
    local app_path = gulp_cpp.AppResourcePath(file):str()
    local save_time = get_modification_time_core(save_path)
    local app_time = get_modification_time_core(app_path)
    
    if save_time == nil then
        return app_time
    elseif app_time == nil then
        return save_time
    elseif save_time > app_time then
        return save_time
    else
        return app_time
    end
end


function hotswap_reload.register_class_file(filename, class_ref, post_reload_function, function_data)
    local file = "scripts/"..filename
    -- don't register twice
    local i = hotswap_reload_data.mapped_files[file]
    if i then
        local item = hotswap_reload_data.list[i]
        hotswap_reload_data.list[i].postfunc = post_reload_function
        --hotswap_reload_data.list[i].func_data = function_data
        --hotswap_reload_data.list[i].new_postfunc = post_reload_function
        hotswap_reload_data.list[i].new_func_data = function_data
        --print("--------------------")
        --print(item.new_postfunc, item.postfunc, post_reload_function, item.new_func_data, item.func_data, function_data)
        --print("====================")
        
        return
    end
    
    -- record details need to detect updates
    local time = get_modification_time(file)
    if time then
        local file_info = { filename = file, mod = time, postfunc = post_reload_function, func_data = function_data, class_r = class_ref}
        table.insert(hotswap_reload_data.list, file_info)
        hotswap_reload_data.mapped_files[file] = #hotswap_reload_data.list
    else
        print(string.format("The file '%s' has no modification time", file))
        print("Not registering", file)
    end   
end

function hotswap_reload.register_source_file(filename, post_reload_function, function_data)
    self:register_class_file(filename, nil, post_reload_function, function_data)
end


local function scan_for_changes(d)
    if #d.list == 0 then return end

    local item = d.list[d.step]
    d.step = d.step + 1
    if d.step > #d.list then d.step = 1 end
    if item then
        local filename = item.filename
        --print("Looking for changes to:", filename)
        local time = get_modification_time(filename)
        if time then
            if time > item.mod then
                print(string.format("Reloading changed file %s", filename))
                --local chunk, err = loadfile(filename)
                --if chunk == nil then
                --    print(err)
                --end
                item.mod = time
                local success, err = pcall(dofile, gulp_cpp.LoadPath(filename):str())
                if not success then
                    print("Problem reloading", name, err)
                else
                    -- loaded OK
                    if item.class_r then
                        local new_class = err
                        --[[
                        for k,v in pairs(item.class_r) do
                            print(type(k), k, type(v), v)
                        end
                        print("--------------------------")
                        for k,v in pairs(item.class_r.__instanceDict) do
                            if string.sub(k, 1, 2) ~= "__" then
                                print(type(k), k, type(v), v)
                            end
                        end
                        print("--------------------------")
                        for k,v in pairs(item.class_r.__instanceDict) do
                            if string.sub(k, 1, 2) == "__" then
                                print(type(k), k, type(v), v)
                            end
                        end
                        print("============================")
                        --]]
                        -- replace all old defintions with new ones...
                        for k,v in pairs(new_class.__instanceDict) do
                            if string.sub(k, 1, 2) ~= "__" then
                                print(type(k), k, type(v), v)
                                item.class_r.__instanceDict[k] = v
                            end
                        end
                        
                    end
                    
                    -- Call local function cleanup, with original data
                    -- These will be the latest version of the function and data just loaded
                    -- So if these are changed we will get the last conversion routines.
                    if item.postfunc then
                        item.postfunc(item.func_data)
                    end
                    
                    --print(item.new_postfunc, item.postfunc, item.new_func_data, item.func_data)
                    --
                    -- now copy across new registered function clean up for next time
                    if item.new_postfunc then
                        item.postfunc = item.new_postfunc
                        item.new_postfunc = nil
                    end
                    if item.new_func_data then
                        item.func_data = item.new_func_data
                        item.new_func_data = nil
                    end
                    --
                    --print(item.new_postfunc, item.postfunc, item.new_func_data, item.func_data)

                    print("@todo: Fix up require!!!")
                    print("@todo: What about local copies of old modules?")
                end
            end
        else
            print(string.format("The file %s has no modification time", time))
            print("Not updating")
        end
        
    end
end


function hotswap_reload.monitor_loop(dt)
    local d = hotswap_reload_data
    if d.enabled then
        d.time = d.time + dt
        if d.time > 0.5 then
            scan_for_changes(d)
            d.time = 0
        end
    end
end

function hotswap_reload.enable()
    hotswap_reload_data.enabled = true
end

function hotswap_reload.disable()
    hotswap_reload_data.enabled = false
end

-- Filename of current file: http://stackoverflow.com/questions/21416798/lua-global-variable-containing-path-to-current-file#21417614
-- 
-- NOTICE: In the case of reloading the reloader, it will run the new version with the new (blank) reload data.
-- This is a problem because we can't detect that we are reloading an old file.
-- (Other files won't have this switching problem obviously!)
-- hotswap_reload.register_source_file("utilities/hotswap_reloader.lua", copy_data_from_old, hotswap_reload)

return hotswap_reload
