-- Forlorn Fox
-- Copyright © 2015 Rob Probin, Tony Park and Kenny Osborne.
-- No redistribution without permission.
--

require("strict")
local class = require("middleclass")
local utf8 = require("utilities/gulp_utf8")
local text_sizes = require("utilities/text_sizes")

local TextEffects = class("TextEffects")

function TextEffects:initialize(row, column, text)
    self:set(row, column, text)
    
    -- default direction
    self._step = 1
    self._dir = "L"
    self:calculate_step()

end

function TextEffects:rotate(angle_in_degrees)
    self._attrs["angle"] = angle_in_degrees
end

function TextEffects:flip_updown()
    self._attrs["flip"] = 1
end

function TextEffects:flip_leftright()
    self._attrs["flip"] = 2
end

function TextEffects:flip_both()
    self._attrs["flip"] = 3
end

function TextEffects:no_flip()
    self._attrs["flip"] = nil
end

function TextEffects:move(row, column)
    self._row = row
    self._column = column
end

function TextEffects:get_position()
    return self._row, self._column
end

function TextEffects:set(row, column, text)
    self._text = text
    if text then
        self:show()
        -- decode into utf32
        self._split = utf8.to32(text)
    end
    
    self._row = row
    self._column = column
    
    self._attrs = {}
    self._colour = 7
end

function TextEffects:colour(c)
    if type(c) == "string" then
        c = get_simple_colour_by_name(c)
    end
    self._colour = c
end

function TextEffects:hide()
    if self._active then
        self._active = false
        gulp._delegate_list:drop_delegate(self)
    end
end

function TextEffects:show()
    if not self._active then
        self._active = true
        gulp._delegate_list:add_delegate(self)
    end
end

function TextEffects:narrow()
    self._attrs["scale_x"] = 0.5
    self._attrs["scale_y"] = 1
    self._step = 0.5
    self:calculate_step()
end

function TextEffects:small(size)
    size = size or 50
    size = size / 100
    self._attrs["scale_x"] = size
    self._attrs["scale_y"] = size
    self._step = size
    self:calculate_step()
end

function TextEffects:down()
    self._dir = "D"
    self:calculate_step()
end

function TextEffects:set_step(line_step, column_step)
    self._line_step = line_step
    self._column_step = column_step
    self._dir = '.'
end


function TextEffects:calculate_step()
    local direction = self._dir
    if direction == "L" then
        self._line_step = 0
        self._column_step = self._step
    elseif direction == "R" then
        self._line_step = 0
        self._column_step = -self._step
    elseif direction == "D" then
        self._line_step = self._step
        self._column_step = 0
    elseif direction == "U" then
        self._line_step = -self._step
        self._column_step = 0
    end
end


-- Too slow? we could cache these with some sort of 
-- C++ printing like display element (currently only standard printing)
function TextEffects:draw(gr)
    local line = self._row
    local column = self._column
    local colour = self._colour
    local attrs = self._attrs
    local text = self._split
    local lstep = self._line_step
    local cstep = self._column_step
    
    text_sizes:set_viewport(gr)
    
    for _, c in ipairs(text) do
        gr:printExT(line, column, colour, c, attrs)
        line = line + lstep
        column = column + cstep
    end
end


return TextEffects
