--

-- both a generic game size
-- and flexible viewport
-- 
-- Aim: Avoids having to specify cell size and instead fit how much
-- you want to print.
-- 

require('strict')
class = require('middleclass')

local text_sizes_class = class('text_sizes_class')

local text_sizes = {}
local mt = {}
setmetatable(text_sizes, mt)
mt.__index = function (t, n)
    if not text_sizes._default then
        error("setup_defaults() not called yet", 2)
    end
    return text_sizes._default[n]
end


text_sizes._list_of_new_ones = setmetatable({}, {__mode = "v"})
text_sizes._next_in_list_of_new_ones = 1

  
function text_sizes:new(characters_across, character_down)
    return text_sizes_class:new(characters_across, character_down)
end

function text_sizes_class:resize(characters_across, characters_down)
    self.characters_across = characters_across
    self.characters_down = characters_down
    self:calibrate()
end


function text_sizes.setup_defaults(characters_across, characters_down)
    text_sizes._default = text_sizes_class:new(characters_across, characters_down)
end

function text_sizes_class:calibrate()
    --local renderer = gulp.app:get_SDL_renderer()
    --print("SDL_GetWindowSize ", gulp_cpp.SDL_GetWindowSize(gulp_cpp.GetGameWindow()))
    --print("SDL_GL_GetDrawableSize", gulp_cpp.SDL_GL_GetDrawableSize(gulp_cpp.GetGameWindow()))
    --print("SDL_GetRendererOutputSize", gulp_cpp.SDL_GetRendererOutputSize(renderer))
    --print("SDL_GetDesktopDisplayMode", gulp_cpp.GetWindowWidth(), gulp_cpp.GetWindowHeight())
    
    --self.width, self.height = gulp_cpp.SDL_GL_GetDrawableSize(gulp_cpp.GetGameWindow())
    --print(">>>", text_sizes.width, text_sizes.height)
    --os.exit()
    --debugMessage(gulp_cpp.GetWindowDiagonalInches())
    local w = gulp_cpp.GetWindowWidth()
    local h = gulp_cpp.GetWindowHeight()
    
    if not self._vp then
        self._vp = gulp_cpp.Viewport()
    end
    -- whole screen
    self._vp:SetRect(0, 0, w, h)
    
    
    if self.characters_across == 0 then
        self.calc_across = true
        self.initial_down = self.characters_down
    elseif self.characters_across < 0 then
        self.calc_across = true
        self.min_across = -self.characters_across
        self.initial_down = self.characters_down
    end
    
    if self.characters_down == 0 then
        self.calc_down = true
        self.initial_across = self.characters_across
    elseif self.characters_down < 0 then
        self.calc_down = true
        self.min_down = -self.characters_down
        self.initial_across = self.characters_across
    end
    
    local across
    local down 
    local pixels
    
    if self.calc_across and self.calc_down then
        -- silly programmer...
        self.calc_across = false
        self.calc_down = false
        self.characters_across = 32
        self.characters_down = 24
    end
    
    if self.calc_across then
        
        pixels = math.floor(h / self.initial_down)
        self.characters_across = math.floor(w / pixels)
        
        if self.min_down ~= nil and self.characters_across < self.min_across then
            pixels = math.floor(w / self.min_across)
            self.characters_across = math.floor(w / pixels)
            self.characters_down = math.floor(h / pixels)
        else
            self.characters_down = self.initial_down
        end
        
    elseif self.calc_down then
    
        pixels = math.floor(w / self.initial_across)
        self.characters_down = math.floor(h / pixels)
    
        if self.min_across ~= nil and self.characters_down < self.min_down then
            pixels = math.floor(h / self.min_down)
            self.characters_across = math.floor(w / pixels)
            self.characters_down = math.floor(h / pixels)
        else
            self.characters_across = self.initial_across
        end
    
    else
        across = w / self.characters_across
        down = h / self.characters_down
        pixels = across
        if across > down then
            pixels = down
        end
        pixels = math.floor(pixels)
    end
    
    self._vp:SetCellSize(pixels)
    
    self._origin_x = (w - (self.characters_across * pixels))/2
    self._origin_y = (h - (self.characters_down * pixels))/2
    self._vp:SetOrigin(self._origin_x, self._origin_y)
    
    self._actual_columns = w/pixels
    self._actual_lines = h/pixels
    self._pixels = pixels

    self._start_x = -self._origin_x/pixels
    self._start_y = -self._origin_y/pixels
    self._end_x = self._actual_columns + self._start_x
    self._end_y = self._actual_lines + self._start_y
end

function text_sizes_class:actual_columns()
    return self._actual_columns
end

function text_sizes_class:actual_lines()
    return self._actual_lines
end

function text_sizes_class:columns()
    return self.characters_across
end

function text_sizes_class:lines()
    return self.characters_down
end
        
function text_sizes_class:window_start_column()
    return self._start_x
end

function text_sizes_class:window_start_line()
    return self._start_y
end

function text_sizes_class:window_end_column()
    return self._end_x
end

function text_sizes_class:window_end_line()
    return self._end_y
end

function text_sizes_class:get_origin()
    return self._origin_x, self._origin_y
end

function text_sizes_class:ScreenCells_to_SDL_Rect(line, column, width, height)
    
    local dstRect = gulp_cpp.SDL_Rect()
    --local dstRect = gulp_cpp.SDL_Rect.new(x, y, self.display_width, self.display_height)
    dstRect.x = self._origin_x + column * self._pixels
    dstRect.y = self._origin_y + line * self._pixels
    dstRect.w = width * self._pixels
    dstRect.h = height * self._pixels
    
    return dstRect
end

function text_sizes_class:get_bg_line_column(w, h)
    -- given the width / height of a background glyph in cells
    -- calculate what line / column and ratio to print it at to 
    -- fill the background as much as possible
    
    -- sanitise inputs
    if w==0 or h==0 then
        return 0,0,1
    end
    
    local origin_l = - ((self._actual_lines - self.characters_down) / 2)
    local origin_c = - ((self._actual_columns - self.characters_across) / 2)
    local w_ratio = self._actual_columns / w
    local h_ratio = self._actual_lines / h
    
    local ratio
    local line
    local column
    
    if w_ratio < h_ratio then
        ratio = w_ratio
        line = ((self._actual_lines - (h*ratio)) / 2) + origin_l
        column = origin_c
    else
        ratio = h_ratio
        line = origin_l
        column = ((self._actual_columns - (w*ratio)) / 2) + origin_c
    end
    
    
    return line, column, ratio
end


function text_sizes_class:set_viewport(gr)
    if not self._vp then
            local err = "FATAL - no text sizes allocated in text_sizes.lua"
            debugMessage(err)
            error(err)
    end
    gr:set_viewport(self._vp)
end

function text_sizes_class:initialize(characters_across, characters_down)
    --local me = {}
    --if type(self) ~= "table" then
    --    print("FATAL - self isn't table")
    --    error("FATAL - self isn't table")
    --end
    
    self:resize(characters_across, characters_down)
    
    -- record weak reference in list to allow resizing automatically
    local i = text_sizes._next_in_list_of_new_ones
    text_sizes._list_of_new_ones[i] = self
    text_sizes._next_in_list_of_new_ones = i+1

    --return me
end

function text_sizes_class:get_cell_size()
    return self._pixels
end

function text_sizes_class:SetPixelCoords()
    self._vp:SetPixelCoords()
    self._mode = "pixel_coords"
end

function text_sizes_class:SetCellCoords()
    self._vp:SetCellCoords()
    self._mode = "cell_coords"
end


function text_sizes.resize_all()
    
    -- resize the default one
    if text_sizes.characters_across then
        text_sizes:calibrate()
    end
    
    -- resize all new items as well
    for k,v in pairs(text_sizes._list_of_new_ones) do
        if v then
            v:calibrate()
        end
    end
end

return text_sizes
