-- Untar
-- Copyright 2017 Rob Probin
-- 
local file_utils = require("utilities/file_utils")

local function unpack_tar(source, dest_dir)
    if source:sub(-7) ~= ".tar.gz" then
        return nil, "Wrong extension for tar.gz"
    end
    local attr, err = file_utils.attributes(dest_dir)
    if attr == nil or attr.mode ~= "directory" then
        return nil, "Target directory doesn't exist for unpack tar"
    end
    attr, err = file_utils.attributes(source)
    if attr == nil or attr.mode ~= "file" then
        return nil, "Source tar.gz doesn't exist for unpack tar"
    end
    
    local f, err = io.open(source, "rb")
    if not f then
        return nil, "Failed to open tar.gz" .. err
    end
    
    local t = f:read(2)
    if t ~= "\x1f\x8b" then
        return nil, "File signature missing from tar.gz"
    end
    f:close()
    
    attr, err = os.execute(string.format('tar -zxf "%s" -C "%s"', source, dest_dir))
    return attr, err
end

-- @todo: Should we delete if something exists already? (unpack_tar())

--[[
-- test functions
print(unpack_tar("test", "output"))
print(unpack_tar("test.tar.gz", "output"))
print(unpack_tar("test.tar.gz", "zip_test_files"))
lfs.mkdir("zip_test_files/untar_test")
print(unpack_tar("Gulpman_Linux/gulpman-0.01g_i386.tar.gz", "zip_test_files/untar_test"))

local file_utils = require("utilities/file_utils")
print(file_utils.remove_recursive("zip_test_files/untar_test"))

--]]

return unpack_tar
